C grcpar.F
C All parameters that users are allowed to change are given by subroutines 
C in this file.
C----------------------------------------------------------------------------
C subroutine grcpar : BASES parameters.
C
C ibswrt = 0 : Create a new output file.
C        = 1 : Use an existing output file; i.e. skip the integration.
C icoup      : Choice of the energy scale (Q^2) for couplings.
C              if icoup = 1  ==> Q^2 = s-hat of the hard interaction.
C                       = 2  ==> Q^2 = average mT^2  (<mT^2>).
C                       = 3  ==> Q^2 = total mT^2  (sum mT^2).
C                       = 4  ==> Q^2 = maximum mT^2  (max mT^2).
C                       = 5  ==> Q^2 = a constant value.
C                       = 6  ==> user-defined Q^2.
C ifact      : Choice of the energy scale (Q^2) for PDF.
C              The definition is the same as icoup.
C              If this is not given explicitly,
C                       = 0  ==> same scale as "icoup".
C igrcgef    : ON/OFF: 0/1 for photon interference in Z/A productions.
C grcq       : Q in GeV; effective only when icoup or ifact = 5.
C ncall      : Number of sampling points in each iteration step.
C grcfile    : BASES output file name.
C----------------------------------------------------------------------------
C subroutine setmas : particle masses and widths.
C
C amz        : Z mass               agz        : Z width
C amh        : SM Higgs mass        agh        : SM Higgs width
C amdq(3)    : bottom quark mass    agdq(3)    : bottom quark width
C amuq(i) and aguq(i) for the up-type quarks (u, c and t), and amdq(i) and 
C agdq(i) for the down-type quarks (d, s and b), respectively.
C Some other parameters are set in setmas but not effective in GR@PPA_4b.
C----------------------------------------------------------------------------
C subroutine grcusrsetq : user-defined energy scale.
C
C The definition is effective only when "icoup" or "ifact" = 6.
C Users must set their-defined energy scale into "grcq" or "grcfaq".
C An example is shown to choose as Q^2 = 1/4*<Et^2>.
C----------------------------------------------------------------------------
C subroutine grcusrcut : user-defined kinematical cut.
C
C The returned argument is "iusrcut".
C If events pass their cuts, iusrcut = 0,
C       not pass             iusrcut = 1.
C Users must carefully check the convergence of the BASES iteration if they 
C define a tight cut using this subroutine.
C----------------------------------------------------------------------------
C Subprocesses inplemented in GR@PPA_4b
C
C For bbbb processes,
C [ISUB] : 400 (gg -> h0(bb)+bb(Higgs2,QCD2))
C        : 401 (qq -> h0(bb)+bb(Higgs2,QCD2))
C        : 402 (gg -> Z/A(bb)+bb(ELWK2,QCD2))
C        : 403 (qq -> Z/A(bb)+bb(ELWK2,QCD2))
C        : 404 (qq -> bbbb (Higgs2,ELWK2))
C        : 405 (gg -> bbbb (QCD4))
C        : 406 (qq -> bbbb (QCD4))
C        : 407 (qq -> Z/A(bb)+Z/A(bb)(ELWK4))
C----------------------------------------------------------------------------
C Kinematical variables provided for user-defined routines.
C See the include file, grchad.inc, for the details.
C
C Four-momentum in the CM frame of the hard interaction
C   i = 1, 2: initial-state partons
C   i = 3, 4, 5, 6: final-state b quarks
C     pgrc(1,i)  : px of i-th particle in CM frame.
C     pgrc(2,i)  : py
C     pgrc(3,i)  : pz
C     pgrc(4,i)  : E
C
C Variables in the laboratory frame
C   initial-state partons (i = 1, 2)
C     plgrc(1,i) : px
C     plgrc(2,i) : py
C     plgrc(3,i) : pz
C     plgrc(4,i) : E
C   final-state b quarks (i = 3, 4, 5, 6)
C     plgrc(1,i) : pt
C     plgrc(2,i) : phi
C     plgrc(3,i) : pseudo-rapidity (eta)
C     plgrc(4,i) : E
C----------------------------------------------------------------------------
C  _/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
C _/_/  Set Initial Parameter for Each Process  _/_/
C_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
      subroutine grcpar(isub,grcfile)
      implicit real*8(a-h,o-z)

      PARAMETER (MXDIM = 50, NDMX = 50 )
      COMMON /BPARM1/ XL(MXDIM),XU(MXDIM),NDIM,NWILD,
     $     IG(MXDIM),NCALL

      include 'grchad.inc'
      integer isub
      character grcfile*60

C...For bbbb processes,
      if (isub.eq.400) then
         ibswrt = 0
         icoup = 5
         grcq = 120.0d0
         ncall = 300000
         grcfile = 'bases_400_mh120'
      endif

      if (isub.eq.401) then
         ibswrt = 0
         icoup = 5
         grcq = 120.0d0
         ncall = 10000
         grcfile = 'bases_401_mh120'
      endif

      if (isub.eq.402) then
         ibswrt = 0
         icoup = 5
         grcq = 91.187d0
         ncall = 1000000
         grcfile = 'bases_402'
      endif

      if (isub.eq.403) then
         ibswrt = 0
         icoup = 5
         grcq = 91.187d0
         ncall = 200000
         grcfile = 'bases_403'
      endif

      if (isub.eq.404) then
         ibswrt = 0
         icoup = 5
         grcq = sqrt(120.0d0**2 + 91.187d0**2)
         ncall = 20000
         grcfile = 'bases_404_mh120'
      endif

      if (isub.eq.405) then
         ibswrt = 0
         icoup = 2
         ncall = 100000
         grcfile = 'bases_405'
      endif

      if (isub.eq.406) then
         ibswrt = 0
         icoup = 2
         ncall = 10000
         grcfile = 'bases_406'
      endif

      if (isub.eq.407) then
         ibswrt = 0
         icoup = 5
         grcq = sqrt(2.0d0)*91.187d0
         ncall = 100000
         grcfile = 'bases_407'
      endif

      return
      end

C  _/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
C _/_/  Set Mass, Width and some Parameters _/_/
C_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
      subroutine setmas
      implicit real*8(a-h,o-z)

      common/pydat2/kchg(500,3),pmas(500,4),parf(2000),vckm(4,4)

      include 'inclm.h'
      include 'inclc.h'

C...Constants
      pi    = acos(- 1.0d0 )
      pi2   = pi * pi
      rad   = pi / 180.0d0
      gevpb = 0.38937966d9

C...Set Coupling (temporary)
      alpha = 1.0d0/128.07d0
      alpha0= 1.0d0/137.0359895d0
      alphas= 0.12d0

C...Set Mass
      amw = 80.41D0            ! W
      amz = 91.187D0           ! Z
      ama = 0.0D0              ! photon
      amg = 0.0D0              ! gluon
      amh = 120.0D0            ! SM Higgs
CCC         amh = 80.0D0
CCC         amh = 160.0D0
      amx = AMW                ! New particle
      amy = AMZ                ! New particle

      amnu(1) = 0.0D0          ! electron neutrino
      amnu(2) = 0.0D0          ! muon neutrino
      amnu(3) = 0.0D0          ! tau neutrino
      amlp(1) = 0.51099906D-3  ! electron
      amlp(2) = 105.658389D-3  ! muon
      amlp(3) = 1.7771D0       ! tau

      amuq(1) = 2.0D-3         ! u quark
      amuq(2) = 1.5D0          ! c quark
      amuq(3) = 175.0D0        ! t quark
      amdq(1) = 5.0D-3         ! d quark
      amdq(2) = 0.5D0          ! s quark
      amdq(3) = 4.75D0          ! b quark

      amcm = AMW
      amcz = AMZ
      amca = AMA
      amcg = AMG

C...Set Width
C      agw = 2.1483d0           ! W
      agw = 2.047d0           ! W
      agz = 2.49D0             ! Z

      agh = 6.5378773d-3       ! SM Higgs (Mh = 120 GeV)
CCC      agh = 4.1610854d-3       ! SM Higgs (Mh = 80 GeV)
CCC      agh = 6.0677468d-2       ! SM Higgs (Mh = 160 GeV)
      agx = AGW                ! New particle
      agy = AGZ                ! New particle

      aguq(1) = 0.0D0          ! u quark
      aguq(2) = 0.0D0          ! c quark
      aguq(3) = 1.586882D0     ! t quark
      agdq(1) = 0.0D0          ! d quark
      agdq(2) = 0.0D0          ! s quark
      agdq(3) = 0.001D0        ! b quark
      agcp = AGW
      agcm = AGW
      agcz = AGZ

C...Set Mass in PYTHIA
      pmas(23,1) = amz
      pmas(25,1) = amh
      pmas(5,1) = amdq(3)

C...Set Width in PYTHIA
      pmas(23,2) = agz
      pmas(25,2) = agh
      pmas(5,2) = agdq(3)

      return
      end


C  _/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
C _/_/  Set User Defined Energy Scale   _/_/
C_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
      subroutine grcusrsetq
      implicit real*8(a-h,o-z)

      common /amjprc/jproc
      include 'grchad.inc'
      include 'inclk.h'

C...User defined Q^2

      COST = GVAL(1)
      SINT = SQRT((1.0D0 - COST)*(1.0D0 + COST))
      PHIT = GVAL(2)
 
      EQT = (GPRI(3)+GVAL(3)-GVAL(4))/2.0D0/GPRI(4)
      PQT = SQRT(EQT**2 - GVAL(3))
 
      PX = PQT*SINT*COS(PHIT)
      PY = PQT*SINT*SIN(PHIT)

      PT2 = PX**2 + PY**2

      grcq = sqrt((2.0d0*PT2 + gval(3)+gval(4))/2.0d0)
      grcfaq = grcq

CCCC...This is a sample for renormalization scale. 
CCCC...The option should be "6".
CCCC...Take Q^2 = 1/4*(Ave.Et^2).
CCC
CCCC...Loop over for the final state particles.
CCC      et = 0.0d0
CCC      do i = 3,6
CCC         pt = sqrt(pgrc(1,i)**2+pgrc(2,i)**2)
CCC         ppp = sqrt(pgrc(1,i)**2+pgrc(2,i)**2+pgrc(3,i)**2)
CCC         et = pgrc(4,i)*pt/ppp + et
CCC      enddo
CCC
CCCC...Set Q^2 = 1/4*(Ave.Et^2)
CCC      grcq = et/4.0d0

      return
      end


C  _/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
C _/_/  Set User Defined Cut  _/_/
C_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
      subroutine grcusrcut(iusrcut)
      implicit real*8(a-h,o-z)

      include 'grchad.inc'
      include 'inclk.h'
      integer iusrcut

C...User can get each particle 4-momentum
C     pgrc(1,i)  : px of i-th particle in CM frame.
C     pgrc(2,i)  : py
C     pgrc(3,i)  : pz
C     pgrc(4,i)  : E
C
C     plgrc(1,i) : pt of i-th particle in lab. frame.
C     plgrc(1,i) : phi
C     plgrc(1,i) : pseudo-rapidity
C     plgrc(1,i) : E
C
C...User must set a "iusrcut" flag if user defined cut are passed or not.
C     iusrcut = 0 : if pass
C             = 1 : if not pass
C
CCCC...Example
CCCC...Take  max Et > 50 GeV.
CCC
CCCC...Loop over for the final state particles.
CCC      etmx = 0.0d0
CCC      do i = 3,6
CCC         pt = sqrt(pgrc(1,i)**2+pgrc(2,i)**2)
CCC         ppp = sqrt(pgrc(1,i)**2+pgrc(2,i)**2+pgrc(3,i)**2)
CCC         etmx = max(etmx,pgrc(4,i)*pt/ppp)
CCC      enddo
CCCC...max Et > 50 GeV.
CCC      if (etmx.gt.50.0d0) then
CCC         iusrcut = 0
CCC      else
CCC         iusrcut = 1
CCC      endif

C      emas = sqrt(pqgrc(4,1)**2-pqgrc(1,1)**2
C     $     -pqgrc(2,1)**2-pqgrc(3,1)**2)
C
C      if (emas.gt.50.0d0.and.emas.lt.200.0d0) then
C         iusrcut = 0
C      else
C         iusrcut = 1
C      endif

      return
      end




