C Program : sample_herw.F
C Purpose : Sample code to connect to HERWIG 6.5
C Date    : Dec.9.2002
C Author  : Soushi Tsuno

      include 'herwig6500.inc'

      INTEGER N
      EXTERNAL HWUDAT
C---MAX NUMBER OF EVENTS OF THIS RUN
      MAXEV=1000
C---PROCESS
      IPROC=-1
C---INITIALISE OTHER COMMON BLOCKS
      CALL HWIGIN
C---USER CAN RESET PARAMETERS AT
C   THIS POINT, OTHERWISE DEFAULT
C   VALUES IN HWIGIN WILL BE USED.

C---COMPUTE PARAMETER-DEPENDENT CONSTANTS
      CALL HWUINC
C---CALL HWUSTA TO MAKE ANY PARTICLE STABLE
      CALL HWUSTA('PI0     ')
C---USER'S INITIAL CALCULATIONS
      CALL HWABEG
C---INITIALISE ELEMENTARY PROCESS
      CALL HWEINI
C---LOOP OVER EVENTS
      DO 100 N=1,MAXEV
C---INITIALISE EVENT
         CALL HWUINE
C---GENERATE HARD SUBPROCESS
         CALL HWEPRO
C---GENERATE PARTON CASCADES
         CALL HWBGEN
C---DO HEAVY OBJECT DECAYS
         CALL HWDHOB
C---DO CLUSTER FORMATION
         CALL HWCFOR
C---DO CLUSTER DECAYS
         CALL HWCDEC
C---DO UNSTABLE PARTICLE DECAYS
         CALL HWDHAD
C---DO HEAVY FLAVOUR HADRON DECAYS
         CALL HWDHVY
C---ADD SOFT UNDERLYING EVENT IF NEEDED
         CALL HWMEVT
C---FINISH EVENT
         CALL HWUFNE
C---USER'S EVENT ANALYSIS
         CALL HWANAL
 100  CONTINUE
C---TERMINATE ELEMENTARY PROCESS
      CALL HWEFIN
C---USER'S TERMINAL CALCULATIONS
      CALL HWAEND

      STOP
      END
C----------------------------------------------------------------------
      SUBROUTINE HWABEG
C     USER'S ROUTINE FOR INITIALIZATION
C----------------------------------------------------------------------
      END
C----------------------------------------------------------------------
      SUBROUTINE HWAEND
C     USER'S ROUTINE FOR TERMINAL CALCULATIONS, HISTOGRAM OUTPUT, ETC
C----------------------------------------------------------------------
      END
C----------------------------------------------------------------------
      SUBROUTINE HWANAL
C     USER'S ROUTINE TO ANALYSE DATA FROM EVENT
C----------------------------------------------------------------------
      END
C----------------------------------------------------------------------

      SUBROUTINE UPINIT
      IMPLICIT NONE

#include "define.h"

C...Include GR@PPA common parameter.
      INCLUDE 'grchad.inc'

C...User process initialization commonblock.
      INTEGER MAXPUP
      PARAMETER (MAXPUP=100)
      INTEGER IDBMUP,PDFGUP,PDFSUP,IDWTUP,NPRUP,LPRUP
      DOUBLE PRECISION EBMUP,XSECUP,XERRUP,XMAXUP
      COMMON/HEPRUP/IDBMUP(2),EBMUP(2),PDFGUP(2),PDFSUP(2),
     &IDWTUP,NPRUP,XSECUP(MAXPUP),XERRUP(MAXPUP),XMAXUP(MAXPUP),
     &LPRUP(MAXPUP)
      SAVE /HEPRUP/

      INTEGER I
      CHARACTER*3 CBEAM
      REAL*8 SIGMAX

#if PDFLIB == 2
C...PDF common parameter.
      CHARACTER*20      PARM(20)
      DOUBLE PRECISION  VALUE(20)
#endif

C...Colliding beams:
C   "PP " (proton on proton) or "PAP" (proton on anti-proton)
CCC      CBEAM = 'PP '
      CBEAM = 'PAP'

C...CM energy. 
      GRCECM = 1960.0D0         ! CM Energy

C...Set variables in the interface common.
      IF(CBEAM.EQ.'PP ') THEN
         IDBMUP(1) = 2212
         IDBMUP(2) = 2212
      ELSEIF(CBEAM.EQ.'PAP') THEN
         IDBMUP(1) = 2212
         IDBMUP(2) = -2212
      ENDIF
      EBMUP(1) = GRCECM/2.0D0
      EBMUP(2) = GRCECM/2.0D0

C...Kinematical cuts.
      GPTCUT   = 10.0D0         ! Pt Cut for each particles
      GETACUT  = 2.5D0          ! Eta Cut
      GRCONCUT = 0.4D0          ! RCone Cut

C   Other cuts can be applied by editing the subroutine grcusrcut 
C   in grcpar.F.

C...Set seeds for random numbers.
      IRSEED = 12345
      NZZ = 2147483647

C...External processes. 
      NPRUP = 1
      LPRUP(1) = 401            ! qq -> h0(bb)+bb(Higgs2,QCD2)

C...Set PDF.
#if PDFLIB == 1
C...This flag is only available for PYTHIA.

#elif PDFLIB == 2
C...Initialization of PDFLIB.
      PARM(1) = 'Init0'
      VALUE(1) = 0.0d0
      CALL PDFSET(PARM, VALUE)

C...Set a PDF of incoming beams: CTEQ 4L as an example.
      PARM(1) = 'Nptype'
      VALUE(1) = 1
      PARM(2) = 'Ngroup'
      VALUE(2) = 4
      PARM(3) = 'Nset'
      VALUE(3) = 32
C      PARM(4) = 'LO'
C      VALUE(4) = 1
C      PARM(5) = 'NFL'
C      VALUE(5) = 5
C      PARM(6) = 'QCDL5'
C      VALUE(6) = 0.181d0
      CALL PDFSET(PARM, VALUE)

C   Tell HERWIG the above PDFLIB setting.
      PDFGUP(1) = VALUE(2)
      PDFSUP(1) = VALUE(3)
      PDFGUP(2) = PDFGUP(1)
      PDFSUP(2) = PDFSUP(1)

#elif PDFLIB == 3
C...Initialization of LHAPDF
C   The location of an appropriate PDFset has to be given 
C   by users here.
      CALL InitPDFset('cteq6.LHpdf')
      CALL InitPDF(0)

C   HERWIG should not care about PDF (see HWIGUP).
      PDFGUP(1) = -1
      PDFGUP(2) = -1
#endif

C...Decide on weighting strategy: unweighted on input.
      IDWTUP = 1

C...Check process.
      DO I = 1,NPRUP
         IF (LPRUP(I).LT.400.OR.LPRUP(I).GT.407) THEN
            WRITE(*,*) 'GRC::Error: Wrong process ID',LPRUP(I),'.'
            STOP
         ENDIF
      ENDDO

C...Only call GR@PPA.
      DO I = 1,NPRUP
         CALL GRCPYGEN(CBEAM,LPRUP(I),1,SIGMAX)
         XSECUP(i) = SIGMAX
         XMAXUP(i) = SIGMAX
      ENDDO

C...Done.
      RETURN
      END

      SUBROUTINE UPEVNT
      IMPLICIT NONE

C...Include GR@PPA common parameter.
      INCLUDE 'grchad.inc'

C...User process event common block.
      INTEGER MAXNUP
      PARAMETER (MAXNUP=500) 
      INTEGER NUP,IDPRUP,IDUP,ISTUP,MOTHUP,ICOLUP
      DOUBLE PRECISION XWGTUP,SCALUP,AQEDUP,AQCDUP,PUP,VTIMUP,SPINUP  
      COMMON/HEPEUP/NUP,IDPRUP,XWGTUP,SCALUP,AQEDUP,AQCDUP,IDUP(MAXNUP),
     &ISTUP(MAXNUP),MOTHUP(2,MAXNUP),ICOLUP(2,MAXNUP),PUP(5,MAXNUP),
     &VTIMUP(MAXNUP),SPINUP(MAXNUP)  
      SAVE /HEPEUP/ 

      REAL*8 SIGEV

C...Only call GR@PPA.
      CALL GRCPYGEN(' ',IDPRUP,0,SIGEV)

C...Ensure proper normalization of weights for MODE=+-2
      XWGTUP = SIGEV

C...Done.
      RETURN
      END
