C Program : sample_pyth.F
C Purpose : Sample code to connect to PYTHIA 6.1xx
C Date    : Dec.10.2001
C Author  : Soushi Tsuno

C Only those subprocesses in which a Higgs boson decaying to a 
C b-quark pair is produced in association with a b-quark pair 
C production via QCD (ISUB = 400 and 401) are activated in this 
C example.

C...All real arithmetic in double precision.
      implicit double precision(a-h,o-z)
C...Three Pythia functions return integers, so need declaring.
      integer pyk,pychge,pycomp
C...EXTERNAL statement links PYDATA on most machines.
      external pydata
C...Commonblocks.
      common/pyjets/n,npad,k(4000,5),p(4000,5),v(4000,5)
      common/pypars/mstp(200),parp(200),msti(200),pari(200)
      common/pydat1/mstu(200),paru(200),mstj(200),parj(200)
      common/pysubs/msel,mselpd,msub(500),kfin(2,-40:40),ckin(200)

C...Counter for number of generated events of each type.
      dimension ncount(8)
      data ncount/8*0/

C...Include GR@PPA common parameter.
      include './inc/grchad.inc'

C...Set Ntuple-----------------------------------------------------
      integer nwpawc
      parameter (nwpawc=300000)
      common/pawc/paw(nwpawc)
      integer neve,psub
      real pj
      common/genep/neve,psub,pj(5,6)

      call hlimit(nwpawc)
      call hropen(1,'genep','bbbb_tev.nt','N',4095,istat)
      call hbnt(10,'genep',' ')
      call hbname(10,'genep',neve,'neve:i,psub:i,pj(5,6):r')
C-----------------------------------------------------------------

C...Number of events and cm energy. 
      nev = 1000                ! Number of events
      grcecm = 2000.0d0         ! CM Energy

C...Kinematical cuts.
      gptcut   = 5.0d0          ! Pt Cut for each particles
      getacut  = 3.0d0        ! Eta Cut
      grconcut = 0.1d0          ! RCone Cut
C   Other cuts can be applies using CKIN parameters of Pythia.
C   Furthermore, users can define their own cuts by editing the 
C   subroutine grcusrcut in grcpar.F.

C...PDF and Coupling.
C   These parameters have to be set before the GR@PPA_4b initialization.
      mstp(51)  = 7             ! CTEC5L
      mstp(58)  = 4             ! Nr. of flavor in PDF
      mstu(101) = 1             ! running alpha_em
      iord_als  = 1             ! first-order running alpha_s
      nfrv_als  = 5             ! Nr. of flavors assumed in alpha_s
      aLam_als  = 0.146d0       ! Lambda used in alpha_s

      mstu(111) = iord_als
      mstu(112) = nfrv_als
      paru(112) = aLam_als

C...GR@PPA_4b initialization (BASES integration).
      call grcpygen('PAP',422,1,sigmax) ! gg -> h0(bb)+bb(Higgs2,QCD2)
      call grcpygen('PAP',423,1,sigmax) ! qq -> h0(bb)+bb(Higgs2,QCD2)

C...Pythia initialization.
      msel = 0
      msub(422) = 1         ! External process on
      msub(423) = 1         ! External process on

C   Switch off unnecessary aspects of Pythia.
C      mstp(61)  = 0             ! Initial state radiation OFF
C      mstp(71)  = 0             ! Final state radiation OFF
C      mstp(81)  = 0             ! Multiple interaction OFF
C      mstp(111) = 0             ! Hadronization OFF
C   Initialization.
      call pyinit('CMS','p','pbar',grcecm)

C...The alpha_s parameters must be set again here if they are non-
C   standard. These parameters are over-written with standard ones 
C   in pyinit according the choice of PDF.
      mstu(111) = iord_als
      mstu(112) = nfrv_als
      paru(112) = aLam_als

C...Event loop.
      do iev = 1,nev

         call pyevnt

         isub = msti(1)
         icase = 1
         if (isub.ge.421 .and. isub.le.427) icase = isub - 399
         ncount(icase) = ncount(icase) + 1
         if (ncount(icase).le.1) then
            write(6,*) ' Following event is subprocess',isub
            call pylist(1)
         endif

         if (mod(iev,1000).eq.0) then
            write(*,*) iev
         endif

C...Data Store.

C...Event Nr..
         neve = iev
C...Process ID.
         psub = isub

C...Set Particle 1,2 (x,y,z).
         do i = 1,2
            do j = 1,4
               pj(j,i) = p(i+2,j)
            enddo
            pj(5,i) = k(i+2,2)
         enddo

C...Set Particle 3,4,5,6 (pt,phi,eta).
         do i = 3,6
            pj(1,i) = pyp(i+4,10)
            pj(2,i) = pyp(i+4,15)
            pj(3,i) = pyp(i+4,19)
            pj(4,i) = pyp(i+4,4)
            pj(5,i) = k(i+4,2)
         enddo

C...Fill ntuple.
         call hfnt(10)

C...End of loop over events.
      enddo

C...Cross section table.
      call pystat(1)

C...Close ntuple.
      call hrout(10,genep,' ')
      call hrend('genep')

      end


C###########################################################
      subroutine pyupev(isub,sigev)
      implicit double precision(a-h,o-z)
      integer isub
      real*8 sigev

C...Only call GR@PPA_4b.
      call grcpygen(' ',isub,0,sigev)

      return
      end

