C Program : sample_pyth_62.F
C Purpose : Sample code to connect to PYTHIA 6.2xx
C Date    : Jan.24.2003
C Author  : Soushi Tsuno

C This is a sample program to embed GR@PPA_4b in the new version of
C PYTHIA (version 6.2xx), where a new interface to external functions
C agreed at the Les Houches workshop in 2001 is implemented. See the
C PYTHIA 6.2 manual for the details. We welcome comments and bug reports.
C S. Tsuno, tsuno@fnal.gov

#include "define.h"

C...All real arithmetic in double precision.
      implicit double precision(a-h,o-z)
C...Three Pythia functions return integers, so need declaring.
      integer pyk,pychge,pycomp
C...EXTERNAL statement links PYDATA on most machines.
      external pydata
C...Commonblocks.
      common/pyjets/n,npad,k(4000,5),p(4000,5),v(4000,5)
      common/pydat1/mstu(200),paru(200),mstj(200),parj(200)
      common/pysubs/msel,mselpd,msub(500),kfin(2,-40:40),ckin(200)
      common/pypars/mstp(200),parp(200),msti(200),pari(200)

C...Counter for number of generated events of each type.
      dimension ncount(8)
      data ncount/8*0/

C...Set Ntuple-----------------------------------------------------
      integer nwpawc
      parameter (nwpawc=300000)
      common/pawc/paw(nwpawc)
      integer neve,psub
      real pj
      common/genep/neve,psub,pj(5,6)

      call hlimit(nwpawc)
      call hropen(1,'genep','bbbb_tev.nt','N',4095,istat)
      call hbnt(10,'genep',' ')
      call hbname(10,'genep',neve,'neve:i,psub:i,pj(5,6):r')
C-----------------------------------------------------------------

C...Number of events. 
      nev = 1000                ! Number of events

#if PDFLIB == 1
C...PDF and Coupling.
C   These parameters have to be set before the GR@PPA_4b initialization.
      mstp(51)  = 7             ! CTEC5L
      mstp(58)  = 4             ! Nr. of flavor in PDF
      mstu(101) = 1             ! running alpha_em
      iord_als  = 1             ! first-order running alpha_s
      nfrv_als  = 5             ! Nr. of flavors assumed in alpha_s
      aLam_als  = 0.146d0       ! Lambda used in alpha_s

      mstu(111) = iord_als
      mstu(112) = nfrv_als
      paru(112) = aLam_als
#endif

C...Pythia initialization.
C   Switch off unnecessary aspects of Pythia.
C      mstp(61)  = 0             ! Initial state radiation
C      mstp(71)  = 0             ! Final state radiation
C      mstp(81)  = 0             ! Multiple interaction
C      mstp(111) = 0             ! Hadronization

C !!! Further initialization is done in the subroutine UPINIT. !!!

C...PYTHIA initialization.
      call pyinit('USER',' ',' ',0,0d0)

#if PDFLIB == 1
C...The alpha_s parameters must be set again here if they are non-
C   standard. These parameters are over-written with standard ones 
C   in pyinit according the choice of PDF.
      mstu(111) = iord_als
      mstu(112) = nfrv_als
      paru(112) = aLam_als
#endif

C...Event loop.
      do iev = 1,nev

         call pyevnt

         isub = msti(1)
         icase = 1
         if (isub.ge.400 .and. isub.le.600) icase = isub - 399
         ncount(icase) = ncount(icase) + 1
         if (ncount(icase).le.1) then
            write(6,*) ' Following event is subprocess',isub
            call pylist(1)
         endif

C...Data Store.
         if (mod(iev,1000).eq.0) then
            write(*,*) iev
         endif

C...Data Store.

C...Event Nr..
         neve = iev
C...Process ID.
         psub = isub

C...Initial-state partons (x,y,z).
         do i = 1,2
            do j = 1,4
               pj(j,i) = p(i+4,j)
            enddo
            pj(5,i) = k(i+4,2)
         enddo

C...Produced b quarks (pt,phi,eta).
         do i = 3,6
            pj(1,i) = pyp(i+5,10)
            pj(2,i) = pyp(i+5,15)
            pj(3,i) = pyp(i+5,19)
            pj(4,i) = pyp(i+5,4)
            pj(5,i) = k(i+5,2)
         enddo

C...Fill ntuple.
         call hfnt(10)

C...End of loop over events.
      enddo

C...Cross section table.
      call pystat(1)

C...Close ntuple.
      call hrout(10,genep,' ')
      call hrend('genep')

      end

      SUBROUTINE UPINIT
      IMPLICIT NONE

C...Include GR@PPA common parameter.
      INCLUDE 'grchad.inc'

C...User process initialization commonblock.
      INTEGER MAXPUP
      PARAMETER (MAXPUP=100)
      INTEGER IDBMUP,PDFGUP,PDFSUP,IDWTUP,NPRUP,LPRUP
      DOUBLE PRECISION EBMUP,XSECUP,XERRUP,XMAXUP
      COMMON/HEPRUP/IDBMUP(2),EBMUP(2),PDFGUP(2),PDFSUP(2),
     &IDWTUP,NPRUP,XSECUP(MAXPUP),XERRUP(MAXPUP),XMAXUP(MAXPUP),
     &LPRUP(MAXPUP)
      SAVE /HEPRUP/

      INTEGER I
      CHARACTER*3 CBEAM
      REAL*8 SIGMAX

#if PDFLIB == 2
C...PDFLIB common parameter.
      CHARACTER*20      PARM(20)
      DOUBLE PRECISION  VALUE(20)
#endif

C...Colliding beams:
C   "PP " (proton on proton) or "PAP" (proton on anti-proton)
CCC      CBEAM = 'PP '
      CBEAM = 'PAP'

C...CM energy in GeV.
      GRCECM = 1960.0D0         ! CM Energy

C...Set variables in the interface common.
      IF(CBEAM.EQ.'PP ') THEN
         IDBMUP(1) = 2212
         IDBMUP(2) = 2212
      ELSEIF(CBEAM.EQ.'PAP') THEN
         IDBMUP(1) = 2212
         IDBMUP(2) = -2212
      ENDIF
      EBMUP(1) = GRCECM/2.0D0
      EBMUP(2) = GRCECM/2.0D0

C...Kinematical cuts.
      GPTCUT   = 10.0D0         ! Pt Cut for each particles
      GETACUT  = 2.5D0          ! Eta Cut
      GRCONCUT = 0.4D0          ! RCone Cut

C   Other cuts can be applies using CKIN parameters of Pythia.
C   Furthermore, users can define their own cuts by editing the 
C   subroutine grcusrcut in grcpar.F.

C...Set seeds for random numbers.
      IRSEED = 12345
      NZZ = 2147483647

C...Number of external processes and the process numbers. 
      NPRUP = 1
      LPRUP(1) = 401            ! qq -> h0(bb)+bb(Higgs2,QCD2)

C...Set PDF.
#if PDFLIB == 1
C   Setting is done in the main program.

#elif PDFLIB == 2
C...Initialization of PDFLIB.
      PARM(1) = 'Init0'
      VALUE(1) = 0.0d0
      CALL PDFSET(PARM, VALUE)

C...Set PDF's of incoming beams: CTEQ 4L.
      PARM(1) = 'Nptype'
      VALUE(1) = 1
      PARM(2) = 'Ngroup'
      VALUE(2) = 4
      PARM(3) = 'Nset'
      VALUE(3) = 32
C      PARM(4) = 'LO'
C      VALUE(4) = 1
C      PARM(5) = 'NFL'
C      VALUE(5) = 5
C      PARM(6) = 'QCDL5'
C      VALUE(6) = 0.181d0
      CALL PDFSET(PARM, VALUE)

C   To tell PYTHIA the above PDFLIB setting
      PDFGUP(1) = VALUE(2)
      PDFSUP(1) = VALUE(3)
      PDFGUP(2) = PDFGUP(1)
      PDFSUP(2) = PDFSUP(1)

#elif PDFLIB == 3
C...Initialization of LHAPDF
      CALL InitPDFset('cteq6.LHpdf')
      CALL InitPDF(0)

      PDFGUP(1) = 4
      PDFSUP(1) = 32
      PDFGUP(2) = PDFGUP(1)
      PDFSUP(2) = PDFSUP(1)
#endif

C...Decide on weighting strategy: unweighted on input.
      IDWTUP = 2

C...Check process.
      DO I = 1,NPRUP
         IF (LPRUP(I).LT.400.OR.LPRUP(I).GT.407) THEN
            WRITE(*,*) 'GRC::Error: Wrong process ID',LPRUP(I),'.'
            STOP
         ENDIF
      ENDDO

C...Only call GR@PPA.
      DO I = 1,NPRUP
         CALL GRCPYGEN(CBEAM,LPRUP(I),1,SIGMAX)
         XSECUP(i) = SIGMAX
         XMAXUP(i) = 1.0D0
      ENDDO

C...Done.
      RETURN
      END

      SUBROUTINE UPEVNT
      IMPLICIT NONE

C...Include GR@PPA common parameter.
      INCLUDE 'grchad.inc'

C...User process event common block.
      INTEGER MAXNUP
      PARAMETER (MAXNUP=500) 
      INTEGER NUP,IDPRUP,IDUP,ISTUP,MOTHUP,ICOLUP
      DOUBLE PRECISION XWGTUP,SCALUP,AQEDUP,AQCDUP,PUP,VTIMUP,SPINUP  
      COMMON/HEPEUP/NUP,IDPRUP,XWGTUP,SCALUP,AQEDUP,AQCDUP,IDUP(MAXNUP),
     &ISTUP(MAXNUP),MOTHUP(2,MAXNUP),ICOLUP(2,MAXNUP),PUP(5,MAXNUP),
     &VTIMUP(MAXNUP),SPINUP(MAXNUP)  
      SAVE /HEPEUP/ 

      REAL*8 SIGEV

C...Only call GR@PPA.
      CALL GRCPYGEN(' ',IDPRUP,0,SIGEV)

C...Ensure proper normalization of weights for MODE=+-2
      XWGTUP = 1.0D0

C...Done.
      RETURN
      END
