C        _/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
C       _/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
C      _/_/ GR@PPA (GRace system @ PP/Anti-p)                _/_/
C     _/_/  File Name  : grcpygen.F(decides all kinematics) _/_/
C    _/_/   Date       : Dec/19/2002                       _/_/
C   _/_/    Author     : Soushi Tsuno (tsuno@fnal.gov)    _/_/
C  _/_/     Institute  : Tsukuba Univ. Japan             _/_/
C _/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
C_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
C  GRCBSIT      BASES initialization.
C  GRCIVAL      Initialization of integration variables.
C  GRCIKIN      Set kinematical parameters.
C  GRCITAU      Set Max/Min of phase space of tau.
C  GRCIYST      Set Max/Min of phase space of yst.
C  GRCICOS      Set Max/Min of phase space of cosine to the beam direction.
C  GRCSPID      Set particle ID.
C  GRCSTAU      Set tau.
C  GRCSYST      Set yst.
C  GRCSCOS      Set cosine(theta).
C  GRCSQ12      Set Q^2 of first cluster.
C  GRCSQ22      Set Q^2 of second cluster.
C  GRCSSPX      Set s^, x1, and x2.
C  GRCSBCS      Set sub system cosine(theta).
C  GRCSENE      Set energy scale.
C  GRCSVAL      Set kinematical variables.
C  GRCSKIN      Set kinematics.
C  GRCFKIN2     Set 2-body kinematics.
C  GRCFKIN3     Set 3-body kinematics.
C  GRCFUSN3     Set fusion type 3-body kinematics.
C  GRCFKIN4     Set 4-body kinematics.
C  GRCFCAS4     Set cascade type 4-body kinematics.
C  GRCFUSN4     Set fusion type 4-body kinematics.
C  GRCFKIN5     Set 3-body + 2-body = 5-body kinematics.
C  GRCFCAS5     Set cascade type 5-body kinematics.
C  GRCFUSN5     Set fusion type 5-body kinematics.
C  GRCFKIN6     Set 3+3 -> (2+1)+(2+1) kinematics.
C  GRCPYPRO     Set input parameters and process IDs.
C  GRCSDCY      Set decay products.
C  GRCPYGEN     Perform Integration/event generation.
C  GRCLOGO      Print the title.
C  GRCSWID      Calculate Boson widths (optional).
C  GRCSCUT      Apply kinematical cuts.
C  GRCSSMR      Smear to z-boosted system.
C  GRCSBWM      Set Breit Wigner type mass of bosons (optional).
C  GRCEXKN      Exchange particle IDs.
C  GRCSWAP      Swap particle ID, if necessary.
C  GRCXSET      Set (phase space) region of the cross section.
C  GRCPSHW      Set the energy scale for parton shower.
C  GRCAPHI      Calculate the difference of phi angles.
C  GRCFFIL      Fill flavor of each particle.
C  GRCPRTNGG    Treat gluon gluon collision from PDF.
C  GRCPRTNQQ    Treat quark anti-quark collision from PDF.
C  GRCPRTNQQB   Treat quark anti-quark' collision from PDF.
C  GRCPRTNQG    Treat quark gluon collision from PDF.
C  GRCPRTNQQA   Treat up(down)-quark up(down)-quark collision from PDF.
C  GRCPRTNQQS   Treat up(down)-quark down(up)-quark collision from PDF.
C  GRCFJQQ      Set two final-state quark-jet flavors.
C  GRCFJQG      Set final-state quark-jet flavor.
C  GRCUSRCUT    Set user-defined kinematical cuts.
C  GRCUSRSETQ   Set user-defined energy scale.
C----------------------------------------
C Initialization of BASES.
C Default iteration = 5 (itmx1, itmx2)
C----------------------------------------
      SUBROUTINE GRCBSIT
      IMPLICIT REAL*8(A-H,O-Z)
 
      PARAMETER (MXDIM = 50, NDMX = 50 )
      COMMON /BPARM1/ XL(MXDIM),XU(MXDIM),NDIM,NWILD,
     $     IG(MXDIM),NCALL
      COMMON /BPARM2/ ACC1,ACC2,ITMX1,ITMX2
 
      INCLUDE 'grchad.inc'
 
C...Set variables for BASES
      NWILD = NDIM
      IF (NWILD.gt.14) NWILD = 14
      DO I =  1,NDIM
         XL(I)  =   0.D0
         XU(I)  =   1.D0
         IG(I)  =   1
      ENDDO
      ITMX1 = 5
      ITMX2 = 5
 
      INBS = 0
      INKV = 0
      INKC = 0
      IN4M = 0
      INAC = 0
 
      NFPRT = 1000000
 
      RETURN
      END
 
C----------------------------------------
C Initialization of kinematics variable.
C Set default parameter.
C----------------------------------------
      SUBROUTINE GRCIVAL
      IMPLICIT REAL*8(A-H,O-Z)
 
      PARAMETER (MXDIM = 50, NDMX = 50 )
      COMMON /BPARM1/ XL(MXDIM),XU(MXDIM),NDIM,NWILD,
     $     IG(MXDIM),NCALL

      INCLUDE 'grchad.inc'
      INTEGER I,J
 
      DO I = 1,32
         XRUN(I) = 0.0D0
         GVAL(I) = 0.0D0
      ENDDO
      DO I = 1,10
         GPRI(I) = 0.0D0
      ENDDO
      DO I = 1,18
         GTAU(I) = 0.0D0
         GYST(I) = 0.0D0
         GCOS3(I) = 0.0D0
         GQ12(I) = 0.0D0
         GQ22(I) = 0.0D0
      ENDDO
      DO I = 1,5
         GSPR(I) = 0.0D0
         GEPS(I) = 0.0D0
      ENDDO
      DO I = 1,8
         GRCDSIG(I) = 0.0D0
      ENDDO
 
      IREJ = 0
      YACOB = 1.0D0
 
      IF (INPFL.LT.1) INPFL = 4
      IF (IFACT.LT.1) IFACT = 0
      IF (ISDEP.LT.1) ISDEP = 0
      IF (IGRCGEF.LT.1) IGRCGEF = 0
      IF (NDIM.LT.7) THEN
         KTYPE = 1
         CXTYPE = 1
      ELSE IF (NDIM.LT.10) THEN
         KTYPE = 2
         CXTYPE = 2
      ELSE IF (NDIM.LT.13) THEN
         KTYPE = 6
         CXTYPE = 5
      ENDIF
      GTAU(1)   = 1
      GTAU(14)  = 1
      GYST(1)   = 1
      GYST(14)  = 1
      GCOS3(1)  = 1
      GCOS3(14) = 1
      GQ12(1)   = 1
      GQ12(14)  = 1
      GQ22(1)   = 1
      GQ22(14)  = 1
      DO I = 1,5
         GCON(I) = 1.0D0
         GSPR(I) = 1
         GEPS(I) = 1.0D0
      ENDDO
 
      IF (GETACUT.LT.0.0001) GETACUT = 100.0D0
      DO I = 1,6
         GRCPTCUT(I)   = GPTCUT
         GRCETACUT(I)  = GETACUT
         GRCRCONCUT(I) = GRCONCUT
      ENDDO

      IF (IGSUB.EQ.408.OR.IGSUB.EQ.412.OR.IGSUB.EQ.464
     $     .OR.IGSUB.EQ.465.OR.IGSUB.EQ.467
     $     .OR.IGSUB.EQ.101.OR.IGSUB.EQ.102.OR.IGSUB.EQ.103
     $     .OR.IGSUB.EQ.111.OR.IGSUB.EQ.112) THEN
         GRCPTCUT(1)   = 0.0D0
         GRCETACUT(1)  = 10.0D0
         GRCRCONCUT(1) = 0.0D0
         GRCPTCUT(2)   = 0.0D0
         GRCETACUT(2)  = 10.0D0
         GRCRCONCUT(2) = 0.0D0
      ENDIF

C...W + jets process
      IF (IGSUB.GT.421.AND.IGSUB.LT.440) THEN
         GRCPTCUT(1)   = 0.0D0
         GRCETACUT(1)  = 10.0D0
         GRCRCONCUT(1) = 0.0D0
         GRCPTCUT(2)   = 0.0D0
         GRCETACUT(2)  = 10.0D0
         GRCRCONCUT(2) = 0.0D0
      ENDIF

C...Z + jets process
      IF (IGSUB.GT.500.AND.IGSUB.LT.510) THEN
         GRCPTCUT(1)   = 0.0D0
         GRCETACUT(1)  = 10.0D0
         GRCRCONCUT(1) = 0.0D0
         GRCPTCUT(2)   = 0.0D0
         GRCETACUT(2)  = 10.0D0
         GRCRCONCUT(2) = 0.0D0
      ENDIF

C...W+gamma and W+2gamma
      IF (IGSUB.EQ.560.OR.IGSUB.EQ.561) THEN
         GRCPTCUT(1)   = 5.0D0
         GRCETACUT(1)  = 3.0D0
         GRCRCONCUT(1) = GRCONCUT
         GRCPTCUT(2)   = 1.0D0
         GRCETACUT(2)  = 10.0D0
         GRCRCONCUT(2) = 0.0D0
      ENDIF

      IF (IGSUB.GT.408.AND.IGSUB.LT.412) THEN
         GRCPTCUT(1)   = 0.0D0
         GRCETACUT(1)  = 100.0D0
         GRCRCONCUT(1) = 0.0D0
      ELSE IF (IGSUB.GT.412.AND.IGSUB.LT.416) THEN
         GRCPTCUT(1)   = 0.0D0
         GRCETACUT(1)  = 100.0D0
         GRCRCONCUT(1) = 0.0D0
      ELSE IF (IGSUB.EQ.418) THEN
         GRCPTCUT(1)   = 0.0D0
         GRCETACUT(1)  = 100.0D0
         GRCRCONCUT(1) = 0.0D0
      ENDIF

      RETURN
      END
 
C----------------
C Set tau limit.
C----------------
      SUBROUTINE GRCITAU
      IMPLICIT REAL*8(A-H,O-Z)
 
#include "define.h"
 
#if PSMODEL > 0
      COMMON/PYSUBS/MSEL,MSELPD,MSUB(500),KFIN(2,-40:40),CKIN(200)
#endif
 
      INCLUDE 'inclk.h'
      INCLUDE 'grchad.inc'
 
      REAL*8 E3,E4,E5,E6,E7,EQ
      REAL*8 TAUMN0,TAUMN1,TAUMN2,TAUMN3,TAUMN4
      REAL*8 TAUMX0,TAUMX1,TAUMX2,TAUMX3,TAUMX4
      REAL*8 Q22MIN,EQ2MIN
 
      IF (IREJ.GT.0) RETURN
 
C...Initilize maximum cut of tau
      IF (GTAU(INT(GTAU(1))+2).EQ.0.0D0) GTAU(INT(GTAU(1))+2) = 1.0D0
 
C...Calculate limits on tau
C...0) due to definition
      TAUMN0 = 0.0D0
      TAUMX0 = 1.0D0
 
C...1) due to limits on subsystem mass
#if PSMODEL > 0
      TAUMN1 = CKIN(1)**2/GRCECM**2
#else
      TAUMN1 = 0.0D0
#endif
      TAUMX1 = 1.0D0
 
#if PSMODEL > 0
      IF (CKIN(2).GT.0.0D0) TAUMX1 = CKIN(2)**2/GRCECM**2
#endif
 
C...2) due to limits on x1 and x2
#if PSMODEL > 0
      TAUMN2 = CKIN(21)*CKIN(23)
      TAUMX2 = CKIN(22)*CKIN(24)
      IF (TAUMX2.EQ.0.0D0) TAUMX2 = 1.0D0
#else
      TAUMN2 = 0.0D0
      TAUMX2 = 1.0D0
#endif
 
C...3) due to limits on xF
      TAUMN3 = 0D0
#if PSMODEL > 0
      TAUMX3 = MAX(1D0 - CKIN(25),1D0 + CKIN(26))
#else
      TAUMX3 = 1.0D0
#endif
 
C...4) due to limits on Total Mass and Pthat
      IF (KTYPE.EQ.1) THEN
         E3 = SQRT(GRCPTCUT(IPX(3)-2)**2+AMASS2(IPX(3)))
         E4 = SQRT(GRCPTCUT(IPX(4)-2)**2+AMASS2(IPX(4)))
         TAUMN4=(E3+E4)**2/GRCECM**2
      ELSE IF (KTYPE.EQ.2.OR.KTYPE.EQ.4) THEN
         E3 = SQRT(GRCPTCUT(IPX(3)-2)**2+AMASS2(IPX(3)))
         E4 = SQRT(GRCPTCUT(IPX(4)-2)**2+AMASS2(IPX(4)))
         E5 = SQRT(GRCPTCUT(IPX(5)-2)**2+AMASS2(IPX(5)))
         TAUMN4 = (E3+E4+E5)**2/GRCECM**2
      ELSE IF (KTYPE.EQ.3) THEN
         E3 = SQRT(GRCPTCUT(IPX(3)-2)**2+AMASS2(IPX(3)))
         EQ = SQRT(GVAL(3)+GRCPTCUT(IPX(3)-2)**2)
         TAUMN4 = (EQ+E3)**2/GRCECM**2
      ELSE IF (KTYPE.EQ.5) THEN
         E5 = SQRT(GRCPTCUT(IPX(5)-2)**2+AMASS2(IPX(5)))
         E6 = SQRT(GRCPTCUT(IPX(6)-2)**2+AMASS2(IPX(6)))
         TAUMN4 = (SQRT(GVAL(3))+E5+E6)**2/GRCECM**2
      ELSE IF (KTYPE.EQ.6) THEN
         E3 = SQRT(GRCPTCUT(IPX(3)-2)**2+AMASS2(IPX(3)))
         E4 = SQRT(GRCPTCUT(IPX(4)-2)**2+AMASS2(IPX(4)))
         E5 = SQRT(GRCPTCUT(IPX(5)-2)**2+AMASS2(IPX(5)))
         E6 = SQRT(GRCPTCUT(IPX(6)-2)**2+AMASS2(IPX(6)))
         TAUMN4 = (E3+E4+E5+E6)**2/GRCECM**2
      ELSE IF (KTYPE.EQ.7) THEN
         TAUMN4 = (SQRT(GVAL(3))+SQRT(GVAL(4)))**2/GRCECM**2
      ELSE IF (KTYPE.EQ.8) THEN
         E5 = SQRT(GRCPTCUT(IPX(5)-2)**2+AMASS2(IPX(5)))
         E6 = SQRT(GRCPTCUT(IPX(6)-2)**2+AMASS2(IPX(6)))
         EQ = SQRT(GVAL(3)+GRCPTCUT(IPX(5)-2)**2)
         Q22MIN = (E5 + EQ)**2
         EQ2MIN = SQRT(Q22MIN+GRCPTCUT(IPX(6)-2)**2)
         TAUMN4 = (E6 + EQ2MIN)**2/GRCECM**2
      ELSE IF (KTYPE.EQ.9) THEN
         E5 = SQRT(GRCPTCUT(IPX(5)-2)**2+AMASS2(IPX(5)))
         E6 = SQRT(GRCPTCUT(IPX(6)-2)**2+AMASS2(IPX(6)))
         TAUMN4 = (SQRT(GVAL(6))+E5+E6)**2/GRCECM**2
      ELSE IF (KTYPE.EQ.10) THEN
         E3 = SQRT(GRCPTCUT(IPX(3)-2)**2+AMASS2(IPX(3)))
         E4 = SQRT(GRCPTCUT(IPX(4)-2)**2+AMASS2(IPX(4)))
         E5 = SQRT(GRCPTCUT(IPX(5)-2)**2+AMASS2(IPX(5)))
         E6 = SQRT(GRCPTCUT(IPX(6)-2)**2+AMASS2(IPX(6)))
         E7 = SQRT(GRCPTCUT(IPX(7)-2)**2+AMASS2(IPX(7)))
         TAUMN4 = (E3+E4+E5+E6+E7)**2/GRCECM**2
      ELSE IF (KTYPE.EQ.11) THEN
         E5 = SQRT(GRCPTCUT(IPX(5)-2)**2+AMASS2(IPX(5)))
         E6 = SQRT(GRCPTCUT(IPX(6)-2)**2+AMASS2(IPX(6)))
         E7 = SQRT(GRCPTCUT(IPX(7)-2)**2+AMASS2(IPX(7)))
         TAUMN4 = (SQRT(GVAL(3))+E5+E6+E7)**2/GRCECM**2
      ELSE IF (KTYPE.EQ.12) THEN
         E7 = SQRT(GRCPTCUT(IPX(7)-2)**2+AMASS2(IPX(7)))
         TAUMN4 = (SQRT(GVAL(3))+SQRT(GVAL(4))+E7)**2/GRCECM**2
      ELSE
         TAUMN4 = 0.0D0
      ENDIF
      TAUMX4 = 1.0D0
 
C...Net effect of all separate limits.
      GTAU(2) = MAX(TAUMN0,TAUMN1,TAUMN2,TAUMN3,TAUMN4,GTAU(2))
      GTAU(INT(GTAU(1))+2) =
     $     MIN(TAUMX0,TAUMX1,TAUMX2,TAUMX3,TAUMX4,GTAU(INT(GTAU(1))+2))

C      gtau(2) = max(taumn0,taumn1,taumn2,taumn3,taumn4)
C      gtau(int(gtau(1))+2) =
C     $     min(taumx0,taumx1,taumx2,taumx3,taumx4)
 
      IF (GTAU(INT(GTAU(1))+2).LE.GTAU(2)) THEN
         IREJ = 1
         WRITE(*,*) 'Warning : Taumin > Taumax'
      ENDIF
 
      RETURN
      END
 
C----------
C Set tau.
C----------
      SUBROUTINE GRCSTAU(RN,TAU,AJACOB)
      IMPLICIT REAL*8(A-H,O-Z)
 
      INCLUDE 'inclk.h'
      INCLUDE 'grchad.inc'
 
      INTEGER KPAR,II
      REAL*8 TAUMIN,TAUMAX,RN,TAU,AJACOB,TAURE,GAMRE,RNA
      REAL*8 THEMIN,THEMAX,THE
      REAL*8 XIMIN,XIMAX,XI
 
      IF (IREJ.GT.0) RETURN
 
      KPAR = 0
      AJACOB = 1.0D0
      GTAU(8) = 0.0D0
      GTAU(INT(GTAU(1))+8) = 1.0D0
      DO II = 1,INT(GTAU(1))
         IF ((RN.GT.GTAU(II+7)).AND.(RN.LE.GTAU(II+8))) THEN
            RNA = (RN - GTAU(II+7))/(GTAU(II+8) - GTAU(II+7))
            KPAR = INT(GTAU(II+13))
            TAUMIN = GTAU(II+1)
            TAUMAX = GTAU(II+2)
            AJACOB = AJACOB/(GTAU(II+8) - GTAU(II+7))
         ENDIF
      ENDDO

      IF (KPAR.EQ.1) THEN
         TAU = TAUMIN*(TAUMAX/TAUMIN)**RNA
         AJACOB = AJACOB*LOG(TAUMAX/TAUMIN)
      ELSE IF (KPAR.EQ.2) THEN
         XIMIN = 1.0D0/TAUMAX
         XIMAX = 1.0D0/TAUMIN
         XI = XIMAX + (XIMIN - XIMAX)*RNA
         TAU = 1.0D0/XI
         AJACOB = AJACOB*(XIMAX - XIMIN)*TAU
      ELSE IF (KPAR.EQ.3) THEN
         XIMIN = 1.0D0/TAUMAX**2
         XIMAX = 1.0D0/TAUMIN**2
         XI = XIMAX + (XIMIN - XIMAX)*RNA
         TAU = 1.0D0/SQRT(XI)
         AJACOB = AJACOB*(XIMAX - XIMIN)*TAU**2*0.5D0
      ELSE IF (KPAR.EQ.4) THEN
         XIMIN = 1.0D0/TAUMAX**3
         XIMAX = 1.0D0/TAUMIN**3
         XI = XIMAX + (XIMIN - XIMAX)*RNA
         TAU = XI**(-1.0D0/3.0D0)
         AJACOB = AJACOB*(XIMAX - XIMIN)*TAU**3/3.0D0
      ELSE IF (KPAR.EQ.5) THEN
         XIMIN = 1.0D0/SQRT(TAUMAX)
         XIMAX = 1.0D0/SQRT(TAUMIN)
         XI = XIMAX + (XIMIN - XIMAX)*RNA
         TAU = 1.0D0/XI**2
         AJACOB = AJACOB*(XIMAX - XIMIN)*SQRT(TAU)*2.0D0
      ELSE IF (KPAR.EQ.6) THEN
         TAU = TAUMIN + (TAUMAX - TAUMIN)*RNA
         AJACOB = AJACOB*(TAUMAX - TAUMIN)/TAU
      ELSE IF (KPAR.EQ.7) THEN
         TAU = TAUMIN + (TAUMAX - TAUMIN)*SQRT(RNA)
         AJACOB = AJACOB*(TAUMAX - TAUMIN)**2
     $        /(2.0D0*(TAU - TAUMIN))
      ELSE IF (KPAR.EQ.8) THEN
         TAURE = AMW**2/GRCECM**2
         GAMRE = AMW*AGW/GRCECM**2
         THEMIN = ATAN((TAUMIN - TAURE)/GAMRE)
         THEMAX = ATAN((TAUMAX - TAURE)/GAMRE)
         THE = THEMIN + (THEMAX - THEMIN)*RNA
         TAU = GAMRE*TAN(THE) + TAURE
         AJACOB = AJACOB*(THEMAX - THEMIN)
     $        *((TAU - TAURE)**2 + GAMRE**2)/(GAMRE*TAU)
      ELSE IF (KPAR.EQ.9) THEN
         TAURE = AMZ**2/GRCECM**2
         GAMRE = AMZ*AGZ/GRCECM**2
         THEMIN = ATAN((TAUMIN - TAURE)/GAMRE)
         THEMAX = ATAN((TAUMAX - TAURE)/GAMRE)
         THE = THEMIN + (THEMAX - THEMIN)*RNA
         TAU = GAMRE*TAN(THE) + TAURE
         AJACOB = AJACOB*(THEMAX - THEMIN)
     $        *((TAU - TAURE)**2 + GAMRE**2)/(GAMRE*TAU)
      ELSE IF (KPAR.EQ.10) THEN
         TAURE = AMUQ(3)**2/GRCECM**2
         GAMRE = AMUQ(3)*AGUQ(3)/GRCECM**2
         THEMIN = ATAN((TAUMIN - TAURE)/GAMRE)
         THEMAX = ATAN((TAUMAX - TAURE)/GAMRE)
         THE = THEMIN + (THEMAX - THEMIN)*RNA
         TAU = GAMRE*TAN(THE) + TAURE
         AJACOB = AJACOB*(THEMAX - THEMIN)
     $        *((TAU - TAURE)**2 + GAMRE**2)/(GAMRE*TAU)
      ENDIF
 
      RETURN
      END
 
C----------------
C Set yst limit.
C----------------
      SUBROUTINE GRCIYST
      IMPLICIT REAL*8(A-H,O-Z)
 
#include "define.h"
 
#if PSMODEL > 0
      COMMON/PYSUBS/MSEL,MSELPD,MSUB(500),KFIN(2,-40:40),CKIN(200)
#endif
 
      INCLUDE 'grchad.inc'
 
      REAL*8 YSTMN0,YSTMN1,YSTMN2,YSTMN3,YEPMN4,YSTMN4
      REAL*8 YSTMX0,YSTMX1,YSTMX2,YSTMX3,YEPMX4,YSTMX4
 
      IF (IREJ.GT.0) RETURN
 
C...Calculate limits on yst
C...0) due to kinematics
      YSTMN0 = LOG(SQRT(GPRI(1)))
      YSTMX0 = -YSTMN0

C...1) due to explicit limits
#if PSMODEL > 0
      YSTMN1 = CKIN(7)
      YSTMX1 = CKIN(8)
      IF (YSTMN1.EQ.0.0D0) YSTMN1 = -10.0D0
      IF (YSTMX1.EQ.0.0D0) YSTMX1 =  10.0D0
#else
      YSTMN1 = -10.0D0
      YSTMX1 =  10.0D0
#endif
 
C...2) due to limits on x1
#if PSMODEL > 0
      YSTMN2 = LOG(MAX(GPRI(1),CKIN(21))/SQRT(GPRI(1)))
      YSTMX2 = LOG(MAX(GPRI(1),CKIN(22))/SQRT(GPRI(1)))
      IF (CKIN(21).EQ.0.0D0) YSTMN2 = -10.0D0
      IF (CKIN(22).EQ.0.0D0) YSTMX2 =  10.0D0
#else
      YSTMN2 = -10.0D0
      YSTMX2 =  10.0D0
#endif
 
C...3) due to limits on x2
#if PSMODEL > 0
      YSTMN3 = -LOG(MAX(GPRI(1),CKIN(24))/SQRT(GPRI(1)))
      YSTMX3 = -LOG(MAX(GPRI(1),CKIN(23))/SQRT(GPRI(1)))
      IF (CKIN(23).EQ.0.0D0) YSTMX3 =  10.0D0
      IF (CKIN(24).EQ.0.0D0) YSTMN3 = -10.0D0
#else
      YSTMN3 = -10.0D0
      YSTMX3 =  10.0D0
#endif
 
C...4) due to limits on xF
#if PSMODEL > 0
      YEPMN4 = 0.5D0*ABS(CKIN(25))/SQRT(GPRI(1))
      YSTMN4 = SIGN(LOG(MAX(1D-20,SQRT(1D0+YEPMN4**2)+YEPMN4)),CKIN(25))
      YEPMX4 = 0.5D0*ABS(CKIN(26))/SQRT(GPRI(1))
      YSTMX4 = SIGN(LOG(MAX(1D-20,SQRT(1D0+YEPMX4**2)+YEPMX4)),CKIN(26))
      IF (CKIN(25).EQ.0.0D0) YSTMN4 = -10.0D0
      IF (CKIN(26).EQ.0.0D0) YSTMX4 =  10.0D0
#else
      YSTMN4 = -10.0D0
      YSTMX4 =  10.0D0
#endif
 
C...Net effect of all separate limits.
      GYST(2) = MAX(YSTMN0,YSTMN1,YSTMN2,YSTMN3,YSTMN4)
      GYST(INT(GYST(1))+2) = MIN(YSTMX0,YSTMX1,YSTMX2,YSTMX3,YSTMX4)
 
      IF (GYST(INT(GYST(1))+2).LE.GYST(2)) THEN
         IREJ = 1
         WRITE(*,*) 'Warning : Ystmin > Ystmax'
      ENDIF
 
      RETURN
      END
 
C----------
C Set yst.
C----------
      SUBROUTINE GRCSYST(RN, YST, AJACOB)
      IMPLICIT REAL*8(A-H,O-Z)
 
      INCLUDE 'inclk.h'
      INCLUDE 'grchad.inc'
 
      INTEGER KPAR,II
      REAL*8 YST,AJACOB,RN,RNA
      REAL*8 YSTMIN,YSTMAX,AUPP,ALOW,YST0
 
      IF (IREJ.GT.0) RETURN
 
      KPAR = 0
      AJACOB = 1.0D0
      GYST(8) = 0.0D0
      GYST(INT(GYST(1))+8) = 1.0D0
      DO II = 1,INT(GYST(1))
         IF (RN.GT.GYST(II+7).AND.RN.LE.GYST(II+8)) THEN
            RNA = (RN - GYST(II+7))/(GYST(II+8) - GYST(II+7))
            KPAR = INT(GYST(II+13))
            YSTMIN = GYST(II+1)
            YSTMAX = GYST(II+2)
            AJACOB = AJACOB/(GYST(II+8) - GYST(II+7))
         ENDIF
      ENDDO
 
      IF (KPAR.EQ.1) THEN
         YST = YSTMIN + (YSTMAX - YSTMIN)*RNA
         AJACOB = AJACOB*(YSTMAX - YSTMIN)
      ELSE IF (KPAR.EQ.2) THEN
         YST = YSTMIN + (YSTMAX - YSTMIN)*SQRT(RNA)
         AJACOB = AJACOB*0.5*(YSTMAX - YSTMIN)**2/(YST - YSTMIN)
      ELSE IF (KPAR.EQ.3) THEN
         YST = YSTMIN + (YSTMAX - YSTMIN)*RNA**(1.0D0/3.0D0)
         AJACOB = AJACOB*(YSTMAX - YSTMIN)**3/(YST - YSTMIN)**2/3.0D0
      ELSE IF (KPAR.EQ.4) THEN
         YST = YSTMIN + (YSTMAX - YSTMIN)*RNA**(3.0/2.0)
         AJACOB = AJACOB*1.5
     $        *((YSTMAX - YSTMIN)**2*(YST - YSTMIN))*(1.0/3.0)
      ELSE IF (KPAR.EQ.5) THEN
         AUPP = ATAN(EXP(YSTMAX))
         ALOW = ATAN(EXP(YSTMIN))
         YST = LOG(TAN(ALOW + (AUPP - ALOW)*RNA))
         AJACOB = AJACOB*(AUPP - ALOW)*2*COSH(YST)
      ELSE IF (KPAR.EQ.6) THEN
         RNA = ACOS(-1.0D0)/3.0D0*(2.0D0*RNA - 1.0D0)
         YST = YSTMAX*TAN(RNA)/TAN(ACOS(-1.0D0)/3.0D0)
         AJACOB = AJACOB*ACOS(-1.0D0)/3.0D0*2.0D0*YSTMAX/COS(RNA)**2
     $        /TAN(ACOS(-1.0D0)/3.0D0)
      ELSE IF (KPAR.EQ.7) THEN
         YST0 = -0.5D0*LOG(GPRI(1))
         AUPP = LOG(MAX(1D-10,EXP(YST0 - YSTMIN) - 1D0))
         ALOW = LOG(MAX(1D-10,EXP(YST0 - YSTMAX) - 1D0))
         YST = YST0 - LOG(1D0 + EXP(ALOW + RNA*(AUPP - ALOW)))
         AJACOB = AJACOB*(AUPP - ALOW)*(1 - EXP(YST - YST0))
      ELSE
         YST0 = -0.5D0*LOG(GPRI(1))
         AUPP = LOG(MAX(1D-10,EXP(YST0 + YSTMIN) - 1D0))
         ALOW = LOG(MAX(1D-10,EXP(YST0 + YSTMAX) - 1D0))
         YST = LOG(1D0 + EXP(AUPP + RNA*(ALOW - AUPP))) -YST0
         AJACOB = AJACOB*(ALOW - AUPP)*(1 - EXP(-1.0*(YST + YST0)))
      ENDIF
 
      RETURN
      END
 
C-----------------
C Set cos3 limit.
C-----------------
      SUBROUTINE GRCICOS
      IMPLICIT REAL*8(A-H,O-Z)
 
#include "define.h"
 
#if PSMODEL > 0
      COMMON/PYSUBS/MSEL,MSELPD,MSUB(500),KFIN(2,-40:40),CKIN(200)
#endif
 
      INCLUDE 'inclk.h'
      INCLUDE 'grchad.inc'
 
      REAL*8 CTNMN0,CTNMN1,CTNMN2,CTNMN3,CTPMX0,CTPMX1,CTPMX2,CTPMX3
      REAL*8 E3,P3,E6,P6,E7,P7
 
      IF (IREJ.GT.0) RETURN
 
C...Calculate limits on cos(theta-hat)
C...0) due to definition
      CTNMN0 = -1.0D0
      CTPMX0 =  1.0D0
 
C...1) due to explicit limits
#if PSMODEL > 0
      CTNMN1 = MIN(0D0,CKIN(27))
      CTPMX1 = MAX(0D0,CKIN(28))
      IF (CKIN(27).EQ.0.0D0) CTNMN1 = -1.0D0
      IF (CKIN(28).EQ.0.0D0) CTNMX1 =  1.0D0
#else
      CTNMN1 = -1.0D0
      CTPMX1 =  1.0D0
#endif
 
C...2) due to limits on Pt
      IF (KTYPE.EQ.1) THEN
         E3 = (GPRI(3)+AMASS2(IPX(3))-AMASS2(IPX(4)))/2.0D0/GPRI(4)
         P3 = SQRT((E3+AMASS1(IPX(3)))*(E3-AMASS1(IPX(3))))
         CTNMN2 = -SQRT(P3**2 - GRCPTCUT(IPX(3)-2)**2)/P3
         CTPMX2 = SQRT(P3**2 - GRCPTCUT(IPX(3)-2)**2)/P3
      ELSE IF (KTYPE.EQ.2.OR.KTYPE.EQ.3) THEN
         E3 = (GPRI(3)+AMASS2(IPX(3))-GVAL(3))/2.0D0/GPRI(4)
         IF (E3.LT.AMASS1(IPX(3))) THEN
            WRITE(*,*) 'GRC::Error: negative sqrt, cos3'
            IREJ = 1
            RETURN
         ENDIF
         P3 = SQRT((E3+AMASS1(IPX(3)))*(E3-AMASS1(IPX(3))))
         IF (P3.LT.GRCPTCUT(IPX(3)-2)) THEN
            WRITE(*,*) 'Warnning: p3 < pt3min'
            IREJ = 1
            RETURN
         ENDIF
         CTNMN2 = -SQRT(P3**2 - GRCPTCUT(IPX(3)-2)**2)/P3
         CTPMX2 = SQRT(P3**2 - GRCPTCUT(IPX(3)-2)**2)/P3
      ELSE IF (KTYPE.EQ.8) THEN
         E6 = (GPRI(3)+AMASS2(IPX(6))-GVAL(4))/2.0D0/GPRI(4)
         IF (E6.LT.AMASS1(IPX(6))) THEN
            WRITE(*,*) 'GRC::Error: negative sqrt, cos3'
            IREJ = 1
            RETURN
         ENDIF
         P6 = SQRT(E6**2 - AMASS2(IPX(6)))
         IF (P6.LT.GRCPTCUT(IPX(6)-2)) THEN
            WRITE(*,*) 'Warnning: p6 < pt6min'
            IREJ = 1
            RETURN
         ENDIF
         CTNMN2 = -SQRT(P6**2 - GRCPTCUT(IPX(6)-2)**2)/P6
         CTPMX2 = SQRT(P6**2 - GRCPTCUT(IPX(6)-2)**2)/P6
      ELSE IF (KTYPE.EQ.11) THEN
         E7 = GVAL(1)
         P7 = SQRT((E7+AMASS1(IPX(7)))*(E7-AMASS1(IPX(7))))
         CTNMN2 = -SQRT(P7**2 - GRCPTCUT(IPX(7)-2)**2)/P7
         CTPMX2 = SQRT(P7**2 - GRCPTCUT(IPX(7)-2)**2)/P7
      ELSE
         CTNMN2 = -1.0D0
         CTPMX2 = 1.0D0
      ENDIF
 
C...3) due to limits on Rapidity (still not available)
      IF (KTYPE.EQ.1) THEN
         E3 = (GPRI(3)+AMASS2(IPX(3))-AMASS2(IPX(4)))/2.0D0/GPRI(4)
         P3 = SQRT((E3+AMASS1(IPX(3)))*(E3-AMASS1(IPX(3))))
C...For true rapidity
C         ctnmn3 = -e3/p3*tanh(grcetacut(ipx(3)-2))
C         ctpmx3 = e3/p3*tanh(grcetacut(ipx(3)-2))
C...For pseudo-rapidity
C         ctnmn3 = -tanh(grcetacut(ipx(3)-2))
C         ctpmx3 = tanh(grcetacut(ipx(3)-2))
 
         CTNMN3 = -1.0D0
         CTPMX3 = 1.0D0
 
      ELSE IF (KTYPE.EQ.2.OR.KTYPE.EQ.3) THEN
         E3 = (GPRI(3)+AMASS2(IPX(3))-GVAL(3))/2.0D0/GPRI(4)
         P3 = SQRT((E3+AMASS1(IPX(3)))*(E3-AMASS1(IPX(3))))
         CTNMN3 = -E3/P3*TANH(GRCETACUT(IPX(3)-2))
         CTPMX3 = E3/P3*TANH(GRCETACUT(IPX(3)-2))
 
         CTNMN3 = -1.0D0
         CTPMX3 = 1.0D0
 
      ELSE IF (KTYPE.EQ.8) THEN
         E6 = (GPRI(3)+AMASS2(IPX(6))-GVAL(4))/2.0D0/GPRI(4)
         P6 = SQRT(E6**2 - AMASS2(IPX(6)))
         CTNMN3 = -E6/P6*TANH(GRCETACUT(IPX(6)-2))
         CTPMX3 = E6/P6*TANH(GRCETACUT(IPX(6)-2))
 
         CTNMN3 = -1.0D0
         CTPMX3 = 1.0D0
      ELSE
         CTNMN3 = -1.0D0
         CTPMX3 = 1.0D0
      ENDIF
 
C...Net effect of all separate limits.
      GCOS3(2) = MAX(CTNMN0,CTNMN1,CTNMN2,CTNMN3)
      GCOS3(INT(GCOS3(1))+2) = MIN(CTPMX0,CTPMX1,CTPMX2,CTPMX3)
 
      IF (GCOS3(INT(GCOS3(1))+2).LE.GCOS3(2)) THEN
         IREJ = 1
         WRITE(*,*) 'Warning : Cos1min > Cos1max'
      ENDIF
 
      RETURN
      END
 
C-----------
C Set cos3.
C-----------
      SUBROUTINE GRCSCOS(RN, COS3, AJACOB)
      IMPLICIT REAL*8(A-H,O-Z)
 
      INCLUDE 'inclk.h'
      INCLUDE 'grchad.inc'
 
      INTEGER KPAR,II
      REAL*8 COS3, AJACOB,RN,RNA
      REAL*8 COS3MIN,COS3MAX,DMIN,DMAX,D,Y,TETA,D13,D23
      REAL*8 EQ1,PQ1,T1MAX,T1MIN,T1,EPS
      REAL*8 MAS1,ABDB,XI,TEMAX,TEMIN,YMAX,YMIN
      REAL*8 E1,E2,E3,E4,P1,P2,P3,P4
      REAL*8 SQM3,SQM4
 
      IF (IREJ.GT.0) RETURN
 
C...Set squared mass on cos(theta-hat)
C...2-2 process
      IF (KTYPE.EQ.1) THEN
         SQM3 = AMASS2(IPX(3))
         SQM4 = AMASS2(IPX(4))
C...3-body final state
      ELSE IF (KTYPE.LT.5) THEN
         SQM3 = AMASS2(IPX(3))
         SQM4 = GVAL(3)
C...4-body final state
      ELSE IF (KTYPE.LT.8) THEN
         SQM3 = GVAL(3)
         SQM4 = GVAL(4)
      ELSE IF (KTYPE.EQ.8) THEN
         SQM3 = (SQRT(GVAL(3)) + AMASS1(IPX(5)))**2
         SQM4 = AMASS2(IPX(6))
      ELSE IF (KTYPE.EQ.11) THEN
         SQM3 = AMASS2(IPX(7))
         SQM4 = AMASS2(IPX(7))
      ELSE IF (KTYPE.EQ.12) THEN
         SQM3 = GVAL(3)
         SQM4 = GVAL(5)
      ENDIF
 
      AJACOB = 1.0D0
      GCOS3(8) = 0.0D0
      GCOS3(INT(GCOS3(1))+8) = 1.0D0
      DO II = 1,INT(GCOS3(1))
         IF (RN.GT.GCOS3(II+7).AND.RN.LE.GCOS3(II+8)) THEN
            RNA = (RN - GCOS3(II+7))/(GCOS3(II+8) - GCOS3(II+7))
            KPAR = INT(GCOS3(II+13))
            COS3MIN = GCOS3(II+1)
            COS3MAX = GCOS3(II+2)
            AJACOB = AJACOB/(GCOS3(II+8) - GCOS3(II+7))
         ENDIF
      ENDDO
 
      E1 = GPRI(3)/2.D0/GPRI(4)
      P1 = E1
      E2 = GPRI(3)/2.D0/GPRI(4)
      P2 = E2
 
      IF (KPAR.EQ.1) THEN
         COS3 = COS3MIN + (COS3MAX - COS3MIN)*RNA
         AJACOB = AJACOB*(COS3MAX - COS3MIN)
C...1/u
      ELSE IF (KPAR.EQ.2) THEN
         E3 = (GPRI(3) + SQM3 - SQM4)/2.D0/GPRI(4)
         P3 = SQRT(E3**2 - SQM3)
         DMIN = 2*(E1*E3 + P1*P3*COS3MIN) - SQM3
         DMAX = 2*(E1*E3 + P1*P3*COS3MAX) - SQM3
         D = DMIN*(DMAX/DMIN)**RNA
         COS3 = E1*E3/P1/P3 - (D + SQM3)/2.D0/P1/P3
         AJACOB = AJACOB*D*LOG(DMAX/DMIN)/2.0D0/P1/P3
C...1/t
      ELSE IF (KPAR.EQ.3) THEN
         E4 = (GPRI(3) + SQM4 - SQM3)/2.D0/GPRI(4)
         P4 = SQRT(E4**2 - SQM4)
         DMIN = 2*(E1*E4 + P1*P4*COS3MIN) - SQM4
         DMAX = 2*(E1*E4 + P1*P4*COS3MAX) - SQM4
         D = DMIN*(DMAX/DMIN)**RNA
         COS3 = -E1*E4/P1/P4 + (D + SQM4)/2.D0/P1/P4
         AJACOB = AJACOB*D*LOG(DMAX/DMIN)/2.0D0/P1/P4
C...1/u + 1/t
      ELSE IF (KPAR.EQ.4) THEN
         E3 = (GPRI(3) + SQM3 - SQM4)/2.D0/GPRI(4)
         P3 = SQRT(E3**2 - SQM3)
         E4 = (GPRI(3) + SQM4 - SQM3)/2.D0/GPRI(4)
         P4 = SQRT(E4**2 - SQM4)
         IF(RNA.LE.0.5D0) THEN
            RNA = RNA*2.0D0
            DMIN = 2*(E1*E3 + P1*P3*COS3MIN) - SQM3
            DMAX = 2*E1*E3 - SQM3
            D = DMIN*(DMAX/DMIN)**RNA
            COS3 = E1*E3/P1/P3 - (D + SQM3)/2.D0/P1/P3
            AJACOB = AJACOB*D*LOG(DMAX/DMIN)/P1/P3
         ELSE
            RNA = 2.0D0 - RNA*2.0D0
            DMIN = 2*(E1*E4 + P1*P4*COS3MIN) - SQM4
            DMAX = 2*E1*E4 - SQM4
            D = DMIN*(DMAX/DMIN)**RNA
            COS3 = -E1*E4/P1/P4 + (D + SQM4)/2.D0/P1/P4
            AJACOB = AJACOB*D*LOG(DMAX/DMIN)/P1/P4
         ENDIF
      ELSE IF (KPAR.EQ.5) THEN
         MAS1 = GCON(1)
         XI = (E1 + P1)/MAS1
         ABDB = 1.0D0 + MAS1**2/P1/(E1 + P1)
         TEMAX = (ABDB + COS3MAX)/(ABDB - COS3MAX)
         TEMIN = (ABDB + COS3MIN)/(ABDB - COS3MIN)
         YMAX = (LOG(TEMAX)/LOG(XI) + 2)/4.D0
         YMIN = (LOG(TEMIN)/LOG(XI) + 2)/4.D0
         IF (COS3MIN.LT.-1.D0+1.D-10) YMIN = 0
         IF (COS3MAX.GT. 1.D0-1.D-10) YMAX = 1
         Y = YMIN + (YMAX - YMIN)*RNA
         TETA = XI**(4*Y - 2)
         COS3 = ABDB*(TETA - 1)/(TETA + 1)
         IF (ABS(COS3).GT.1) COS3 = SIGN(1.D0,COS3)
         D13 = 4*E1/(TETA + 1)
         D23 = TETA*D13
         AJACOB = AJACOB*LOG(XI)/2.D0/E1/P1*D13*D23*(YMAX-YMIN)
      ELSE IF (KPAR.EQ.6) THEN
         EQ1 = (GPRI(3) + SQM3 - SQM4)/2.D0/GPRI(4)
         PQ1 = SQRT(EQ1**2 - SQM3)
         T1MAX = -SQM3 + 2*(E1*EQ1 + P1*PQ1)
         T1MIN = -SQM3 + 2*(E1*EQ1 - P1*PQ1)
         T1 = T1MIN + (T1MAX - T1MIN)*RNA
         COS3 = (-T1 - SQM3 + E1*EQ1*2)/2.D0/P1/PQ1
         AJACOB = AJACOB*(T1MAX-T1MIN)/(2.0D0*P1*PQ1)
      ELSE IF (KPAR.EQ.7) THEN
         EQ1 = (GPRI(3) + SQM3 - SQM4)/2.D0/GPRI(4)
         PQ1 = SQRT(EQ1**2 - SQM3)
         T1MAX = -SQM3 + 2*(E1*EQ1 + P1*PQ1)
         T1MIN = -SQM3 + 2*(E1*EQ1 - P1*PQ1)
         T1 = T1MIN*(T1MAX/T1MIN)**RNA
         COS3 = (-T1 - SQM3 + E1*EQ1*2)/2.D0/P1/PQ1
         AJACOB = AJACOB*T1*LOG(T1MAX/T1MIN)/(2*P1*PQ1)
      ELSE IF (KPAR.EQ.8) THEN
         EPS = GCON(1)
         DMIN = -LOG(1.0D0 + 2.0D0/EPS)/2.0D0
         DMAX = LOG(1.0D0 + 2.0D0/EPS)/2.0D0
         D = DMIN + (DMAX - DMIN)*RNA
         COS3 = COS3MAX*(1.0D0 + EPS)*TANH(D)
         AJACOB = AJACOB*(DMAX - DMIN)/COSH(D)**2*(1.0D0 + EPS)
     $        *COS3MAX
 
         COS3 = (COS3MAX-COS3MIN)/2.0D0*COS3+(COS3MAX+COS3MIN)/2.0D0
         AJACOB = AJACOB*(COS3MAX-COS3MIN)/2.0D0
      ENDIF
 
      RETURN
      END
 
C----------
C Set Q12.
C----------
      SUBROUTINE GRCSQ12(RN, Q12, AJACOB)
      IMPLICIT REAL*8(A-H,O-Z)
 
      INCLUDE 'inclk.h'
      INCLUDE 'grchad.inc'
 
      INTEGER KPAR,II
      REAL*8 RN,RNA,Q12,AJACOB,Q12MIN,Q12MAX
      REAL*8 THE,THEMAX,THEMIN
      REAL*8 EQ3,E3MIN,XIMIN,XIMAX,XI,E3MAX,E3
      REAL*8 E7MIN
 
      IF (IREJ.GT.0) RETURN
      IF (GQ12(INT(GQ12(1))+2).EQ.0) GQ12(INT(GQ12(1))+2)=1.0D16
 
C...3-body final state, Q^2 is NOT a resonance(glu/gam splitting).
      IF (KTYPE.EQ.2) THEN
         EQ3 = SQRT(AMASS2(IPX(3)) + GRCPTCUT(IPX(3)-2)**2)
         GQ12(2) = MAX(GQ12(2),(AMASS1(IPX(4))+AMASS1(IPX(5)))**2)
         GQ12(INT(GQ12(1))+2) = MIN(GQ12(INT(GQ12(1))+2),
     $        (GPRI(4)-EQ3)**2-GRCPTCUT(IPX(3)-2)**2)
 
C...3-body final state, Q^2 is a resonance production(W+-, Z, top).
      ELSE IF (KTYPE.EQ.3) THEN
         EQ3 = SQRT(AMASS2(IPX(3)) + GRCPTCUT(IPX(3)-2)**2)
         GQ12(2) = MAX(GQ12(2),(AMASS1(IPX(4))+AMASS1(IPX(5)))**2)
         GQ12(INT(GQ12(1))+2) = MIN(GQ12(INT(GQ12(1))+2),
     $        (GRCECM-EQ3)**2-GRCPTCUT(IPX(3)-2)**2)
 
C...4-body final state, Q^2(ij) : resonance, Q^2(kl) : non-resonance.
C...4-body final state, Q^2(ij) : resonance, Q^2(kl) : resonance.
C...4-body final state, Cascade Decay, (345) + 6 -> 3 + (45) + 6
C...4-body Fusion 1 -> 5 + A, 2 -> 6 + B, A + B -> 3 + 4
      ELSE IF (KTYPE.EQ.5.OR.KTYPE.EQ.7
     $        .OR.KTYPE.EQ.8.OR.KTYPE.EQ.9) THEN
         GQ12(2) = MAX(GQ12(2),(AMASS1(IPX(3))+AMASS1(IPX(4)))**2)
         GQ12(INT(GQ12(1))+2) = MIN(GQ12(INT(GQ12(1))+2),
     $        (GRCECM-AMASS1(IPX(5))-AMASS1(IPX(6)))**2)
 
C...4-body final state, Q^2(ij) : non-resonance, Q^2(kl) : non-resonance.
      ELSE IF (KTYPE.EQ.6) THEN
         GQ12(2) = MAX(GQ12(2),(AMASS1(IPX(3))+AMASS1(IPX(4)))**2)
         GQ12(INT(GQ12(1))+2) = MIN(GQ12(INT(GQ12(1))+2),
     $        (GPRI(4)-AMASS1(IPX(5))-AMASS1(IPX(6)))**2)

C...5-body Fusion 1 -> Q45+A, 2 -> Q67+B, A+B -> 3, Q45 -> 4+5, Q67 -> 6+7.
      ELSE IF (KTYPE.EQ.10) THEN
         E3MIN = SQRT(GRCPTCUT(IPX(3)-2)**2+AMASS2(IPX(3)))
         GQ12(2) = MAX(GQ12(2),(AMASS1(IPX(4))+AMASS1(IPX(5)))**2)
         GQ12(INT(GQ12(1))+2) = MIN(GQ12(INT(GQ12(1))+2),
     $        (GPRI(4)-E3MIN-AMASS1(IPX(6))-AMASS1(IPX(7)))**2)
      ELSE IF (KTYPE.EQ.11.OR.KTYPE.EQ.12) THEN
         E7MIN = SQRT(GRCPTCUT(IPX(7)-2)**2+AMASS2(IPX(7)))
         GQ12(2) = MAX(GQ12(2),(AMASS1(IPX(3))+AMASS1(IPX(4)))**2)
         GQ12(INT(GQ12(1))+2) = MIN(GQ12(INT(GQ12(1))+2),
     $        (GRCECM-AMASS1(IPX(5))-AMASS1(IPX(6))-E7MIN)**2)
      ENDIF
 
      IF (GQ12(2).GT.GQ12(INT(GQ12(1))+2)) THEN
         IREJ = 1
C         WRITE(*,*) 'Warning : Q12min > Q12max ',GQ12(2),
C     $        GQ12(INT(GQ12(1))+2)
      ENDIF
 
      AJACOB = 1.0D0
      GQ12(8) = 0.0D0
      GQ12(INT(GQ12(1))+8) = 1.0D0
      DO II = 1,INT(GQ12(1))
         IF (RN.GT.GQ12(II+7).AND.RN.LE.GQ12(II+8)) THEN
            RNA = (RN - GQ12(II+7))/(GQ12(II+8) - GQ12(II+7))
            KPAR = INT(GQ12(II+13))
            Q12MIN = GQ12(II+1)
            Q12MAX = GQ12(II+2)
            AJACOB = AJACOB/(GQ12(II+8) - GQ12(II+7))
         ENDIF
      ENDDO

      IF (KPAR.EQ.1) THEN
         Q12 = Q12MIN + (Q12MAX - Q12MIN)*RNA
         AJACOB = AJACOB*(Q12MAX - Q12MIN)
      ELSE IF (KPAR.EQ.2) THEN
         Q12 = Q12MIN*(Q12MAX/Q12MIN)**RNA
         AJACOB = AJACOB*Q12*LOG(Q12MAX/Q12MIN)
      ELSE IF (KPAR.EQ.3) THEN
         XIMIN = 1.0D0/SQRT(Q12MAX)
         XIMAX = 1.0D0/SQRT(Q12MIN)
         XI = XIMAX + (XIMIN - XIMAX)*RNA
         Q12 = 1.0D0/XI**2
         AJACOB = AJACOB*(XIMAX - XIMIN)*2.0D0/XI**3
      ELSE IF (KPAR.EQ.4) THEN
         XIMIN = 1.0D0/Q12MAX
         XIMAX = 1.0D0/Q12MIN
         XI = XIMAX + (XIMIN - XIMAX)*RNA
         Q12 = 1.0D0/XI
         AJACOB = AJACOB*(XIMAX - XIMIN)*Q12**2
      ELSE IF (KPAR.EQ.5) THEN
         XIMIN = 1.0D0/Q12MAX**2
         XIMAX = 1.0D0/Q12MIN**2
         XI = XIMAX + (XIMIN - XIMAX)*RNA
         Q12 = 1.0D0/SQRT(XI)
         AJACOB = AJACOB*(XIMAX - XIMIN)*Q12**3*0.5D0
      ELSE IF (KPAR.EQ.6) THEN
         Q12 = Q12MIN + (Q12MAX - Q12MIN)*SQRT(RNA)
         AJACOB = AJACOB*(Q12MAX - Q12MIN)**2
     $        /(2.0D0*(Q12 - Q12MIN))
      ELSE IF (KPAR.EQ.7) THEN
         Q12 = Q12MIN + (Q12MAX - Q12MIN)*RNA**(1.0D0/3.0D0)
         AJACOB = AJACOB*(Q12MAX - Q12MIN)**3/(Q12 - Q12MIN)**2/3.0D0
      ELSE IF (KPAR.EQ.8) THEN
         Q12 = Q12MIN + (Q12MAX - Q12MIN)*RNA**(2.0D0/3.0D0)
         AJACOB = AJACOB*2.0D0/3.0D0*(Q12MAX - Q12MIN)**(3.0D0/2.0D0)
     $        /SQRT(Q12 - Q12MIN)
      ELSE IF (KPAR.EQ.9) THEN
         IF (KTYPE.GT.4) THEN
            IREJ = 1
            WRITE(*,*) 'GRC::Error: Wrong ktype in Q12'
         ENDIF
         E3MIN = GRCPTCUT(IPX(3)-2)
         E3MAX = (GPRI(3) - Q12MIN)/2.0D0/GPRI(4)
         E3 = E3MIN*(E3MAX/E3MIN)**RNA
         AJACOB = AJACOB*E3*LOG(E3MAX/E3MIN)
         Q12 = GPRI(3) - 2.0D0*GPRI(4)*E3
         AJACOB = AJACOB*2.0D0*GPRI(4)
      ELSE IF (KPAR.EQ.10) THEN
         THEMIN = ATAN((Q12MIN - AMW**2)/(AMW*AGW))
         THEMAX = ATAN((Q12MAX - AMW**2)/(AMW*AGW))
         THE = THEMIN + (THEMAX - THEMIN)*RNA
         Q12 = AMW*AGW*TAN(THE) + AMW**2
         AJACOB = AJACOB*(THEMAX - THEMIN)
     $        *((Q12 - AMW**2)**2 + (AMW*AGW)**2)/(AMW*AGW)
      ELSE IF (KPAR.EQ.11) THEN
         THEMIN = ATAN((Q12MIN - AMZ**2)/(AMZ*AGZ))
         THEMAX = ATAN((Q12MAX - AMZ**2)/(AMZ*AGZ))
         THE = THEMIN + (THEMAX - THEMIN)*RNA
         Q12 = AMZ*AGZ*TAN(THE) + AMZ**2
         AJACOB = AJACOB*(THEMAX - THEMIN)
     $        *((Q12 - AMZ**2)**2 + (AMZ*AGZ)**2)/(AMZ*AGZ)
      ELSE IF (KPAR.EQ.12) THEN
         THEMIN = ATAN((Q12MIN - AMUQ(3)**2)/(AMUQ(3)*AGUQ(3)))
         THEMAX = ATAN((Q12MAX - AMUQ(3)**2)/(AMUQ(3)*AGUQ(3)))
         THE = THEMIN + (THEMAX - THEMIN)*RNA
         Q12 = AMUQ(3)*AGUQ(3)*TAN(THE) + AMUQ(3)**2
         AJACOB = AJACOB*(THEMAX - THEMIN)
     $        *((Q12 - AMUQ(3)**2)**2
     $        + (AMUQ(3)*AGUQ(3))**2)/(AMUQ(3)*AGUQ(3))
      ELSE IF (KPAR.EQ.13) THEN
         THEMIN = ATAN((Q12MIN - AMH**2)/(AMH*AGH))
         THEMAX = ATAN((Q12MAX - AMH**2)/(AMH*AGH))
         THE = THEMIN + (THEMAX - THEMIN)*RNA
         Q12 = AMH*AGH*TAN(THE) + AMH**2
         AJACOB = AJACOB*(THEMAX - THEMIN)
     $        *((Q12 - AMH**2)**2 + (AMH*AGH)**2)/(AMH*AGH)
 
         IF (Q12.LT.1.0D0) THEN
            WRITE(*,*) Q12,THE,RNA,TAN(THE)
         ENDIF
 
      ENDIF
 
      RETURN
      END
 
C----------
C Set Q22.
C----------
      SUBROUTINE GRCSQ22(RN, Q22, AJACOB)
      IMPLICIT REAL*8(A-H,O-Z)
 
      INCLUDE 'inclk.h'
      INCLUDE 'grchad.inc'
 
      INTEGER KPAR,II
      REAL*8 RN,RNA,Q22,AJACOB,Q22MIN,Q22MAX
      REAL*8 THE,THEMAX,THEMIN
      REAL*8 E6,E3MIN,XIMIN,XIMAX,XI,E7MIN
 
      IF (IREJ.GT.0) RETURN
      IF (GQ22(INT(GQ22(1))+2).EQ.0) GQ22(INT(GQ22(1))+2)=1.0D16
 
C...4-body final state, Q^2(ij) : resonance, Q^2(kl) : non-resonance.
C...4-body final state, Q^2(ij) : non-resonance, Q^2(kl) : non-resonance.
      IF (KTYPE.EQ.5.OR.KTYPE.EQ.6) THEN
         GQ22(2) = MAX(GQ22(2),(AMASS1(IPX(5))+AMASS1(IPX(6)))**2)
         GQ22(INT(GQ22(1))+2) = MIN(GQ22(INT(GQ22(1))+2),
     $        (GPRI(4)-SQRT(GVAL(3)))**2)
 
C...4-body final state, Q^2(ij) : resonance, Q^2(kl) : resonance.
      ELSE IF (KTYPE.EQ.7) THEN
         GQ22(2) = MAX(GQ22(2),(AMASS1(IPX(5))+AMASS1(IPX(6)))**2)
         GQ22(INT(GQ22(1))+2) = MIN(GQ22(INT(GQ22(1))+2),
     $        (GRCECM-SQRT(GVAL(3)))**2)

C...Now is only Z/gamma pair production.
         IF (RN.LT.GQ22(9)) THEN
            GQ22(INT(GQ22(1))+1) = MIN(GQ22(INT(GQ22(1))+1),
     $           (GRCECM-SQRT(GVAL(3)))**2)
         ELSE
            IF ((GRCECM-SQRT(GVAL(3)))**2.LT.GQ22(INT(GQ22(1))+1)) THEN
               IREJ = 1
               RETURN
            ENDIF
         ENDIF
 
C...4-body final state, Cascade Decay, (345) + 6 -> 3 + (45) + 6
      ELSE IF (KTYPE.EQ.8) THEN
         E6 = SQRT(GRCPTCUT(IPX(6)-2)**2+AMASS2(IPX(6)))
         GQ22(2) = MAX(GQ22(2),(AMASS1(IPX(5))+SQRT(GVAL(3)))**2)
         GQ22(INT(GQ22(1))+2) = MIN(GQ22(INT(GQ22(1))+2),
     $        (GPRI(4)-E6)**2-GRCPTCUT(IPX(6)-2)**2)
 
C...5-body Fusion 1 -> Q45+A, 2 -> Q67+B, A+B -> 3, Q45 -> 4+5, Q67 -> 6+7.
      ELSE IF (KTYPE.EQ.10) THEN
         E3MIN = SQRT(GRCPTCUT(IPX(3)-2)**2+AMASS2(IPX(3)))
         GQ22(2) = MAX(GQ22(2),(AMASS1(IPX(6))+AMASS1(IPX(7)))**2)
         GQ22(INT(GQ22(1))+2) = MIN(GQ22(INT(GQ22(1))+2),
     $        (GPRI(4)-E3MIN-SQRT(GVAL(3)))**2)
      ELSE IF (KTYPE.EQ.11) THEN
         E7MIN = SQRT(GRCPTCUT(IPX(7)-2)**2+AMASS1(IPX(7))**2)
         GQ22(2) = MAX(GQ22(2),(AMASS1(IPX(5))+AMASS1(IPX(6)))**2)
         GQ22(INT(GQ22(1))+2) = MIN(GQ22(INT(GQ22(1))+2),
     $        (GPRI(4)-SQRT(GVAL(3))-E7MIN)**2)
      ELSE IF (KTYPE.EQ.12) THEN
         E7MIN = SQRT(GRCPTCUT(IPX(7)-2)**2+AMASS1(IPX(7))**2)
         GQ22(2) = MAX(GQ22(2),(AMASS1(IPX(5))+AMASS1(IPX(6)))**2)
         GQ22(INT(GQ22(1))+2) = MIN(GQ22(INT(GQ22(1))+2),
     $        (GRCECM-SQRT(GVAL(3))-E7MIN)**2)
      ENDIF
 
      IF (GQ22(2).GT.GQ22(INT(GQ22(1))+2)) THEN
         IREJ = 1
C         WRITE(*,*) 'Warning : Q22min > Q22max'
         RETURN
      ENDIF
 
      AJACOB = 1.0D0
      GQ22(8) = 0.0D0
      GQ22(INT(GQ22(1))+8) = 1.0D0
      DO II = 1,INT(GQ22(1))
         IF (RN.GT.GQ22(II+7).AND.RN.LE.GQ22(II+8)) THEN
            RNA = (RN - GQ22(II+7))/(GQ22(II+8) - GQ22(II+7))
            KPAR = INT(GQ22(II+13))
            Q22MIN = GQ22(II+1)
            Q22MAX = GQ22(II+2)
            AJACOB = AJACOB/(GQ22(II+8) - GQ22(II+7))
         ENDIF
      ENDDO
 
      IF (Q22MIN.GT.Q22MAX) THEN
         IREJ = 1
         WRITE(*,*) 'Warning : Q22min > Q22max'
         RETURN
      ENDIF

      IF (KPAR.EQ.1) THEN
         Q22 = Q22MIN + (Q22MAX - Q22MIN)*RNA
         AJACOB = AJACOB*(Q22MAX - Q22MIN)
      ELSE IF (KPAR.EQ.2) THEN
         Q22 = Q22MIN*(Q22MAX/Q22MIN)**RNA
         AJACOB = AJACOB*Q22*LOG(Q22MAX/Q22MIN)
      ELSE IF (KPAR.EQ.3) THEN
         XIMIN = 1.0D0/SQRT(Q22MAX)
         XIMAX = 1.0D0/SQRT(Q22MIN)
         XI = XIMAX + (XIMIN - XIMAX)*RNA
         Q22 = 1.0D0/XI**2
         AJACOB = AJACOB*(XIMAX - XIMIN)*2.0D0/XI**3
      ELSE IF (KPAR.EQ.4) THEN
         XIMIN = 1.0D0/Q22MAX
         XIMAX = 1.0D0/Q22MIN
         XI = XIMAX + (XIMIN - XIMAX)*RNA
         Q22 = 1.0D0/XI
         AJACOB = AJACOB*(XIMAX - XIMIN)*Q22**2
      ELSE IF (KPAR.EQ.5) THEN
         XIMIN = 1.0D0/Q22MAX**2
         XIMAX = 1.0D0/Q22MIN**2
         XI = XIMAX + (XIMIN - XIMAX)*RNA
         Q22 = 1.0D0/SQRT(XI)
         AJACOB = AJACOB*(XIMAX - XIMIN)*Q22**3*0.5D0
      ELSE IF (KPAR.EQ.6) THEN
         Q22 = Q22MIN + (Q22MAX - Q22MIN)*SQRT(RNA)
         AJACOB = AJACOB*(Q22MAX - Q22MIN)**2
     $        /(2.0D0*(Q22 - Q22MIN))
      ELSE IF (KPAR.EQ.7) THEN
         Q22 = Q22MIN + (Q22MAX - Q22MIN)*RNA**(1.0D0/3.0D0)
         AJACOB = AJACOB*(Q22MAX - Q22MIN)**3/(Q22 - Q22MIN)**2/3.0D0
      ELSE IF (KPAR.EQ.8) THEN
         Q22 = Q22MIN + (Q22MAX - Q22MIN)*RNA**(2.0D0/3.0D0)
         AJACOB = AJACOB*2.0D0/3.0D0*(Q22MAX - Q22MIN)**(3.0D0/2.0D0)
     $        /SQRT(Q22 - Q22MIN)
      ELSE IF (KPAR.EQ.9) THEN
C        Empty
      ELSE IF (KPAR.EQ.10) THEN
         THEMIN = ATAN((Q22MIN - AMW**2)/(AMW*AGW))
         THEMAX = ATAN((Q22MAX - AMW**2)/(AMW*AGW))
         THE = THEMIN + (THEMAX - THEMIN)*RNA
         Q22 = AMW*AGW*TAN(THE) + AMW**2
         AJACOB = AJACOB*(THEMAX - THEMIN)
     $        *((Q22 - AMW**2)**2 + (AMW*AGW)**2)/(AMW*AGW)
      ELSE IF (KPAR.EQ.11) THEN
         THEMIN = ATAN((Q22MIN - AMZ**2)/(AMZ*AGZ))
         THEMAX = ATAN((Q22MAX - AMZ**2)/(AMZ*AGZ))
         THE = THEMIN + (THEMAX - THEMIN)*RNA
         Q22 = AMZ*AGZ*TAN(THE) + AMZ**2
         AJACOB = AJACOB*(THEMAX - THEMIN)
     $        *((Q22 - AMZ**2)**2 + (AMZ*AGZ)**2)/(AMZ*AGZ)
      ELSE IF (KPAR.EQ.12) THEN
         THEMIN = ATAN((Q22MIN - AMUQ(3)**2)/(AMUQ(3)*AGUQ(3)))
         THEMAX = ATAN((Q22MAX - AMUQ(3)**2)/(AMUQ(3)*AGUQ(3)))
         THE = THEMIN + (THEMAX - THEMIN)*RNA
         Q22 = AMUQ(3)*AGUQ(3)*TAN(THE) + AMUQ(3)**2
         AJACOB = AJACOB*(THEMAX - THEMIN)
     $        *((Q22 - AMUQ(3)**2)**2
     $        + (AMUQ(3)*AGUQ(3))**2)/(AMUQ(3)*AGUQ(3))
      ELSE IF (KPAR.EQ.13) THEN
         THEMIN = ATAN((Q22MIN - AMH**2)/(AMH*AGH))
         THEMAX = ATAN((Q22MAX - AMH**2)/(AMH*AGH))
         THE = THEMIN + (THEMAX - THEMIN)*RNA
         Q22 = AMH*AGH*TAN(THE) + AMH**2
         AJACOB = AJACOB*(THEMAX - THEMIN)
     $        *((Q22 - AMH**2)**2 + (AMH*AGH)**2)/(AMH*AGH)
      ENDIF
 
      RETURN
      END
 
C--------------------------------
C Set s^,x1,x2 from tau and yst.
C--------------------------------
      SUBROUTINE GRCSSPX
      IMPLICIT REAL*8(A-H,O-Z)
 
      INCLUDE 'inclk.h'
      INCLUDE 'grchad.inc'
 
      REAL*8 RNA,TAU,YST,AJACOB
 
C...Valiable tau,y for PDF
      CALL GRCITAU
      RNA = XRUN(1)
      CALL GRCSTAU(RNA, TAU, AJACOB)
      GPRI(1) = TAU
      YACOB = YACOB*AJACOB

      CALL GRCIYST
      RNA = XRUN(2)
      CALL GRCSYST(RNA, YST, AJACOB)
      GPRI(2) = YST
      YACOB = YACOB*AJACOB
 
C...Set x1,x2,s,w
      GPRI(5) = SQRT(GPRI(1))*EXP(GPRI(2))
      GPRI(6) = SQRT(GPRI(1))*EXP(-GPRI(2))
      GPRI(3) = GPRI(1)*GRCECM**2
      GPRI(4) = SQRT(GPRI(1))*GRCECM
C...Set beta,gamma
      GPRI(7) = TANH(GPRI(2))
      GPRI(8) = COSH(GPRI(2))

      RETURN
      END
 
C----------------------------
C Set subsequent cos(theta).
C----------------------------
      SUBROUTINE GRCSBCS(KPAR,EPS,RN,SUBCOS,AJACOB,SCSMIN,SCSMAX)
      IMPLICIT REAL*8(A-H,O-Z)
 
      INCLUDE 'inclk.h'
      INCLUDE 'grchad.inc'
 
      INTEGER KPAR
      REAL*8 SUBCOS,RN,RNA,AJACOB,EPS,SCSMIN,SCSMAX
      REAL*8 PF4(4,8),Q1(4),Q2(4),PK1(4),PK2(4)
      REAL*8 E1,E2,E5,E6,EE1,EE2,EQ1,EQ2
      REAL*8 P1,P5,P2,P6,PP1,PP2,PQ1
      REAL*8 COS1,SIN1,SIN3,SIN4,PHI1,ETA
      REAL*8 XI,XIMIN,XIMAX
      REAL*8 D,DMIN,DMAX
      REAL*8 V,VMIN,VMAX
      REAL*8 G,GMIN,GMAX
      REAL*8 D13,D24,D14,D23
 
      IF (KPAR.EQ.2.AND.EPS.LT.1.0D-5) THEN
         IREJ = 3
         WRITE(*,*) "ERROR: WRONG SUBSYSTEM PARAMETER"
      ENDIF
 
      IF (KPAR.EQ.1) THEN
         SUBCOS = SCSMIN + (SCSMAX - SCSMIN)*RN
         AJACOB = SCSMAX - SCSMIN
      ELSE IF (KPAR.EQ.2) THEN
         DMIN = -LOG(1.0D0 + 2.0D0/EPS)/2.0D0
         DMAX = LOG(1.0D0 + 2.0D0/EPS)/2.0D0
         D = DMIN + (DMAX - DMIN)*RN
         SUBCOS = (1.0D0 + EPS)*TANH(D)
         AJACOB = (DMAX - DMIN)/COSH(D)**2*(1.0D0 + EPS)
 
         SUBCOS = (SCSMAX-SCSMIN)/2.0D0*SUBCOS+(SCSMAX+SCSMIN)/2.0D0
         AJACOB = AJACOB*(SCSMAX-SCSMIN)/2.0D0
C...1/u
      ELSE IF (KPAR.EQ.3) THEN
         E1 = (GPRI(3)+AMASS2(1)-AMASS2(2))/2.D0/GPRI(4)
         P1 = SQRT((E1+AMASS1(1))*(E1-AMASS1(1)))
         E5 = (GPRI(4) - GVAL(1) + GVAL(2))/2.0D0
         P5 = SQRT(E5**2 - AMASS2(IPX(5)))
 
         DMIN = 2*(E1*E5 + P1*P5*SCSMIN) - AMASS2(IPX(5))
         DMAX = 2*(E1*E5 + P1*P5*SCSMAX) - AMASS2(IPX(5))
 
         D = DMIN*(DMAX/DMIN)**RN
         SUBCOS = E1*E5/P1/P5 - (D + AMASS2(IPX(5)))/2.D0/P1/P5
         AJACOB = D*LOG(DMAX/DMIN)/2.0D0/P1/P5
C...1/t
      ELSE IF (KPAR.EQ.4) THEN
         E2 = (GPRI(3)+AMASS2(2)-AMASS2(1))/2.D0/GPRI(4)
         P2 = SQRT((E2+AMASS1(2))*(E2-AMASS1(2)))
         E6 = (GPRI(4) - GVAL(1) - GVAL(2))/2.0D0
         P6 = SQRT(E6**2 - AMASS2(IPX(6)))
 
         DMIN = 2*(E2*E6 + P2*P6*SCSMIN) - AMASS2(IPX(6))
         DMAX = 2*(E2*E6 + P2*P6*SCSMAX) - AMASS2(IPX(6))
 
         XIMIN = 1.0D0/DMAX
         XIMAX = 1.0D0/DMIN
         XI = XIMAX + (XIMIN - XIMAX)*RN
         SUBCOS = -E2*E6/P2/P6 + (1.0D0/XI + AMASS2(IPX(6)))/2.D0/P2/P6
         AJACOB = (XIMAX - XIMIN)/XI**2/2.0D0/P2/P6
 
C...1/u
      ELSE IF (KPAR.EQ.5) THEN
         E1 = (GPRI(3)+AMASS2(1)-AMASS2(2))/2.D0/GPRI(4)
         P1 = SQRT((E1+AMASS1(1))*(E1-AMASS1(1)))
         E5 = (GPRI(4) - GVAL(1) + GVAL(2))/2.0D0
         P5 = SQRT(E5**2 - AMASS2(IPX(5)))
 
         VMIN = -LOG(1.0D0 + 2.0D0/EPS)/2.0D0
         VMAX = LOG(1.0D0 + 2.0D0/EPS)/2.0D0
         V = VMIN + (VMAX - VMIN)*RN
 
         RNA = (1.0D0 + EPS)*TANH(V)
         AJACOB = (VMAX - VMIN)/COSH(V)**2*(1.0D0 + EPS)
 
         RNA = (RNA + 1.0D0)/2.0D0
         AJACOB = AJACOB/2.0D0
 
 
         DMAX = 2.0D0*E1*E5 - 2.0D0*P1*P5*SCSMIN - AMASS2(IPX(5))
         DMIN = 2.0D0*E1*E5 - 2.0D0*P1*P5*SCSMAX - AMASS2(IPX(5))
 
         D = DMIN*(DMAX/DMIN)**RNA
         SUBCOS = (2.0D0*E1*E5 - D - AMASS2(IPX(5)))/2.0D0/P1/P5
         AJACOB = AJACOB*D*LOG(DMAX/DMIN)/2.0D0/P1/P5
 
C...1/t
      ELSE IF (KPAR.EQ.6) THEN
         E2 = (GPRI(3)+AMASS2(2)-AMASS2(1))/2.D0/GPRI(4)
         P2 = SQRT((E2+AMASS1(2))*(E2-AMASS1(2)))
         E6 = (GPRI(4) - GVAL(1) - GVAL(2))/2.0D0
         P6 = SQRT(E6**2 - AMASS2(IPX(6)))
 
         VMIN = -LOG(1.0D0 + 2.0D0/EPS)/2.0D0
         VMAX = LOG(1.0D0 + 2.0D0/EPS)/2.0D0
         V = VMIN + (VMAX - VMIN)*RN
 
         RNA = (1.0D0 + EPS)*TANH(V)
         AJACOB = (VMAX - VMIN)/COSH(V)**2*(1.0D0 + EPS)
 
         RNA = (RNA + 1.0D0)/2.0D0
         AJACOB = AJACOB/2.0D0
 
 
         DMIN = 2.0D0*E2*E6 + 2.0D0*P2*P6*SCSMIN - AMASS2(IPX(6))
         DMAX = 2.0D0*E2*E6 + 2.0D0*P2*P6*SCSMAX - AMASS2(IPX(6))
 
         IF (DMAX.LT.DMIN) THEN
            WRITE(*,*) SCSMIN,SCSMAX,DMAX,DMIN
         ENDIF
 
         D = DMIN*(DMAX/DMIN)**RNA
         SUBCOS = -(2.0D0*E2*E6 - D - AMASS2(IPX(6)))/2.0D0/P2/P6
         AJACOB = AJACOB*D*LOG(DMAX/DMIN)/2.0D0/P2/P6
 
      ELSE IF (KPAR.EQ.7) THEN
         E1 = (GPRI(3)+AMASS2(1)-AMASS2(2))/2.D0/GPRI(4)
         P1 = SQRT((E1+AMASS1(1))*(E1-AMASS1(1)))
         E2 = (GPRI(3)+AMASS2(2)-AMASS2(1))/2.D0/GPRI(4)
         P2 = SQRT((E2+AMASS1(2))*(E2-AMASS1(2)))
 
C...Particle 1
         PF4(1,1) = 0
         PF4(2,1) = 0
         PF4(3,1) = P1
         PF4(4,1) = E1
 
C...Particle 2
         PF4(1,2) = 0
         PF4(2,2) = 0
         PF4(3,2) = -P2
         PF4(4,2) = E2
 
C...q1 q2
         COS1 = GVAL(1)
         SIN1 = SQRT((1.0D0 - COS1)*(1.0D0 + COS1))
         PHI1 = GVAL(2)
 
         EQ1 = (GPRI(3) + GVAL(3) - GVAL(4))/2.D0/GPRI(4)
         PQ1 = SQRT(EQ1*EQ1 - GVAL(3))
         Q1(1) = PQ1*SIN1*COS(PHI1)
         Q1(2) = PQ1*SIN1*SIN(PHI1)
         Q1(3) = PQ1*COS1
         Q1(4) = EQ1
 
         EQ2 = (GPRI(3) + GVAL(4) - GVAL(3))/2.D0/GPRI(4)
         Q2(1) = -Q1(1)
         Q2(2) = -Q1(2)
         Q2(3) = -Q1(3)
         Q2(4) = EQ2
 
         E5 = (GVAL(4)+AMASS2(IPX(5))-AMASS2(IPX(6)))
     $        /2.0D0/SQRT(GVAL(4))
         P5 = SQRT((E5-AMASS1(IPX(5)))*(E5+AMASS1(IPX(5))))
         E6 = (GVAL(4)+AMASS2(IPX(6))-AMASS2(IPX(5)))
     $        /2.0D0/SQRT(GVAL(4))
         P6 = P5
 
C...p1-p5
         IF (RN.LT.0.25D0) THEN
C            idg=1
            CALL LABTOK(PF4(1,1),Q2,PF4(1,1),PF4(1,2), PK1,PK2)
            RN = RN*4.0D0
            AJACOB = 4.0D0
            EE1 = PK1(4)
            PP1 = SQRT((EE1-AMASS1(1))*(EE1+AMASS1(1)))
            ETA = 0.0D0
 
            DMIN = 2.0D0*(EE1*AMASS2(IPX(5))/(E5+P5)+P5*AMASS2(1)
     $           /(EE1+PP1)+ETA*PP1*P5)-AMASS2(1)-AMASS2(IPX(5))
 
            EPS = 0
            IF (DMIN.LE.0) THEN
               EPS = AMASS2(1) - DMIN
               DMIN = DMIN + EPS
            ENDIF
 
C            eta = 2.0d0
            ETA = 1.0D0
            DMAX = 2.0D0*(EE1*AMASS2(IPX(5))/(E5+P5)+P5*AMASS2(1)
     $           /(EE1+PP1)+ETA*PP1*P5)-AMASS2(1)-AMASS2(IPX(5))+EPS
 
            IF (DMIN.GT.DMAX) GOTO 999
 
            D = DMIN*(DMAX/DMIN)**RN
            AJACOB = AJACOB*D*LOG(DMAX/DMIN)/PP1/P5/2.D0
            D = D - EPS
C            d1 = d
            D = (D+AMASS2(1)+AMASS2(IPX(5)))/2.0D0
            SUBCOS = EE1*AMASS2(IPX(5))/(E5+P5)+P5*AMASS2(1)/(EE1+PP1)-D
            SUBCOS = 1.D0 + SUBCOS/PP1/P5
 
            IF(ABS(SUBCOS).GT.1) GOTO 999
 
            SIN3 = SQRT((1.0D0-SUBCOS)*(1.0D0+SUBCOS))
 
            D13 = D
            D24 = E6*PK2(4)+P6*(PK2(1)*SIN3*COS(GVAL(8))
     $           +PK2(2)*SIN3*SIN(GVAL(8))+PK2(3)*SUBCOS)
            D14 = E6*PK1(4)+P6*(PK1(1)*SIN3*COS(GVAL(8))
     $           +PK1(2)*SIN3*SIN(GVAL(8))+PK1(3)*SUBCOS)
            D23 = E5*PK2(4)-P5*(PK2(1)*SIN3*COS(GVAL(8))
     $           +PK2(2)*SIN3*SIN(GVAL(8))+PK2(3)*SUBCOS)
 
C            if (d13.gt.min(d24,d14,d23)) goto 999
            IF (D13.GT.D24) GOTO 999
 
         ELSE IF (RN.LT.0.5D0) THEN
C...p2-p6
C            idg=2
            CALL LABTOK(PF4(1,2),Q2,PF4(1,1),PF4(1,2), PK1,PK2)
            RN = (RN - 0.25D0)*4.0D0
            AJACOB = 4.0D0
            EE2 = PK2(4)
            PP2 = SQRT((EE2-AMASS1(2))*(EE2+AMASS1(2)))
            ETA=0.D0
 
            GMIN = 2.0D0*(EE2*AMASS2(IPX(6))/(E6+P6)+P6*AMASS2(2)
     $           /(EE2+PP2)+ETA*PP2*P6)-AMASS2(2)-AMASS2(IPX(6))
            EPS=0
            IF (GMIN.LE.0) THEN
               EPS = AMASS2(2) - GMIN
               GMIN = GMIN+EPS
            ENDIF
 
C            eta = 2.0d0
            ETA = 1.0D0
            GMAX=2.0D0*(EE2*AMASS2(IPX(6))/(E6+P6)+P6*AMASS2(2)
     $           /(EE2+PP2)+ETA*PP2*P6)-AMASS2(2)-AMASS2(IPX(6))+EPS
 
            IF (GMIN.GT.GMAX) GOTO 999
 
            G = GMIN*(GMAX/GMIN)**RN
            AJACOB = AJACOB*G*LOG(GMAX/GMIN)/PP2/P6/2.0D0
            G = G - EPS
C            g1 = g
            G = (G+AMASS2(2)+AMASS2(IPX(6)))/2.0D0
            SUBCOS = EE2*AMASS2(IPX(6))/(E6+P6)+P6*AMASS2(2)/(EE2+PP2)-G
            SUBCOS = 1.0D0 + SUBCOS/PP2/P6
 
C            subcos = -subcos
 
            IF (ABS(SUBCOS).GT.1) GOTO 999
 
            SIN4 = SQRT((1.0D0 - SUBCOS)*(1.0D0 + SUBCOS))
 
            D13 = E5*PK1(4)+P5*(PK1(1)*SIN4*COS(GVAL(8))
     $           +PK1(2)*SIN4*SIN(GVAL(8))+PK1(3)*SUBCOS)
            D24 = G
            D14 = E6*PK1(4)-P6*(PK1(1)*SIN4*COS(GVAL(8))
     $           +PK1(2)*SIN4*SIN(GVAL(8))+PK1(3)*SUBCOS)
            D23 = E5*PK2(4)+P5*(PK2(1)*SIN4*COS(GVAL(8))
     $           +PK2(2)*SIN4*SIN(GVAL(8))+PK2(3)*SUBCOS)
 
C            if (d24.gt.min(d13,d14,d23)) goto 999
            IF (D24.GT.D13) GOTO 999
 
         ELSE IF (RN.LT.0.75D0) THEN
C...p1-p6
C         idg=3
            CALL LABTOK(PF4(1,1),Q2,PF4(1,1),PF4(1,2), PK1,PK2)
            RN = (RN - 0.5D0)*4.0D0
            AJACOB = 4.0D0
            EE1 = PK1(4)
            PP1 = SQRT((EE1-AMASS1(1))*(EE1+AMASS1(1)))
            ETA = 0.0D0
 
            DMIN = 2.0D0*(EE1*AMASS2(IPX(6))/(E6+P6)+P6*AMASS2(1)
     $           /(EE1+PP1)+ETA*PP1*P6)-AMASS2(1)-AMASS2(IPX(6))
            EPS = 0
            IF (DMIN.LE.0) THEN
               EPS = AMASS2(1) - DMIN
               DMIN = DMIN + EPS
            END IF
 
C            eta = 2.0d0
            ETA = 1.0D0
            DMAX = 2.0D0*(EE1*AMASS2(IPX(6))/(E6+P6)+P6*AMASS2(1)
     $           /(EE1+PP1)+ETA*PP1*P6)-AMASS2(1)-AMASS2(IPX(6))+EPS
 
            IF (DMIN.GT.DMAX) GOTO 999
 
            D = DMIN*(DMAX/DMIN)**RN
            AJACOB = AJACOB*D*LOG(DMAX/DMIN)/PP1/P6/2.0D0
            D = D - EPS
            D = (D + AMASS2(1) + AMASS2(IPX(6)))/2.0D0
            SUBCOS = EE1*AMASS2(IPX(6))/(E6+P6)+P6*AMASS2(1)/(EE1+PP1)-D
            SUBCOS = 1.0D0 + SUBCOS/PP1/P6
 
C            subcos = -subcos
 
            IF(ABS(SUBCOS).GT.1) GOTO 999
 
            SIN4 = SQRT((1.0D0 - SUBCOS)*(1.0D0 + SUBCOS))
            D14 = D
            D13 = E5*PK1(4)+P5*(PK1(1)*SIN4*COS(GVAL(8))
     $           +PK1(2)*SIN4*SIN(GVAL(8))+PK1(3)*SUBCOS)
            D24 = E6*PK2(4)-P6*(PK2(1)*SIN4*COS(GVAL(8))
     $           +PK2(2)*SIN4*SIN(GVAL(8))+PK2(3)*SUBCOS)
            D23 = E5*PK2(4)+P5*(PK2(1)*SIN4*COS(GVAL(8))
     $           +PK2(2)*SIN4*SIN(GVAL(8))+PK2(3)*SUBCOS)
 
C            if (d14.gt.min(d24,d13,d23)) goto 999
            IF (D14.GT.D23) GOTO 999
 
         ELSE
C...p2-p5
C         idg=4
            CALL LABTOK(PF4(1,2),Q2,PF4(1,1),PF4(1,2), PK1,PK2)
            RN = (RN - 0.75D0)*4.0D0
            AJACOB = 4.0D0
            EE2 = PK2(4)
            PP2 = SQRT((EE2-AMASS1(2))*(EE2+AMASS1(2)))
            ETA = 0.D0
            GMIN = 2.0D0*(EE2*AMASS2(IPX(5))/(E5+P5)+P5*AMASS2(2)
     $           /(EE2+PP2)+ETA*PP2*P5)-AMASS2(2)-AMASS2(IPX(5))
            EPS = 0.0D0
            IF (GMIN.LE.0) THEN
               EPS = AMASS2(2) - GMIN
               GMIN = GMIN + EPS
            END IF
 
C            eta = 2.0d0
            ETA = 1.0D0
            GMAX = 2.0D0*(EE2*AMASS2(IPX(5))/(E5+P5)+P5*AMASS2(2)
     $           /(EE2+PP2)+ETA*PP2*P5)-AMASS2(2)-AMASS2(IPX(5))+EPS
 
            IF (GMIN.GT.GMAX) GOTO 999
 
            G = GMIN*(GMAX/GMIN)**RN
            AJACOB = AJACOB*G*LOG(GMAX/GMIN)/PP2/P5/2.0D0
            G = G - EPS
            SUBCOS = EE2*AMASS2(IPX(5))/(E5+P5)+P5*AMASS2(2)/(EE2+PP2)-G
            SUBCOS = 1.0D0 + SUBCOS/PP2/P5
 
            IF(ABS(SUBCOS).GT.1) GOTO 999
 
            SIN3 = SQRT((1.0D0 - SUBCOS)*(1.0D0 + SUBCOS))
 
            D13 = E5*PK1(4)-P5*(PK1(1)*SIN3*COS(GVAL(8))
     $           +PK1(2)*SIN3*SIN(GVAL(8))+PK1(3)*SUBCOS)
            D24 = E6*PK2(4)+P6*(PK2(1)*SIN3*COS(GVAL(8))
     $           +PK2(2)*SIN3*SIN(GVAL(8))+PK2(3)*SUBCOS)
            D14 = E6*PK1(4)+P6*(PK1(1)*SIN3*COS(GVAL(8))
     $           +PK1(2)*SIN3*SIN(GVAL(8))+PK1(3)*SUBCOS)
            D23 = G
 
C            if (d23.gt.min(d13,d14,d24)) goto 999
            IF (D23.GT.D14) GOTO 999
 
         ENDIF
      ENDIF
 
      RETURN
 999  CONTINUE
      IREJ = 1
      RETURN
      END
 
C**********************************************************************
C*======================                                              *
C* FUNCTION DRN( ISEED)                                               *
C*======================                                              *
C*  Machine-independent Random number generator                       *
C*     General purpose Version,  OK as long as >= 32 bits             *
C*((Arguement))                                                       *
C*  ISEED: Seed                                                       *
C*                                                                    *
C**********************************************************************
 
*     REAL FUNCTION DRN*8(ISEED)
      DOUBLE PRECISION FUNCTION DRN(ISEED)
 
#include "define.h"
 
      COMMON/RANDM/RDM(31),RM1,RM2,IA1,IC1,M1,IX1,
     .                             IA2,IC2,M2,IX2,
     .                             IA3,IC3,M3,IX3
 
      REAL*8 PYR
 
C Generate Next number in sequence
 
      IX1    = MOD( IA1*IX1+IC1, M1 )
      IX2    = MOD( IA2*IX2+IC2, M2 )
      IX3    = MOD( IA3*IX3+IC3, M3 )
      J      = 1 + (31*IX3)/M3
 
#if PSMODEL > 0
      DRN    = PYR(0)
C      DRN    = RDM(J)
#else
      DRN    = RDM(J)
#endif
 
      RDM(J) = ( FLOAT(IX1)+FLOAT(IX2)*RM2 )*RM1
 
C Omit following statement if function arguement passed by value:
 
      ISEED = IX1
      RETURN
      END
 
C-----------------------------------------
C Set energy scale and coupling constant.
C-----------------------------------------
      SUBROUTINE GRCSENE
      IMPLICIT REAL*8(A-H,O-Z)
 
#include "define.h"
 
      INCLUDE 'inclk.h'
      INCLUDE 'grchad.inc'
 
      INTEGER I
      REAL*8 PT2,PM2,PT2MX

C...Set renormalization scale
C...Q^2 = s^
      IF (ICOUP.EQ.1) THEN
         GRCQ = GPRI(4)
 
C...Q^2 = Average Mt^2
      ELSE IF (ICOUP.EQ.2) THEN
         PT2 = 0.0D0
         PM2 = 0.0D0
         DO I = 3,NNF
            PT2 = PGRC(1,I)**2 + PGRC(2,I)**2 + PT2
            PM2 = AMASS2(I) + PM2
         ENDDO
         GRCQ = SQRT((PT2 + PM2)/(NNF - 2))
 
C...Q^2 = Total Mt^2
      ELSE IF (ICOUP.EQ.3) THEN
         PT2 = 0.0D0
         PM2 = 0.0D0
         DO I = 3,NNF
            PT2 = PGRC(1,I)**2 + PGRC(2,I)**2 + PT2
            PM2 = AMASS2(I) + PM2
         ENDDO
         GRCQ = SQRT(PT2 + PM2)
 
C...Q^2 = max Mt^2
      ELSE IF (ICOUP.EQ.4) THEN
         PT2MX = 0.0D0
         DO I = 3,NNF
            PT2MX = MAX(PT2MX,
     $           PGRC(1,IPX(I))**2+PGRC(2,IPX(I))**2+AMASS2(IPX(I)))
         ENDDO
         GRCQ = SQRT(PT2MX)
 
C...Q^2 = constant
      ELSE IF (ICOUP.EQ.5) THEN
         IF (GRCQ.EQ.0.0D0) THEN
            WRITE(*,*) 'GRC::Error: Set grcq energy scale'
         ENDIF
 
C...User original scale
      ELSE IF (ICOUP.EQ.6) THEN
         CALL GRCUSRSETQ
      ENDIF
 
C...Set factorization scale
C...as same as the renormalization scale
      IF (IFACT.EQ.0) THEN
         GRCFAQ = GRCQ
C...Q^2 = s^
      ELSE IF (IFACT.EQ.1) THEN
         GRCFAQ = GPRI(4)
 
C...Q^2 = Average Mt^2
      ELSE IF (IFACT.EQ.2) THEN
         PT2 = 0.0D0
         PM2 = 0.0D0
         DO I = 3,NNF
            PT2 = PGRC(1,I)**2 + PGRC(2,I)**2 + PT2
            PM2 = AMASS2(I) + PM2
         ENDDO
         GRCFAQ = SQRT((PT2 + PM2)/(NNF - 2))
 
C...Q^2 = Total Mt^2
      ELSE IF (IFACT.EQ.3) THEN
         PT2 = 0.0D0
         PM2 = 0.0D0
         DO I = 3,NNF
            PT2 = PGRC(1,I)**2 + PGRC(2,I)**2 + PT2
            PM2 = AMASS2(I) + PM2
         ENDDO
         GRCFAQ = SQRT(PT2 + PM2)
 
C...Q^2 = max Mt^2
      ELSE IF (IFACT.EQ.4) THEN
         PT2MX = 0.0D0
         DO I = 3,NNF
            PT2MX = MAX(PT2MX,
     $           PGRC(1,IPX(I))**2+PGRC(2,IPX(I))**2+AMASS2(IPX(I)))
         ENDDO
         GRCFAQ = SQRT(PT2MX)
 
C...Q^2 = constant
      ELSE IF (IFACT.EQ.5) THEN
         IF (GRCFAQ.EQ.0.0D0) THEN
            WRITE(*,*) 'GRC::Error: Set grcfaq energy scale'
         ENDIF
 
C...User original scale
      ELSE IF (IFACT.EQ.6) THEN
         CALL GRCUSRSETQ
      ENDIF

C...Set coupling constants
#if PDFLIB == 1
C...PYTHIA Alpha_em, Alpha_s
      ALPHA = PYALEM(GRCQ**2)
      ALPHAS = PYALPS(GRCQ**2)
#elif PDFLIB == 2
C...PDF lib Alpha_em, Alpha_s
C      alpha = 1.0d0/128.90228d0
C      alpha = 1.0d0/128.899994d0
C...Set Alpha_s
      ALPHAS = ALPHAS2(GRCQ)
#elif PDFLIB == 3
C...Set Alpha_s
      ALPHAS = alphasPDF(GRCQ)
#endif
 
      RETURN
      END
 
C-----------------------------------
C Width correction of vector bosons
C-----------------------------------
      SUBROUTINE GRCSWID(Q2)
      IMPLICIT REAL*8(A-H,O-Z)
 
#include "define.h"
 
C...Particle properties + some flavour parameters.
      COMMON/PYDAT2/KCHG(500,4),PMAS(500,4),PARF(2000),VCKM(4,4)
 
      INCLUDE 'inclk.h'
      INCLUDE 'grchad.inc'
 
      INTEGER I,IMR,IR
      REAL*8 Q2,EB,PB,PHASE
      REAL*8 ALA,SINW,EALP,SALP,GW,WF,WD
 
      INTEGER ICBU(6),ICBD(6)
      DATA ICBU/1,2,1,1,2,2/
      DATA ICBD/1,2,2,3,1,3/
 
C...Now only W boson...
C...Search external particles
      IMR = 0
      DO I=3,NNF
         IF (ABS(KFCODE(I)).EQ.24) THEN
            IMR = IMR + 1
            IR = I
         ELSE IF (ABS(KFCODE(I)).EQ.23) THEN
            IMR = IMR + 1
            IR = I
            WRITE(*,*) 's-dependent width: NOT supported yet...'
            STOP
         ELSE IF (ABS(KFCODE(I)).EQ.25) THEN
            IMR = IMR + 1
            IR = I
            WRITE(*,*) 's-dependent width: NOT supported yet...'
            STOP
         ENDIF
      ENDDO
 
      IF (IMR.EQ.1) THEN
         ALA = 0
         SINW = 1 - (AMW/AMZ)**2

C...Set coupling constants
#if PDFLIB == 1
C...PYTHIA Alpha_em, Alpha_s
         EALP = PYALEM(Q2)
         SALP = PYALPS(Q2)
#elif PDFLIB == 2
C...PDF lib Alpha_em, Alpha_s
         EALP = 1.0D0/128.90228D0
         SALP = ALPHAS2(SQRT(Q2))
#elif PDFLIB == 3
C...Set Alpha_s
         EALP = 1.0D0/128.90228D0
         SALP = alphasPDF(GRCQ)
#endif

         GW = 4*PI*EALP/SINW
 
C...W -> u-type d-type (Quark sector)
         DO I = 1,6
            EB = (AMW**2 + AMUQ(ICBU(I))**2 - AMDQ(ICBD(I))**2)
     $           /2.0D0/AMW
            PB = SQRT( (EB - AMUQ(ICBU(I)))*(EB + AMUQ(ICBU(I))) )
            PHASE = 2.0D0*PB/16.0D0/PI/AMW**2
            WF = 3.0D0*(1 + SALP/PI)
            WD = GW/3.0D0*AMW**2
            ALA = PHASE*WD*WF*VCKM(ICBU(I),ICBD(I)) + ALA
         ENDDO
 
C...W -> l nu (Lepton sector)
         DO I = 1,3
            EB = (AMW**2 + AMLP(I)**2 - AMNU(I)**2)/2.0D0/AMW
            PB = SQRT( (EB - AMLP(I))*(EB + AMLP(I)) )
            PHASE = 2.0D0*PB/16.0D0/PI/AMW**2
            WF = 1.0D0
            WD = GW/3.0D0*AMW**2
            ALA = PHASE*WD*WF + ALA
         ENDDO
 
C...Set Width
         AGW = ALA
 
      ELSE
         WRITE(*,*) 's-dependence width:',
     $        ' NOT supported double resonance...'
         STOP
      ENDIF
 
      RETURN
      END
 
C----------------------------
C Set kinematical variables.
C----------------------------
      SUBROUTINE GRCSVAL
      IMPLICIT REAL*8(A-H,O-Z)

#include "define.h"
 
      PARAMETER (MXDIM = 50, NDMX = 50 )
      COMMON /BPARM1/ XL(MXDIM),XU(MXDIM),NDIM,NWILD,
     $     IG(MXDIM),NCALL
 
      INCLUDE 'inclk.h'
      INCLUDE 'grchad.inc'

C...Grace Variable
      INTEGER IDCYST
      REAL*8 RNA,AJACOB
      REAL*8 E1,E2,E3,E4,E5,E6,E7,P3,P4,P5,P6,P7
      REAL*8 EQ,PQ,EQ1,EQ2,PQ1,PQ2,EQ34,PQ34,EQ56,PQ56,SUME45
      REAL*8 E3MIN,E4MIN,E5MIN,E6MIN,SUMMIN,SUMMAX,SUBMIN,SUBMAX
      REAL*8 E7MIN,E7MAX,E56MIN,E56MAX

      REAL*8 Q12,Q22,Q34,Q45,Q67,QM
      REAL*8 QMIN,QMAX,QE3MIN,QE3MAX,QESMIN,QESMAX
      REAL*8 Q56MIN,Q56MAX,Q567MIN,Q567MAX
      REAL*8 RMAS,RWID,THEMIN,THEMAX,THE,XCP
 
      REAL*8 COS1,SIN1,COS4,SIN4,COS5,SIN5,COS6,COS7,SIN7
      REAL*8 COS34,COS45,COS67,COSIJ,COSKL,EQIJ,SINIJ
      REAL*8 COSQ1,SINQ1,COSQ2,SINQ2,COSQ34,SINQ34,SINQ56
      REAL*8 COS4MIN,COS4MIN1,COS4MIN2
      REAL*8 COS4MAX,COS4MAX1,COS4MAX2
      REAL*8 COS5MIN,COS5MIN1,COS5MIN2,COS5MIN3,COS5MIN4
      REAL*8 COS5MAX,COS5MAX1,COS5MAX2,COS5MAX3,COS5MAX4
      REAL*8 COS6MIN,COS6MIN1,COS6MIN2,COS6MIN3,COS6MIN4
      REAL*8 COS6MAX,COS6MAX1,COS6MAX2,COS6MAX3,COS6MAX4
      REAL*8 COSQ2MIN,COSQ1MIN,COSQ2MIN1,COSQ2MIN2
      REAL*8 COSQ2MAX,COSQ1MAX,COSQ2MAX1,COSQ2MAX2

      REAL*8 RANSUM,XSND(30)

C######################
C# Set 2-body process #
C######################
      IF (NNF.EQ.4) THEN
         IF (IGSUB.EQ.421) THEN
C...Total cross section with all decay particles.
            YACOB = YACOB/GRCWBR(17)

C...Pick up final state flavor.
            RANSUM = XRUN(6)*GDCYSUM
            IDCYST = 0
            DO WHILE (RANSUM.GT.0.0D0)
               IDCYST = IDCYST + 1
               RANSUM = RANSUM - GDCYRAT(IDCYST)
            ENDDO
            KFCODE(3) = IGWDCY(2*GDCYPR1(IDCYST)-1)
            KFCODE(4) = IGWDCY(2*GDCYPR1(IDCYST))
            AMASS1(3) = GWDCYMS(ABS(KFCODE(3)))
            AMASS1(4) = GWDCYMS(ABS(KFCODE(4)))
            AMASS2(3) = AMASS1(3)**2
            AMASS2(4) = AMASS1(4)**2
            AMLP(1) = AMASS1(3)
            AMNU(1) = AMASS1(4)

C...Apply Br. ratio.
            YACOB = YACOB*GDCYSUM

         ELSE IF (IGSUB.EQ.500) THEN
C...Z decays.
            XSND(1) = 0.0D0
            DO I = 2,IGNDCY+1
               XSND(I) = XSND(I-1)+GDCYRAT(I-1)/GDCYSUM
            ENDDO

            RNA = XRUN(6)
            DO I = 1,IGNDCY
               IF (RNA.GT.XSND(I).AND.RNA.LT.XSND(I+1)) THEN
                  YACOB = YACOB/(XSND(I+1)-XSND(I))
                  KFCODE(3) = IGZDCY(2*GDCYPR1(I)-1)
                  KFCODE(4) = IGZDCY(2*GDCYPR1(I))
               ENDIF
            ENDDO
            AMASS1(3) = GWDCYMS(ABS(KFCODE(3)))
            AMASS1(4) = GWDCYMS(ABS(KFCODE(4)))
            AMASS2(3) = AMASS1(3)**2
            AMASS2(4) = AMASS1(4)**2
            AMLP(1) = AMASS1(3)

C...Colour freedom
            IF (ABS(KFCODE(3)).LT.7) THEN
               YACOB = YACOB*3.0D0
            ENDIF
         ENDIF

C...Set tau, yst, s, w, x1, x2, beta, and gamma
         CALL GRCSSPX

C...Set Cos1, Phi1
         RNA = XRUN(3)
         CALL GRCICOS
         CALL GRCSCOS(RNA, COS1, AJACOB)
         GVAL(1) = COS1
         YACOB = YACOB*AJACOB
 
         GVAL(2) = XRUN(4)*2.0D0*PI
         YACOB = YACOB*2.0D0*PI
 
C...Set Pt
         E3 = (GPRI(3)+AMASS2(3)-AMASS2(4))/2.0D0/GPRI(4)
         SIN1 = SQRT((1.0D0 - COS1)*(1.0D0 + COS1))
         GVAL(3) = SQRT(E3**2 - AMASS2(3))*SIN1
 
C...Set Flavor
         IF (NDIM.GT.4.AND.GPRI(10).EQ.0.0D0) THEN
            GPRI(10) = XRUN(5)
         ENDIF
 
C######################
C# Set 3-body process #
C######################
      ELSE IF (NNF.EQ.5) THEN 
C##############################################
C# Q^2 is NOT a resonance(glu/gam splitting). #
C##############################################
         IF (KTYPE.LT.4) THEN

            IF (KTYPE.EQ.2) THEN
 
C...s-dependence width
               IF (ISDEP.EQ.1) THEN
C...Special case
                  QMIN = 1.0D0
                  QMAX = (GRCECM-AMASS1(IPX(4))-AMASS1(IPX(5)))**2
 
C...Set W mass for wbb
                  IF (IGSUB.EQ.451) THEN
                     RMAS = AMW
                     RWID = AGW
                     RNA = XRUN(9)
C...Set Z mass for wbb
                  ELSE IF (IGSUB.EQ.410) THEN
                     RMAS = AMZ
                     RWID = AGZ
                     RNA = XRUN(8)
                  ELSE IF (IGSUB.EQ.411) THEN
                     RMAS = AMZ
                     RWID = AGZ
                     RNA = XRUN(9)
C...Set W mass for wcc
                  ELSE IF (IGSUB.EQ.457) THEN
                     RMAS = AMW
                     RWID = AGW
                     RNA = XRUN(9)
C...Set Z mass for wcc
                  ELSE IF (IGSUB.EQ.414) THEN
                     RMAS = AMZ
                     RWID = AGZ
                     RNA = XRUN(8)
                  ELSE IF (IGSUB.EQ.415) THEN
                     RMAS = AMZ
                     RWID = AGZ
                     RNA = XRUN(9)
                  ELSE
                     WRITE(*,*) "NOW ONLY SUPPORT..."
                     WRITE(*,*) "...451,410,411,457,414,415..."
                     STOP
                  ENDIF
 
                  THEMIN = ATAN((QMIN - RMAS**2)/(RMAS*RWID))
                  THEMAX = ATAN((QMAX - RMAS**2)/(RMAS*RWID))
                  THE = THEMIN + (THEMAX - THEMIN)*RNA
                  QM = RMAS*RWID*TAN(THE) + RMAS**2
                  YACOB = YACOB*(THEMAX - THEMIN)
     $                 *((QM-RMAS**2)**2+(RMAS*RWID)**2)/(RMAS*RWID)
 
C...s-dep. Q^2
                  YACOB = YACOB*QM*RWID/PI/RMAS
     $                 /((QM-RMAS**2)**2+(RMAS*RWID)**2)
 
                  AMASS2(IPX(3)) = QM
                  AMASS1(IPX(3)) = SQRT(QM)
                  GVAL(7) = SQRT(QM)
 
               ENDIF

#if PSMODEL > 0
C...Apply Br. ratio.
               IF (IGSUB.EQ.410.OR.IGSUB.EQ.411
     $              .OR.IGSUB.EQ.414.OR.IGSUB.EQ.415) THEN
                  YACOB = YACOB*GDCYSUM
               ENDIF
#endif

               IF (IGSUB.EQ.560) THEN
C...W decays.
                  XSND(1) = 0.0D0
                  DO I = 2,IGNDCY+1
                     XSND(I) = XSND(I-1)+GDCYRAT(I-1)/GDCYSUM
                  ENDDO
                  
                  RNA = XRUN(9)
                  DO I = 1,IGNDCY
                     IF (RNA.GT.XSND(I).AND.RNA.LT.XSND(I+1)) THEN
                        YACOB = YACOB/(XSND(I+1)-XSND(I))
                        KFCODE(3) = IGWDCY(2*GDCYPR1(I)-1)
                        KFCODE(4) = IGWDCY(2*GDCYPR1(I))
                     ENDIF
                  ENDDO
                  AMASS1(3) = GWDCYMS(ABS(KFCODE(3)))
                  AMASS1(4) = GWDCYMS(ABS(KFCODE(4)))
                  AMASS2(3) = AMASS1(3)**2
                  AMASS2(4) = AMASS1(4)**2
                  AMLP(1) = AMASS1(3)
                  AMNU(1) = AMASS1(4)

C...Colour freedom
                  IF (ABS(KFCODE(3)).LT.7) THEN
                     YACOB = YACOB*3.0D0
                  ENDIF
               ENDIF
 
C...Set tau, yst, s, w, x1, x2, beta, and gamma
               CALL GRCSSPX
C...Set Q2(ij)
               RNA = XRUN(5)
               CALL GRCSQ12(RNA, Q12, AJACOB)
               GVAL(3) = Q12
               YACOB = YACOB*AJACOB
 
C###############################################
C# Q^2 is a resonance production(W+-, Z, top). #
C###############################################
            ELSE IF (KTYPE.EQ.3) THEN

               IF (IGSUB.EQ.422.OR.IGSUB.EQ.423
     $              .OR.IGSUB.EQ.464) THEN
C...W decays.
                  XSND(1) = 0.0D0
                  DO I = 2,IGNDCY+1
                     XSND(I) = XSND(I-1)+GDCYRAT(I-1)/GDCYSUM
                  ENDDO

                  RNA = XRUN(9)
                  DO I = 1,IGNDCY
                     IF (RNA.GT.XSND(I).AND.RNA.LT.XSND(I+1)) THEN
                        YACOB = YACOB/(XSND(I+1)-XSND(I))
                        KFCODE(3) = IGWDCY(2*GDCYPR1(I)-1)
                        KFCODE(4) = IGWDCY(2*GDCYPR1(I))
                     ENDIF
                  ENDDO
                  AMASS1(3) = GWDCYMS(ABS(KFCODE(3)))
                  AMASS1(4) = GWDCYMS(ABS(KFCODE(4)))
                  AMASS2(3) = AMASS1(3)**2
                  AMASS2(4) = AMASS1(4)**2
                  AMLP(1) = AMASS1(3)
                  AMNU(1) = AMASS1(4)

C...Colour freedom
                  IF (ABS(KFCODE(3)).LT.7) THEN
                     YACOB = YACOB*3.0D0
                  ENDIF

               ELSE IF (IGSUB.EQ.501.OR.IGSUB.EQ.502) THEN
C...Z decays.
                  XSND(1) = 0.0D0
                  DO I = 2,IGNDCY+1
                     XSND(I) = XSND(I-1)+GDCYRAT(I-1)/GDCYSUM
                  ENDDO

                  RNA = XRUN(9)
                  DO I = 1,IGNDCY
                     IF (RNA.GT.XSND(I).AND.RNA.LT.XSND(I+1)) THEN
                        YACOB = YACOB/(XSND(I+1)-XSND(I))
                        KFCODE(3) = IGZDCY(2*GDCYPR1(I)-1)
                        KFCODE(4) = IGZDCY(2*GDCYPR1(I))
                     ENDIF
                  ENDDO
                  AMASS1(3) = GWDCYMS(ABS(KFCODE(3)))
                  AMASS1(4) = GWDCYMS(ABS(KFCODE(4)))
                  AMASS2(3) = AMASS1(3)**2
                  AMASS2(4) = AMASS1(4)**2
                  AMLP(1) = AMASS1(3)

C...Colour freedom
                  IF (ABS(KFCODE(3)).LT.7) THEN
                     YACOB = YACOB*3.0D0
                  ENDIF
               ENDIF

C...Set Q2(ij)
               RNA = XRUN(5)
               CALL GRCSQ12(RNA, Q12, AJACOB)
               GVAL(3) = Q12
               YACOB = YACOB*AJACOB
C...Set tau, yst, s, w, x1, x2, beta, and gamma
               CALL GRCSSPX
            ENDIF
 
C...Set Cos1, Phi1
            RNA = XRUN(3)
            CALL GRCICOS
            CALL GRCSCOS(RNA, COS1, AJACOB)
            GVAL(1) = COS1
            YACOB = YACOB*AJACOB
 
            GVAL(2) = XRUN(4)*2.0D0*PI
            YACOB = YACOB*2.0D0*PI
 
C...Set Cos(ij), Phi(ij) in (ij) CM frame
            GVAL(5) = XRUN(7)*2.0D0*PI - GVAL(2)
            YACOB = YACOB*2.0D0*PI
 
            RNA = XRUN(6)
            CALL GRCSBCS(INT(GSPR(1)),GEPS(1),RNA,
     $           COSIJ,AJACOB,-1.0D0,1.0D0)
            GVAL(4) = COSIJ
            YACOB = YACOB*AJACOB
 
C...Set Pt
            EQIJ = (GPRI(3)+AMASS2(IPX(3))-GVAL(3))/2.0D0/GPRI(4)
            SINIJ = SQRT((1.0D0 - GVAL(1))*(1.0D0 + GVAL(1)))
            GVAL(6) = SQRT(EQIJ**2 - AMASS2(IPX(3)))*SINIJ
 
C#################################
C# 3-Body Fusion type production #
C#################################
         ELSE IF (KTYPE.EQ.4) THEN
 
C...Fermion Propagator
C...Set tau, yst, s, w, x1, x2, beta, and gamma
            CALL GRCSSPX
 
            E1 = (GPRI(3)+AMASS2(1)-AMASS2(2))/2.D0/GPRI(4)
            E2 = (GPRI(3)+AMASS2(2)-AMASS2(1))/2.D0/GPRI(4)
C...Set E3
            E4MIN = SQRT(GRCPTCUT(IPX(4)-2)**2+AMASS2(IPX(4)))
            E5MIN = SQRT(GRCPTCUT(IPX(5)-2)**2+AMASS2(IPX(5)))
 
            QE3MIN = SQRT(GRCPTCUT(IPX(3)-2)**2+AMASS2(IPX(3)))
C            qe3max = gpri(4)-e4min-e5min
            QE3MAX = (GPRI(3) + AMASS2(IPX(3)))/2.0D0/GPRI(4)
 
            GVAL(1) = QE3MIN*(QE3MAX/QE3MIN)**XRUN(3)
            YACOB = YACOB*GVAL(1)*LOG(QE3MAX/QE3MIN)
 
            IF (QE3MIN.GT.QE3MAX) THEN
               IREJ = 1
               WRITE(*,*) 'qe3min.gt.qe3max'
               GO TO 999
            ENDIF
 
C...Set E4-E5
            SUME45 = GPRI(4) - GVAL(1)
            QESMIN = -SUME45 + 2.0D0*E4MIN
            QESMAX = SUME45 - 2.0D0*E5MIN
 
C            qesmin = -sume45 + amass1(ipx(4)) + amass1(ipx(5))
C            qesmax = sume45 - amass1(ipx(4)) - amass1(ipx(5))
 
            GVAL(2) = QESMIN + (QESMAX - QESMIN)*XRUN(4)
            YACOB = YACOB*(QESMAX - QESMIN)
 
            IF (QESMIN.GT.QESMAX) THEN
               IREJ = 1
C               write(*,*) 'qesmin.gt.qesmax'
               GO TO 999
            ENDIF
 
C...Set Phi4
            GVAL(5) = XRUN(7)*2.0D0*PI
            YACOB = YACOB*2.0D0*PI
 
            E3 = GVAL(1)
            P3 = SQRT(E3**2 - AMASS2(IPX(3)))
 
            E4 = (SUME45 + GVAL(2))/2.0D0
            E5 = (SUME45 - GVAL(2))/2.0D0
            P4 = SQRT(E4**2 - AMASS2(IPX(4)))
            P5 = SQRT(E5**2 - AMASS2(IPX(5)))
 
            IF (E3.LT.AMASS1(IPX(3))) THEN
               IREJ = 1
               WRITE(*,*) 'e3.lt.amass1(ipx(3))',GVAL(1),GVAL(2)
               GO TO 999
            ENDIF
            IF (E4.LT.AMASS1(IPX(4))) THEN
               IREJ = 1
               WRITE(*,*) 'e4.lt.amass1(ipx(4))',GVAL(1),GVAL(2)
               GO TO 999
            ENDIF
            IF (E5.LT.AMASS1(IPX(5))) THEN
               IREJ = 1
               WRITE(*,*) 'e5.lt.amass1(ipx(5))',GVAL(1),GVAL(2)
               GO TO 999
            ENDIF
 
            IF (P4+P5-P3.LT.0) THEN
               IREJ = 1
C               write(*,*) p4+p5-p3,p3-p4+p5,p3+p4-p5,gpri(4),p3
               GO TO 999
            ENDIF
            IF (P3-P4+P5.LT.0) THEN
               IREJ = 1
C               write(*,*) p4+p5-p3,p3-p4+p5,p3+p4-p5
               GO TO 999
            ENDIF
            IF (P3+P4-P5.LT.0) THEN
               IREJ = 1
C               write(*,*) p4+p5-p3,p3-p4+p5,p3+p4-p5
               GO TO 999
            ENDIF
 
C...Set Cos4
            IF (P4.LT.GRCPTCUT(IPX(4)-2)) THEN
               IREJ = 1
               GO TO 999
            ENDIF
C...Due to Pt
            COS4MIN1 = -SQRT(P4**2-GRCPTCUT(IPX(4)-2)**2)/P4
            COS4MAX1 = SQRT(P4**2-GRCPTCUT(IPX(4)-2)**2)/P4
C...Due to Eta
            COS4MIN2 = -E4/P4*TANH(GRCETACUT(IPX(4)-2))
            COS4MAX2 = E4/P4*TANH(GRCETACUT(IPX(4)-2))
 
            COS4MIN = MAX(-1.0D0,COS4MIN1,COS4MIN2)
            COS4MAX = MIN(1.0D0,COS4MAX1,COS4MAX2)
 
            RNA = XRUN(5)
            CALL GRCSBCS(INT(GSPR(1)),GEPS(1),RNA,
     $           COS4,AJACOB,COS4MIN,COS4MAX)
            GVAL(3) = COS4
            YACOB = YACOB*AJACOB
 
            SIN4 = SQRT((1.0D0 + COS4)*(1.0D0 - COS4))
 
C Check!!
            IF (P5.LT.GRCPTCUT(IPX(5)-2)) THEN
               IREJ = 1
               GO TO 999
            ENDIF
 
C...Due to Pt
            COS5MIN1 = -SQRT(P5**2-GRCPTCUT(IPX(5)-2)**2)/P5
            COS5MAX1 = SQRT(P5**2-GRCPTCUT(IPX(5)-2)**2)/P5
C...Due to Eta
            COS5MIN2 = -E5/P5*TANH(GRCETACUT(IPX(5)-2))
            COS5MAX2 = E5/P5*TANH(GRCETACUT(IPX(5)-2))
C...Due to Cosine Constraint
            COS5MIN3 = -(P3 + P4*COS4)/P5
            COS5MAX3 = -(-P3 + P4*COS4)/P5
C...Due to Phi Constraint
            XCP = (P3**2-P4**2-P5**2)/2.0D0/P4/P5
            IF (XCP.GT.0) THEN
               COS5MIN4 = XCP*(COS4 - SIN4*SQRT(1.0D0/XCP**2 - 1.0D0))
               COS5MAX4 = XCP*(COS4 + SIN4*SQRT(1.0D0/XCP**2 - 1.0D0))
            ELSE
               COS5MIN4 = XCP*(COS4 + SIN4*SQRT(1.0D0/XCP**2 - 1.0D0))
               COS5MAX4 = XCP*(COS4 - SIN4*SQRT(1.0D0/XCP**2 - 1.0D0))
            ENDIF
 
C...Set Cos5
            COS5MIN = MAX(-1.0D0,COS5MIN1,COS5MIN2,COS5MIN3,COS5MIN4)
            COS5MAX = MIN(1.0D0,COS5MAX1,COS5MAX2,COS5MAX3,COS5MAX4)
 
            IF (COS5MIN.GT.COS5MAX) THEN
               IREJ = 1
               WRITE(*,*) 'cos5min.gt.cos5max',COS5MIN,COS5MAX,XCP
               WRITE(*,*) 'min',COS5MIN1,COS5MIN2
               WRITE(*,*) 'max',COS5MAX1,COS5MAX2
            ENDIF
 
            RNA = XRUN(6)
            CALL GRCSBCS(INT(GSPR(2)),GEPS(2),RNA,
     $           COS5,AJACOB,COS5MIN,COS5MAX)
            GVAL(4) = COS5
            YACOB = YACOB*AJACOB
 
         ENDIF
 
C...Set Flavor
         IF (NDIM.GT.7.AND.GPRI(10).EQ.0.0D0) THEN
            GPRI(10) = XRUN(8)
         ENDIF

C######################
C# Set 4-body process #
C######################
      ELSE IF (NNF.EQ.6) THEN 

         IF (IGSUB.EQ.408.OR.IGSUB.EQ.412.OR.IGSUB.EQ.424
     $        .OR.IGSUB.EQ.425.OR.IGSUB.EQ.426.OR.IGSUB.EQ.427
     $        .OR.IGSUB.EQ.428.OR.IGSUB.EQ.429.OR.IGSUB.EQ.430
     $        .OR.IGSUB.EQ.465.OR.IGSUB.EQ.467.OR.IGSUB.EQ.561) THEN
C...W decays.
            XSND(1) = 0.0D0
            DO I = 2,IGNDCY+1
               XSND(I) = XSND(I-1)+GDCYRAT(I-1)/GDCYSUM
            ENDDO

            RNA = XRUN(12)
            DO I = 1,IGNDCY
               IF (RNA.GT.XSND(I).AND.RNA.LT.XSND(I+1)) THEN
                  YACOB = YACOB/(XSND(I+1)-XSND(I))
                  KFCODE(3) = IGWDCY(2*GDCYPR1(I)-1)
                  KFCODE(4) = IGWDCY(2*GDCYPR1(I))
               ENDIF
            ENDDO
            AMASS1(3) = GWDCYMS(ABS(KFCODE(3)))
            AMASS1(4) = GWDCYMS(ABS(KFCODE(4)))
            AMASS2(3) = AMASS1(3)**2
            AMASS2(4) = AMASS1(4)**2
            AMLP(1) = AMASS1(3)
            AMNU(1) = AMASS1(4)

C...Colour freedom
            IF (ABS(KFCODE(3)).LT.7) THEN
               YACOB = YACOB*3.0D0
            ENDIF

CC...Total cross section with all decay particles.
C            YACOB = YACOB/GRCWBR(18)
C
CC...Pick up final state flavor.
C            RANSUM = XRUN(12)*GDCYSUM
C            IDCYST = 0
C
C            DO WHILE (RANSUM.GT.0.0D0)
C               IDCYST = IDCYST + 1
C               RANSUM = RANSUM - GDCYRAT(IDCYST)
C            ENDDO
C            KFCODE(3) =  IGWDCY(2*GDCYPR1(IDCYST)-1)
C            KFCODE(4) =  IGWDCY(2*GDCYPR1(IDCYST))
C            AMASS1(3) = GWDCYMS(ABS(KFCODE(3)))
C            AMASS1(4) = GWDCYMS(ABS(KFCODE(4)))
C            AMASS2(3) = AMASS1(3)**2
C            AMASS2(4) = AMASS1(4)**2
C            AMLP(1) = AMASS1(3)
C            AMNU(1) = AMASS1(4)
C
CC...Apply Br. ratio.
C            YACOB = YACOB*GDCYSUM

         ELSE IF (IGSUB.EQ.503.OR.IGSUB.EQ.504.OR.IGSUB.EQ.505
     $           .OR.IGSUB.EQ.506.OR.IGSUB.EQ.507
     $           .OR.IGSUB.EQ.508.OR.IGSUB.EQ.509) THEN
C...Z decays.
            XSND(1) = 0.0D0
            DO I = 2,IGNDCY+1
               XSND(I) = XSND(I-1)+GDCYRAT(I-1)/GDCYSUM
            ENDDO

            RNA = XRUN(12)
            DO I = 1,IGNDCY
               IF (RNA.GT.XSND(I).AND.RNA.LT.XSND(I+1)) THEN
                  YACOB = YACOB/(XSND(I+1)-XSND(I))
                  KFCODE(3) = IGZDCY(2*GDCYPR1(I)-1)
                  KFCODE(4) = IGZDCY(2*GDCYPR1(I))
               ENDIF
            ENDDO
            AMASS1(3) = GWDCYMS(ABS(KFCODE(3)))
            AMASS1(4) = GWDCYMS(ABS(KFCODE(4)))
            AMASS2(3) = AMASS1(3)**2
            AMASS2(4) = AMASS1(4)**2
            AMLP(1) = AMASS1(3)

C...Colour freedom
            IF (ABS(KFCODE(3)).LT.7) THEN
               YACOB = YACOB*3.0D0
            ENDIF

         ELSE IF (IGSUB.EQ.550.OR.IGSUB.EQ.551.OR.IGSUB.EQ.552) THEN
C...Z-pair production.
C...Z decays.
            XSND(1) = 0.0D0
            DO I = 2,IGNDCY+1
               XSND(I) = XSND(I-1)+GDCYRAT(I-1)/GDCYSUM
            ENDDO

            RNA = XRUN(13)
            DO I = 1,IGNDCY
               IF (RNA.GT.XSND(I).AND.RNA.LT.XSND(I+1)) THEN
                  YACOB = YACOB/(XSND(I+1)-XSND(I))
                  IF (IGSUB.EQ.550) THEN
                     KFCODE(3) = IGZDCY(2*GDCYPR1(I)-1)
                     KFCODE(4) = IGZDCY(2*GDCYPR1(I))
                     KFCODE(5) = -IGZDCY(2*GDCYPR2(I)-1)
                     KFCODE(6) = -IGZDCY(2*GDCYPR2(I))
                  ELSE IF (IGSUB.EQ.551) THEN
                     KFCODE(3) = IGWDCY(2*GDCYPR1(I)-1)
                     KFCODE(4) = IGWDCY(2*GDCYPR1(I))
                     KFCODE(5) = -IGWDCY(2*GDCYPR2(I)-1)
                     KFCODE(6) = -IGWDCY(2*GDCYPR2(I))
                  ELSE IF (IGSUB.EQ.552) THEN
                     KFCODE(3) = IGZDCY(2*GDCYPR1(I)-1)
                     KFCODE(4) = IGZDCY(2*GDCYPR1(I))
                     KFCODE(5) = IGWDCY(2*GDCYPR2(I))
                     KFCODE(6) = IGWDCY(2*GDCYPR2(I)-1)
                  ENDIF
               ENDIF
            ENDDO

            AMASS1(3) = GWDCYMS(ABS(KFCODE(3)))
            AMASS1(4) = GWDCYMS(ABS(KFCODE(4)))
            AMASS1(5) = GWDCYMS(ABS(KFCODE(5)))
            AMASS1(6) = GWDCYMS(ABS(KFCODE(6)))
            AMASS2(3) = AMASS1(3)**2
            AMASS2(4) = AMASS1(4)**2
            AMASS2(5) = AMASS1(5)**2
            AMASS2(6) = AMASS1(6)**2
            AMLP(1) = AMASS1(3)

C...Colour freedom
            IF (ABS(KFCODE(3)).LT.7) THEN
               YACOB = YACOB*3.0D0
            ENDIF
            IF (ABS(KFCODE(5)).LT.7) THEN
               YACOB = YACOB*3.0D0
            ENDIF
         ENDIF

         IF (KTYPE.NE.9) THEN
C#########################################
C# Q^2(ij) : reso., Q^2(kl) : non-reso.. #
C#########################################
            IF (KTYPE.EQ.5) THEN
C...Set Q12(ij)
               RNA = XRUN(5)
               CALL GRCSQ12(RNA, Q12, AJACOB)
               GVAL(3) = Q12
               YACOB = YACOB*AJACOB

C...Set tau, yst, s, w, x1, x2, beta, and gamma
               CALL GRCSSPX
C...Set Q22(kl)
               RNA = XRUN(6)
               CALL GRCSQ22(RNA, Q22, AJACOB)
               GVAL(4) = Q22
               YACOB = YACOB*AJACOB
 
C#############################################
C# Q^2(ij) : non-reso., Q^2(kl) : non-reso.. #
C#############################################
            ELSE IF (KTYPE.EQ.6) THEN
 
C...Set tau, yst, s, w, x1, x2, beta, and gamma
               CALL GRCSSPX
C...Set Q12(ij)
               RNA = XRUN(5)
               CALL GRCSQ12(RNA, Q12, AJACOB)
               GVAL(3) = Q12
               YACOB = YACOB*AJACOB
C...Set Q22(kl)
               RNA = XRUN(6)
               CALL GRCSQ22(RNA, Q22, AJACOB)
               GVAL(4) = Q22
               YACOB = YACOB*AJACOB
 
C#####################################
C# Q^2(ij) : reso., Q^2(kl) : reso.. #
C#####################################
            ELSE IF (KTYPE.EQ.7) THEN
 
C...Set Q12(ij)
               RNA = XRUN(5)
               CALL GRCSQ12(RNA, Q12, AJACOB)
               GVAL(3) = Q12
               YACOB = YACOB*AJACOB
C...Set Q22(kl)
               RNA = XRUN(6)
               CALL GRCSQ22(RNA, Q22, AJACOB)
               GVAL(4) = Q22
               YACOB = YACOB*AJACOB
C...Set tau, yst, s, w, x1, x2, beta, and gamma
               CALL GRCSSPX
 
C#########################################
C# Cascade Decay (3+4+5) + 6 , (3+4) + 5 #
C#########################################
            ELSE IF (KTYPE.EQ.8) THEN
 
C...Set Q12(ij)
               RNA = XRUN(5)
               CALL GRCSQ12(RNA, Q12, AJACOB)
               GVAL(3) = Q12
               YACOB = YACOB*AJACOB
C...Set tau, yst, s, w, x1, x2, beta, and gamma
               CALL GRCSSPX
C...Set Q22(kl)
               RNA = XRUN(6)
               CALL GRCSQ22(RNA, Q22, AJACOB)
               GVAL(4) = Q22
               YACOB = YACOB*AJACOB
            ENDIF
 
C...Set Cos1, Phi1
            RNA = XRUN(3)
            CALL GRCICOS
            CALL GRCSCOS(RNA, COS1, AJACOB)
            GVAL(1) = COS1
            YACOB = YACOB*AJACOB
 
            GVAL(2) = XRUN(4)*2.0D0*PI
            YACOB = YACOB*2.0D0*PI
 
C...Set Cos(ij), Phi(ij) in (ij) CM frame
            GVAL(6) = XRUN(8)*2.0D0*PI - GVAL(2)
            YACOB = YACOB*2.0D0*PI
 
            RNA = XRUN(7)
C            CALL GRCSBCS(INT(GSPR(1)),GEPS(1),RNA,
C     $           COSIJ,AJACOB,-0.999D0,0.999D0)

            CALL GRCSBCS(INT(GSPR(1)),GEPS(1),RNA,
     $           COSIJ,AJACOB,-1.0D0,1.0D0)

            GVAL(5) = COSIJ
            YACOB = YACOB*AJACOB
 
C...Set Cos(kl), Phi(kl) in (kl) CM frame
            GVAL(8) = XRUN(10)*2.0D0*PI - GVAL(2)
            YACOB = YACOB*2.0D0*PI
 
            RNA = XRUN(9)
C            CALL GRCSBCS(INT(GSPR(2)),GEPS(2),RNA,
C     $           COSKL,AJACOB,-0.999D0,0.999D0)

            CALL GRCSBCS(INT(GSPR(2)),GEPS(2),RNA,
     $           COSKL,AJACOB,-1.0D0,1.0D0)

            GVAL(7) = COSKL
            YACOB = YACOB*AJACOB
 
C########################################################
C# 4-body Fusion 1 -> 5 + A, 2 -> 6 + B, A + B -> 3 + 4 #
C########################################################
         ELSE IF (KTYPE.EQ.9) THEN
 
C...Fermion Propagator
C...Set Q(56)
            RNA = XRUN(5)
            CALL GRCSQ12(RNA, Q34, AJACOB)
            GVAL(6) = Q34
            YACOB = YACOB*AJACOB
C...Set tau, yst, s, w, x1, x2, beta, and gamma
            CALL GRCSSPX
C...Set E5+E6
            SUMMIN = (GPRI(3) - GVAL(6))/2.0D0/GPRI(4)
            SUMMAX = GPRI(4) - SQRT(GVAL(6))
 
            IF (SUMMIN.GT.SUMMAX) THEN
               IREJ = 1
               WRITE(*,*) 'summin.gt.summax'
               GO TO 999
            ENDIF
 
            GVAL(1) = SUMMIN + (SUMMAX - SUMMIN)*XRUN(3)
            YACOB = YACOB*(SUMMAX - SUMMIN)
 
C...Set E5-E6
            E5MIN = SQRT(GRCPTCUT(IPX(5)-2)**2+AMASS2(IPX(5)))
            E6MIN = SQRT(GRCPTCUT(IPX(6)-2)**2+AMASS2(IPX(6)))
 
            SUBMIN = -GVAL(1) + 2.0D0*E5MIN
            SUBMAX =  GVAL(1) - 2.0D0*E6MIN
 
            IF (SUBMIN.GT.SUBMAX) THEN
               IREJ = 1
C               write(*,*) 'submin.gt.submax'
               GO TO 999
            ENDIF
 
            GVAL(2) = SUBMIN + (SUBMAX - SUBMIN)*XRUN(4)
            YACOB = YACOB*(SUBMAX - SUBMIN)
 
C...Set Energy and momentum
            E1 = (GPRI(3)+AMASS2(1)-AMASS2(2))/2.D0/GPRI(4)
            E2 = (GPRI(3)+AMASS2(2)-AMASS2(1))/2.D0/GPRI(4)
 
            EQ = GPRI(4) - GVAL(1)
            PQ = SQRT(EQ**2 - GVAL(6))
 
            E5 = (GVAL(1) - GVAL(2))/2.0D0
            E6 = (GVAL(1) + GVAL(2))/2.0D0
            P5 = SQRT(E5**2 - AMASS2(IPX(5)))
            P6 = SQRT(E6**2 - AMASS2(IPX(6)))
 
            IF (EQ.LT.SQRT(GVAL(6))) THEN
               IREJ = 1
               WRITE(*,*) 'eq.lt.sqrt(q34)',GVAL(1),GVAL(2)
               GO TO 999
            ENDIF
            IF (E5.LT.AMASS1(IPX(5))) THEN
               IREJ = 1
               WRITE(*,*) 'e5.lt.amass1(',IPX(5),')',GVAL(1),GVAL(2)
               GO TO 999
            ENDIF
            IF (E6.LT.AMASS1(IPX(6))) THEN
               IREJ = 1
               WRITE(*,*) 'e6.lt.amass1(',IPX(6),')',GVAL(1),GVAL(2)
               GO TO 999
            ENDIF
 
            IF (P5+P6-PQ.LT.0) THEN
               IREJ = 1
C               write(*,*) p4+p5-p3,p3-p4+p5,p3+p4-p5,gpri(4),p3
               GO TO 999
            ENDIF
            IF (PQ-P5+P6.LT.0) THEN
               IREJ = 1
C               write(*,*) p4+p5-p3,p3-p4+p5,p3+p4-p5
               GO TO 999
            ENDIF
            IF (PQ+P5-P6.LT.0) THEN
               IREJ = 1
C               write(*,*) p4+p5-p3,p3-p4+p5,p3+p4-p5
               GO TO 999
            ENDIF
 
C...Set Cos5
            IF (P5.LT.GRCPTCUT(IPX(5)-2)) THEN
               IREJ = 1
               GO TO 999
            ENDIF
C...Due to Pt
            COS5MIN1 = -SQRT(P5**2-GRCPTCUT(IPX(5)-2)**2)/P5
            COS5MAX1 = SQRT(P5**2-GRCPTCUT(IPX(5)-2)**2)/P5
C...Due to Eta
            COS5MIN2 = -E5/P5*TANH(GRCETACUT(IPX(5)-2))
            COS5MAX2 = E5/P5*TANH(GRCETACUT(IPX(5)-2))
 
            COS5MIN = MAX(-1.0D0,COS5MIN1,COS5MIN2)
            COS5MAX = MIN(1.0D0,COS5MAX1,COS5MAX2)
 
            RNA = XRUN(8)
            CALL GRCSBCS(INT(GSPR(1)),GEPS(1),RNA,
     $           COS5,AJACOB,COS5MIN,COS5MAX)
            GVAL(3) = COS5
            YACOB = YACOB*AJACOB
 
            SIN5 = SQRT((1.0D0 + COS5)*(1.0D0 - COS5))
 
C Check!!
            IF (P6.LT.GRCPTCUT(IPX(6)-2)) THEN
               IREJ = 1
               GO TO 999
            ENDIF
C...Due to Pt
            COS6MIN1 = -SQRT(P6**2-GRCPTCUT(IPX(6)-2)**2)/P6
            COS6MAX1 = SQRT(P6**2-GRCPTCUT(IPX(6)-2)**2)/P6
C...Due to Eta
            COS6MIN2 = -E6/P6*TANH(GRCETACUT(IPX(6)-2))
            COS6MAX2 = E6/P6*TANH(GRCETACUT(IPX(6)-2))
C...Due to Cosine Constraint
            COS6MIN3 = -(PQ + P5*COS5)/P6
            COS6MAX3 = -(-PQ + P5*COS5)/P6
C...Due to Phi Constraint
            XCP = (PQ**2-P5**2-P6**2)/2.0D0/P5/P6
            IF (XCP.GT.0) THEN
               COS6MIN4 = XCP*(COS5 - SIN5*SQRT(1.0D0/XCP**2 - 1.0D0))
               COS6MAX4 = XCP*(COS5 + SIN5*SQRT(1.0D0/XCP**2 - 1.0D0))
            ELSE
               COS6MIN4 = XCP*(COS5 + SIN5*SQRT(1.0D0/XCP**2 - 1.0D0))
               COS6MAX4 = XCP*(COS5 - SIN5*SQRT(1.0D0/XCP**2 - 1.0D0))
            ENDIF
 
C...Set Cos6
            COS6MIN = MAX(-1.0D0,COS6MIN1,COS6MIN2,COS6MIN3,COS6MIN4)
            COS6MAX = MIN(1.0D0,COS6MAX1,COS6MAX2,COS6MAX3,COS6MAX4)
 
            IF (COS6MIN.GT.COS6MAX) THEN
               IREJ = 1
               GO TO 999
C               write(*,*) 'cos6min.gt.cos6max',cos6min,cos6max,xcp
C               write(*,*) 'min',cos6min1,cos6min2
C               write(*,*) 'max',cos6max1,cos6max2
            ENDIF
 
            RNA = XRUN(6)
            CALL GRCSBCS(INT(GSPR(2)),GEPS(2),RNA,
     $           COS6,AJACOB,COS6MIN,COS6MAX)
            GVAL(4) = COS6
            YACOB = YACOB*AJACOB
 
C...Set Phi5
            GVAL(5) = XRUN(7)*2.0D0*PI
            YACOB = YACOB*2.0D0*PI
 
C...Set Cos34, Phi34 in 34 CM frame
            RNA = XRUN(9)
            CALL GRCSBCS(INT(GSPR(3)),GEPS(3),RNA,
     $           COS34,AJACOB,-1.0D0,1.0D0)
            GVAL(7) = COS34
            YACOB = YACOB*AJACOB
 
            GVAL(8) = XRUN(10)*2.0D0*PI - GVAL(5)
            YACOB = YACOB*2.0D0*PI
 
         ENDIF

C...Set Flavor
         IF (NDIM.GT.10.AND.GPRI(10).EQ.0.0D0) THEN
            GPRI(10) = XRUN(11)
         ENDIF

C######################
C# Set 5-body process #
C######################
      ELSE IF (NNF.EQ.7) THEN 
 
         IF (IGSUB.GT.418) THEN

            IF (IGSUB.EQ.431.OR.IGSUB.EQ.432.OR.IGSUB.EQ.433
     $           .OR.IGSUB.EQ.434.OR.IGSUB.EQ.435.OR.IGSUB.EQ.436
     $           .OR.IGSUB.EQ.437.OR.IGSUB.EQ.438) THEN
C...W decays.
               XSND(1) = 0.0D0
               DO I = 2,IGNDCY+1
                  XSND(I) = XSND(I-1)+GDCYRAT(I-1)/GDCYSUM
               ENDDO

               RNA = XRUN(15)
               DO I = 1,IGNDCY
                  IF (RNA.GT.XSND(I).AND.RNA.LT.XSND(I+1)) THEN
                     YACOB = YACOB/(XSND(I+1)-XSND(I))
                     KFCODE(3) = IGWDCY(2*GDCYPR1(I)-1)
                     KFCODE(4) = IGWDCY(2*GDCYPR1(I))
                  ENDIF
               ENDDO
               AMASS1(3) = GWDCYMS(ABS(KFCODE(3)))
               AMASS1(4) = GWDCYMS(ABS(KFCODE(4)))
               AMASS2(3) = AMASS1(3)**2
               AMASS2(4) = AMASS1(4)**2
               AMLP(1) = AMASS1(3)
               AMNU(1) = AMASS1(4)

C...Colour freedom
               IF (ABS(KFCODE(3)).LT.7) THEN
                  YACOB = YACOB*3.0D0
               ENDIF

            ENDIF

C#########################################
C# Q^2(ij) : reso., Q^2(kl) : non-reso.. #
C#########################################
            IF (KTYPE.EQ.11) THEN

C...Set Q12(ij)
               RNA = XRUN(5)
               CALL GRCSQ12(RNA, Q12, AJACOB)
               GVAL(3) = Q12
               YACOB = YACOB*AJACOB

C...Set tau, yst, s, w, x1, x2, beta, and gamma
               CALL GRCSSPX

C...Set Q22(kl)
               RNA = XRUN(6)
               CALL GRCSQ22(RNA, Q22, AJACOB)
               GVAL(4) = Q22
               YACOB = YACOB*AJACOB

C...Set P56 + P34
               SUMMIN = GRCPTCUT(IPX(7)-2)
               SUMMAX = SQRT((GPRI(4)-SQRT(GVAL(3))-SQRT(GVAL(4)))**2
     $              -AMASS2(IPX(7))**2)
               
               IF (SUMMIN.GT.SUMMAX) THEN
                  IREJ = 1
                  WRITE(*,*) 'summin.gt.summax'
                  GO TO 999
               ENDIF

               GVAL(1) = SUMMIN + (SUMMAX - SUMMIN)*XRUN(3)
               YACOB = YACOB*(SUMMAX - SUMMIN)

C...Set P56 - P34
               SUBMIN = -GVAL(1)
               SUBMAX =  GVAL(1)
 
               IF (SUBMIN.GT.SUBMAX) THEN
                  IREJ = 1
                  write(*,*) 'submin.gt.submax'
                  GO TO 999
               ENDIF
 
               GVAL(2) = SUBMIN + (SUBMAX - SUBMIN)*XRUN(4)
               YACOB = YACOB*(SUBMAX - SUBMIN)

C...Set Cos7, Phi7
               RNA = XRUN(7)
               CALL GRCICOS
               CALL GRCSCOS(RNA, COS1, AJACOB)
               GVAL(5) = COS1
               YACOB = YACOB*AJACOB
 
               GVAL(7) = XRUN(9)*2.0D0*PI
               YACOB = YACOB*2.0D0*PI

C...Q(56) system
               PQ56 = (GVAL(1)+GVAL(2))/2.0D0
               EQ56 = SQRT(PQ56**2+GVAL(4))
C...Q(34) system
               PQ34 = (GVAL(1)-GVAL(2))/2.0D0
               EQ34 = SQRT(PQ34**2+GVAL(3))
C...Particle 7
               E7 = GPRI(4) - EQ56 - EQ34
               P7 = SQRT((E7+AMASS1(IPX(7)))*(E7-AMASS1(IPX(7))))
               COS7 = GVAL(5)
               SIN7 = SQRT((1.0D0 + COS7)*(1.0D0 - COS7))

C...Due to Cosine Constraint
               COSQ2MIN1 = -(PQ34 + P7*COS7)/PQ56
               COSQ2MAX1 = (PQ34 - P7*COS7)/PQ56
C...Set Cos_56
               COSQ2MIN = MAX(-1.0D0,COSQ2MIN1)
               COSQ2MAX = MIN( 1.0D0,COSQ2MAX1)
 
               IF (COSQ2MIN.GT.COSQ2MAX) THEN
                  IREJ = 1
C                  WRITE(*,*) 'cosq2min.gt.cosq2max',COSQ2MIN,COSQ2MAX
                  GO TO 999
               ENDIF
 
               RNA = XRUN(8)
               CALL GRCSBCS(INT(GSPR(3)),GEPS(3),RNA,
     $              COSQ2,AJACOB,COSQ2MIN,COSQ2MAX)
               GVAL(6) = COSQ2
               YACOB = YACOB*AJACOB

               SINQ56 = SQRT((1.0D0 + GVAL(6))*(1.0D0 - GVAL(6)))
               COSQ34 = -(PQ56*GVAL(6) + P7*COS7)/PQ34

               IF (ABS(COSQ34).GT.1.0D0) THEN
                  IREJ = 1
                  GO TO 999
               ENDIF
               SINQ34 = SQRT((1.0D0 + COSQ34)*(1.0D0 - COSQ34))

C...Check kinematical variables
               PT7 = P7*SIN7
               PT34 = PQ34*SINQ34
               PT56 = PQ56*SINQ56

               IF (PT7.LT.PT56-PT34) THEN
                  IREJ = 1
                  GO TO 999
               ENDIF
               IF (PT7.LT.PT34-PT56) THEN
                  IREJ = 1
                  GO TO 999
               ENDIF
               IF (PT7.GT.PT34+PT56) THEN
                  IREJ = 1
                  GO TO 999
               ENDIF

C...Set Cos(34), Phi(34) in (34) CM frame
               GVAL(9) = XRUN(11)*2.0D0*PI
               YACOB = YACOB*2.0D0*PI
 
               RNA = XRUN(10)
               CALL GRCSBCS(INT(GSPR(1)),GEPS(1),RNA,
     $              COSIJ,AJACOB,-1.0D0,1.0D0)
               GVAL(8) = COSIJ
               YACOB = YACOB*AJACOB

C...Set Cos(56), Phi(56) in (56) CM frame
               GVAL(11) = XRUN(13)*2.0D0*PI - GVAL(2)
               YACOB = YACOB*2.0D0*PI
 
               RNA = XRUN(12)
               CALL GRCSBCS(INT(GSPR(2)),GEPS(2),RNA,
     $              COSKL,AJACOB,-1.0D0,1.0D0)
               GVAL(10) = COSKL
               YACOB = YACOB*AJACOB

C...Set Flavor
               IF (NDIM.GT.15.AND.GPRI(10).EQ.0.0D0) THEN
                  GPRI(10) = XRUN(14)
               ENDIF

C############################################
C# Cascade Decay (3+4) + (5+6+7), (5+6) + 7 #
C############################################
            ELSE IF (KTYPE.EQ.12) THEN

C...Set Q12(ij)
               RNA = XRUN(5)
               CALL GRCSQ12(RNA, Q12, AJACOB)
               GVAL(3) = Q12
               YACOB = YACOB*AJACOB
C...Set Q22(kl)
               RNA = XRUN(6)
               CALL GRCSQ22(RNA, Q22, AJACOB)
               GVAL(4) = Q22
               YACOB = YACOB*AJACOB

C...Set tau, yst, s, w, x1, x2, beta, and gamma
               CALL GRCSSPX

               E7MIN = SQRT(GRCPTCUT(IPX(7)-2)**2+AMASS1(IPX(7))**2)
               Q567MIN = (SQRT(GVAL(4))+E7MIN)**2
     $              -GRCPTCUT(IPX(7)-2)**2
               Q567MAX = (GPRI(4)-SQRT(GVAL(3)))**2

               IF (Q567MIN.GT.Q567MAX) THEN
                  IREJ = 1
                  WRITE(*,*) 'q567min.gt.q567max'
                  GO TO 999
               ENDIF
C               GVAL(5) = Q567MIN + (Q567MAX - Q567MIN)*XRUN(7)
C               YACOB = YACOB*(Q567MAX - Q567MIN)

               GVAL(5) = Q567MIN*(Q567MAX/Q567MIN)**XRUN(7)
               YACOB = YACOB*GVAL(5)*LOG(Q567MAX/Q567MIN)

C...Set Cos1, Phi1
               RNA = XRUN(3)
               CALL GRCICOS
               CALL GRCSCOS(RNA, COS1, AJACOB)
               GVAL(1) = COS1
               YACOB = YACOB*AJACOB
 
               GVAL(2) = XRUN(4)*2.0D0*PI
               YACOB = YACOB*2.0D0*PI

C...Set Cos(567), Phi(567) in (567) CM frame
               GVAL(7) = XRUN(9)*2.0D0*PI - GVAL(2)
               YACOB = YACOB*2.0D0*PI

               RNA = XRUN(8)
               CALL GRCSBCS(INT(GSPR(1)),GEPS(1),RNA,
     $              COS567,AJACOB,-1.0D0,1.0D0)
               GVAL(6) = COS567
               YACOB = YACOB*AJACOB

C...Set Cos(34), Phi(34) in (34) CM frame
               GVAL(9) = XRUN(11)*2.0D0*PI
               YACOB = YACOB*2.0D0*PI
 
               RNA = XRUN(10)
               CALL GRCSBCS(INT(GSPR(2)),GEPS(2),RNA,
     $              COSIJ,AJACOB,-1.0D0,1.0D0)
               GVAL(8) = COSIJ
               YACOB = YACOB*AJACOB

C...Set Cos(56), Phi(56) in (56) CM frame
               GVAL(11) = XRUN(13)*2.0D0*PI
               YACOB = YACOB*2.0D0*PI
 
               RNA = XRUN(12)
               CALL GRCSBCS(INT(GSPR(3)),GEPS(3),RNA,
     $              COSKL,AJACOB,-1.0D0,1.0D0)
               GVAL(10) = COSKL
               YACOB = YACOB*AJACOB

C...Set Flavor
               IF (NDIM.GT.15.AND.GPRI(10).EQ.0.0D0) THEN
                  GPRI(10) = XRUN(14)
               ENDIF
            ENDIF

C...Special Case W + bbbb
         ELSE IF (IGSUB.EQ.418) THEN
C...Fermion Propagator
            E3MIN = SQRT(GRCPTCUT(IPX(3)-2)**2+AMASS2(IPX(3)))
 
C...s-dependence width
            IF (ISDEP.EQ.1) THEN
C...Special case
               IF (IGSUB.EQ.418) THEN
                  WRITE(*,*) 'stiil not available for 418...'
                  GO TO 999
C...Set W mass
                  QMIN = 1.0D0
                  QMAX = (GRCECM-AMASS1(IPX(4))-AMASS1(IPX(5))
     $                 -AMASS1(IPX(6))-AMASS1(IPX(7)))**2
                  
                  THEMIN = ATAN((QMIN - AMW**2)/(AMW*AGW))
                  THEMAX = ATAN((QMAX - AMW**2)/(AMW*AGW))
                  THE = THEMIN + (THEMAX - THEMIN)*XRUN(15)
                  QM = AMW*AGW*TAN(THE) + AMW**2
                  YACOB = YACOB*(THEMAX - THEMIN)
     $                 *((QM-AMW**2)**2+(AMW*AGW)**2)/(AMW*AGW)
 
C...s-dep. Q^2
                  YACOB = YACOB*QM*AGW/PI/AMW
     $                 /((QM-AMW**2)**2+(AMW*AGW)**2)
 
                  AMASS2(IPX(3)) = QM
                  AMASS1(IPX(3)) = SQRT(QM)
               ELSE
                  WRITE(*,*) "NOW ONLY SUPPORT 418..."
                  STOP
               ENDIF
            ENDIF
 
C...Set tau, yst, s, w, x1, x2, beta, and gamma
            CALL GRCSSPX
 
C...Set Q45
            RNA = XRUN(5)
            CALL GRCSQ12(RNA, Q45, AJACOB)
            GVAL(3) = Q45
            YACOB = YACOB*AJACOB
C...Set Q67
            RNA = XRUN(6)
            CALL GRCSQ22(RNA, Q67, AJACOB)
            GVAL(4) = Q67
            YACOB = YACOB*AJACOB
 
C...Set Eq1+Eq2
C            summin = sqrt(gval(3)) + sqrt(gval(4))
            SUMMIN = (GPRI(3) - AMASS2(IPX(3)))/2.0D0/GPRI(4)
            SUMMAX = GPRI(4) - E3MIN
            
            IF (SUMMIN.GT.SUMMAX) THEN
               IREJ = 1
               WRITE(*,*) 'summin.gt.summax'
               GO TO 999
            ENDIF
 
            GVAL(1) = SUMMIN + (SUMMAX - SUMMIN)*XRUN(3)
            YACOB = YACOB*(SUMMAX - SUMMIN)
 
C...Set Eq1-Eq2
            SUBMIN = -GVAL(1) + 2.0D0*SQRT(GVAL(4))
            SUBMAX =  GVAL(1) - 2.0D0*SQRT(GVAL(3))
 
            IF (SUBMIN.GT.SUBMAX) THEN
               IREJ = 1
C               write(*,*) 'submin.gt.submax'
               GO TO 999
            ENDIF
 
            GVAL(2) = SUBMIN + (SUBMAX - SUBMIN)*XRUN(4)
            YACOB = YACOB*(SUBMAX - SUBMIN)
 
C...Set Energy and momentum
            E1 = (GPRI(3)+AMASS2(1)-AMASS2(2))/2.D0/GPRI(4)
            E2 = (GPRI(3)+AMASS2(2)-AMASS2(1))/2.D0/GPRI(4)
 
            E3 = GPRI(4) - GVAL(1)
            P3 = SQRT(E3**2 - AMASS2(IPX(3)))
 
            EQ1  = (GVAL(1) - GVAL(2))/2.0D0
            EQ2  = (GVAL(1) + GVAL(2))/2.0D0
            PQ1 = SQRT(EQ1**2 - GVAL(3))
            PQ2 = SQRT(EQ2**2 - GVAL(4))
 
            GVAL(12) = PQ1
            GVAL(13) = PQ2
 
            IF (EQ1.LT.SQRT(GVAL(3))) THEN
               IREJ = 1
               WRITE(*,*) 'eq1.lt.sqrt(q45)'
               GO TO 999
            ENDIF
            IF (EQ2.LT.SQRT(GVAL(4))) THEN
               IREJ = 1
               WRITE(*,*) 'eq2.lt.sqrt(q67)'
               GO TO 999
            ENDIF
 
            IF (PQ1+PQ2-P3.LT.0) THEN
               IREJ = 1
C               write(*,*) 'pq1+pq2-p3.lt.0',p3,pq1,pq2
               GO TO 999
            ENDIF
            IF (P3-PQ1+PQ2.LT.0) THEN
               IREJ = 1
C               write(*,*) 'p3-pq1+pq2.lt.0',p3,pq1,pq2
               GO TO 999
            ENDIF
            IF (P3+PQ1-PQ2.LT.0) THEN
               IREJ = 1
C               write(*,*) 'p3+pq1-pq2.lt.0',p3,pq1,pq2
               GO TO 999
            ENDIF
 
C...Set CosQ1
            COSQ1MIN = -1.0D0
            COSQ1MAX =  1.0D0
 
            RNA = XRUN(7)
            CALL GRCSBCS(INT(GSPR(1)),GEPS(1),RNA,
     $           COSQ1,AJACOB,COSQ1MIN,COSQ1MAX)
            GVAL(5) = COSQ1
            YACOB = YACOB*AJACOB
 
            SINQ1 = SQRT((1.0D0 + COSQ1)*(1.0D0 - COSQ1))
 
C Check!!
C...Due to Cosine Constraint
            COSQ2MIN1 = (-PQ1*COSQ1 - P3)/PQ2
            COSQ2MAX1 = (-PQ1*COSQ1 + P3)/PQ2
 
C...Due to Phi Constraint
            XCP = (P3**2-PQ1**2-PQ2**2)/2.0D0/PQ1/PQ2
            IF (XCP.GT.0) THEN
               COSQ2MIN2 = XCP*(COSQ1-SINQ1*SQRT(1.0D0/XCP**2-1.0D0))
               COSQ2MAX2 = XCP*(COSQ1+SINQ1*SQRT(1.0D0/XCP**2-1.0D0))
            ELSE
               COSQ2MIN2 = XCP*(COSQ1+SINQ1*SQRT(1.0D0/XCP**2-1.0D0))
               COSQ2MAX2 = XCP*(COSQ1-SINQ1*SQRT(1.0D0/XCP**2-1.0D0))
            ENDIF

C...Set CosQ2
            COSQ2MIN = MAX(-1.0D0,COSQ2MIN1,COSQ2MIN2)
            COSQ2MAX = MIN( 1.0D0,COSQ2MAX1,COSQ2MAX2)
 
            IF (COSQ2MIN.GT.COSQ2MAX) THEN
               IREJ = 1
               WRITE(*,*) 'cosq2min.gt.cosq2max',COSQ2MIN,COSQ2MAX,XCP
               GO TO 999
            ENDIF
 
            RNA = XRUN(8)
            CALL GRCSBCS(INT(GSPR(2)),GEPS(2),RNA,
     $           COSQ2,AJACOB,COSQ2MIN,COSQ2MAX)
            GVAL(6) = COSQ2
            YACOB = YACOB*AJACOB
 
            SINQ2 = SQRT((1.0D0 - COSQ2)*(1.0D0 + COSQ2))
 
C...Set PhiQ1
            GVAL(7) = XRUN(9)*2.0D0*PI
            YACOB = YACOB*2.0D0*PI
 
C...Set Cos45, Phi45 in 45 CM frame
            RNA = XRUN(10)
            CALL GRCSBCS(INT(GSPR(3)),GEPS(3),RNA,
     $           COS45,AJACOB,-1.0D0,1.0D0)
            GVAL(8) = COS45
            YACOB = YACOB*AJACOB
 
            GVAL(9) = XRUN(11)*2.0D0*PI
            YACOB = YACOB*2.0D0*PI
 
C...Set Cos67, Phi67 in 67 CM frame
            RNA = XRUN(12)
            CALL GRCSBCS(INT(GSPR(4)),GEPS(4),RNA,
     $           COS67,AJACOB,-1.0D0,1.0D0)
            GVAL(10) = COS67
            YACOB = YACOB*AJACOB
 
            GVAL(11) = XRUN(13)*2.0D0*PI
            YACOB = YACOB*2.0D0*PI
 
C...Set Flavor
            IF (NDIM.GT.13.AND.GPRI(10).EQ.0.0D0) THEN
               GPRI(10) = XRUN(14)
            ENDIF
         ENDIF

C######################
C# Set 6-body process #
C######################
      ELSE IF (NNF.EQ.8) THEN 

C...Scecial case : top pair production
C...1 + 2 -> tt -> {3 + (45)} + {6 + (78)}
         IF (IGSUB.EQ.419.OR.IGSUB.EQ.420) THEN
C...Total cross section with all decay particles.
            YACOB = YACOB/GRCWBR(17)/GRCWBR(18)

C...Pick up final state flavor.
            RANSUM = XRUN(17)*GDCYSUM
            IDCYST = 0
            DO WHILE (RANSUM.GT.0.0D0)
               IDCYST = IDCYST + 1
               RANSUM = RANSUM - GDCYRAT(IDCYST)
            ENDDO

            KFCODE(4) =  IGWDCY(2*GDCYPR1(IDCYST)-1)
            KFCODE(5) =  IGWDCY(2*GDCYPR1(IDCYST))
            KFCODE(7) = -IGWDCY(2*GDCYPR2(IDCYST)-1)
            KFCODE(8) = -IGWDCY(2*GDCYPR2(IDCYST))

            AMASS1(4) = GWDCYMS(ABS(KFCODE(4)))
            AMASS1(5) = GWDCYMS(ABS(KFCODE(5)))
            AMASS1(7) = GWDCYMS(ABS(KFCODE(7)))
            AMASS1(8) = GWDCYMS(ABS(KFCODE(8)))

            AMASS2(4) = AMASS1(4)**2
            AMASS2(5) = AMASS1(5)**2
            AMASS2(7) = AMASS1(7)**2
            AMASS2(8) = AMASS1(8)**2

C...Apply Br. ratio.
            YACOB = YACOB*GDCYSUM

C...Q^2(45):reso., Q^2(78):reso. type.
            KTYPE = 7
            IPX(3) = 4
            IPX(4) = 5
            IPX(5) = 7
            IPX(6) = 8

C...Set Q12(45)
            RNA = XRUN(11)
            CALL GRCSQ12(RNA, Q12, AJACOB)
            GVAL(9) = Q12
            YACOB = YACOB*AJACOB

C...Set Q22(78)
            RNA = XRUN(12)
            CALL GRCSQ22(RNA, Q22, AJACOB)
            GVAL(10) = Q22
            YACOB = YACOB*AJACOB

C...Set top resonance mass
            IPX(3) = 3
            IPX(4) = 4
            IPX(5) = 5
            IPX(6) = 6
            QMIN = GVAL(9) + AMASS2(IPX(3))
            QMAX = (GRCECM - AMASS1(IPX(6)) - SQRT(GVAL(10)))**2
 
            THEMIN = ATAN((QMIN - AMUQ(3)**2)/(AMUQ(3)*AGUQ(3)))
            THEMAX = ATAN((QMAX - AMUQ(3)**2)/(AMUQ(3)*AGUQ(3)))
            THE = THEMIN + (THEMAX - THEMIN)*XRUN(5)
            QM = AMUQ(3)*AGUQ(3)*TAN(THE) + AMUQ(3)**2
            YACOB = YACOB*(THEMAX - THEMIN)
     $           *((QM-AMUQ(3)**2)**2
     $           +(AMUQ(3)*AGUQ(3))**2)/(AMUQ(3)*AGUQ(3))
            GVAL(3) = QM

C...Set anti-top resonance mass
            QMIN = GVAL(10) + AMASS2(IPX(6))
            QMAX = (GRCECM - SQRT(GVAL(3)))**2
 
            THEMIN = ATAN((QMIN - AMUQ(3)**2)/(AMUQ(3)*AGUQ(3)))
            THEMAX = ATAN((QMAX - AMUQ(3)**2)/(AMUQ(3)*AGUQ(3)))
            THE = THEMIN + (THEMAX - THEMIN)*XRUN(6)
            QM = AMUQ(3)*AGUQ(3)*TAN(THE) + AMUQ(3)**2
            YACOB = YACOB*(THEMAX - THEMIN)
     $           *((QM-AMUQ(3)**2)**2
     $           +(AMUQ(3)*AGUQ(3))**2)/(AMUQ(3)*AGUQ(3))
            GVAL(4) = QM

            IPX(3) = 4
            IPX(4) = 5
            IPX(5) = 7
            IPX(6) = 8
C...Set tau, yst, s, w, x1, x2, beta, and gamma
            CALL GRCSSPX

            IPX(3) = 3
            IPX(4) = 4
            IPX(5) = 5
            IPX(6) = 6
C...Set Cos1, Phi1
            RNA = XRUN(3)
            CALL GRCICOS
            CALL GRCSCOS(RNA, COS1, AJACOB)
            GVAL(1) = COS1
            YACOB = YACOB*AJACOB
 
            GVAL(2) = XRUN(4)*2.0D0*PI
            YACOB = YACOB*2.0D0*PI

C...Set Cos{3(45)}, Phi{3(45)} in 3(45) CM frame
            RNA = XRUN(7)
            CALL GRCSBCS(INT(GSPR(1)),GEPS(1),RNA,
     $           COSIJ,AJACOB,-1.0D0,1.0D0)
            GVAL(5) = COSIJ
            YACOB = YACOB*AJACOB

            GVAL(6) = XRUN(8)*2.0D0*PI - GVAL(2)
            YACOB = YACOB*2.0D0*PI
 
C...Set Cos{6(78)}, Phi{6(78)} in 6(78) CM frame
            RNA = XRUN(9)
            CALL GRCSBCS(INT(GSPR(2)),GEPS(2),RNA,
     $           COSKL,AJACOB,-1.0D0,1.0D0)
            GVAL(7) = COSKL
            YACOB = YACOB*AJACOB

            GVAL(8) = XRUN(10)*2.0D0*PI - GVAL(2)
            YACOB = YACOB*2.0D0*PI
 
C...Set Cos(45), Phi(45) in (45) CM frame
            RNA = XRUN(13)
            CALL GRCSBCS(INT(GSPR(3)),GEPS(3),RNA,
     $           COSIJ,AJACOB,-1.0D0,1.0D0)
            GVAL(11) = COSIJ
            YACOB = YACOB*AJACOB

            GVAL(12) = XRUN(14)*2.0D0*PI
            YACOB = YACOB*2.0D0*PI
 
C...Set Cos(78), Phi(78) in (78) CM frame
            RNA = XRUN(15)
            CALL GRCSBCS(INT(GSPR(4)),GEPS(4),RNA,
     $           COSKL,AJACOB,-1.0D0,1.0D0)
            GVAL(13) = COSKL
            YACOB = YACOB*AJACOB

            GVAL(14) = XRUN(16)*2.0D0*PI
            YACOB = YACOB*2.0D0*PI

C...Set Flavor
            IF (NDIM.GT.17.AND.GPRI(10).EQ.0.0D0) THEN
               GPRI(10) = XRUN(18)
            ENDIF
            KTYPE = 13

         ELSE
C...W/Z +4 jets processes.
            IF (IGSUB.EQ.439) THEN
C...Total cross section with all decay particles.
               YACOB = YACOB/GRCWBR(18)

C...Pick up final state flavor.
               RANSUM = XRUN(18)*GDCYSUM
               IDCYST = 0
               DO WHILE (RANSUM.GT.0.0D0)
                  IDCYST = IDCYST + 1
                  RANSUM = RANSUM - GDCYRAT(IDCYST)
               ENDDO
               KFCODE(3) =  IGWDCY(2*GDCYPR1(IDCYST)-1)
               KFCODE(4) =  IGWDCY(2*GDCYPR1(IDCYST))
               AMASS1(3) = GWDCYMS(ABS(KFCODE(3)))
               AMASS1(4) = GWDCYMS(ABS(KFCODE(4)))
               AMASS2(3) = AMASS1(3)**2
               AMASS2(4) = AMASS1(4)**2
               AMLP(1) = AMASS1(3)
               AMNU(1) = AMASS1(4)

C...Apply Br. ratio.
               YACOB = YACOB*GDCYSUM
            ENDIF

C...Q^2(34):reso. W/Z, Q^2(78):gluon. type.
            KTYPE = 7
            IPX(3) = 3
            IPX(4) = 4
            IPX(5) = 7
            IPX(6) = 8

C...Set Q12(34)
            RNA = XRUN(11)
            CALL GRCSQ12(RNA, Q12, AJACOB)
            GVAL(9) = Q12
            YACOB = YACOB*AJACOB

C...Set Q22(78)
            RNA = XRUN(12)
            CALL GRCSQ22(RNA, Q22, AJACOB)
            GVAL(10) = Q22
            YACOB = YACOB*AJACOB

C...Set large gluon mass 3(45)
            IPX(3) = 3
            IPX(4) = 4
            IPX(5) = 5
            IPX(6) = 6
            QMIN = GVAL(9) + AMASS2(5)
            QMAX = (GRCECM - AMASS1(6) - SQRT(GVAL(10)))**2

            RNA = XRUN(5)
            QM = QMIN*(QMAX/QMIN)**RNA
            YACOB = YACOB*QM*LOG(QMAX/QMIN)
            GVAL(3) = QM

C...Set another side large gluon mass 6(78)
            QMIN = GVAL(10) + AMASS2(6)
            QMAX = (GRCECM - SQRT(GVAL(3)))**2

            RNA = XRUN(6)
            QM = QMIN*(QMAX/QMIN)**RNA
            YACOB = YACOB*QM*LOG(QMAX/QMIN)
            GVAL(4) = QM
 
            IPX(3) = 3
            IPX(4) = 4
            IPX(5) = 7
            IPX(6) = 8
C...Set tau, yst, s, w, x1, x2, beta, and gamma
            CALL GRCSSPX

            IPX(3) = 3
            IPX(4) = 4
            IPX(5) = 5
            IPX(6) = 6
C...Set Cos1, Phi1
            RNA = XRUN(3)
            CALL GRCICOS
            CALL GRCSCOS(RNA, COS1, AJACOB)
            GVAL(1) = COS1
            YACOB = YACOB*AJACOB
 
            GVAL(2) = XRUN(4)*2.0D0*PI
            YACOB = YACOB*2.0D0*PI

C...Set Cos{5(34)}, Phi{5(34)} in 5(34) CM frame
            RNA = XRUN(7)
            CALL GRCSBCS(INT(GSPR(1)),GEPS(1),RNA,
     $           COSIJ,AJACOB,-1.0D0,1.0D0)
            GVAL(5) = COSIJ
            YACOB = YACOB*AJACOB

            GVAL(6) = XRUN(8)*2.0D0*PI - GVAL(2)
            YACOB = YACOB*2.0D0*PI

C...Set Cos{6(78)}, Phi{6(78)} in 6(78) CM frame
            RNA = XRUN(9)
            CALL GRCSBCS(INT(GSPR(2)),GEPS(2),RNA,
     $           COSKL,AJACOB,-1.0D0,1.0D0)
            GVAL(7) = COSKL
            YACOB = YACOB*AJACOB

            GVAL(8) = XRUN(10)*2.0D0*PI - GVAL(2)
            YACOB = YACOB*2.0D0*PI
 
C...Set Cos(34), Phi(34) in (34) CM frame
            RNA = XRUN(13)
            CALL GRCSBCS(INT(GSPR(3)),GEPS(3),RNA,
     $           COSIJ,AJACOB,-1.0D0,1.0D0)
            GVAL(11) = COSIJ
            YACOB = YACOB*AJACOB

            GVAL(12) = XRUN(14)*2.0D0*PI
            YACOB = YACOB*2.0D0*PI

C...Set Cos(78), Phi(78) in (78) CM frame
            RNA = XRUN(15)
            CALL GRCSBCS(INT(GSPR(4)),GEPS(4),RNA,
     $           COSKL,AJACOB,-1.0D0,1.0D0)
            GVAL(13) = COSKL
            YACOB = YACOB*AJACOB

            GVAL(14) = XRUN(16)*2.0D0*PI
            YACOB = YACOB*2.0D0*PI

C...Set Flavor
            IF (NDIM.GT.16.AND.GPRI(10).EQ.0.0D0) THEN
               GPRI(10) = XRUN(17)
            ENDIF
            KTYPE = 13
            IPX(3) = 5
            IPX(4) = 3
            IPX(5) = 4
            IPX(6) = 6
            IPX(7) = 7
            IPX(8) = 8
         ENDIF
      ENDIF

C...Check Flag
      IF (IREJ.GT.0) GO TO 999

C...Fill temp.Kinematics
      CALL GRCSKIN(PGRC,AJACOB)
 
C...Set Coupling constant
      CALL GRCSENE
 
      IF (YACOB.LT.0) THEN
         IREJ = 1
      ENDIF
 
      RETURN
 999  CONTINUE
      YACOB = 0.0D0
      RETURN
      END
 
C------------------------
C Set n-body kinematics.
C------------------------
      SUBROUTINE GRCSKIN(PF4,AJACOB)
      IMPLICIT REAL* 8(A-H,O-Z)
 
      INCLUDE 'inclk.h'
      INCLUDE 'grchad.inc'
 
      REAL*8 PF4(4,8),AJACOB

      IF (NNF.EQ.4) THEN
         CALL GRCFKIN2(PF4,AJACOB)
      ELSE IF (NNF.EQ.5) THEN
         IF (KTYPE.LT.4) THEN
            CALL GRCFKIN3(PF4,AJACOB)
         ELSE IF (KTYPE.EQ.4) THEN
            CALL GRCFUSN3(PF4,AJACOB)
         ENDIF
      ELSE IF (NNF.EQ.6) THEN
         IF (KTYPE.EQ.8) THEN
            CALL GRCFCAS4(PF4,AJACOB)
         ELSE IF (KTYPE.EQ.9) THEN
            CALL GRCFUSN4(PF4,AJACOB)
         ELSE
            CALL GRCFKIN4(PF4,AJACOB)
         ENDIF
      ELSE IF (NNF.EQ.7) THEN
         IF (KTYPE.EQ.10) THEN
            CALL GRCFUSN5(PF4,AJACOB)
         ELSE IF (KTYPE.EQ.11) THEN
            CALL GRCFKIN5(PF4,AJACOB)
         ELSE IF (KTYPE.EQ.12) THEN
            CALL GRCFCAS5(PF4,AJACOB)
         ENDIF
      ELSE IF (NNF.EQ.8) THEN
         IF (KTYPE.EQ.13) THEN
            CALL GRCFKIN6(PF4,AJACOB)
         ENDIF
      ELSE
         IREJ = 1
         WRITE(*,*) 'GRC::Error: Set wrong ktype.'
      ENDIF

      RETURN
      END
 
C-----------------------
C Set kinematical cuts.
C-----------------------
      SUBROUTINE GRCSCUT
      IMPLICIT REAL*8(A-H,O-Z)
 
      INCLUDE 'inclk.h'
      INCLUDE 'grchad.inc'
 
      INTEGER I,J,IUSRCUT
      REAL*8 PPZ,PPP
      REAL*8 SUMX,SUMY,SUMPT,DPHI,RCON
      INTEGER IGPCOMB(157)

C...Symmetric conversion array
      DATA IGPCOMB/0,0,0,0,0,1,1,1,1,0,1,0,1,0,0,1,1,0,0,1,1,0,0,1,1,
     $             0,0,1,1,0,0,1,1,0,0,1,1,0,0,0,0,0,0,0,0,1,1,1,1,1,
     $             1,1,1,0,0,1,1,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
     $             0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
     $             0,1,1,1,1,0,0,0,0,1,1,1,1,0,0,1,1,0,0,1,1,0,0,0,1,
     $             1,1,0,0,0,1,1,1,0,0,0,0,1,1,1,1,0,0,0,0,1,1,1,1,0,
     $             0,0,0,1,1,1,1/

C...Set 4-Momenmum for Initial State Particles
      DO I=1,4
         PLGRC(I,1) = PGRC(I,1)
         PLGRC(I,2) = PGRC(I,2)
      ENDDO

      SUMX = 0.0D0
      SUMY = 0.0D0
C...Set Px,Py,Pz to Pt,Phi,Eta for Final State Particles
      DO I=3,NNF
 
C...Set pt,phi,eta,E
         PLGRC(1,I) = SQRT(PGRC(1,I)**2+PGRC(2,I)**2)
         IF (PGRC(2,I).GT.0) THEN
            PLGRC(2,I) = ACOS(PGRC(1,I)/PLGRC(1,I))
         ELSE
            PLGRC(2,I) = -ACOS(PGRC(1,I)/PLGRC(1,I))
         ENDIF

C...Set pz
         IF (IGPCOMB(GRCFLAV+1).EQ.0) THEN
            PPZ = GPRI(8)*(PGRC(3,I) + GPRI(7)*PGRC(4,I))
         ELSE IF (IGPCOMB(GRCFLAV+1).EQ.1) THEN
            PPZ = GPRI(8)*(-PGRC(3,I) + GPRI(7)*PGRC(4,I))
         ELSE
            IREJ = 1
            WRITE(*,*) 'GRC::Error: Wrong flavor in GRCSCUT.'
         ENDIF
         PPP = SQRT(PGRC(1,I)**2+PGRC(2,I)**2+PPZ**2)
C...Set energy
         PLGRC(4,I) = GPRI(8)*(PGRC(4,I) + GPRI(7)*PGRC(3,I))
C...Set pseudo-rapidity
         PLGRC(3,I) = 0.5D0*LOG((PPP+PPZ)/(PPP-PPZ))
C...Set "true" rapidity
C         plgrc(3,i) = 0.5d0*log((plgrc(4,i)+ppz)/(plgrc(4,i)-ppz))

C...Pt Cut
         IF (PLGRC(1,I).LT.GRCPTCUT(I-2)) IREJ = 2
C...Eta Cut
         IF (ABS(PLGRC(3,I)).GT.GRCETACUT(I-2)) IREJ = 2
 
         SUMX = PGRC(1,I) + SUMX
         SUMY = PGRC(2,I) + SUMY
      ENDDO
C...Set SumPt
      SUMPT = SQRT(SUMX**2 + SUMY**2)
C...MET Cut
      IF (SUMPT.LT.GRCMETCUT) IREJ = 2
      IF (IREJ.GT.0) GO TO 999

C...Set Rcone
      DO I = 3,NNF
         DO J = 3,NNF
            IF (I.NE.J) THEN
               CALL GRCAPHI(PLGRC(2,I),PLGRC(2,J),DPHI)
               RCON = SQRT(DPHI**2 + (PLGRC(3,I) - PLGRC(3,J))**2)
C...Rcone Cut
               IF (RCON.LT.GRCRCONCUT(I-2)
     $              .AND.GRCRCONCUT(J-2).GT.0.0D0) THEN
                  IREJ = 2
                  GO TO 999
               ENDIF
            ENDIF
         ENDDO
      ENDDO

CCC      CALL GRCAPHI(PLGRC(2,5),PLGRC(2,6),DPHI)
CCC      RCON = SQRT(DPHI**2 + (PLGRC(3,5) - PLGRC(3,6))**2)
CCC      IF (RCON.LT.GRCRCONCUT(I-2)) THEN
CCC         IREJ = 2
CCC         GO TO 999
CCC      ENDIF

C...Call user defined cut
      IUSRCUT = 0
      CALL GRCUSRCUT(IUSRCUT)
      IF (IUSRCUT.EQ.1) THEN
         IREJ = 2
         GO TO 999
      ENDIF
 
      RETURN
 999  CONTINUE
      YACOB = 0.0D0
      RETURN
      END
 
C------------------
C Smear to PYTHIA.
C------------------
      SUBROUTINE GRCSSMR
      IMPLICIT REAL*8(A-H,O-Z)
 
#include "define.h"
 
      INCLUDE 'inclk.h'
      INCLUDE 'grchad.inc'
 
#if PSMODEL == 61
      COMMON/PYPARS/MSTP(200),PARP(200),MSTI(200),PARI(200)
      COMMON/PYUPPR/NUP,KUP(20,7),NFUP,IFUP(10,2),PUP(20,5),Q2UP(0:10)
#else
      COMMON/PYPARS/MSTP(200),PARP(200),MSTI(200),PARI(200)
C...User process event common block.
      INTEGER MAXNUP
      PARAMETER (MAXNUP=500)
      INTEGER NUP,IDPRUP,IDUP,ISTUP,MOTHUP,ICOLUP
      DOUBLE PRECISION XWGTUP,SCALUP,AQEDUP,AQCDUP,PUP,VTIMUP,SPINUP
      COMMON/HEPEUP/NUP,IDPRUP,XWGTUP,SCALUP,AQEDUP,AQCDUP,IDUP(MAXNUP),
     &ISTUP(MAXNUP),MOTHUP(2,MAXNUP),ICOLUP(2,MAXNUP),PUP(5,MAXNUP),
     &VTIMUP(MAXNUP),SPINUP(MAXNUP)
      SAVE /HEPEUP/
#endif

      INTEGER LECT
      COMMON/HELICITY/LECT(4000)

      INTEGER I,IMR,ISEL
      REAL*8 XMRESO(4),AJACOB,X1MASS
      INTEGER IGPCOMB(157)
      CHARACTER*60 XFMT

C...Symmetric conversion array
      DATA IGPCOMB/0,0,0,0,0,1,1,1,1,0,1,0,1,0,0,1,1,0,0,1,1,0,0,1,1,
     $             0,0,1,1,0,0,1,1,0,0,1,1,0,0,0,0,0,0,0,0,1,1,1,1,1,
     $             1,1,1,0,0,1,1,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
     $             0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
     $             0,1,1,1,1,0,0,0,0,1,1,1,1,0,0,1,1,0,0,1,1,0,0,0,1,
     $             1,1,0,0,0,1,1,1,0,0,0,0,1,1,1,1,0,0,0,0,1,1,1,1,0,
     $             0,0,0,1,1,1,1/

      IF (IBSWRT.LT.2) THEN
 
C...Set Phase Region
         CALL GRCXSET(ISEL)
         GRCIREG = ISEL

C...Breit Wigner mstp(42) On:1/Off:0
#if PSMODEL > 0
         IF (MSTP(42).EQ.1) THEN
            IF (ISDEP.EQ.0) THEN
               CALL GRCSBWM(PGRC)
            ELSE
               CALL GRCSKIN(PGRC,AJACOB)
            ENDIF
         ELSE IF (MSTP(42).EQ.0) THEN
            CALL GRCSKIN(PGRC,AJACOB)
         ENDIF
#else
         CALL GRCSKIN(PGRC,AJACOB)
#endif

C...Apply Kinematical Cuts
         CALL GRCSCUT

C...Set Kinematics to PYTHIA
         IF (KFCODE(1).LT.7) THEN
            AMASS1(1) = GWDCYMS(ABS(KFCODE(1)))
         ELSE
            AMASS1(1) = AMG
         ENDIF
         IF (KFCODE(2).LT.7) THEN
            AMASS1(2) = GWDCYMS(ABS(KFCODE(2)))
         ELSE
            AMASS1(2) = AMG
         ENDIF

         IF (IGPCOMB(GRCFLAV+1).EQ.1) THEN
            DO I = 3,NNF
               PGRC(3,I) = -PGRC(3,I)
            ENDDO
            DO I = 1,4
               PQGRC(3,I) = -PQGRC(3,I)
            ENDDO
         ENDIF

C...Set Particle Combination
         CALL GRCSWAP(ISEL)

C...Set Color String
         CALL SPDETC(ISEL)

#if PSMODEL > 61 
CC...Set Helicity State
C         CALL SPDETH(ISEL)
C         DO I = 1,NUP
C            SPINUP(I) = LECT(I)
C         ENDDO

C...For tmp...
         DO I = 1,NUP
            SPINUP(I) = 0.0D0
         ENDDO
#endif

C+-------------------------------------------------+
#ifdef MCDBWRT
C...For special usage.
C...int(1*nup)+int(3*(nup-2))+int(2*nup)+int(nup)+real(5*nup)+3
         XFMT = '(##i4,##i3,##i4,##f4.0,##f11.4)'
         WRITE(XFMT(2:3),FMT='(i2)') NUP
         WRITE(XFMT(7:8),FMT='(i2)') 3*(NUP-2)
         WRITE(XFMT(12:13),FMT='(i2)') 2*NUP
         WRITE(XFMT(17:18),FMT='(i2)') NUP
         WRITE(XFMT(24:25),FMT='(i2)') 5*NUP+3
         WRITE(13,FMT=XFMT) 
     $        (IDUP(J),J=1,NUP),(ISTUP(J),J=3,NUP),
     $        ((MOTHUP(I,J),I=1,2),J=3,NUP),
     $        ((ICOLUP(I,J),I=1,2),J=1,NUP),
     $        (SPINUP(J),J=1,NUP),
     $        ((PUP(I,J),I=1,5),J=1,NUP),SCALUP,AQEDUP,AQCDUP
#endif
C+-------------------------------------------------+

C...For special usage. 
      ELSE IF (IBSWRT.EQ.2) THEN

C...For special usage.
C...int(1*nup)+int(3*(nup-2))+int(2*nup)+int(nup)+real(5*nup)+3
         XFMT = '(##i4,##i3,##i4,##f4.0,##f11.4)'
         WRITE(XFMT(2:3),FMT='(i2)') NUP
         WRITE(XFMT(7:8),FMT='(i2)') 3*(NUP-2)
         WRITE(XFMT(12:13),FMT='(i2)') 2*NUP
         WRITE(XFMT(17:18),FMT='(i2)') NUP
         WRITE(XFMT(24:25),FMT='(i2)') 5*NUP+3
         READ(14,FMT=XFMT) 
     $        (IDUP(J),J=1,NUP),(ISTUP(J),J=3,NUP),
     $        ((MOTHUP(I,J),I=1,2),J=3,NUP),
     $        ((ICOLUP(I,J),I=1,2),J=1,NUP),
     $        (SPINUP(J),J=1,NUP),
     $        ((PUP(I,J),I=1,5),J=1,NUP),SCALUP,AQEDUP,AQCDUP

C...Status codes for entries.
         ISTUP(1) = -1
         ISTUP(2) = -1
      ENDIF
 
      RETURN
      END
 
C-----------------
C Set B.W masses.
C-----------------
      SUBROUTINE GRCSBWM(PEBW)
      IMPLICIT REAL*8(A-H,O-Z)
 
#include "define.h"
 
      INCLUDE 'inclk.h'
      INCLUDE 'grchad.inc'
 
#if PSMODEL > 0
      COMMON/PYPARS/MSTP(200),PARP(200),MSTI(200),PARI(200)
#else
      REAL*8 PARP(50)
#endif
 
      INTEGER I,IDUMMY,IMR,IMRX(4)
      REAL*8 XMR,XMRG,XMRMIN,XMRMAX
      REAL*8 AMRDN,AMRUP,AMRTO,XMRESO(4)
      REAL*8 PQ1,PQ2,COSQ1,COSQ2,SINQ1,SINQ2,AAA,BBB,CCC
      REAL*8 PEBW(4,8),AJACOB
      EXTERNAL DRN
 
C...Decide W, Z, H
      IMR = 0
      DO I=3,NNF
         IF (ABS(KFCODE(I)).EQ.24) THEN
            IMR = IMR + 1
            IMRX(IMR) = I
            XMR = AMW
            XMRG = AGW
         ELSE IF (ABS(KFCODE(I)).EQ.23) THEN
            IMR = IMR + 1
            IMRX(IMR) = I
            XMR = AMZ
            XMRG = AGZ
         ELSE IF (ABS(KFCODE(I)).EQ.25) THEN
            IMR = IMR + 1
            IMRX(IMR) = I
            XMR = AMH
            XMRG = AGH
         ENDIF
      ENDDO
 
      IF (IMR.EQ.0) THEN
CCC         write(*,*) 'Error: No B.W. particle'
      ENDIF
 
C...Breit Wigner Mass
      DO I=1,IMR
         IF (KTYPE.LT.10) THEN
            XMRMIN = PARP(42)
            XMRMAX = PGRC(4,IMRX(IMR))
         ELSE
            PQ1 = GVAL(12)
            PQ2 = GVAL(13)
            COSQ1 = GVAL(5)
            COSQ2 = GVAL(6)
            SINQ1 = SQRT((1.0D0 - COSQ1)*(1.0D0 + COSQ1))
            SINQ2 = SQRT((1.0D0 - COSQ2)*(1.0D0 + COSQ2))
 
            AAA = (PQ1*COSQ1 + PQ2*COSQ2)**2
            BBB = (PQ1*SINQ1 - PQ2*SINQ2)**2
            CCC = (PQ1*SINQ1 + PQ2*SINQ2)**2
 
            XMRMIN = SQRT(MAX(PARP(42)**2,
     $           PGRC(4,IMRX(IMR))**2-AAA-CCC))
            XMRMAX = SQRT(PGRC(4,IMRX(IMR))**2-AAA-BBB)
         ENDIF
 
         AMRDN = ATAN((XMRMIN**2 - XMR**2)/(XMR*XMRG))
         AMRUP = ATAN((XMRMAX**2 - XMR**2)/(XMR*XMRG))
 
         AMRTO = AMRDN + (AMRUP - AMRDN)*DRN(IDUMMY)
         XMRESO(I) = SQRT(XMR**2 + XMR*XMRG*TAN(AMRTO))
         AMASS1(IMRX(IMR)) = XMRESO(I)
         AMASS2(IMRX(IMR)) = XMRESO(I)**2
 
      ENDDO
 
C...Set new kinematics
      CALL GRCSKIN(PEBW,AJACOB)
 
      RETURN
      END
 
C------------------------
C Set 2-body kinematics.
C------------------------
      SUBROUTINE GRCFKIN2(PF4,AJACOB)
      IMPLICIT REAL*8(A-H,O-Z)
 
      INCLUDE 'inclk.h'
      INCLUDE 'grchad.inc'
 
      INTEGER I
      REAL*8 FACT
      REAL*8 E1,E2,E3,P3,COS3,SIN3,PHI3,E4,Z1,Z2
      REAL*8 PF4(4,8),AJACOB
 
C...beta function
      BETA(Z1,Z2)=SQRT(1.0D0-2.0D0*(Z1+Z2)+(Z1-Z2)**2)
 
      FACT = 0.38937966E9/(2.0D0*GPRI(3))

      AJACOB = 0.0D0 
      IF (NNF.NE.4) THEN
         WRITE(*,*) 'Error: wrong Nr. of partcle'
      ENDIF

C...Check invariant mass
      IF (GPRI(3).LT.(AMASS1(IPX(3))+AMASS1(IPX(4)))**2) THEN
         IREJ = 3
         GO TO 999
      ENDIF
 
C...Set Energy and Momentum
      E1 = (GPRI(3)+AMASS2(IPX(1))-AMASS2(IPX(2)))/2.D0/GPRI(4)
      E2 = (GPRI(3)+AMASS2(IPX(2))-AMASS2(IPX(1)))/2.D0/GPRI(4)
 
C...Particle 1
      PF4(1,IPX(1)) = 0
      PF4(2,IPX(1)) = 0
      PF4(3,IPX(1)) = SQRT((E1-AMASS1(IPX(1)))*(E1+AMASS1(IPX(1))))
      PF4(4,IPX(1)) = E1
 
C...Particle 2
      PF4(1,IPX(2)) = 0
      PF4(2,IPX(2)) = 0
      PF4(3,IPX(2)) = -SQRT((E2-AMASS1(IPX(2)))*(E2+AMASS1(IPX(2))))
      PF4(4,IPX(2)) = E2
 
C...Particle 3
      E3 = (GPRI(3)+AMASS2(IPX(3))-AMASS2(IPX(4)))/2.D0/GPRI(4)
      P3 = SQRT((E3-AMASS1(IPX(3)))*(E3+AMASS1(IPX(3))))
      COS3 = GVAL(1)
      SIN3 = SQRT((1.0D0 - COS3)*(1.0D0 + COS3))
      PHI3 = GVAL(2)
 
      PF4(1,IPX(3)) = P3*SIN3*COS(PHI3)
      PF4(2,IPX(3)) = P3*SIN3*SIN(PHI3)
      PF4(3,IPX(3)) = P3*COS3
      PF4(4,IPX(3)) = E3
 
C...Particle 4
      E4 = (GPRI(3)+AMASS2(IPX(4))-AMASS2(IPX(3)))/2.D0/GPRI(4)
 
      PF4(1,IPX(4)) = -PF4(1,IPX(3))
      PF4(2,IPX(4)) = -PF4(2,IPX(3))
      PF4(3,IPX(4)) = -PF4(3,IPX(3))
      PF4(4,IPX(4)) = E4

      PQGRC(1,1) = 0.0d0
      PQGRC(2,1) = 0.0d0
      PQGRC(3,1) = PF4(3,IPX(1))+PF4(3,IPX(2))
      PQGRC(4,1) = PF4(4,IPX(1))+PF4(4,IPX(2))
 
      DO I=1,NNF
         IF (PF4(4,IPX(I)).LT.AMASS1(IPX(I))) IREJ = 3
      ENDDO
 
      AJACOB = FACT/32.D0/PI2
     $     *BETA(AMASS2(IPX(3))/GPRI(3),AMASS2(IPX(4))/GPRI(3))
 
      RETURN
 999  CONTINUE
      YACOB = 0.0D0
      RETURN
      END
 
C------------------------
C Set 3-body kinematics.
C------------------------
      SUBROUTINE GRCFKIN3(PF4,AJACOB)
      IMPLICIT REAL*8(A-H,O-Z)
 
      INCLUDE 'inclk.h'
      INCLUDE 'grchad.inc'
 
      INTEGER I
      REAL*8 FACT
      REAL*8 E1,E2,EQ1,PQ1,Z1,Z2
      REAL*8 EIP1,PIP1,COSIP1,SINIP1,PHIIP1
      REAL*8 EIP2,PIP2,COSIP2,SINIP2,PHIIP2,EIP3
      REAL*8 PF4(4,8),PW(4),AJACOB
 
C...beta function
      BETA(Z1,Z2)=SQRT(1.0D0-2.0D0*(Z1+Z2)+(Z1-Z2)**2)
 
      FACT = 0.38937966E9/(2.0D0*GPRI(3))

      AJACOB = 0.0D0 
      IF (NNF.NE.5) THEN
         WRITE(*,*) 'Error: wrong Nr. of partcle'
      ENDIF

C...Check invariant mass
      IF (GVAL(3).LT.(AMASS1(IPX(4))+AMASS1(IPX(5)))**2) THEN
         IREJ = 3
         GO TO 999
      ENDIF
 
C...Set Energy and Momentum
      E1 = (GPRI(3)+AMASS2(IPX(1))-AMASS2(IPX(2)))/2.D0/GPRI(4)
      E2 = (GPRI(3)+AMASS2(IPX(2))-AMASS2(IPX(1)))/2.D0/GPRI(4)
 
C...Particle 1
      PF4(1,IPX(1)) = 0
      PF4(2,IPX(1)) = 0
      PF4(3,IPX(1)) = SQRT((E1-AMASS1(IPX(1)))*(E1+AMASS1(IPX(1))))
      PF4(4,IPX(1)) = E1
 
C...Particle 2
      PF4(1,IPX(2)) = 0
      PF4(2,IPX(2)) = 0
      PF4(3,IPX(2)) = -SQRT((E2-AMASS1(IPX(2)))*(E2+AMASS1(IPX(2))))
      PF4(4,IPX(2)) = E2
 
C...Particle ipx(3)
      EIP1 = (GPRI(3)+AMASS2(IPX(3))-GVAL(3))/2.0D0/GPRI(4)
      PIP1 = SQRT((EIP1-AMASS1(IPX(3)))*(EIP1+AMASS1(IPX(3))))
      COSIP1 = GVAL(1)
      SINIP1 = SQRT((1.0D0-COSIP1)*(1.0D0+COSIP1))
      PHIIP1 = GVAL(2)
 
      PF4(1,IPX(3)) = PIP1*SINIP1*COS(PHIIP1)
      PF4(2,IPX(3)) = PIP1*SINIP1*SIN(PHIIP1)
      PF4(3,IPX(3)) = PIP1*COSIP1
      PF4(4,IPX(3)) = EIP1
 
C...Particle ipx(4) in pw CM frame
      EIP2 = (GVAL(3)+AMASS2(IPX(4))-AMASS2(IPX(5)))
     $     /2.D0/SQRT(GVAL(3))
      PIP2 = SQRT((EIP2-AMASS1(IPX(4)))*(EIP2+AMASS1(IPX(4))))
      COSIP2 = GVAL(4)
      SINIP2 = SQRT((1.0D0-COSIP2)*(1.0D0+COSIP2))
      PHIIP2 = GVAL(5)
 
      PF4(1,IPX(4)) = PIP2*SINIP2*COS(PHIIP2)
      PF4(2,IPX(4)) = PIP2*SINIP2*SIN(PHIIP2)
      PF4(3,IPX(4)) = PIP2*COSIP2
      PF4(4,IPX(4)) = EIP2
 
C...Particle ipx(5) in pw CM frame
      EIP3 = (GVAL(3)+AMASS2(IPX(5))-AMASS2(IPX(4)))
     $     /2.D0/SQRT(GVAL(3))
      PF4(1,IPX(5)) = -PF4(1,IPX(4))
      PF4(2,IPX(5)) = -PF4(2,IPX(4))
      PF4(3,IPX(5)) = -PF4(3,IPX(4))
      PF4(4,IPX(5)) = EIP3
 
C...q1
      EQ1 = (GPRI(3) + GVAL(3) - AMASS2(IPX(3)))/2.0D0/GPRI(4)
      PQ1 = SQRT(EQ1*EQ1 - GVAL(3))
      PW(1) = -PQ1*SINIP1*COS(PHIIP1)
      PW(2) = -PQ1*SINIP1*SIN(PHIIP1)
      PW(3) = -PQ1*COSIP1
      PW(4) = EQ1

      PQGRC(1,1) = PW(1)
      PQGRC(2,1) = PW(2)
      PQGRC(3,1) = PW(3)
      PQGRC(4,1) = PW(4)

C...Boost from 3-4 CM frame to 1-2 CM frame (Lab-frame)
      CALL WTOLAB(PF4(1,IPX(4)),PF4(1,IPX(5)),PW,
     $     PF4(1,IPX(4)),PF4(1,IPX(5)))
 
      DO I=1,NNF
         IF (PF4(4,IPX(I)).LT.AMASS1(IPX(I))) IREJ = 3
      ENDDO
 
      AJACOB = FACT/((2*PI)*(32.D0*PI2)*(32.D0*PI2))
     $     *BETA(GVAL(3)/GPRI(3),AMASS2(IPX(3))/GPRI(3))
     $     *BETA(AMASS2(IPX(4))/GVAL(3),AMASS2(IPX(5))/GVAL(3))
 
      RETURN
 999  CONTINUE
      YACOB = 0.0D0
      RETURN
      END
 
C------------------------------------
C Set 3-body fusion type kinematics.
C------------------------------------
      SUBROUTINE GRCFUSN3(PF4,AJACOB)
      IMPLICIT REAL*8(A-H,O-Z)
 
      INCLUDE 'inclk.h'
      INCLUDE 'grchad.inc'
 
      INTEGER I
      REAL*8 FACT
      REAL*8 E1,E2,E3,E4,E5,P1,P3,P4,P5,PT3,PT4,PT5
      REAL*8 COS3,COS4,COS5,SIN3,SIN4,SIN5,PHI3,PHI4,PHI5
      REAL*8 COSP34,SINP34
      REAL*8 PF4(4,8),AJACOB
 
      FACT = 0.38937966E9/(2.0D0*GPRI(3))

      AJACOB = 0.0D0 
      IF (NNF.NE.5) THEN
         WRITE(*,*) 'Error: wrong Nr. of partcle'
      ENDIF
      IF (KTYPE.NE.4) THEN
         WRITE(*,*) 'Error: wrong ktype'
      ENDIF
 
C...Set Energy and Momentum
      E1 = (GPRI(3)+AMASS2(IPX(1))-AMASS2(IPX(2)))/2.D0/GPRI(4)
      P1 = SQRT((E1-AMASS1(IPX(1)))*(E1+AMASS1(IPX(1))))
      E2 = (GPRI(3)+AMASS2(IPX(2))-AMASS2(IPX(1)))/2.D0/GPRI(4)
 
C...Particle ipx(1)
      PF4(1,IPX(1)) = 0
      PF4(2,IPX(1)) = 0
      PF4(3,IPX(1)) = P1
      PF4(4,IPX(1)) = E1
 
C...Particle ipx(2)
      PF4(1,IPX(2)) = 0
      PF4(2,IPX(2)) = 0
      PF4(3,IPX(2)) = -SQRT((E2-AMASS1(IPX(2)))*(E2+AMASS1(IPX(2))))
      PF4(4,IPX(2)) = E2
 
C...Fermion Propagator
C...Particle ipx(4)
      E4 = (GPRI(4) - GVAL(1) + GVAL(2))/2.0D0
      P4 = SQRT(E4**2 - AMASS2(IPX(4)))
 
      COS4 = GVAL(3)
      SIN4 = SQRT((1.0D0 - COS4)*(1.0D0 + COS4))
      PT4 = P4*SIN4
 
      PHI4 = GVAL(5)
 
      PF4(1,IPX(4)) = P4*SIN4*COS(PHI4)
      PF4(2,IPX(4)) = P4*SIN4*SIN(PHI4)
      PF4(3,IPX(4)) = P4*COS4
      PF4(4,IPX(4)) = E4
 
C...Particle ipx(5)
      E5 = (GPRI(4) - GVAL(1) - GVAL(2))/2.0D0
      P5 = SQRT(E5**2 - AMASS2(IPX(5)))
 
      COS5 = GVAL(4)
      SIN5 = SQRT((1.0D0 - COS5)*(1.0D0 + COS5))
      PT5 = P5*SIN5
 
C...Particle ipx(3)
      E3 = GVAL(1)
      P3 = SQRT((E3 - AMASS1(IPX(3)))*(E3 + AMASS1(IPX(3))))
 
      COS3 = (P5*COS5 + P4*COS4)/P3
      SIN3 = SQRT((1.0D0 - COS3)*(1.0D0 + COS3))
      PT3 = P3*SIN3
 
      PHI5 = PHI4 + ACOS((PT3**2-PT4**2-PT5**2)/2.0D0/PT4/PT5)
 
      PF4(1,IPX(5)) = P5*SIN5*COS(PHI5)
      PF4(2,IPX(5)) = P5*SIN5*SIN(PHI5)
      PF4(3,IPX(5)) = P5*COS5
      PF4(4,IPX(5)) = E5
 
C...Particle 3
      PF4(1,IPX(3)) = -PF4(1,IPX(4)) - PF4(1,IPX(5))
      PF4(2,IPX(3)) = -PF4(2,IPX(4)) - PF4(2,IPX(5))
      PF4(3,IPX(3)) = -PF4(3,IPX(4)) - PF4(3,IPX(5))
      PF4(4,IPX(3)) = GVAL(1)
 
      DO I=1,NNF
         IF (PF4(4,IPX(I)).LT.AMASS1(IPX(I))) IREJ = 3
      ENDDO
 
C...Set Jacobian
      AJACOB = FACT/8.0D0/(2.0D0*PI)**5/SIN4/SIN5
     $     /ABS(SIN(ACOS((PT3**2-PT4**2-PT5**2)/2.0D0/PT4/PT5)))
 
      RETURN
      END
 
C------------------------
C Set 4-body kinematics.
C------------------------
      SUBROUTINE GRCFKIN4(PF4,AJACOB)
      IMPLICIT REAL*8(A-H,O-Z)
 
      INCLUDE 'inclk.h'
      INCLUDE 'grchad.inc'
 
      INTEGER I
      REAL*8 FACT
      REAL*8 E1,E2,EQ1,PQ1,EQ2,COS1,SIN1,PHI1,Z1,Z2
      REAL*8 EI1Q1,PI1Q1,COSI1Q1,SINI1Q1,PHII1Q1,EI2Q1
      REAL*8 EI1Q2,PI1Q2,COSI1Q2,SINI1Q2,PHII1Q2,EI2Q2
      REAL*8 PF4(4,8),Q1(4),Q2(4),AJACOB
 
C...beta function
      BETA(Z1,Z2)=SQRT(1.0D0-2.0D0*(Z1+Z2)+(Z1-Z2)**2)
 
      FACT = 0.38937966E9/(2.0D0*GPRI(3))

      AJACOB = 0.0D0
      IF (NNF.NE.6) THEN
         WRITE(*,*) 'Error: wrong Nr. of partcle'
      ENDIF

C...Check invariant mass
      IF (GVAL(3).LT.(AMASS1(IPX(3))+AMASS1(IPX(4)))**2) THEN
         IREJ = 3
         GO TO 999
      ENDIF
      IF (GVAL(4).LT.(AMASS1(IPX(5))+AMASS1(IPX(6)))**2) THEN
         IREJ = 3
         GO TO 999
      ENDIF

C...Set Energy and Momentum
      E1 = (GPRI(3)+AMASS2(1)-AMASS2(2))/2.D0/GPRI(4)
      E2 = (GPRI(3)+AMASS2(2)-AMASS2(1))/2.D0/GPRI(4)

C...Particle 1
      PF4(1,IPX(1)) = 0
      PF4(2,IPX(1)) = 0
      PF4(3,IPX(1)) = SQRT((E1-AMASS1(1))*(E1+AMASS1(1)))
      PF4(4,IPX(1)) = E1
 
C...Particle 2
      PF4(1,IPX(2)) = 0
      PF4(2,IPX(2)) = 0
      PF4(3,IPX(2)) = -SQRT((E2-AMASS1(2))*(E2+AMASS1(2)))
      PF4(4,IPX(2)) = E2

C...Particle ipx(3) in q1 CM frame
      EI1Q1 = (GVAL(3)+AMASS2(IPX(3))-AMASS2(IPX(4)))
     $     /2.D0/SQRT(GVAL(3))
      IF (EI1Q1.LT.AMASS1(IPX(3))) GO TO 999

      PI1Q1 = SQRT((EI1Q1-AMASS1(IPX(3)))*(EI1Q1+AMASS1(IPX(3))))
      COSI1Q1 = GVAL(5)
      SINI1Q1 = SQRT((1.0D0-COSI1Q1)*(1.0D0+COSI1Q1))
      PHII1Q1 = GVAL(6)
 
      PF4(1,IPX(3)) = PI1Q1*SINI1Q1*COS(PHII1Q1)
      PF4(2,IPX(3)) = PI1Q1*SINI1Q1*SIN(PHII1Q1)
      PF4(3,IPX(3)) = PI1Q1*COSI1Q1
      PF4(4,IPX(3)) = EI1Q1

C...Particle ipx(4) in q1 CM frame
      EI2Q1 = (GVAL(3)+AMASS2(IPX(4))-AMASS2(IPX(3)))
     $     /2.D0/SQRT(GVAL(3))
      IF (EI2Q1.LT.AMASS1(IPX(4))) GO TO 999

      PF4(1,IPX(4)) = -PF4(1,IPX(3))
      PF4(2,IPX(4)) = -PF4(2,IPX(3))
      PF4(3,IPX(4)) = -PF4(3,IPX(3))
      PF4(4,IPX(4)) = EI2Q1

C...Particle ipx(5) in q2 CM frame
      EI1Q2 = (GVAL(4)+AMASS2(IPX(5))-AMASS2(IPX(6)))
     $     /2.D0/SQRT(GVAL(4))
      IF (EI1Q2.LT.AMASS1(IPX(5))) GO TO 999

      PI1Q2 = SQRT((EI1Q2-AMASS1(IPX(5)))*(EI1Q2+AMASS1(IPX(5))))
      COSI1Q2 = GVAL(7)
      SINI1Q2 = SQRT((1.0D0-COSI1Q2)*(1.0D0+COSI1Q2))
      PHII1Q2 = GVAL(8)
 
      IF (GSPR(1).EQ.7.OR.GSPR(2).EQ.7) THEN
         IF (XRUN(9).GE.0.25.AND.XRUN(9).LT.0.75) THEN
            COSI1Q2 = -COSI1Q2
            SINI1Q2 = -SINI1Q2
         ENDIF
      ENDIF
 
      PF4(1,IPX(5)) = PI1Q2*SINI1Q2*COS(PHII1Q2)
      PF4(2,IPX(5)) = PI1Q2*SINI1Q2*SIN(PHII1Q2)
      PF4(3,IPX(5)) = PI1Q2*COSI1Q2
      PF4(4,IPX(5)) = EI1Q2
 
C...Particle ipx(6) in q2 CM frame
      EI2Q2 = (GVAL(4)+AMASS2(IPX(6))-AMASS2(IPX(5)))
     $     /2.D0/SQRT(GVAL(4))
      IF (EI2Q2.LT.AMASS1(IPX(6))) GO TO 999

      PF4(1,IPX(6)) = -PF4(1,IPX(5))
      PF4(2,IPX(6)) = -PF4(2,IPX(5))
      PF4(3,IPX(6)) = -PF4(3,IPX(5))
      PF4(4,IPX(6)) = EI2Q2
 
C...q1 q2
      COS1 = GVAL(1)
      SIN1 = SQRT((1.0D0 - COS1)*(1.0D0 + COS1))
      PHI1 = GVAL(2)
 
      EQ1 = (GPRI(3) + GVAL(3) - GVAL(4))/2.D0/GPRI(4)
      PQ1 = SQRT(EQ1*EQ1 - GVAL(3))
      Q1(1) = PQ1*SIN1*COS(PHI1)
      Q1(2) = PQ1*SIN1*SIN(PHI1)
      Q1(3) = PQ1*COS1
      Q1(4) = EQ1

      PQGRC(1,1) = Q1(1)
      PQGRC(2,1) = Q1(2)
      PQGRC(3,1) = Q1(3)
      PQGRC(4,1) = Q1(4)
 
      EQ2 = (GPRI(3) + GVAL(4) - GVAL(3))/2.D0/GPRI(4)
      Q2(1) = -Q1(1)
      Q2(2) = -Q1(2)
      Q2(3) = -Q1(3)
      Q2(4) = EQ2

      PQGRC(1,2) = Q2(1)
      PQGRC(2,2) = Q2(2)
      PQGRC(3,2) = Q2(3)
      PQGRC(4,2) = Q2(4)
 
C...boost from q1 CM frame to 1-2 CM frame (Lab-frame)
      CALL WTOLAB(PF4(1,IPX(3)),PF4(1,IPX(4)),Q1,
     $     PF4(1,IPX(3)),PF4(1,IPX(4)))
 
C...boost from q2 CM frame to 1-2 CM frame (Lab-frame)
      CALL WTOLAB(PF4(1,IPX(5)),PF4(1,IPX(6)),Q2,
     $     PF4(1,IPX(5)),PF4(1,IPX(6)))
 
      DO I=1,NNF
         IF (PF4(4,IPX(I)).LT.AMASS1(IPX(I))) IREJ = 3
      ENDDO

      AJACOB = FACT/(2*PI)**2/(32*PI2)**3
     $     *BETA(GVAL(3)/GPRI(3),GVAL(4)/GPRI(3))
     $     *BETA(AMASS2(IPX(3))/GVAL(3),AMASS2(IPX(4))/GVAL(3))
     $     *BETA(AMASS2(IPX(5))/GVAL(4),AMASS2(IPX(6))/GVAL(4))

      RETURN
 999  CONTINUE
      IREJ = 3
      YACOB = 0.0D0
      RETURN
      END
 
C-------------------------------------
C Set 4-body cascade type kinematics.
C-------------------------------------
      SUBROUTINE GRCFCAS4(PF4,AJACOB)
      IMPLICIT REAL*8(A-H,O-Z)
 
      INCLUDE 'inclk.h'
      INCLUDE 'grchad.inc'
 
      INTEGER I
      REAL*8 FACT
      REAL*8 E1,E2,COS1,SIN1,PHI1,Z1,Z2
      REAL*8 EI1Q1,PI1Q1,COSI1Q1,SINI1Q1,PHII1Q1,EI2Q1
      REAL*8 EI1Q2,PI1Q2,COSQ1,SINQ1,PHIQ1
      REAL*8 EQ1,EQ2,PQ1,PQ2
      REAL*8 PF4(4,8),Q1(4),Q2(4),AJACOB
 
C...beta function
      BETA(Z1,Z2)=SQRT(1.0D0-2.0D0*(Z1+Z2)+(Z1-Z2)**2)
 
      FACT = 0.38937966E9/(2.0D0*GPRI(3))

      AJACOB = 0.0D0 
      IF (NNF.NE.6) THEN
         WRITE(*,*) 'Error: wrong Nr. of partcle'
      ENDIF
 
C...Set Energy and Momentum
      E1 = (GPRI(3)+AMASS2(IPX(1))-AMASS2(IPX(2)))/2.D0/GPRI(4)
      E2 = (GPRI(3)+AMASS2(IPX(2))-AMASS2(IPX(1)))/2.D0/GPRI(4)
 
C...Particle 1
      PF4(1,IPX(1)) = 0
      PF4(2,IPX(1)) = 0
      PF4(3,IPX(1)) = SQRT((E1-AMASS1(IPX(1)))*(E1+AMASS1(IPX(1))))
      PF4(4,IPX(1)) = E1
 
C...Particle 2
      PF4(1,IPX(2)) = 0
      PF4(2,IPX(2)) = 0
      PF4(3,IPX(2)) = -SQRT((E2-AMASS1(IPX(2)))*(E2+AMASS1(IPX(2))))
      PF4(4,IPX(2)) = E2
 
C...q2
      COS1 = GVAL(1)
      SIN1 = SQRT((1.0D0 - COS1)*(1.0D0 + COS1))
      PHI1 = GVAL(2)
 
      EQ2 = (GPRI(3)+GVAL(4)-AMASS2(IPX(6)))/2.0D0/GPRI(4)
      PQ2 = SQRT(EQ2**2 - GVAL(4))
 
      Q2(1) = PQ2*SIN1*COS(PHI1)
      Q2(2) = PQ2*SIN1*SIN(PHI1)
      Q2(3) = PQ2*COS1
      Q2(4) = EQ2

      PQGRC(1,2) = Q2(1)
      PQGRC(2,2) = Q2(2)
      PQGRC(3,2) = Q2(3)
      PQGRC(4,2) = Q2(4)
 
C...Particle ipx(6) in q1 CM frame
      PF4(1,IPX(6)) = -Q2(1)
      PF4(2,IPX(6)) = -Q2(2)
      PF4(3,IPX(6)) = -Q2(3)
      PF4(4,IPX(6)) = (GPRI(3)+AMASS2(IPX(6))-GVAL(4))/2.0D0/GPRI(4)
 
C...q1
      EQ1 = (GVAL(4)+GVAL(3)-AMASS2(IPX(5)))/2.D0/SQRT(GVAL(4))
      PQ1 = SQRT(EQ1**2 - GVAL(3))
      COSQ1 = GVAL(7)
      SINQ1 = SQRT((1.0D0-COSQ1)*(1.0D0+COSQ1))
      PHIQ1 = GVAL(8)
 
      Q1(1) = PQ1*SINQ1*COS(PHIQ1)
      Q1(2) = PQ1*SINQ1*SIN(PHIQ1)
      Q1(3) = PQ1*COSQ1
      Q1(4) = EQ1
 
C...Particle ipx(5) in q1 CM frame
      PF4(1,IPX(5)) = -Q1(1)
      PF4(2,IPX(5)) = -Q1(2)
      PF4(3,IPX(5)) = -Q1(3)
      PF4(4,IPX(5)) = (GVAL(4)+AMASS2(IPX(5))-GVAL(3))
     $     /2.D0/SQRT(GVAL(4))
 
C...Particle ipx(3) in q1 CM frame
      EI1Q1 = (GVAL(3)+AMASS2(IPX(3))-AMASS2(IPX(4)))
     $     /2.0D0/SQRT(GVAL(3))
      PI1Q1 = SQRT((EI1Q1-AMASS1(IPX(3)))*(EI1Q1+AMASS1(IPX(3))))
      COSI1Q1 = GVAL(5)
      SINI1Q1 = SQRT((1.0D0-COSI1Q1)*(1.0D0+COSI1Q1))
      PHII1Q1 = GVAL(6)
 
      PF4(1,IPX(3)) = PI1Q1*SINI1Q1*COS(PHII1Q1)
      PF4(2,IPX(3)) = PI1Q1*SINI1Q1*SIN(PHII1Q1)
      PF4(3,IPX(3)) = PI1Q1*COSI1Q1
      PF4(4,IPX(3)) = EI1Q1
 
C...Particle ipx(4) in q1 CM frame
      EI2Q1 = (GVAL(3)+AMASS2(IPX(4))-AMASS2(IPX(3)))
     $     /2.0D0/SQRT(GVAL(3))
      PF4(1,IPX(4)) = -PF4(1,IPX(3))
      PF4(2,IPX(4)) = -PF4(2,IPX(3))
      PF4(3,IPX(4)) = -PF4(3,IPX(3))
      PF4(4,IPX(4)) = EI2Q1
 
C...boost from q1 CM frame to q2 CM frame
      CALL WTOLAB(PF4(1,IPX(3)),PF4(1,IPX(4)),Q1,
     $     PF4(1,IPX(3)),PF4(1,IPX(4)))
 
C...boost from q2 CM frame to LAB frame
      CALL WTOLAB(PF4(1,IPX(3)),PF4(1,IPX(4)),Q2,
     $     PF4(1,IPX(3)),PF4(1,IPX(4)))
 
C...boost from q2 CM frame to LAB frame (Lab-frame)
      CALL WTOLAB(PF4(1,IPX(5)),Q1(1),Q2,PF4(1,IPX(5)),Q1(1))

      PQGRC(1,1) = Q1(1)
      PQGRC(2,1) = Q1(2)
      PQGRC(3,1) = Q1(3)
      PQGRC(4,1) = Q1(4)
 
      DO I=1,6
         IF (PF4(4,IPX(I)).LT.AMASS1(IPX(I))) IREJ = 3
      ENDDO
 
      AJACOB = FACT/(2*PI)**2/(32*PI2)**3
     $     *BETA(GVAL(4)/GPRI(3),AMASS2(IPX(6))/GPRI(3))
     $     *BETA(GVAL(3)/GVAL(4),AMASS2(IPX(5))/GVAL(4))
     $     *BETA(AMASS2(IPX(3))/GVAL(3),AMASS2(IPX(4))/GVAL(3))
 
      RETURN
      END
 
C------------------------------------
C Set 4-body fusion type kinematics.
C------------------------------------
      SUBROUTINE GRCFUSN4(PF4,AJACOB)
      IMPLICIT REAL*8(A-H,O-Z)
 
      INCLUDE 'inclk.h'
      INCLUDE 'grchad.inc'
 
      INTEGER I
      REAL*8 FACT
      REAL*8 E1,E2,E5,E6,P1,P2,P5,P6,PT5,PT6,EQ,PQ,PTQ,Z1,Z2
      REAL*8 E3Q34,E4Q34,P3Q34,COS5,COS6,SIN5,SIN6,PHI5,PHI6
      REAL*8 COSQ,SINQ,COSQ34,SINQ34,PHIQ34
      REAL*8 PF4(4,8),AJACOB
      REAL*8 PQ1(4)
 
C...beta function
      BETA(Z1,Z2)=SQRT(1.0D0-2.0D0*(Z1+Z2)+(Z1-Z2)**2)
 
      FACT = 0.38937966E9/(2.0D0*GPRI(3))

      AJACOB = 0.0D0 
      IF (NNF.NE.6) THEN
         WRITE(*,*) 'Error: wrong Nr. of partcle'
      ENDIF
 
C...Set Energy and Momentum
      E1 = (GPRI(3)+AMASS2(IPX(1))-AMASS2(IPX(2)))/2.D0/GPRI(4)
      P1 = SQRT((E1-AMASS1(IPX(1)))*(E1+AMASS1(IPX(1))))
      E2 = (GPRI(3)+AMASS2(IPX(2))-AMASS2(IPX(1)))/2.D0/GPRI(4)
      P2 = SQRT((E2-AMASS1(IPX(2)))*(E2+AMASS1(IPX(2))))
 
C...Particle 1
      PF4(1,IPX(1)) = 0
      PF4(2,IPX(1)) = 0
      PF4(3,IPX(1)) = P1
      PF4(4,IPX(1)) = E1
 
C...Particle 2
      PF4(1,IPX(2)) = 0
      PF4(2,IPX(2)) = 0
      PF4(3,IPX(2)) = -P2
      PF4(4,IPX(2)) = E2
 
C...Fermion Propagator
C...Particle ipx(5)
      E5 = (GVAL(1) - GVAL(2))/2.0D0
      P5 = SQRT(E5**2 - AMASS2(IPX(5)))
 
      COS5 = GVAL(3)
      SIN5 = SQRT((1.0D0 - COS5)*(1.0D0 + COS5))
      PT5 = P5*SIN5
 
      PHI5 = GVAL(5)
 
      PF4(1,IPX(5)) = P5*SIN5*COS(PHI5)
      PF4(2,IPX(5)) = P5*SIN5*SIN(PHI5)
      PF4(3,IPX(5)) = P5*COS5
      PF4(4,IPX(5)) = E5
 
C...Particle ipx(6)
      E6 = (GVAL(1) + GVAL(2))/2.0D0
      P6 = SQRT(E6**2 - AMASS2(IPX(6)))
 
      COS6 = GVAL(4)
      SIN6 = SQRT((1.0D0 - COS6)*(1.0D0 + COS6))
      PT6 = P6*SIN6
 
C...Particle 3-4 System
      EQ = GPRI(4) - GVAL(1)
      PQ = SQRT(EQ**2 - GVAL(6))
 
      COSQ = (P6*COS6 + P5*COS5)/PQ
      SINQ = SQRT((1.0D0 - COSQ)*(1.0D0 + COSQ))
      PTQ = PQ*SINQ
 
      PHI6 = PHI5 + ACOS((PTQ**2-PT5**2-PT6**2)/2.0D0/PT5/PT6)
 
      PF4(1,IPX(6)) = P6*SIN6*COS(PHI6)
      PF4(2,IPX(6)) = P6*SIN6*SIN(PHI6)
      PF4(3,IPX(6)) = P6*COS6
      PF4(4,IPX(6)) = E6
 
      PQ1(1) = -PF4(1,IPX(5)) - PF4(1,IPX(6))
      PQ1(2) = -PF4(2,IPX(5)) - PF4(2,IPX(6))
      PQ1(3) = -PF4(3,IPX(5)) - PF4(3,IPX(6))
      PQ1(4) = EQ

      PQGRC(1,1) = PQ1(1)
      PQGRC(2,1) = PQ1(2)
      PQGRC(3,1) = PQ1(3)
      PQGRC(4,1) = PQ1(4)
 
C...Particle 3,4 in Q34 CM frame
      E3Q34 = (GVAL(6)+AMASS2(IPX(3))-AMASS2(IPX(4)))
     $     /2.0D0/SQRT(GVAL(6))
      P3Q34 = SQRT((E3Q34-AMASS1(IPX(3)))*(E3Q34+AMASS1(IPX(3))))
      COSQ34 = GVAL(7)
      SINQ34 = SQRT((1.0D0-COSQ34)*(1.0D0+COSQ34))
      PHIQ34 = GVAL(8)
 
      PF4(1,IPX(3)) = P3Q34*SINQ34*COS(PHIQ34)
      PF4(2,IPX(3)) = P3Q34*SINQ34*SIN(PHIQ34)
      PF4(3,IPX(3)) = P3Q34*COSQ34
      PF4(4,IPX(3)) = E3Q34
 
      E4Q34 = (GVAL(6)+AMASS2(IPX(4))-AMASS2(IPX(3)))
     $     /2.0D0/SQRT(GVAL(6))
      PF4(1,IPX(4)) = -PF4(1,IPX(3))
      PF4(2,IPX(4)) = -PF4(2,IPX(3))
      PF4(3,IPX(4)) = -PF4(3,IPX(3))
      PF4(4,IPX(4)) = E4Q34
 
C...boost from q34 CM frame to 1-2 CM frame (Lab-frame)
      CALL WTOLAB(PF4(1,IPX(3)),PF4(1,IPX(4)),PQ1,
     $     PF4(1,IPX(3)),PF4(1,IPX(4)))
 
      DO I=1,NNF
         IF (PF4(4,IPX(I)).LT.AMASS1(IPX(I))) IREJ = 3
      ENDDO
 
      AJACOB = FACT/8.0D0/(2.0D0*PI)**8/SIN5/SIN6/8.0D0
     $     /ABS(SIN(ACOS((PTQ**2-PT5**2-PT6**2)/2.0D0/PT5/PT6)))
     $     *BETA(AMASS2(IPX(3))/GVAL(6),AMASS2(IPX(4))/GVAL(6))
 
      RETURN
      END

C------------------------------------------
C Set 5-body cluster mass type kinematics.
C------------------------------------------
      SUBROUTINE GRCFKIN5(PF4,AJACOB)
      IMPLICIT REAL*8(A-H,O-Z)
 
      INCLUDE 'inclk.h'
      INCLUDE 'grchad.inc'

      INTEGER I
      REAL*8 FACT
      REAL*8 E1,E2,E3,E4,E5,E6,E7,P1,P2,P3,P5,P7,Z1,Z2
      REAL*8 COS7,SIN7,PHI7,COS34,SIN34,PHI34,COS56,SIN56,PHI56
      REAL*8 EQ34,PQ34,EQ56,PQ56
      REAL*8 COSQ34,SINQ34,COSQ56,SINQ56,PHIQ56
      REAL*8 PT34,PT56,PT7,PHIX
      REAL*8 PF4(4,8),AJACOB
      REAL*8 PQ1(4),PQ2(4)
 
C...beta function
      BETA(Z1,Z2)=SQRT(1.0D0-2.0D0*(Z1+Z2)+(Z1-Z2)**2)
 
      FACT = 0.38937966E9/(2.0D0*GPRI(3))

      AJACOB = 0.0D0 
      IF (NNF.NE.7) THEN
         WRITE(*,*) 'Error: wrong Nr. of partcle'
      ENDIF
 
C...Set Energy and Momentum
      E1 = (GPRI(3)+AMASS2(IPX(1))-AMASS2(IPX(2)))/2.D0/GPRI(4)
      P1 = SQRT((E1-AMASS1(IPX(1)))*(E1+AMASS1(IPX(1))))
      E2 = (GPRI(3)+AMASS2(IPX(2))-AMASS2(IPX(1)))/2.D0/GPRI(4)
      P2 = SQRT((E2-AMASS1(IPX(2)))*(E2+AMASS1(IPX(2))))
 
C...Particle 1
      PF4(1,IPX(1)) = 0
      PF4(2,IPX(1)) = 0
      PF4(3,IPX(1)) = P1
      PF4(4,IPX(1)) = E1
 
C...Particle 2
      PF4(1,IPX(2)) = 0
      PF4(2,IPX(2)) = 0
      PF4(3,IPX(2)) = -P2
      PF4(4,IPX(2)) = E2

C...Q(56) system
      PQ56 = (GVAL(1)+GVAL(2))/2.0D0
      EQ56 = SQRT(PQ56**2+GVAL(4))
C...Q(34) system
      PQ34 = (GVAL(1)-GVAL(2))/2.0D0
      EQ34 = SQRT(PQ34**2+GVAL(3))
C...Particle 7
      E7 = GPRI(4) - EQ56 - EQ34
      P7 = SQRT((E7+AMASS1(IPX(7)))*(E7-AMASS1(IPX(7))))
      COS7 = GVAL(5)
      SIN7 = SQRT((1.0D0 + COS7)*(1.0D0 - COS7))
      PHI7 = GVAL(7)

      PF4(1,IPX(7)) = P7*SIN7*COS(PHI7)
      PF4(2,IPX(7)) = P7*SIN7*SIN(PHI7)
      PF4(3,IPX(7)) = P7*COS7
      PF4(4,IPX(7)) = E7

C...Q(56) system
      COSQ56 = GVAL(6)
      SINQ56 = SQRT((1.0D0 + COSQ56)*(1.0D0 - COSQ56))

C...Q(34) system
      COSQ34 = -(PQ56*COSQ56 + P7*COS7)/PQ34

      IF (ABS(COSQ34).GT.1.0D0) THEN
         IREJ = 3
         GO TO 999
      ENDIF
      SINQ34 = SQRT((1.0D0 + COSQ34)*(1.0D0 - COSQ34))

      PT7 = P7*SIN7
      PT34 = PQ34*SINQ34
      PT56 = PQ56*SINQ56
      PHIX = (PT34**2-PT7**2-PT56**2)/(2.0D0*PT7*PT56)

      IF (ABS(PHIX).GT.1.0D0) THEN
         IREJ = 3
         GO TO 999
      ENDIF
      PHIQ56 = PHI7 + ACOS(PHIX)

C...Store Q(56) and Q(34) system
      PQ2(1) = PQ56*SINQ56*COS(PHIQ56)
      PQ2(2) = PQ56*SINQ56*SIN(PHIQ56)
      PQ2(3) = PQ56*COSQ56
      PQ2(4) = EQ56

      PQ1(1) = -(PF4(1,IPX(7))+PQ2(1))
      PQ1(2) = -(PF4(2,IPX(7))+PQ2(2))
      PQ1(3) = -(PF4(3,IPX(7))+PQ2(3))
      PQ1(4) = EQ34

C...Check energy.
      IF (ABS(SQRT(PQ1(1)**2+PQ1(2)**2)-PT34).GT.0.0001D0) THEN
         IREJ = 3
         GO TO 999
      ENDIF

      PQGRC(1,1) = PQ1(1)
      PQGRC(2,1) = PQ1(2)
      PQGRC(3,1) = PQ1(3)
      PQGRC(4,1) = PQ1(4)

      PQGRC(1,2) = PQ2(1)
      PQGRC(2,2) = PQ2(2)
      PQGRC(3,2) = PQ2(3)
      PQGRC(4,2) = PQ2(4)

C...Particle 3 in Q34 CM frame
      E3 = (GVAL(3)+AMASS2(IPX(3))-AMASS2(IPX(4)))
     $     /2.0D0/SQRT(GVAL(3))
      P3 = SQRT((E3 - AMASS1(IPX(3)))*(E3 + AMASS1(IPX(3))))
      COS34 = GVAL(8)
      SIN34 = SQRT((1.0D0 - COS34)*(1.0D0 + COS34))
      PHI34 = GVAL(9)
 
      PF4(1,IPX(3)) = P3*SIN34*COS(PHI34)
      PF4(2,IPX(3)) = P3*SIN34*SIN(PHI34)
      PF4(3,IPX(3)) = P3*COS34
      PF4(4,IPX(3)) = E3

C...Particle 4 in Q34 CM frame
      E4 = (GVAL(3)+AMASS2(IPX(4))-AMASS2(IPX(3)))
     $     /2.0D0/SQRT(GVAL(3))
      PF4(1,IPX(4)) = -PF4(1,IPX(3))
      PF4(2,IPX(4)) = -PF4(2,IPX(3))
      PF4(3,IPX(4)) = -PF4(3,IPX(3))
      PF4(4,IPX(4)) = E4

C...boost from Q34 CM frame to 1-2 CM frame (Lab-frame)
      CALL WTOLAB(PF4(1,IPX(3)),PF4(1,IPX(4)),PQ1,
     $     PF4(1,IPX(3)),PF4(1,IPX(4)))

C...Particle 5 in Q56 CM frame
      E5 = (GVAL(4)+AMASS2(IPX(5))-AMASS2(IPX(6)))
     $     /2.0D0/SQRT(GVAL(4))
      P5 = SQRT((E5 - AMASS1(IPX(5)))*(E5 + AMASS1(IPX(5))))
      COS56 = GVAL(10)
      SIN56 = SQRT((1.0D0 - COS56)*(1.0D0 + COS56))
      PHI56 = GVAL(11)

      PF4(1,IPX(5)) = P5*SIN56*COS(PHI56)
      PF4(2,IPX(5)) = P5*SIN56*SIN(PHI56)
      PF4(3,IPX(5)) = P5*COS56
      PF4(4,IPX(5)) = E5

C...Particle 6 in Q56 CM frame
      E6 = (GVAL(4)+AMASS2(IPX(6))-AMASS2(IPX(5)))
     $     /2.0D0/SQRT(GVAL(4))
      PF4(1,IPX(6)) = -PF4(1,IPX(5))
      PF4(2,IPX(6)) = -PF4(2,IPX(5))
      PF4(3,IPX(6)) = -PF4(3,IPX(5))
      PF4(4,IPX(6)) = E6
 
C...boost from Q56 CM frame to 1-2 CM frame (Lab-frame)
      CALL WTOLAB(PF4(1,IPX(5)),PF4(1,IPX(6)),PQ2,
     $     PF4(1,IPX(5)),PF4(1,IPX(6)))

      DO I=1,NNF
         IF (PF4(4,IPX(I)).LT.AMASS1(IPX(I))) IREJ = 3
      ENDDO
 
C...Set Jacobian
      AJACOB = FACT/8.0D0/(2.0D0*PI)**11/SINQ34/SINQ56/64.0D0
     $     /ABS(SIN(ACOS((PT7**2-PT34**2-PT56**2)/2.0D0/PT34/PT56)))
     $     *BETA(AMASS2(IPX(3))/GVAL(3),AMASS2(IPX(4))/GVAL(3))
     $     *BETA(AMASS2(IPX(5))/GVAL(4),AMASS2(IPX(6))/GVAL(4))

      RETURN
 999  CONTINUE
      YACOB = 0.0D0
      RETURN
      END

C-------------------------------------
C Set 5-body cascade type kinematics.
C-------------------------------------
      SUBROUTINE GRCFCAS5(PF4,AJACOB)
      IMPLICIT REAL*8(A-H,O-Z)
 
      INCLUDE 'inclk.h'
      INCLUDE 'grchad.inc'
 
      INTEGER I
      REAL*8 FACT
      REAL*8 E1,E2,E3,E4,E5,E6,E7,P3,P5,P7,E34,P34,E56
      REAL*8 COS1,SIN1,PHI1,COS7,SIN7,PHI7
      REAL*8 COS34,SIN34,PHI34,COS56,SIN56,PHI56
      REAL*8 Z1,Z2

      REAL*8 PF4(4,8),PQ567(4),PQ34(4),PQ56(4),AJACOB
 
C...beta function
      BETA(Z1,Z2)=SQRT(1.0D0-2.0D0*(Z1+Z2)+(Z1-Z2)**2)
 
      FACT = 0.38937966E9/(2.0D0*GPRI(3))

      AJACOB = 0.0D0 
      IF (NNF.NE.7) THEN
         WRITE(*,*) 'Error: wrong Nr. of partcle'
         GO TO 999
      ENDIF
 
C...Set Energy and Momentum
      E1 = (GPRI(3)+AMASS2(IPX(1))-AMASS2(IPX(2)))/2.D0/GPRI(4)
      E2 = (GPRI(3)+AMASS2(IPX(2))-AMASS2(IPX(1)))/2.D0/GPRI(4)

      IF (E1.LT.AMASS2(IPX(1))) GO TO 999
      IF (E2.LT.AMASS2(IPX(2))) GO TO 999
 
C...Particle 1
      PF4(1,IPX(1)) = 0
      PF4(2,IPX(1)) = 0
      PF4(3,IPX(1)) = SQRT((E1-AMASS1(IPX(1)))*(E1+AMASS1(IPX(1))))
      PF4(4,IPX(1)) = E1
 
C...Particle 2
      PF4(1,IPX(2)) = 0
      PF4(2,IPX(2)) = 0
      PF4(3,IPX(2)) = -SQRT((E2-AMASS1(IPX(2)))*(E2+AMASS1(IPX(2))))
      PF4(4,IPX(2)) = E2
 
C...Q34
      COS1 = GVAL(1)
      SIN1 = SQRT((1.0D0 - COS1)*(1.0D0 + COS1))
      PHI1 = GVAL(2)

      E34 = (GPRI(3)+GVAL(3)-GVAL(5))/2.0D0/GPRI(4)
      IF (E34.LT.SQRT(GVAL(3))) GO TO 999

      P34 = SQRT(E34**2 - GVAL(3))

      PQ34(1) = P34*SIN1*COS(PHI1)
      PQ34(2) = P34*SIN1*SIN(PHI1)
      PQ34(3) = P34*COS1
      PQ34(4) = E34

      PQGRC(1,1) = PQ34(1)
      PQGRC(2,1) = PQ34(2)
      PQGRC(3,1) = PQ34(3)
      PQGRC(4,1) = PQ34(4)

C...Particle 3 in Q34 CM frame
      E3 = (GVAL(3)+AMASS2(IPX(3))-AMASS2(IPX(4)))
     $     /2.0D0/SQRT(GVAL(3))
      IF (E3.LT.AMASS1(IPX(3))) GO TO 999

      P3 = SQRT((E3 - AMASS1(IPX(3)))*(E3 + AMASS1(IPX(3))))
      COS34 = GVAL(8)
      SIN34 = SQRT((1.0D0 - COS34)*(1.0D0 + COS34))
      PHI34 = GVAL(9)
 
      PF4(1,IPX(3)) = P3*SIN34*COS(PHI34)
      PF4(2,IPX(3)) = P3*SIN34*SIN(PHI34)
      PF4(3,IPX(3)) = P3*COS34
      PF4(4,IPX(3)) = E3

C...Particle 4 in Q34 CM frame
      E4 = (GVAL(3)+AMASS2(IPX(4))-AMASS2(IPX(3)))
     $     /2.0D0/SQRT(GVAL(3))
      PF4(1,IPX(4)) = -PF4(1,IPX(3))
      PF4(2,IPX(4)) = -PF4(2,IPX(3))
      PF4(3,IPX(4)) = -PF4(3,IPX(3))
      PF4(4,IPX(4)) = E4

C...boost from Q34 CM frame to 1-2 CM frame (Lab-frame)
      CALL WTOLAB(PF4(1,IPX(3)),PF4(1,IPX(4)),PQ34,
     $     PF4(1,IPX(3)),PF4(1,IPX(4)))

C...Particle system (567) in Q567 CM frame
      PQ567(1) = -PQ34(1)
      PQ567(2) = -PQ34(2)
      PQ567(3) = -PQ34(3)
      PQ567(4) = (GPRI(3)+GVAL(5)-GVAL(3))/2.0D0/GPRI(4)

      PQGRC(1,3) = PQ567(1)
      PQGRC(2,3) = PQ567(2)
      PQGRC(3,3) = PQ567(3)
      PQGRC(4,3) = PQ567(4)

C...Particle 7 in Q567 CM frame
      E7 = (GVAL(5)+AMASS2(IPX(7))-GVAL(4))/2.0D0/SQRT(GVAL(5))
      IF (E7.LT.AMASS1(IPX(7))) GO TO 999

      P7 = SQRT((E7 - AMASS1(IPX(7)))*(E7 + AMASS1(IPX(7))))
      COS7 = GVAL(6)
      SIN7 = SQRT((1.0D0 - COS7)*(1.0D0 + COS7))
      PHI7 = GVAL(7)

      PF4(1,IPX(7)) = P7*SIN7*COS(PHI7)
      PF4(2,IPX(7)) = P7*SIN7*SIN(PHI7)
      PF4(3,IPX(7)) = P7*COS7
      PF4(4,IPX(7)) = E7

C...Particle system (56) in Q567 CM frame
      E56 = (GVAL(5)+GVAL(4)-AMASS2(IPX(7)))/2.0D0/SQRT(GVAL(5))

      PQ56(1) = -PF4(1,IPX(7))
      PQ56(2) = -PF4(2,IPX(7))
      PQ56(3) = -PF4(3,IPX(7))
      PQ56(4) = E56

      PQGRC(1,2) = PQ56(1)
      PQGRC(2,2) = PQ56(2)
      PQGRC(3,2) = PQ56(3)
      PQGRC(4,2) = PQ56(4)

C...Particle 5 in Q56 CM frame
      E5 = (GVAL(4)+AMASS2(IPX(5))-AMASS2(IPX(6)))
     $     /2.0D0/SQRT(GVAL(4))
      IF (E5.LT.AMASS1(IPX(5))) GO TO 999

      P5 = SQRT((E5 - AMASS1(IPX(5)))*(E5 + AMASS1(IPX(5))))
      COS56 = GVAL(10)
      SIN56 = SQRT((1.0D0 - COS56)*(1.0D0 + COS56))
      PHI56 = GVAL(11)
 
      PF4(1,IPX(5)) = P5*SIN56*COS(PHI56)
      PF4(2,IPX(5)) = P5*SIN56*SIN(PHI56)
      PF4(3,IPX(5)) = P5*COS56
      PF4(4,IPX(5)) = E5

C...Particle 4 in Q34 CM frame
      E6 = (GVAL(4)+AMASS2(IPX(6))-AMASS2(IPX(5)))
     $     /2.0D0/SQRT(GVAL(4))
      PF4(1,IPX(6)) = -PF4(1,IPX(5))
      PF4(2,IPX(6)) = -PF4(2,IPX(5))
      PF4(3,IPX(6)) = -PF4(3,IPX(5))
      PF4(4,IPX(6)) = E6

C...boost from Q56 CM frame to Q567 CM frame (Lab-frame)
      CALL WTOLAB(PF4(1,IPX(5)),PF4(1,IPX(6)),PQ56,
     $     PF4(1,IPX(5)),PF4(1,IPX(6)))

C...boost from Q567 CM frame to 1-2 CM frame (Lab-frame)
      CALL WTOLAB(PF4(1,IPX(7)),PQGRC(1,2),PQ567,
     $     PF4(1,IPX(7)),PQGRC(1,2))
      CALL WTOLAB(PF4(1,IPX(5)),PF4(1,IPX(6)),PQ567,
     $     PF4(1,IPX(5)),PF4(1,IPX(6)))

      DO I = 1,NNF
         IF (PF4(4,IPX(I)).LT.AMASS1(IPX(I))) GO TO 999
      ENDDO

C...Set Jacobian
      AJACOB = FACT/(2.0D0*PI)**11/8.0D0**4
     $     *BETA(GVAL(3)/GPRI(3),GVAL(5)/GPRI(3))
     $     *BETA(AMASS2(IPX(3))/GVAL(3),AMASS2(IPX(4))/GVAL(3))
     $     *BETA(AMASS2(IPX(7))/GVAL(5),GVAL(4)/GVAL(5))
     $     *BETA(AMASS2(IPX(5))/GVAL(4),AMASS2(IPX(6))/GVAL(4))

      RETURN
 999  CONTINUE
      IREJ = 3
      YACOB = 0.0D0
      END

C------------------------------------
C Set 5-body fusion type kinematics.
C------------------------------------
      SUBROUTINE GRCFUSN5(PF4,AJACOB)
      IMPLICIT REAL*8(A-H,O-Z)
 
      INCLUDE 'inclk.h'
      INCLUDE 'grchad.inc'
 
      INTEGER I
      REAL*8 FACT
      REAL*8 E1,E2,E3,E4,E5,E6,E7,P1,P2,P3,P4,P6,Z1,Z2
      REAL*8 EQ1,EQ2,PT3,PQ1P,PQ2P,PT1Q,PT2Q
      REAL*8 COS3,SIN3,COSQ1,SINQ1,COSQ2,SINQ2,PHIQ1,PHIQ2
      REAL*8 COS45,SIN45,PHI45,COS67,SIN67,PHI67
      REAL*8 PF4(4,8),AJACOB
      REAL*8 PQ1(4),PQ2(4)
 
C...beta function
      BETA(Z1,Z2)=SQRT(1.0D0-2.0D0*(Z1+Z2)+(Z1-Z2)**2)
 
      FACT = 0.38937966E9/(2.0D0*GPRI(3))

      AJACOB = 0.0D0 
      IF (NNF.NE.7) THEN
         WRITE(*,*) 'Error: wrong Nr. of partcle'
      ENDIF
 
C...Set Energy and Momentum
      E1 = (GPRI(3)+AMASS2(IPX(1))-AMASS2(IPX(2)))/2.D0/GPRI(4)
      P1 = SQRT((E1-AMASS1(IPX(1)))*(E1+AMASS1(IPX(1))))
      E2 = (GPRI(3)+AMASS2(IPX(2))-AMASS2(IPX(1)))/2.D0/GPRI(4)
      P2 = SQRT((E2-AMASS1(IPX(2)))*(E2+AMASS1(IPX(2))))
 
C...Particle 1
      PF4(1,IPX(1)) = 0
      PF4(2,IPX(1)) = 0
      PF4(3,IPX(1)) = P1
      PF4(4,IPX(1)) = E1
 
C...Particle 2
      PF4(1,IPX(2)) = 0
      PF4(2,IPX(2)) = 0
      PF4(3,IPX(2)) = -P2
      PF4(4,IPX(2)) = E2
 
      EQ1 = (GVAL(1) - GVAL(2))/2.0D0
      PQ1P = SQRT(EQ1**2 - GVAL(3))
      COSQ1 = GVAL(5)
      SINQ1 = SQRT((1.0D0 + COSQ1)*(1.0D0 - COSQ1))
      PHIQ1 = GVAL(7)
 
C...Fermion Propagator 45
      PQ1(1) = PQ1P*SINQ1*COS(PHIQ1)
      PQ1(2) = PQ1P*SINQ1*SIN(PHIQ1)
      PQ1(3) = PQ1P*COSQ1
      PQ1(4) = EQ1

      PQGRC(1,1) = PQ1(1)
      PQGRC(2,1) = PQ1(2)
      PQGRC(3,1) = PQ1(3)
      PQGRC(4,1) = PQ1(4)
 
      EQ2  = (GVAL(1) + GVAL(2))/2.0D0
      PQ2P = SQRT(EQ2**2 - GVAL(4))
      COSQ2 = GVAL(6)
      SINQ2 = SQRT((1.0D0 + COSQ2)*(1.0D0 - COSQ2))
 
      E3 = GPRI(4) - GVAL(1)
      P3 = SQRT((E3 + AMASS1(IPX(3)))*(E3 - AMASS1(IPX(3))))
      COS3 = -(PQ1P*COSQ1 + PQ2P*COSQ2)/P3
      SIN3 = SQRT((1.0D0 + COS3)*(1.0D0 - COS3))
 
      PT1Q = PQ1P*SINQ1
      PT2Q = PQ2P*SINQ2
      PT3  = P3*SIN3
 
      PHIQ2 = PHIQ1 + ACOS((PT3**2-PT1Q**2-PT2Q**2)/2.0D0/PT1Q/PT2Q)
 
C...Fermion Propagator 67
      PQ2(1) = PQ2P*SINQ2*COS(PHIQ2)
      PQ2(2) = PQ2P*SINQ2*SIN(PHIQ2)
      PQ2(3) = PQ2P*COSQ2
      PQ2(4) = EQ2

      PQGRC(1,2) = PQ2(1)
      PQGRC(2,2) = PQ2(2)
      PQGRC(3,2) = PQ2(3)
      PQGRC(4,2) = PQ2(4)
 
C...Particle ipx(4) in Q45 CM frame
      E4 = (GVAL(3)+AMASS2(IPX(4))-AMASS2(IPX(5)))
     $     /2.0D0/SQRT(GVAL(3))
      P4 = SQRT((E4 - AMASS1(IPX(4)))*(E4 + AMASS1(IPX(4))))
      COS45 = GVAL(8)
      SIN45 = SQRT((1.0D0 - COS45)*(1.0D0 + COS45))
      PHI45 = GVAL(9)
 
      PF4(1,IPX(4)) = P4*SIN45*COS(PHI45)
      PF4(2,IPX(4)) = P4*SIN45*SIN(PHI45)
      PF4(3,IPX(4)) = P4*COS45
      PF4(4,IPX(4)) = E4
 
C...Particle ipx(5) in Q45 CM frame
      E5 = (GVAL(3)+AMASS2(IPX(5))-AMASS2(IPX(4)))
     $     /2.0D0/SQRT(GVAL(3))
      PF4(1,IPX(5)) = -PF4(1,IPX(4))
      PF4(2,IPX(5)) = -PF4(2,IPX(4))
      PF4(3,IPX(5)) = -PF4(3,IPX(4))
      PF4(4,IPX(5)) = E5
 
C...boost from Q45 CM frame to 1-2 CM frame (Lab-frame)
      CALL WTOLAB(PF4(1,IPX(4)),PF4(1,IPX(5)),PQ1,
     $     PF4(1,IPX(4)),PF4(1,IPX(5)))
 
C...Particle ipx(6) in Q67 CM frame
      E6 = (GVAL(4)+AMASS2(IPX(6))-AMASS2(IPX(7)))
     $     /2.0D0/SQRT(GVAL(4))
      P6 = SQRT((E6 - AMASS1(IPX(6)))*(E6 + AMASS1(IPX(6))))
      COS67 = GVAL(10)
      SIN67 = SQRT((1.0D0 - COS67)*(1.0D0 + COS67))
      PHI67 = GVAL(11)
 
      PF4(1,IPX(6)) = P6*SIN67*COS(PHI67)
      PF4(2,IPX(6)) = P6*SIN67*SIN(PHI67)
      PF4(3,IPX(6)) = P6*COS67
      PF4(4,IPX(6)) = E6
 
C...Particle ipx(7) in Q67 CM frame
      E7 = (GVAL(4)+AMASS2(IPX(7))-AMASS2(IPX(6)))
     $     /2.0D0/SQRT(GVAL(4))
      PF4(1,IPX(7)) = -PF4(1,IPX(6))
      PF4(2,IPX(7)) = -PF4(2,IPX(6))
      PF4(3,IPX(7)) = -PF4(3,IPX(6))
      PF4(4,IPX(7)) = E7
 
C...boost from Q67 CM frame to 1-2 CM frame (Lab-frame)
      CALL WTOLAB(PF4(1,IPX(6)),PF4(1,IPX(7)),PQ2,
     $     PF4(1,IPX(6)),PF4(1,IPX(7)))
 
C...Particle ipx(3)
      PF4(1,IPX(3)) = -PQ1(1) - PQ2(1)
      PF4(2,IPX(3)) = -PQ1(2) - PQ2(2)
      PF4(3,IPX(3)) = -PQ1(3) - PQ2(3)
      PF4(4,IPX(3)) = E3
 
      DO I=1,NNF
         IF (PF4(4,IPX(I)).LT.AMASS1(IPX(I))) IREJ = 3
      ENDDO
 
C...Set Jacobian
      AJACOB = FACT/8.0D0/(2.0D0*PI)**11/SINQ1/SINQ2/64.0D0
     $     /ABS(SIN(ACOS((PT3**2-PT1Q**2-PT2Q**2)/2.0D0/PT1Q/PT2Q)))
     $     *BETA(AMASS2(IPX(4))/GVAL(3),AMASS2(IPX(5))/GVAL(3))
     $     *BETA(AMASS2(IPX(6))/GVAL(4),AMASS2(IPX(7))/GVAL(4))
 
      RETURN
      END

C------------------------
C Set 6-body kinematics.
C------------------------
      SUBROUTINE GRCFKIN6(PF4,AJACOB)
      IMPLICIT REAL*8(A-H,O-Z)
 
      INCLUDE 'inclk.h'
      INCLUDE 'grchad.inc'
 
      INTEGER I
      REAL*8 FACT
      REAL*8 E1,E2,EQT,PQT,COST,SINT,PHIT,Z1,Z2
      REAL*8 EI1Q1,PI1Q1,COSI1Q1,SINI1Q1,PHII1Q1
      REAL*8 EI1Q2,PI1Q2,COSI1Q2,SINI1Q2,PHII1Q2
      REAL*8 EQ45,PQ45,COSQ45,SINQ45,PHIQ45
      REAL*8 EQ78,PQ78,COSQ78,SINQ78,PHIQ78
      REAL*8 PF4(4,8),QT1(4),QT2(4),Q45(4),Q78(4),AJACOB
 
C...beta function
      BETA(Z1,Z2)=SQRT(1.0D0-2.0D0*(Z1+Z2)+(Z1-Z2)**2)
 
      FACT = 0.38937966E9/(2.0D0*GPRI(3))

      AJACOB = 0.0D0 
      IF (NNF.NE.8) THEN
         WRITE(*,*) 'Error: wrong Nr. of partcle'
      ENDIF
 
C...Set Energy and Momentum
      E1 = (GPRI(3)+AMASS2(IPX(1))-AMASS2(IPX(2)))/2.D0/GPRI(4)
      E2 = (GPRI(3)+AMASS2(IPX(2))-AMASS2(IPX(1)))/2.D0/GPRI(4)
 
C...Particle 1
      PF4(1,IPX(1)) = 0
      PF4(2,IPX(1)) = 0
      PF4(3,IPX(1)) = SQRT((E1-AMASS1(IPX(1)))*(E1+AMASS1(IPX(1))))
      PF4(4,IPX(1)) = E1
 
C...Particle 2
      PF4(1,IPX(2)) = 0
      PF4(2,IPX(2)) = 0
      PF4(3,IPX(2)) = -SQRT((E2-AMASS1(IPX(2)))*(E2+AMASS1(IPX(2))))
      PF4(4,IPX(2)) = E2

C...top pair
      COST = GVAL(1)
      SINT = SQRT((1.0D0 - COST)*(1.0D0 + COST))
      PHIT = GVAL(2)
 
      EQT = (GPRI(3)+GVAL(3)-GVAL(4))/2.0D0/GPRI(4)
      PQT = SQRT(EQT**2 - GVAL(3))
 
      QT1(1) = PQT*SINT*COS(PHIT)
      QT1(2) = PQT*SINT*SIN(PHIT)
      QT1(3) = PQT*COST
      QT1(4) = EQT

      QT2(1) = -QT1(1)
      QT2(2) = -QT1(2)
      QT2(3) = -QT1(3)
      QT2(4) = (GPRI(3)+GVAL(4)-GVAL(3))/2.0D0/GPRI(4)

      PQGRC(1,1) = QT1(1)
      PQGRC(2,1) = QT1(2)
      PQGRC(3,1) = QT1(3)
      PQGRC(4,1) = QT1(4)

      PQGRC(1,2) = QT2(1)
      PQGRC(2,2) = QT2(2)
      PQGRC(3,2) = QT2(3)
      PQGRC(4,2) = QT2(4)

C...q45
      EQ45 = (GVAL(3)+GVAL(9)-AMASS2(IPX(3)))/2.D0/SQRT(GVAL(3))
      IF (EQ45.lt.SQRT(GVAL(9))) GO TO 999
      PQ45 = SQRT(EQ45**2 - GVAL(9))
      COSQ45 = GVAL(5)
      SINQ45 = SQRT((1.0D0-COSQ45)*(1.0D0+COSQ45))
      PHIQ45 = GVAL(6)

      Q45(1) = PQ45*SINQ45*COS(PHIQ45)
      Q45(2) = PQ45*SINQ45*SIN(PHIQ45)
      Q45(3) = PQ45*COSQ45
      Q45(4) = EQ45

C...Particle ipx(3) in q45 CM frame
      PF4(1,IPX(3)) = -Q45(1)
      PF4(2,IPX(3)) = -Q45(2)
      PF4(3,IPX(3)) = -Q45(3)
      PF4(4,IPX(3)) = (GVAL(3)+AMASS2(IPX(3))-GVAL(9))
     $     /2.D0/SQRT(GVAL(3))
 
C...Particle ipx(4) in q45 CM frame
      EI1Q1 = (GVAL(9)+AMASS2(IPX(4))-AMASS2(IPX(5)))
     $     /2.0D0/SQRT(GVAL(9))
      PI1Q1 = SQRT((EI1Q1-AMASS1(IPX(4)))*(EI1Q1+AMASS1(IPX(4))))
      COSI1Q1 = GVAL(11)
      SINI1Q1 = SQRT((1.0D0-COSI1Q1)*(1.0D0+COSI1Q1))
      PHII1Q1 = GVAL(12)
 
      PF4(1,IPX(4)) = PI1Q1*SINI1Q1*COS(PHII1Q1)
      PF4(2,IPX(4)) = PI1Q1*SINI1Q1*SIN(PHII1Q1)
      PF4(3,IPX(4)) = PI1Q1*COSI1Q1
      PF4(4,IPX(4)) = EI1Q1
 
C...Particle ipx(5) in q1 CM frame
      PF4(1,IPX(5)) = -PF4(1,IPX(4))
      PF4(2,IPX(5)) = -PF4(2,IPX(4))
      PF4(3,IPX(5)) = -PF4(3,IPX(4))
      PF4(4,IPX(5)) = (GVAL(9)+AMASS2(IPX(5))-AMASS2(IPX(4)))
     $     /2.0D0/SQRT(GVAL(9))

C...boost from q45 CM frame to q3(45) CM frame
      CALL WTOLAB(PF4(1,IPX(4)),PF4(1,IPX(5)),Q45,
     $     PF4(1,IPX(4)),PF4(1,IPX(5)))
 
C...boost from q3(45) CM frame to LAB frame
      CALL WTOLAB(PF4(1,IPX(4)),PF4(1,IPX(5)),QT1,
     $     PF4(1,IPX(4)),PF4(1,IPX(5)))
 
      CALL WTOLAB(PF4(1,IPX(3)),Q45(1),QT1,PF4(1,IPX(3)),Q45(1))

      PQGRC(1,3) = Q45(1)
      PQGRC(2,3) = Q45(2)
      PQGRC(3,3) = Q45(3)
      PQGRC(4,3) = Q45(4)

C...q78
      EQ78 = (GVAL(4)+GVAL(10)-AMASS2(IPX(6)))/2.D0/SQRT(GVAL(4))
      IF (EQ78.lt.SQRT(GVAL(10))) GO TO 999
      PQ78 = SQRT(EQ78**2 - GVAL(10))
      COSQ78 = GVAL(7)
      SINQ78 = SQRT((1.0D0-COSQ78)*(1.0D0+COSQ78))
      PHIQ78 = GVAL(8)

      Q78(1) = PQ78*SINQ78*COS(PHIQ78)
      Q78(2) = PQ78*SINQ78*SIN(PHIQ78)
      Q78(3) = PQ78*COSQ78
      Q78(4) = EQ78

C...Particle ipx(6) in q45 CM frame
      PF4(1,IPX(6)) = -Q78(1)
      PF4(2,IPX(6)) = -Q78(2)
      PF4(3,IPX(6)) = -Q78(3)
      PF4(4,IPX(6)) = (GVAL(4)+AMASS2(IPX(6))-GVAL(10))
     $     /2.D0/SQRT(GVAL(4))

C...Particle ipx(7) in q78 CM frame
      EI1Q2 = (GVAL(10)+AMASS2(IPX(7))-AMASS2(IPX(8)))
     $     /2.0D0/SQRT(GVAL(10))
      PI1Q2 = SQRT((EI1Q2-AMASS1(IPX(7)))*(EI1Q2+AMASS1(IPX(7))))
      COSI1Q2 = GVAL(13)
      SINI1Q2 = SQRT((1.0D0-COSI1Q2)*(1.0D0+COSI1Q2))
      PHII1Q2 = GVAL(14)
 
      PF4(1,IPX(7)) = PI1Q2*SINI1Q2*COS(PHII1Q2)
      PF4(2,IPX(7)) = PI1Q2*SINI1Q2*SIN(PHII1Q2)
      PF4(3,IPX(7)) = PI1Q2*COSI1Q2
      PF4(4,IPX(7)) = EI1Q2
 
C...Particle ipx(8) in q78 CM frame
      PF4(1,IPX(8)) = -PF4(1,IPX(7))
      PF4(2,IPX(8)) = -PF4(2,IPX(7))
      PF4(3,IPX(8)) = -PF4(3,IPX(7))
      PF4(4,IPX(8)) = (GVAL(10)+AMASS2(IPX(8))-AMASS2(IPX(7)))
     $     /2.0D0/SQRT(GVAL(10))

C...boost from q78 CM frame to q6(78) CM frame
      CALL WTOLAB(PF4(1,IPX(7)),PF4(1,IPX(8)),Q78,
     $     PF4(1,IPX(7)),PF4(1,IPX(8)))

C...boost from q6(78) CM frame to LAB frame
      CALL WTOLAB(PF4(1,IPX(7)),PF4(1,IPX(8)),QT2,
     $     PF4(1,IPX(7)),PF4(1,IPX(8)))

      CALL WTOLAB(PF4(1,IPX(6)),Q78(1),QT2,PF4(1,IPX(6)),Q78(1))

      PQGRC(1,4) = Q78(1)
      PQGRC(2,4) = Q78(2)
      PQGRC(3,4) = Q78(3)
      PQGRC(4,4) = Q78(4)
 
      DO I=1,8
         IF (PF4(4,IPX(I)).LT.AMASS1(IPX(I))) GO TO 999
      ENDDO
 
      AJACOB = FACT/(2*PI)**4/(32*PI2)**5
     $     *BETA(GVAL(3)/GPRI(3),GVAL(4)/GPRI(3))
     $     *BETA(GVAL(9)/GVAL(3),AMASS2(IPX(3))/GVAL(3))
     $     *BETA(AMASS2(IPX(4))/GVAL(9),AMASS2(IPX(5))/GVAL(9))
     $     *BETA(GVAL(10)/GVAL(4),AMASS2(IPX(6))/GVAL(4))
     $     *BETA(AMASS2(IPX(7))/GVAL(10),AMASS2(IPX(8))/GVAL(10))

      RETURN
 999  CONTINUE
      IREJ = 3
      AJACOB = 0.0D0
      END
 
C---------------------------
C Set particle combination.
C---------------------------
      SUBROUTINE GRCSPID
      IMPLICIT REAL*8(A-H,O-Z)
 
      INCLUDE 'grchad.inc'
 
      INTEGER ICH3(18),ICH4(18),ICH5(18),ICH6(18),ICH7(18),ICH8(18)
      DATA ICH3/3,3,4,5,3,3,3,3,3,3,4,4,4,4,5,5,3,3/
      DATA ICH4/4,4,3,3,4,4,5,5,6,6,5,5,6,6,6,6,4,4/
      DATA ICH5/0,5,5,4,5,6,4,6,4,5,3,6,3,5,3,4,5,5/
      DATA ICH6/0,0,0,0,6,5,6,4,5,4,6,3,5,3,4,3,6,6/
      DATA ICH7/0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,7/
      DATA ICH8/0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8/
 
      NNF = 0
      IPX(1) = 1
      IPX(2) = 2
      IPX(3) = ICH3(CXTYPE)
      IPX(4) = ICH4(CXTYPE)
      IPX(5) = ICH5(CXTYPE)
      IPX(6) = ICH6(CXTYPE)
      IPX(7) = ICH7(CXTYPE)
      IPX(8) = ICH8(CXTYPE)
 
      IF (KTYPE.EQ.1) THEN
         NNF = 4
      ELSE IF (KTYPE.LT.5) THEN
         NNF = 5
      ELSE IF (KTYPE.LT.10) THEN
         NNF = 6
      ELSE IF (KTYPE.LT.13) THEN
         NNF = 7
      ELSE IF (KTYPE.EQ.13) THEN
         NNF = 8
      ELSE
         WRITE(*,*) 'wrong ktype.'
         STOP
      ENDIF
 
      RETURN
      END
 
C-----------------------
C Exchange particle ID.
C-----------------------
      SUBROUTINE GRCEXKN(I1,I2)
      IMPLICIT REAL*8(A-H,O-Z)
 
#include "define.h"
 
#if PSMODEL == 61
      COMMON/PYUPPR/NUP,KUP(20,7),NFUP,IFUP(10,2),PUP(20,5),Q2UP(0:10)
#elif PSMODEL == 62
C...User process event common block.
      INTEGER MAXNUP
      PARAMETER (MAXNUP=500)
      INTEGER NUP,IDPRUP,IDUP,ISTUP,MOTHUP,ICOLUP
      DOUBLE PRECISION XWGTUP,SCALUP,AQEDUP,AQCDUP,PUP,VTIMUP,SPINUP
      COMMON/HEPEUP/NUP,IDPRUP,XWGTUP,SCALUP,AQEDUP,AQCDUP,IDUP(MAXNUP),
     &ISTUP(MAXNUP),MOTHUP(2,MAXNUP),ICOLUP(2,MAXNUP),PUP(5,MAXNUP),
     &VTIMUP(MAXNUP),SPINUP(MAXNUP)
      SAVE /HEPEUP/
#endif
 
      INCLUDE 'inclk.h'
      INCLUDE 'grchad.inc'
 
      INTEGER I,I1,I2
      REAL*8 PPTMP(4),PUTMP(4),TMPMAS
 
      DO I=1,4
         PPTMP(I) = PGRC(I,I1)
         PGRC(I,I1) = PGRC(I,I2)
         PGRC(I,I2) = PPTMP(I)
 
#if PSMODEL == 61
         PUTMP(I) = PUP(I1,I)
         PUP(I1,I) = PUP(I2,I)
         PUP(I2,I) = PUTMP(I)
#elif PSMODEL == 62
         PUTMP(I) = PUP(I,I1)
         PUP(I,I1) = PUP(I,I2)
         PUP(I,I2) = PUTMP(I)
#endif
      ENDDO
 
      TMPMAS = AMASS1(I1)
      AMASS1(I1) = AMASS1(I2)
      AMASS1(I2) = TMPMAS
      AMASS2(I1) = AMASS1(I1)**2
      AMASS2(I2) = AMASS1(I2)**2
 
#if PSMODEL == 61
      TMPMAS = PUP(I1,5)
      PUP(I1,5) = PUP(I2,5)
      PUP(I2,5) = TMPMAS
#elif PSMODEL == 62
      TMPMAS = PUP(5,I1)
      PUP(5,I1) = PUP(5,I2)
      PUP(5,I2) = TMPMAS
#endif
 
      IF (PGRC(4,I1).LT.AMASS1(I1)) IREJ = 3
      IF (PGRC(4,I2).LT.AMASS1(I2)) IREJ = 3
 
      RETURN
      END
 
C------------------------------------
C Set dPhi between Particle A and B.
C------------------------------------
      SUBROUTINE GRCAPHI(PHIA,PHIB,PHIC)
      IMPLICIT REAL*8(A-H,O-Z)
 
      REAL*8 PHIA,PHIB,PHIC
 
      IF (ABS(PHIA - PHIB).GT.ACOS(-1.0)) THEN
         PHIC = 2.0*ACOS(-1.0) - ABS(PHIA - PHIB)
      ELSE
         PHIC = ABS(PHIA - PHIB)
      ENDIF
 
      RETURN
      END
 
C    _/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
C   _/_/  Set momentum fraction from PDF  _/_/
C  _/_/  Date   10/11/2000               _/_/
C _/_/  Author Soushi Tsuno             _/_/
C_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
C
C subroutine grcprtngg
C  _______________________
C | Initial |             |
C |  State  | gluon,gluon |
C +=========+=============+
C | grcflav |      0      |
C +---------+-------------+
C
C subroutine grcprtnqq
C  _____________________________________________________________________
C | Initial |  _ |  _ |  _ |  _ | _  | _  | _  | _  |  _ | _  |  _ | _  |
C |  State  | uu | dd | cc | ss | uu | dd | cc | ss | bb | bb | tt | tt |
C +=========+====+====+====+====+====+====+====+====+====+====+====+====+
C | grcflav |  1 |  2 |  3 |  4 |  5 |  6 |  7 |  8 |  9 | 10 | 11 | 12 |
C +---------+----+----+----+----+----+----+----+----+----+----+----+----+
C            _______________________________________
C           |  _ |  _ |  _ |  _ | _  | _  | _  | _  |
C           | uc | ds | cu | sd | uc | ds | cu | sd |
C           +====+====+====+====+====+====+====+====+
C           | 133| 134| 135| 136| 137| 138| 139| 140|
C           +----+----+----+----+----+----+----+----+
C b-content  _______________________________________
C           |  _ |  _ |  _ |  _ | _  | _  | _  | _  |
C           | db | sb | bd | bs | bd | bs | db | sb |
C           +====+====+====+====+====+====+====+====+
C           | 141| 142| 143| 144| 145| 146| 147| 148|
C           +----+----+----+----+----+----+----+----+
C top-content_______________________________________
C           |  _ |  _ |  _ |  _ | _  | _  | _  | _  |
C           | ut | ct | tu | tc | ut | ct | tu | tc |
C           +====+====+====+====+====+====+====+====+
C           | 149| 150| 151| 152| 153| 154| 155| 156|
C           +----+----+----+----+----+----+----+----+
C
C subroutine grcprtnqqb
C Without quark mixing
C  _________________________________________________
C | Initial |  _ |  _ | _  | _  | _  | _  |  _ |  _ |
C |  State  | ud | cs | du | sc | ud | cs | du | sc |
C +=========+====+====+====+====+====+====+====+====+
C | grcflav | 13 | 14 | 15 | 16 | 17 | 18 | 19 | 20 |
C +---------+----+----+----+----+----+----+----+----+
C With quark mixing
C  _________________________________________________
C | Initial |  _ |  _ | _  | _  | _  | _  |  _ |  _ |
C |  State  | us | cd | su | dc | us | cd | su | dc |
C +=========+====+====+====+====+====+====+====+====+
C | grcflav | 21 | 22 | 23 | 24 | 25 | 26 | 27 | 28 |
C +---------+----+----+----+----+----+----+----+----+
C            _______________________________________
C           |  _ |  _ | _  | _  | _  | _  |  _ |  _ |
C           | ub | cb | bu | bc | ub | cb | bu | bc |
C           +====+====+====+====+====+====+====+====+
C           | 29 | 30 | 31 | 32 | 33 | 34 | 35 | 36 |
C           +----+----+----+----+----+----+----+----+
C
C subroutine grcprtnqg
C  _________________________________________________
C | Initial |    |    |    |    | _  | _  | _  | _  |
C |  State  | ug | dg | cg | sg | ug | dg | cg | sg |
C +=========+====+====+====+====+====+====+====+====+
C | grcflav | 37 | 38 | 39 | 40 | 41 | 42 | 43 | 44 |
C +---------+----+----+----+----+----+----+----+----+
C            _______________________________________
C           |    |    |    |    |  _ |  _ |  _ |  _ |
C           | gu | gd | gc | gs | gu | gd | gc | gs |
C           +====+====+====+====+====+====+====+====+
C           | 45 | 46 | 47 | 48 | 49 | 50 | 51 | 52 |
C           +----+----+----+----+----+----+----+----+
C            _______________________________________
C           |    | _  |    |  _ |    | _  |    |  _ |
C           | bg | bg | gb | gb | tg | tg | gt | gt |
C           +====+====+====+====+====+====+====+====+
C           | 53 | 54 | 55 | 56 | 57 | 58 | 59 | 60 |
C           +----+----+----+----+----+----+----+----+
C
C subroutine grcprtnqqa
C  _____________________________________________________________________
C | Initial |    |    |    |    | __ | __ | __ | __ |    | __ |    | __ |
C |  State  | uu | dd | cc | ss | uu | dd | cc | ss | bb | bb | tt | tt |
C +=========+====+====+====+====+====+====+====+====+====+====+====+====+
C | grcflav | 61 | 62 | 63 | 64 | 65 | 66 | 67 | 68 | 69 | 70 | 71 | 72 |
C +---------+----+----+----+----+----+----+----+----+----+----+----+----+
C            _______________________________________
C           |    |    |    |    | __ | __ | __ | __ |
C           | uc | ds | cu | sd | uc | ds | cu | sd |
C           +====+====+====+====+====+====+====+====+
C           | 73 | 74 | 75 | 76 | 77 | 78 | 79 | 80 |
C           +----+----+----+----+----+----+----+----+
C            _______________________________________
C           |    |    |    |    | __ | __ | __ | __ |
C           | bd | bs | db | sb | bd | bs | db | sb |
C           +====+====+====+====+====+====+====+====+
C           | 81 | 82 | 83 | 84 | 85 | 86 | 87 | 88 |
C           +----+----+----+----+----+----+----+----+
C            _______________________________________
C           |    |    |    |    | __ | __ | __ | __ |
C           | tu | tc | ut | ct | tu | tc | ut | ct |
C           +====+====+====+====+====+====+====+====+
C           | 89 | 90 | 91 | 92 | 93 | 94 | 95 | 96 |
C           +----+----+----+----+----+----+----+----+
C
C subroutine grcprtnqqs
C  _________________________________________________
C | Initial |    |    |    |    |    |    |    |    |
C |  State  | ud | us | cd | cs | du | dc | su | sc |
C +=========+====+====+====+====+====+====+====+====+
C | grcflav | 97 | 98 | 99 | 100| 101| 102| 103| 104|
C +---------+----+----+----+----+----+----+----+----+
C            _______________________________________
C           | __ | __ | __ | __ | __ | __ | __ | __ |
C           | ud | us | cd | cs | du | dc | su | sc |
C           +====+====+====+====+====+====+====+====+
C           | 105| 106| 107| 108| 109| 110| 111| 112|
C           +----+----+----+----+----+----+----+----+
C            _______________________________________
C           |    |    |    |    | __ | __ | __ | __ |
C           | ub | cb | bu | bc | ub | cb | bu | bc |
C           +====+====+====+====+====+====+====+====+
C           | 113| 114| 115| 116| 117| 118| 119| 120|
C           +----+----+----+----+----+----+----+----+
C            ___________________________________________________________
C           |    |    |    |    |    |    | __ | __ | __ | __ | __ | __ |
C           | td | ts | tb | dt | st | bt | td | ts | tb | dt | st | bt |
C           +====+====+====+====+====+====+====+====+====+====+====+====+
C           | 121| 122| 123| 124| 125| 126| 127| 128| 129| 130| 131| 132|
C           +----+----+----+----+----+----+----+----+----+----+----+----+
C
C----------------------
C Initial gluon gluon.
C----------------------
      SUBROUTINE GRCPRTNGG
      IMPLICIT REAL*8(A-H,O-Z)
 
#include "define.h"
 
      INCLUDE 'grchad.inc'
 
#if PSMODEL > 0
C...Parameters.
      COMMON/PYPARS/MSTP(200),PARP(200),MSTI(200),PARI(200)
#endif

C...PYTHIA PDF
      REAL*8 XPP(-25:25),XPPBAR(-25:25)
C...LHA PDF
      REAL*8 FLHA1(-6:6),FLHA2(-6:6)
 
C...Grace Variable
      INTEGER ISEL
      REAL*8 SUM
 
      IF (GPRI(5).GT.1.OR.GPRI(6).GT.1) THEN
         IREJ = 1
         GO TO 999
      ENDIF

#if PDFLIB == 1
C...Internal PDF from PYTHIA
      IF (IGPP.EQ.0) THEN
         CALL PYPDFU( 2212,GPRI(5),GRCFAQ**2,XPP)
         CALL PYPDFU( 2212,GPRI(6),GRCFAQ**2,XPPBAR)
      ELSE IF (IGPP.EQ.1) THEN
         CALL PYPDFU( 2212,GPRI(5),GRCFAQ**2,XPP)
         CALL PYPDFU(-2212,GPRI(6),GRCFAQ**2,XPPBAR)
      ENDIF

C...gluon from p, gluon from pbar
      SUM = XPP(0)*XPPBAR(0)

#elif PDFLIB == 2
C...External PDF
      CALL STRUCTM(GPRI(5),GRCFAQ,
     $     UPV1,DNV1,USEA1,DSEA1,STR1,CHM1,BOT1,TOP1,GL1)
      CALL STRUCTM(GPRI(6),GRCFAQ,
     $     UPV2,DNV2,USEA2,DSEA2,STR2,CHM2,BOT2,TOP2,GL2)
 
C...gluon from p, gluon from pbar
      SUM = GL1*GL2

#elif PDFLIB == 3
C...LHA PDF
      CALL evolvePDF(GPRI(5),GRCFAQ,FLHA1)
      CALL evolvePDF(GPRI(6),GRCFAQ,FLHA2)

C...gluon from p, gluon from pbar
      SUM = FLHA1(0)*FLHA2(0)
#else


#endif

C...Reject very small/large x
      IF (SUM.EQ.0.0D0) THEN
C         write(*,*) 'reject',sum,GPRI(5),GPRI(6),GRCFAQ
         GO TO 999
      ENDIF

      GRCFLAV = 0
      YACOB = SUM*YACOB

C...Set flavor
      CALL GRCFFIL

      RETURN
 999  CONTINUE
      YACOB = 0.0D0
      RETURN
      END
 
C---------------------------
C Initial quark anti-quark.
C---------------------------
      SUBROUTINE GRCPRTNQQ
      IMPLICIT REAL*8(A-H,O-Z)
 
#include "define.h"
 
      INCLUDE 'grchad.inc'
 
#if PSMODEL > 0
C...Parameters.
      COMMON/PYPARS/MSTP(200),PARP(200),MSTI(200),PARI(200)
#endif
 
C...Proton Variable
      INTEGER I
      REAL*8 TERM(36)

C...PYTHIA PDF
      REAL*8 XPP(-25:25),XPPBAR(-25:25)
C...LHA PDF
      REAL*8 FLHA1(-6:6),FLHA2(-6:6)

C...Grace Variable
      INTEGER ISEL,INPA
      REAL*8 SUM,RANSUM
      INTEGER FMAX,FMIN
      REAL*8 RNA,GFLA(18)

      INTEGER IGFLVTMP(36)
      DATA IGFLVTMP/  1,  2,  3,  4,  5,  6,  7,  8,133,134,135,136,
     $              137,138,139,140,  9, 10,141,142,143,144,145,146,
     $              147,148, 11, 12,149,150,151,152,153,154,155,156/

      IF (GPRI(5).GT.1.OR.GPRI(6).GT.1) THEN
         IREJ = 1
         GO TO 999
      ENDIF
 
#if PSMODEL > 0
      IF (INPFL.NE.MSTP(58)) THEN
         WRITE(*,*) 'Mismacth Nr. of flavor in PDF: INPFL/MSTP(58): ',
     $        INPFL,'/',MSTP(58)
         WRITE(*,*) 'processed as INPFL:',INPFL
         MSTP(58) = INPFL
      ENDIF
#endif

      IF (INPFL.GT.4.AND.IGSUB.GT.399.AND.IGSUB.LT.416) THEN
         WRITE(*,*) 'GR@PPA : We do not support b,t contents of PDF'
         WRITE(*,*) '         for the 400,401,...,415.'
         WRITE(*,*) 'Set INPFL <= 4.'
         STOP
      ENDIF

#if PDFLIB == 1
C...Internal PDF from PYTHIA
      IF (IGPP.EQ.0) THEN
         CALL PYPDFU( 2212,GPRI(5),GRCFAQ**2,XPP)
         CALL PYPDFU( 2212,GPRI(6),GRCFAQ**2,XPPBAR)
      ELSE IF (IGPP.EQ.1) THEN
         CALL PYPDFU( 2212,GPRI(5),GRCFAQ**2,XPP)
         CALL PYPDFU(-2212,GPRI(6),GRCFAQ**2,XPPBAR)
      ENDIF
 
C...Loop over quark flavours (up to b).
      INPA = 8
      TERM(1) = XPP(2)*XPPBAR(-2) ! grcflav:  1, u     u-bar
      TERM(2) = XPP(1)*XPPBAR(-1) ! grcflav:  2, d     d-bar
      TERM(3) = XPP(4)*XPPBAR(-4) ! grcflav:  3, c     c-bar
      TERM(4) = XPP(3)*XPPBAR(-3) ! grcflav:  4, s     s-bar
      TERM(5) = XPP(-2)*XPPBAR(2) ! grcflav:  5, u-bar u
      TERM(6) = XPP(-1)*XPPBAR(1) ! grcflav:  6, d-bar d
      TERM(7) = XPP(-4)*XPPBAR(4) ! grcflav:  7, c-bar c
      TERM(8) = XPP(-3)*XPPBAR(3) ! grcflav:  8, s-bar s
      
      IF (MSTP(58).GT.4) THEN
         INPA = 10
         TERM(9)  = XPP(5)*XPPBAR(-5) ! grcflav:  9, b     b-bar
         TERM(10) = XPP(-5)*XPPBAR(5) ! grcflav: 10, b-bar b
      ENDIF

      IF (MSTP(58).EQ.6) THEN
         INPA = 12
         TERM(11) = XPP(6)*XPPBAR(-6) ! grcflav: 11, t     t-bar
         TERM(12) = XPP(-6)*XPPBAR(6) ! grcflav: 12, t-bar t
      ENDIF

      IF (IGSUB.EQ.428.OR.IGSUB.EQ.436.OR.IGSUB.EQ.504
     $     .OR.IGSUB.EQ.551) THEN
         INPA = 16
         TERM(9)  = XPP(2)*XPPBAR(-4) ! grcflav: 133, u     c-bar
         TERM(10) = XPP(1)*XPPBAR(-3) ! grcflav: 134, d     s-bar
         TERM(11) = XPP(4)*XPPBAR(-2) ! grcflav: 135, c     u-bar
         TERM(12) = XPP(3)*XPPBAR(-1) ! grcflav: 136, s     d-bar
         TERM(13) = XPP(-2)*XPPBAR(4) ! grcflav: 137, u-bar c
         TERM(14) = XPP(-1)*XPPBAR(3) ! grcflav: 138, d-bar s
         TERM(15) = XPP(-4)*XPPBAR(2) ! grcflav: 139, c-bar u
         TERM(16) = XPP(-3)*XPPBAR(1) ! grcflav: 140, s-bar d

         IF (INPFL.GT.4) THEN
            INPA = 26
            TERM(17) = XPP(5)*XPPBAR(-5) ! grcflav:   9, b     b-bar
            TERM(18) = XPP(-5)*XPPBAR(5) ! grcflav:  10, b-bar b
            TERM(19) = XPP(1)*XPPBAR(-5) ! grcflav: 141, d     b-bar
            TERM(20) = XPP(3)*XPPBAR(-5) ! grcflav: 142, s     b-bar
            TERM(21) = XPP(5)*XPPBAR(-1) ! grcflav: 143, b     d-bar
            TERM(22) = XPP(5)*XPPBAR(-3) ! grcflav: 144, b     s-bar
            TERM(23) = XPP(-5)*XPPBAR(1) ! grcflav: 145, b-bar d
            TERM(24) = XPP(-5)*XPPBAR(3) ! grcflav: 146, b-bar s
            TERM(25) = XPP(-1)*XPPBAR(5) ! grcflav: 147, d-bar b
            TERM(26) = XPP(-3)*XPPBAR(5) ! grcflav: 148, s-bar b
         ENDIF

         IF (INPFL.EQ.6) THEN
            INPA = 36
            TERM(27) = XPP(6)*XPPBAR(-6) ! grcflav:  11, t     t-bar
            TERM(28) = XPP(-6)*XPPBAR(6) ! grcflav:  12, t-bar t
            TERM(29) = XPP(2)*XPPBAR(-6) ! grcflav: 149, u     t-bar
            TERM(30) = XPP(4)*XPPBAR(-6) ! grcflav: 150, c     t-bar
            TERM(31) = XPP(6)*XPPBAR(-2) ! grcflav: 151, t     u-bar
            TERM(32) = XPP(6)*XPPBAR(-4) ! grcflav: 152, t     c-bar
            TERM(33) = XPP(-2)*XPPBAR(6) ! grcflav: 153, u-bar t
            TERM(34) = XPP(-4)*XPPBAR(6) ! grcflav: 154, c-bar t
            TERM(35) = XPP(-6)*XPPBAR(2) ! grcflav: 155, t-bar u
            TERM(36) = XPP(-6)*XPPBAR(4) ! grcflav: 156, t-bar c
         ENDIF
      ENDIF

#elif PDFLIB == 2
C...External PDF
      IF (IGPP.EQ.0) THEN
         CALL STRUCTM(GPRI(5),GRCFAQ,
     $        UPV1,DNV1,USEA1,DSEA1,STR1,CHM1,BOT1,TOP1,GL1)
         CALL STRUCTM(GPRI(6),GRCFAQ,
     $        UPV2,DNV2,USEA2,DSEA2,STR2,CHM2,BOT2,TOP2,GL2)
         UPV1 = UPV1 + USEA1
         DNV1 = DNV1 + DSEA1
         UPV2 = UPV2 + USEA2
         DNV2 = DNV2 + DSEA2
      ELSE IF (IGPP.EQ.1) THEN
         CALL STRUCTM(GPRI(5),GRCFAQ,
     $        UPV1,DNV1,USEA1,DSEA1,STR1,CHM1,BOT1,TOP1,GL1)
         CALL STRUCTM(GPRI(6),GRCFAQ,
     $        USEA2,DSEA2,UPV2,DNV2,STR2,CHM2,BOT2,TOP2,GL2)
         UPV1 = UPV1 + USEA1
         DNV1 = DNV1 + DSEA1
         USEA2 = UPV2 + USEA2
         DSEA2 = DNV2 + DSEA2
      ENDIF

      INPA = 8
      TERM(1) = UPV1*USEA2      ! grcflav:  1, u     u-bar
      TERM(2) = DNV1*DSEA2      ! grcflav:  2, d     d-bar
      TERM(3) = CHM1*CHM2       ! grcflav:  3, c     c-bar
      TERM(4) = STR1*STR2       ! grcflav:  4, s     s-bar
      TERM(5) = USEA1*UPV2      ! grcflav:  5, u-bar u
      TERM(6) = DSEA1*DNV2      ! grcflav:  6, d-bar d
      TERM(7) = CHM1*CHM2       ! grcflav:  7, c-bar c
      TERM(8) = STR1*STR2       ! grcflav:  8, s-bar s

      IF (INPFL.GT.4) THEN
         INPA = 10
         TERM(9)  = BOT1*BOT2   ! grcflav:  9, b     b-bar
         TERM(10) = BOT1*BOT2   ! grcflav: 10, b-bar b
      ENDIF

      IF (INPFL.EQ.6) THEN
         INPA = 12
         TERM(11) = TOP1*TOP2    ! grcflav: 11, t     t-bar
         TERM(12) = TOP1*TOP2    ! grcflav: 12, t-bar t
      ENDIF

      IF (IGSUB.EQ.428.OR.IGSUB.EQ.436.OR.IGSUB.EQ.504
     $     .OR.IGSUB.EQ.551) THEN
         INPA = 16
         TERM(9)  = UPV1*CHM2   ! grcflav: 133, u     c-bar
         TERM(10) = DNV1*STR2   ! grcflav: 134, d     s-bar
         TERM(11) = CHM1*USEA2  ! grcflav: 135, c     u-bar
         TERM(12) = STR1*DSEA2  ! grcflav: 136, s     d-bar
         TERM(13) = USEA1*CHM2  ! grcflav: 137, u-bar c
         TERM(14) = DSEA1*STR2  ! grcflav: 138, d-bar s
         TERM(15) = CHM1*UPV2   ! grcflav: 139, c-bar u
         TERM(16) = STR1*DNV2   ! grcflav: 140, s-bar d

         IF (INPFL.GT.4) THEN
            INPA = 26
            TERM(17) = BOT1*BOT2  ! grcflav:   9, b     b-bar
            TERM(18) = BOT1*BOT2  ! grcflav:  10, b-bar b
            TERM(19) = DNV1*BOT2  ! grcflav: 141, d     b-bar
            TERM(20) = STR1*BOT2  ! grcflav: 142, s     b-bar
            TERM(21) = BOT1*DSEA2 ! grcflav: 143, b     d-bar
            TERM(22) = BOT1*STR2  ! grcflav: 144, b     s-bar
            TERM(23) = BOT1*DNV2  ! grcflav: 145, b-bar d
            TERM(24) = BOT1*STR2  ! grcflav: 146, b-bar s
            TERM(25) = DSEA1*BOT2 ! grcflav: 147, d-bar b
            TERM(26) = STR1*BOT2  ! grcflav: 148, s-bar b
         ENDIF

         IF (INPFL.EQ.6) THEN
            INPA = 36
            TERM(27) = TOP1*TOP2  ! grcflav:  11, t     t-bar
            TERM(28) = TOP1*TOP2  ! grcflav:  12, t-bar t
            TERM(29) = UPV1*TOP2  ! grcflav: 149, u     t-bar
            TERM(30) = CHM1*TOP2  ! grcflav: 150, c     t-bar
            TERM(31) = TOP1*USEA2 ! grcflav: 151, t     u-bar
            TERM(32) = TOP1*CHM2  ! grcflav: 152, t     c-bar
            TERM(33) = USEA1*TOP2 ! grcflav: 153, u-bar t
            TERM(34) = CHM1*TOP2  ! grcflav: 154, c-bar t
            TERM(35) = TOP1*UPV2  ! grcflav: 155, t-bar u
            TERM(36) = TOP1*CHM2  ! grcflav: 156, t-bar c
         ENDIF
      ENDIF

#elif PDFLIB == 3
C...LHA PDF
      CALL evolvePDF(GPRI(5),GRCFAQ,FLHA1)
      CALL evolvePDF(GPRI(6),GRCFAQ,FLHA2)

C...Loop over quark flavours (up to b).
      INPA = 8
      TERM(1) = FLHA1(2)*FLHA2(-2*(1-2*IGPP)) ! grcflav:  1, u     u-bar
      TERM(2) = FLHA1(1)*FLHA2(-1*(1-2*IGPP)) ! grcflav:  2, d     d-bar
      TERM(3) = FLHA1(4)*FLHA2(-4*(1-2*IGPP)) ! grcflav:  3, c     c-bar
      TERM(4) = FLHA1(3)*FLHA2(-3*(1-2*IGPP)) ! grcflav:  4, s     s-bar
      TERM(5) = FLHA1(-2)*FLHA2(2*(1-2*IGPP)) ! grcflav:  5, u-bar u
      TERM(6) = FLHA1(-1)*FLHA2(1*(1-2*IGPP)) ! grcflav:  6, d-bar d
      TERM(7) = FLHA1(-4)*FLHA2(4*(1-2*IGPP)) ! grcflav:  7, c-bar c
      TERM(8) = FLHA1(-3)*FLHA2(3*(1-2*IGPP)) ! grcflav:  8, s-bar s

      IF (INPFL.GT.4) THEN
         INPA = 10
         TERM(9)  = FLHA1(5)*FLHA2(-5*(1-2*IGPP)) ! grcflav:  9, b     b-bar
         TERM(10) = FLHA1(-5)*FLHA2(5*(1-2*IGPP)) ! grcflav: 10, b-bar b
      ENDIF

      IF (INPFL.EQ.6) THEN
         INPA = 12
         TERM(11) = FLHA1(6)*FLHA2(-6*(1-2*IGPP)) ! grcflav: 11, t     t-bar
         TERM(12) = FLHA1(-6)*FLHA2(6*(1-2*IGPP)) ! grcflav: 12, t-bar t
      ENDIF

      IF (IGSUB.EQ.428.OR.IGSUB.EQ.436.OR.IGSUB.EQ.504
     $     .OR.IGSUB.EQ.551) THEN
         INPA = 16
         TERM(9)  = FLHA1(2)*FLHA2(-4*(1-2*IGPP)) ! grcflav: 133, u     c-bar
         TERM(10) = FLHA1(1)*FLHA2(-3*(1-2*IGPP)) ! grcflav: 134, d     s-bar
         TERM(11) = FLHA1(4)*FLHA2(-2*(1-2*IGPP)) ! grcflav: 135, c     u-bar
         TERM(12) = FLHA1(3)*FLHA2(-1*(1-2*IGPP)) ! grcflav: 136, s     d-bar
         TERM(13) = FLHA1(-2)*FLHA2(4*(1-2*IGPP)) ! grcflav: 137, u-bar c
         TERM(14) = FLHA1(-1)*FLHA2(3*(1-2*IGPP)) ! grcflav: 138, d-bar s
         TERM(15) = FLHA1(-4)*FLHA2(2*(1-2*IGPP)) ! grcflav: 139, c-bar u
         TERM(16) = FLHA1(-3)*FLHA2(1*(1-2*IGPP)) ! grcflav: 140, s-bar d

         IF (INPFL.GT.4) THEN
            INPA = 26
            TERM(17) = FLHA1(5)*FLHA2(-5*(1-2*IGPP)) ! grcflav:   9, b    b-bar
            TERM(18) = FLHA1(-5)*FLHA2(5*(1-2*IGPP)) ! grcflav:  10, b-bar b
            TERM(19) = FLHA1(1)*FLHA2(-5*(1-2*IGPP)) ! grcflav: 141, d    b-bar
            TERM(20) = FLHA1(3)*FLHA2(-5*(1-2*IGPP)) ! grcflav: 142, s    b-bar
            TERM(21) = FLHA1(5)*FLHA2(-1*(1-2*IGPP)) ! grcflav: 143, b    d-bar
            TERM(22) = FLHA1(5)*FLHA2(-3*(1-2*IGPP)) ! grcflav: 144, b    s-bar
            TERM(23) = FLHA1(-5)*FLHA2(1*(1-2*IGPP)) ! grcflav: 145, b-bar d
            TERM(24) = FLHA1(-5)*FLHA2(3*(1-2*IGPP)) ! grcflav: 146, b-bar s
            TERM(25) = FLHA1(-1)*FLHA2(5*(1-2*IGPP)) ! grcflav: 147, d-bar b
            TERM(26) = FLHA1(-3)*FLHA2(5*(1-2*IGPP)) ! grcflav: 148, s-bar b
         ENDIF

         IF (INPFL.EQ.6) THEN
            INPA = 36
            TERM(27) = FLHA1(6)*FLHA2(-6*(1-2*IGPP)) ! grcflav:  11, t    t-bar
            TERM(28) = FLHA1(-6)*FLHA2(6*(1-2*IGPP)) ! grcflav:  12, t-bar t
            TERM(29) = FLHA1(2)*FLHA2(-6*(1-2*IGPP)) ! grcflav: 149, u    t-bar
            TERM(30) = FLHA1(4)*FLHA2(-6*(1-2*IGPP)) ! grcflav: 150, c    t-bar
            TERM(31) = FLHA1(6)*FLHA2(-2*(1-2*IGPP)) ! grcflav: 151, t    u-bar
            TERM(32) = FLHA1(6)*FLHA2(-4*(1-2*IGPP)) ! grcflav: 152, t    c-bar
            TERM(33) = FLHA1(-2)*FLHA2(6*(1-2*IGPP)) ! grcflav: 153, u-bar t
            TERM(34) = FLHA1(-4)*FLHA2(6*(1-2*IGPP)) ! grcflav: 154, c-bar t
            TERM(35) = FLHA1(-6)*FLHA2(2*(1-2*IGPP)) ! grcflav: 155, t-bar u
            TERM(36) = FLHA1(-6)*FLHA2(4*(1-2*IGPP)) ! grcflav: 156, t-bar c
         ENDIF
      ENDIF
#endif

C      IF (INPFL.EQ.5.AND.TERM(9).EQ.0.0D0) THEN
C         WRITE(*,*) 'Error: PDF, wrong num. of flavor(5).'
C         STOP
C      ELSE IF (INPFL.EQ.6.AND.TERM(11).EQ.0.0D0) THEN
C         WRITE(*,*) 'Error: PDF, wrong num. of flavor(6).'
C         STOP
C      ENDIF
 
      GFLA(1) = 1
      GFLA(2) = 0.0D0
      GFLA(INT(GFLA(1))+2) = 1.0D0
      GFLA(10) = 1.0D0
      GFLA(INT(GFLA(1))+10) = INPA
      DO I=1,INT(GFLA(1))
         IF (GPRI(10).GT.GFLA(I+1).AND.GPRI(10).LE.GFLA(I+2)) THEN
            RNA = (GPRI(10) - GFLA(I+1))/(GFLA(I+2) - GFLA(I+1))
            FMIN = INT(GFLA(I+9))
            FMAX = INT(GFLA(I+10))
            YACOB = YACOB/(GFLA(I+2) - GFLA(I+1))
         ENDIF
      ENDDO
 
      IF (FMIN.EQ.1) THEN
         FMIN = 0
      ENDIF
 
      SUM = 0.0D0
      DO I = FMIN+1,FMAX
         SUM = SUM + TERM(I)
      ENDDO

C...Reject very small/large x
      IF (SUM.EQ.0.0D0) THEN
C         write(*,*) 'reject',sum,GPRI(5),GPRI(6),GRCFAQ
         GO TO 999
      ENDIF
 
C...Select one of the possible terms at random.
      RANSUM = RNA*SUM
      ISEL = FMIN
      DO WHILE (RANSUM.GT.0.0D0)
         ISEL = ISEL + 1
         RANSUM = RANSUM - TERM(ISEL)
      ENDDO

      IF (ISEL.EQ.0) THEN
         WRITE(*,*) 'GRC::Error Mismatch intial flavor, ISEL=0 .'
         GO TO 999
      ENDIF

      GRCFLAV = ISEL
      YACOB = SUM*YACOB

      IF (IGSUB.EQ.428.OR.IGSUB.EQ.436.OR.IGSUB.EQ.504
     $     .OR.IGSUB.EQ.551) THEN
         GRCFLAV = IGFLVTMP(ISEL)
      ENDIF

C...Set flavor
      CALL GRCFFIL

      RETURN
 999  CONTINUE
      IREJ = 2
      YACOB = 0.0D0
      RETURN
      END
 
C----------------------------
C Initial quark anti-quark'.
C----------------------------
      SUBROUTINE GRCPRTNQQB
      IMPLICIT REAL*8(A-H,O-Z)
 
#include "define.h"
 
      INCLUDE 'grchad.inc'

#if PSMODEL > 0
C...Parameters.
      COMMON/PYPARS/MSTP(200),PARP(200),MSTI(200),PARI(200)
#endif
 
C...Proton Variable
      REAL*8 TERM(16)

C...PYTHIA PDF
      REAL*8 XPP(-25:25),XPPBAR(-25:25)
C...LHA PDF
      REAL*8 FLHA1(-6:6),FLHA2(-6:6)
 
C...Grace Variable
      INTEGER I,ISEL,INPA,FLAGPA
      REAL*8 SUM,XS1,XS2,XSS,RNA,RANSUM

      IF (GPRI(5).GT.1.OR.GPRI(6).GT.1) THEN
         IREJ = 1
         GO TO 999
      ENDIF

C...Without CKM
      IF (GRCCKM(1,1).EQ.1.0D0.AND.GRCCKM(1,2).EQ.0.0D0) THEN
         XS1 = 1.0D0
         XS2 = 0.0D0
C...With CKM
      ELSE
         XS1 = 50.0D0
         XS2 = 2.0D0
      ENDIF

C...For Z/gamma processes.
      IF (IGSUB.EQ.507) THEN
         XS1 = 5.0D0
         XS2 = 2.0D0
      ENDIF

      XSS = XS1/(XS1 + XS2)

#if PSMODEL > 0
      IF (INPFL.NE.MSTP(58)) THEN
         WRITE(*,*) 'Mismacth Nr. of flavor in PDF: INPFL/MSTP(58): ',
     $        INPFL,'/',MSTP(58)
         WRITE(*,*) 'processed as INPFL:',INPFL
         MSTP(58) = INPFL
      ENDIF
#endif

      IF (INPFL.GT.4.AND.IGSUB.GT.399.AND.IGSUB.LT.416) THEN
         WRITE(*,*) 'GR@PPA : We do not support b,t contents of PDF'
         WRITE(*,*) '         for the 400,401,...,415.'
         WRITE(*,*) 'Set INPFL <= 4.'
         STOP
      ENDIF

#if PDFLIB == 1
C...Internal PDF from PYTHIA
      IF (IGPP.EQ.0) THEN
         CALL PYPDFU( 2212,GPRI(5),GRCFAQ**2,XPP)
         CALL PYPDFU( 2212,GPRI(6),GRCFAQ**2,XPPBAR)
      ELSE IF (IGPP.EQ.1) THEN
         CALL PYPDFU( 2212,GPRI(5),GRCFAQ**2,XPP)
         CALL PYPDFU(-2212,GPRI(6),GRCFAQ**2,XPPBAR)
      ENDIF

      IF (GPRI(10).LT.XSS) THEN
         FLAGPA = 1
         RNA = GPRI(10)/XSS
         YACOB = YACOB/XSS

C...Loop over quark flavours (up to b).
C...Diagonal component of CKM
         INPA = 8
         TERM(1) = XPP(2)*XPPBAR(-1) ! grcflav: 13, u     d-bar
         TERM(2) = XPP(4)*XPPBAR(-3) ! grcflav: 14, c     s-bar
         TERM(3) = XPP(-1)*XPPBAR(2) ! grcflav: 15, d-bar u
         TERM(4) = XPP(-3)*XPPBAR(4) ! grcflav: 16, s-bar c
         TERM(5) = XPP(-2)*XPPBAR(1) ! grcflav: 17, u-bar d
         TERM(6) = XPP(-4)*XPPBAR(3) ! grcflav: 18, c-bar s
         TERM(7) = XPP(1)*XPPBAR(-2) ! grcflav: 19, d     u-bar
         TERM(8) = XPP(3)*XPPBAR(-4) ! grcflav: 20, s     c-bar
      ELSE
         FLAGPA = 2
         RNA = (GPRI(10) - XSS)/(1.0D0 - XSS)
         YACOB = YACOB/(1.0D0 - XSS)

C...Off-Diagonal component of CKM
         INPA = 8
         TERM(1) = XPP(2)*XPPBAR(-3) ! grcflav: 21, u     s-bar
         TERM(2) = XPP(4)*XPPBAR(-1) ! grcflav: 22, c     d-bar
         TERM(3) = XPP(-3)*XPPBAR(2) ! grcflav: 23, s-bar u
         TERM(4) = XPP(-1)*XPPBAR(4) ! grcflav: 24, d-bar c
         TERM(5) = XPP(-2)*XPPBAR(3) ! grcflav: 25, u-bar s
         TERM(6) = XPP(-4)*XPPBAR(1) ! grcflav: 26, c-bar d
         TERM(7) = XPP(3)*XPPBAR(-2) ! grcflav: 27, s     u-bar
         TERM(8) = XPP(1)*XPPBAR(-4) ! grcflav: 28, d     c-bar

         IF (INPFL.EQ.5) THEN
            INPA = 16
            TERM(9)  = XPP(2)*XPPBAR(-5) ! grcflav: 29, u     b-bar
            TERM(10) = XPP(4)*XPPBAR(-5) ! grcflav: 30, c     b-bar
            TERM(11) = XPP(-5)*XPPBAR(2) ! grcflav: 31, b-bar u
            TERM(12) = XPP(-5)*XPPBAR(4) ! grcflav: 32, b-bar c
            TERM(13) = XPP(-2)*XPPBAR(5) ! grcflav: 33, u-bar b
            TERM(14) = XPP(-4)*XPPBAR(5) ! grcflav: 34, c-bar b
            TERM(15) = XPP(5)*XPPBAR(-2) ! grcflav: 35, b     u-bar
            TERM(16) = XPP(5)*XPPBAR(-4) ! grcflav: 36, b     c-bar
         ENDIF
 
         IF (INPFL.EQ.6) THEN
            WRITE(*,*) 'Error: No mixing with top'
            STOP
         ENDIF
      ENDIF

#elif PDFLIB == 2
C...External PDF
      IF (IGPP.EQ.0) THEN
         CALL STRUCTM(GPRI(5),GRCFAQ,
     $        UPV1,DNV1,USEA1,DSEA1,STR1,CHM1,BOT1,TOP1,GL1)
         CALL STRUCTM(GPRI(6),GRCFAQ,
     $        UPV2,DNV2,USEA2,DSEA2,STR2,CHM2,BOT2,TOP2,GL2)
         UPV1 = UPV1 + USEA1
         DNV1 = DNV1 + DSEA1
         UPV2 = UPV2 + USEA2
         DNV2 = DNV2 + DSEA2
      ELSE IF (IGPP.EQ.1) THEN
         CALL STRUCTM(GPRI(5),GRCFAQ,
     $        UPV1,DNV1,USEA1,DSEA1,STR1,CHM1,BOT1,TOP1,GL1)
         CALL STRUCTM(GPRI(6),GRCFAQ,
     $        USEA2,DSEA2,UPV2,DNV2,STR2,CHM2,BOT2,TOP2,GL2)
         UPV1 = UPV1 + USEA1
         DNV1 = DNV1 + DSEA1
         USEA2 = UPV2 + USEA2
         DSEA2 = DNV2 + DSEA2
      ENDIF
 
      IF (GPRI(10).LT.XSS) THEN
         FLAGPA = 1
         RNA = GPRI(10)/XSS
         YACOB = YACOB/XSS

C...Loop over quark flavours (up to b).
C...Diagonal component of CKM
         INPA = 8
         TERM(1) = UPV1*DSEA2   ! grcflav: 13, u     d-bar
         TERM(2) = CHM1*STR2    ! grcflav: 14, c     s-bar
         TERM(3) = DSEA1*UPV2   ! grcflav: 15, d-bar u
         TERM(4) = STR1*CHM2    ! grcflav: 16, s-bar c
         TERM(5) = USEA1*DNV2   ! grcflav: 17, u-bar d
         TERM(6) = CHM1*STR2    ! grcflav: 18, c-bar s
         TERM(7) = DNV1*USEA2   ! grcflav: 19, d     u-bar
         TERM(8) = STR1*CHM2    ! grcflav: 20, s     c-bar
      ELSE
         FLAGPA = 2
         RNA = (GPRI(10) - XSS)/(1.0D0 - XSS)
         YACOB = YACOB/(1.0D0 - XSS)

C...Off-Diagonal component of CKM
         INPA = 8
         TERM(1) = UPV1*STR2    ! grcflav: 21, u     s-bar
         TERM(2) = CHM1*DSEA2   ! grcflav: 22, c     d-bar
         TERM(3) = STR1*UPV2    ! grcflav: 23, s-bar u
         TERM(4) = DSEA1*CHM2   ! grcflav: 24, d-bar c
         TERM(5) = USEA1*STR2   ! grcflav: 25, u-bar s
         TERM(6) = CHM1*DNV2    ! grcflav: 26, c-bar d
         TERM(7) = STR1*USEA2   ! grcflav: 27, s     u-bar
         TERM(8) = DNV1*CHM2    ! grcflav: 28, d     c-bar

         IF (INPFL.EQ.5) THEN
            INPA = 16
            TERM(9)  = UPV1*BOT2  ! grcflav: 29, u     b-bar
            TERM(10) = CHM1*BOT2  ! grcflav: 30, c     b-bar
            TERM(11) = BOT1*UPV2  ! grcflav: 31, b-bar u
            TERM(12) = BOT1*CHM2  ! grcflav: 32, b-bar c
            TERM(13) = USEA1*BOT2 ! grcflav: 33, u-bar b
            TERM(14) = CHM1*BOT2  ! grcflav: 34, c-bar b
            TERM(15) = BOT1*USEA2 ! grcflav: 35, b u-bar
            TERM(16) = BOT1*CHM2  ! grcflav: 36, b c-bar
         ENDIF
 
         IF (INPFL.EQ.6) THEN
            WRITE(*,*) 'Error: No mixing with top'
            STOP
         ENDIF
      ENDIF

#elif PDFLIB == 3
C...LHA PDF
      CALL evolvePDF(GPRI(5),GRCFAQ,FLHA1)
      CALL evolvePDF(GPRI(6),GRCFAQ,FLHA2)

      IF (GPRI(10).LT.XSS) THEN
         FLAGPA = 1
         RNA = GPRI(10)/XSS
         YACOB = YACOB/XSS

C...Loop over quark flavours (up to b).
C...Diagonal component of CKM
         INPA = 8
         TERM(1) = FLHA1(2)*FLHA2(-1*(1-2*IGPP)) ! grcflav: 13, u     d-bar
         TERM(2) = FLHA1(4)*FLHA2(-3*(1-2*IGPP)) ! grcflav: 14, c     s-bar
         TERM(3) = FLHA1(-1)*FLHA2(2*(1-2*IGPP)) ! grcflav: 15, d-bar u
         TERM(4) = FLHA1(-3)*FLHA2(4*(1-2*IGPP)) ! grcflav: 16, s-bar c
         TERM(5) = FLHA1(-2)*FLHA2(1*(1-2*IGPP)) ! grcflav: 17, u-bar d
         TERM(6) = FLHA1(-4)*FLHA2(3*(1-2*IGPP)) ! grcflav: 18, c-bar s
         TERM(7) = FLHA1(1)*FLHA2(-2*(1-2*IGPP)) ! grcflav: 19, d     u-bar
         TERM(8) = FLHA1(3)*FLHA2(-4*(1-2*IGPP)) ! grcflav: 20, s     c-bar
      ELSE
         FLAGPA = 2
         RNA = (GPRI(10) - XSS)/(1.0D0 - XSS)
         YACOB = YACOB/(1.0D0 - XSS)

C...Off-Diagonal component of CKM
         INPA = 8
         TERM(1) = FLHA1(2)*FLHA2(-3*(1-2*IGPP)) ! grcflav: 21, u     s-bar
         TERM(2) = FLHA1(4)*FLHA2(-1*(1-2*IGPP)) ! grcflav: 22, c     d-bar
         TERM(3) = FLHA1(-3)*FLHA2(2*(1-2*IGPP)) ! grcflav: 23, s-bar u
         TERM(4) = FLHA1(-1)*FLHA2(4*(1-2*IGPP)) ! grcflav: 24, d-bar c
         TERM(5) = FLHA1(-2)*FLHA2(3*(1-2*IGPP)) ! grcflav: 25, u-bar s
         TERM(6) = FLHA1(-4)*FLHA2(1*(1-2*IGPP)) ! grcflav: 26, c-bar d
         TERM(7) = FLHA1(3)*FLHA2(-2*(1-2*IGPP)) ! grcflav: 27, s     u-bar
         TERM(8) = FLHA1(1)*FLHA2(-4*(1-2*IGPP)) ! grcflav: 28, d     c-bar

         IF (INPFL.EQ.5) THEN
            INPA = 16
            TERM(9)  = XPP(2)*FLHA2(-5*(1-2*IGPP)) ! grcflav: 29, u     b-bar
            TERM(10) = XPP(4)*FLHA2(-5*(1-2*IGPP)) ! grcflav: 30, c     b-bar
            TERM(11) = XPP(-5)*FLHA2(2*(1-2*IGPP)) ! grcflav: 31, b-bar u
            TERM(12) = XPP(-5)*FLHA2(4*(1-2*IGPP)) ! grcflav: 32, b-bar c
            TERM(13) = XPP(-2)*FLHA2(5*(1-2*IGPP)) ! grcflav: 33, u-bar b
            TERM(14) = XPP(-4)*FLHA2(5*(1-2*IGPP)) ! grcflav: 34, c-bar b
            TERM(15) = XPP(5)*FLHA2(-2*(1-2*IGPP)) ! grcflav: 35, b     u-bar
            TERM(16) = XPP(5)*FLHA2(-4*(1-2*IGPP)) ! grcflav: 36, b     c-bar
         ENDIF
 
         IF (INPFL.EQ.6) THEN
            WRITE(*,*) 'Error: No mixing with top'
            STOP
         ENDIF
      ENDIF

#endif

      SUM = 0.0D0
      DO I = 1,INPA
         SUM = SUM + TERM(I)
      ENDDO

C...Reject very small/large x
      IF (SUM.EQ.0.0D0) THEN
C         write(*,*) 'reject',sum,GPRI(5),GPRI(6),GRCFAQ
         GO TO 999
      ENDIF

C...Select one of the possible terms at random.
      RANSUM = RNA*SUM
      ISEL = 0
      DO WHILE (RANSUM.GT.0.0D0)
         ISEL = ISEL + 1
         RANSUM = RANSUM - TERM(ISEL)
      ENDDO

      IF (ISEL.EQ.0) THEN
         WRITE(*,*) 'GRC::Error Mismatch intial flavor, ISEL=0 .'
         GO TO 999
      ENDIF
 
      IF (FLAGPA.EQ.1) THEN
         GRCFLAV = ISEL + 12
      ELSE IF (FLAGPA.EQ.2) THEN
         GRCFLAV = ISEL + 20
      ELSE
         WRITE(*,*) 'Error: strange random num. for PDF.',RNA
         GO TO 999
      ENDIF
      YACOB = SUM*YACOB

C...Set flavor
      CALL GRCFFIL

      RETURN
 999  CONTINUE
      IREJ = 2
      YACOB = 0.0D0
      RETURN
      END

C----------------------
C Initial quark gluon.
C----------------------
      SUBROUTINE GRCPRTNQG
      IMPLICIT REAL*8(A-H,O-Z)
 
#include "define.h"
 
      INCLUDE 'grchad.inc'
 
#if PSMODEL > 0
C...Parameters.
      COMMON/PYPARS/MSTP(200),PARP(200),MSTI(200),PARI(200)
#endif
 
C...Proton Variable
      INTEGER I
      REAL*8 TERM(24)

C...PYTHIA PDF
      REAL*8 XPP(-25:25),XPPBAR(-25:25)
C...LHA PDF
      REAL*8 FLHA1(-6:6),FLHA2(-6:6)
 
C...Grace Variable
      INTEGER ISEL,INPA
      REAL*8 SUM,RANSUM
      INTEGER FMAX,FMIN
      REAL*8 RNA,GFLA(18)

      IF (GPRI(5).GT.1.OR.GPRI(6).GT.1) THEN
         IREJ = 1
         GO TO 999
      ENDIF

#if PSMODEL > 0
      IF (INPFL.NE.MSTP(58)) THEN
         WRITE(*,*) 'Mismacth Nr. of flavor in PDF: INPFL/MSTP(58): ',
     $        INPFL,'/',MSTP(58)
         WRITE(*,*) 'processed as INPFL:',INPFL
         MSTP(58) = INPFL
      ENDIF
#endif

      IF (INPFL.GT.4.AND.IGSUB.GT.399.AND.IGSUB.LT.416) THEN
         WRITE(*,*) 'GR@PPA : We do not support b,t contents of PDF'
         WRITE(*,*) '         for the 400,401,...,415.'
         WRITE(*,*) 'Set INPFL <= 4.'
         STOP
      ENDIF

#if PDFLIB == 1
C...Internal PDF from PYTHIA
      IF (IGPP.EQ.0) THEN
         CALL PYPDFU( 2212,GPRI(5),GRCFAQ**2,XPP)
         CALL PYPDFU( 2212,GPRI(6),GRCFAQ**2,XPPBAR)
      ELSE IF (IGPP.EQ.1) THEN
         CALL PYPDFU( 2212,GPRI(5),GRCFAQ**2,XPP)
         CALL PYPDFU(-2212,GPRI(6),GRCFAQ**2,XPPBAR)
      ENDIF

C...Loop over quark flavours (up to b).
      INPA = 16
      TERM(1)  = XPP(2)*XPPBAR(0)  ! grcflav: 37, u     gluon
      TERM(2)  = XPP(1)*XPPBAR(0)  ! grcflav: 38, d     gluon
      TERM(3)  = XPP(4)*XPPBAR(0)  ! grcflav: 39, c     gluon
      TERM(4)  = XPP(3)*XPPBAR(0)  ! grcflav: 40, s     gluon
      TERM(5)  = XPP(-2)*XPPBAR(0) ! grcflav: 41, u-bar gluon
      TERM(6)  = XPP(-1)*XPPBAR(0) ! grcflav: 42, d-bar gluon
      TERM(7)  = XPP(-4)*XPPBAR(0) ! grcflav: 43, c-bar gluon
      TERM(8)  = XPP(-3)*XPPBAR(0) ! grcflav: 44, s-bar gluon
      TERM(9)  = XPP(0)*XPPBAR(2)  ! grcflav: 45, gluon u    
      TERM(10) = XPP(0)*XPPBAR(1)  ! grcflav: 46, gluon d    
      TERM(11) = XPP(0)*XPPBAR(4)  ! grcflav: 47, gluon c    
      TERM(12) = XPP(0)*XPPBAR(3)  ! grcflav: 48, gluon s    
      TERM(13) = XPP(0)*XPPBAR(-2) ! grcflav: 49, gluon u-bar
      TERM(14) = XPP(0)*XPPBAR(-1) ! grcflav: 50, gluon d-bar
      TERM(15) = XPP(0)*XPPBAR(-4) ! grcflav: 51, gluon c-bar
      TERM(16) = XPP(0)*XPPBAR(-3) ! grcflav: 52, gluon s-bar

      IF (INPFL.GT.4) THEN
         INPA = 20
         TERM(17) = XPP(5)*XPPBAR(0)  ! grcflav: 53, b     gluon
         TERM(18) = XPP(-5)*XPPBAR(0) ! grcflav: 54, b-bar gluon
         TERM(19) = XPP(0)*XPPBAR(5)  ! grcflav: 55, gluon b
         TERM(20) = XPP(0)*XPPBAR(-5) ! grcflav: 56, gluon b-bar
      ENDIF

      IF (INPFL.EQ.6) THEN
         INPA = 24
         TERM(21) = XPP(6)*XPPBAR(0)  ! grcflav: 57, t     gluon
         TERM(22) = XPP(-6)*XPPBAR(0) ! grcflav: 58, t-bar gluon
         TERM(23) = XPP(0)*XPPBAR(6)  ! grcflav: 59, gluon t
         TERM(24) = XPP(0)*XPPBAR(-6) ! grcflav: 60, gluon t-bar
      ENDIF

#elif PDFLIB == 2
C...External PDF
      IF (IGPP.EQ.0) THEN
         CALL STRUCTM(GPRI(5),GRCFAQ,
     $        UPV1,DNV1,USEA1,DSEA1,STR1,CHM1,BOT1,TOP1,GL1)
         CALL STRUCTM(GPRI(6),GRCFAQ,
     $        UPV2,DNV2,USEA2,DSEA2,STR2,CHM2,BOT2,TOP2,GL2)
         UPV1 = UPV1 + USEA1
         DNV1 = DNV1 + DSEA1
         UPV2 = UPV2 + USEA2
         DNV2 = DNV2 + DSEA2
      ELSE IF (IGPP.EQ.1) THEN
         CALL STRUCTM(GPRI(5),GRCFAQ,
     $        UPV1,DNV1,USEA1,DSEA1,STR1,CHM1,BOT1,TOP1,GL1)
         CALL STRUCTM(GPRI(6),GRCFAQ,
     $        USEA2,DSEA2,UPV2,DNV2,STR2,CHM2,BOT2,TOP2,GL2)
         UPV1 = UPV1 + USEA1
         DNV1 = DNV1 + DSEA1
         USEA2 = UPV2 + USEA2
         DSEA2 = DNV2 + DSEA2
      ENDIF

C...Loop over quark flavours (up to b).
      INPA = 16
      TERM(1)  = UPV1*GL2       ! grcflav: 37, u     gluon
      TERM(2)  = DNV1*GL2       ! grcflav: 38, d     gluon
      TERM(3)  = CHM1*GL2       ! grcflav: 39, c     gluon
      TERM(4)  = STR1*GL2       ! grcflav: 40, s     gluon
      TERM(5)  = USEA1*GL2      ! grcflav: 41, u-bar gluon
      TERM(6)  = DSEA1*GL2      ! grcflav: 42, d-bar gluon
      TERM(7)  = CHM1*GL2       ! grcflav: 43, c-bar gluon
      TERM(8)  = STR1*GL2       ! grcflav: 44, s-bar gluon
      TERM(9)  = GL1*UPV2       ! grcflav: 45, gluon u
      TERM(10) = GL1*DNV2       ! grcflav: 46, gluon d
      TERM(11) = GL1*CHM2       ! grcflav: 47, gluon c
      TERM(12) = GL1*STR2       ! grcflav: 48, gluon s
      TERM(13) = GL1*USEA2      ! grcflav: 49, gluon u-bar
      TERM(14) = GL1*DSEA2      ! grcflav: 50, gluon d-bar
      TERM(15) = GL1*CHM2       ! grcflav: 51, gluon c-bar
      TERM(16) = GL1*STR2       ! grcflav: 52, gluon s-bar

      IF (INPFL.GT.4) THEN
         INPA = 20
         TERM(17) = BOT1*GL2    ! grcflav: 53, b     gluon
         TERM(18) = BOT1*GL2    ! grcflav: 54, b-bar gluon
         TERM(19) = GL1*BOT2    ! grcflav: 55, gluon b
         TERM(20) = GL1*BOT2    ! grcflav: 56, gluon b-bar
      ENDIF

      IF (INPFL.EQ.6) THEN
         INPA = 24
         TERM(21) = TOP1*GL2    ! grcflav: 57, t     gluon
         TERM(22) = TOP1*GL2    ! grcflav: 58, t-bar gluon
         TERM(23) = GL1*TOP2    ! grcflav: 59, gluon t
         TERM(24) = GL1*TOP2    ! grcflav: 60, gluon t-bar
      ENDIF

#elif PDFLIB == 3
C...LHA PDF
      CALL evolvePDF(GPRI(5),GRCFAQ,FLHA1)
      CALL evolvePDF(GPRI(6),GRCFAQ,FLHA2)

C...Loop over quark flavours (up to b).
      INPA = 16
      TERM(1)  = FLHA1(2)*FLHA2(0)             ! grcflav: 37, u     gluon
      TERM(2)  = FLHA1(1)*FLHA2(0)             ! grcflav: 38, d     gluon
      TERM(3)  = FLHA1(4)*FLHA2(0)             ! grcflav: 39, c     gluon
      TERM(4)  = FLHA1(3)*FLHA2(0)             ! grcflav: 40, s     gluon
      TERM(5)  = FLHA1(-2)*FLHA2(0)            ! grcflav: 41, u-bar gluon
      TERM(6)  = FLHA1(-1)*FLHA2(0)            ! grcflav: 42, d-bar gluon
      TERM(7)  = FLHA1(-4)*FLHA2(0)            ! grcflav: 43, c-bar gluon
      TERM(8)  = FLHA1(-3)*FLHA2(0)            ! grcflav: 44, s-bar gluon
      TERM(9)  = FLHA1(0)*FLHA2(2*(1-2*IGPP))  ! grcflav: 45, gluon u    
      TERM(10) = FLHA1(0)*FLHA2(1*(1-2*IGPP))  ! grcflav: 46, gluon d    
      TERM(11) = FLHA1(0)*FLHA2(4*(1-2*IGPP))  ! grcflav: 47, gluon c    
      TERM(12) = FLHA1(0)*FLHA2(3*(1-2*IGPP))  ! grcflav: 48, gluon s    
      TERM(13) = FLHA1(0)*FLHA2(-2*(1-2*IGPP)) ! grcflav: 49, gluon u-bar
      TERM(14) = FLHA1(0)*FLHA2(-1*(1-2*IGPP)) ! grcflav: 50, gluon d-bar
      TERM(15) = FLHA1(0)*FLHA2(-4*(1-2*IGPP)) ! grcflav: 51, gluon c-bar
      TERM(16) = FLHA1(0)*FLHA2(-3*(1-2*IGPP)) ! grcflav: 52, gluon s-bar

      IF (INPFL.GT.4) THEN
         INPA = 20
         TERM(17) = FLHA1(5)*FLHA2(0)             ! grcflav: 53, b     gluon
         TERM(18) = FLHA1(-5)*FLHA2(0)            ! grcflav: 54, b-bar gluon
         TERM(19) = FLHA1(0)*FLHA2(5*(1-2*IGPP))  ! grcflav: 55, gluon b
         TERM(20) = FLHA1(0)*FLHA2(-5*(1-2*IGPP)) ! grcflav: 56, gluon b-bar
      ENDIF

      IF (INPFL.EQ.6) THEN
         INPA = 24
         TERM(21) = FLHA1(6)*FLHA2(0)             ! grcflav: 57, t     gluon
         TERM(22) = FLHA1(-6)*FLHA2(0)            ! grcflav: 58, t-bar gluon
         TERM(23) = FLHA1(0)*FLHA2(6*(1-2*IGPP))  ! grcflav: 59, gluon t
         TERM(24) = FLHA1(0)*FLHA2(-6*(1-2*IGPP)) ! grcflav: 60, gluon t-bar
      ENDIF
#endif

C      IF (INPFL.EQ.5.AND.TERM(17).EQ.0.0D0) THEN
C         WRITE(*,*) 'Error: PDF, wrong num. of flavor(5).'
C         STOP
C      ELSE IF (INPFL.EQ.6.AND.TERM(21).EQ.0.0D0) THEN
C         WRITE(*,*) 'Error: PDF, wrong num. of flavor(6).'
C         STOP
C      ENDIF
 
      GFLA(1) = 1
      GFLA(2) = 0.0D0
      GFLA(INT(GFLA(1))+2) = 1.0D0
      GFLA(10) = 1.0D0
      GFLA(INT(GFLA(1))+10) = INPA
      DO I=1,INT(GFLA(1))
         IF (GPRI(10).GT.GFLA(I+1).AND.GPRI(10).LE.GFLA(I+2)) THEN
            RNA = (GPRI(10) - GFLA(I+1))/(GFLA(I+2) - GFLA(I+1))
            FMIN = INT(GFLA(I+9))
            FMAX = INT(GFLA(I+10))
            YACOB = YACOB/(GFLA(I+2) - GFLA(I+1))
         ENDIF
      ENDDO
 
      IF (FMIN.EQ.1) THEN
         FMIN = 0
      ENDIF
 
      SUM = 0.0D0
      DO I = FMIN+1,FMAX
         SUM = SUM + TERM(I)
      ENDDO

C...Reject very small/large x
      IF (SUM.EQ.0.0D0) THEN
C         write(*,*) 'reject',sum,GPRI(5),GPRI(6),GRCFAQ
         GO TO 999
      ENDIF
 
C...Select one of the possible terms at random.
      RANSUM = RNA*SUM
      ISEL = FMIN
      DO WHILE (RANSUM.GT.0.0D0)
         ISEL = ISEL + 1
         RANSUM = RANSUM - TERM(ISEL)
      ENDDO

      IF (ISEL.EQ.0) THEN
         WRITE(*,*) 'GRC::Error Mismatch intial flavor, ISEL=0 .'
         GO TO 999
      ENDIF

      GRCFLAV = ISEL + 36
      YACOB = SUM*YACOB

C...Set flavor
      CALL GRCFFIL
 
      RETURN
 999  CONTINUE
      IREJ = 2
      YACOB = 0.0D0
      RETURN
      END

C-----------------------------------------
C Initial up(down)-quark up(down)-quark'.
C-----------------------------------------
      SUBROUTINE GRCPRTNQQA
      IMPLICIT REAL*8(A-H,O-Z)
 
#include "define.h"
 
      INCLUDE 'grchad.inc'
 
#if PSMODEL > 0
C...Parameters.
      COMMON/PYPARS/MSTP(200),PARP(200),MSTI(200),PARI(200)
#endif
 
C...Proton Variable
      INTEGER I
      REAL*8 TERM(24)

C...PYTHIA PDF
      REAL*8 XPP(-25:25),XPPBAR(-25:25)
C...LHA PDF
      REAL*8 FLHA1(-6:6),FLHA2(-6:6)

C...Grace Variable
      INTEGER ISEL,INPA
      REAL*8 SUM,RANSUM,RNA,XS1,XS2,XSS
      INTEGER FLAGPA

      IF (GPRI(5).GT.1.OR.GPRI(6).GT.1) THEN
         IREJ = 1
         GO TO 999
      ENDIF

C...Weight PDF
      XS1 = 0.5D0
      XS2 = 0.5D0
C      XS1 = 0.0D0
C      XS2 = 1.0D0
      XSS = XS1/(XS1 + XS2)

#if PSMODEL > 0
      IF (INPFL.NE.MSTP(58)) THEN
         WRITE(*,*) 'Mismacth Nr. of flavor in PDF: INPFL/MSTP(58): ',
     $        INPFL,'/',MSTP(58)
         WRITE(*,*) 'processed as INPFL:',INPFL
         MSTP(58) = INPFL
      ENDIF
#endif

#if PDFLIB == 1
C...Internal PDF from PYTHIA
      IF (IGPP.EQ.0) THEN
         CALL PYPDFU( 2212,GPRI(5),GRCFAQ**2,XPP)
         CALL PYPDFU( 2212,GPRI(6),GRCFAQ**2,XPPBAR)
      ELSE IF (IGPP.EQ.1) THEN
         CALL PYPDFU( 2212,GPRI(5),GRCFAQ**2,XPP)
         CALL PYPDFU(-2212,GPRI(6),GRCFAQ**2,XPPBAR)
      ENDIF

      IF (GPRI(10).LE.XSS) THEN
         FLAGPA = 1
         RNA = GPRI(10)/XSS
         YACOB = YACOB/XSS

C...Loop over quark flavours (up to b).
         INPA = 8
         TERM(1) = XPP(2)*XPPBAR(2)   ! grcflav: 61, u     u
         TERM(2) = XPP(1)*XPPBAR(1)   ! grcflav: 62, d     d
         TERM(3) = XPP(4)*XPPBAR(4)   ! grcflav: 63, c     c
         TERM(4) = XPP(3)*XPPBAR(3)   ! grcflav: 64, s     s
         TERM(5) = XPP(-2)*XPPBAR(-2) ! grcflav: 65, u-bar u-bar
         TERM(6) = XPP(-1)*XPPBAR(-1) ! grcflav: 66, d-bar d-bar
         TERM(7) = XPP(-4)*XPPBAR(-4) ! grcflav: 67, c-bar c-bar
         TERM(8) = XPP(-3)*XPPBAR(-3) ! grcflav: 68, s-bar s-bar

         IF (MSTP(58).GT.4) THEN
            INPA = 10
            TERM(9)  = XPP(5)*XPPBAR(5)   ! grcflav: 69, b     b
            TERM(10) = XPP(-5)*XPPBAR(-5) ! grcflav: 70, b-bar b-bar
         ENDIF
      
         IF (MSTP(58).EQ.6) THEN
            INPA = 12
            TERM(11) = XPP(6)*XPPBAR(6)   ! grcflav: 71, t     t
            TERM(12) = XPP(-6)*XPPBAR(-6) ! grcflav: 72, t-bar t-bar
         ENDIF
      ELSE
         FLAGPA = 2
         RNA = (GPRI(10) - XSS)/(1.0D0 - XSS)
         YACOB = YACOB/(1.0D0 - XSS)

C...Loop over quark flavours (up to b).
         INPA = 8
         TERM(1) = XPP(2)*XPPBAR(4)    ! grcflav: 73, u     c
         TERM(2) = XPP(1)*XPPBAR(3)    ! grcflav: 74, d     s
         TERM(3) = XPP(4)*XPPBAR(2)    ! grcflav: 75, c     u
         TERM(4) = XPP(3)*XPPBAR(1)    ! grcflav: 76, s     d
         TERM(5) = XPP(-2)*XPPBAR(-4)  ! grcflav: 77, u-bar c-bar
         TERM(6) = XPP(-1)*XPPBAR(-3)  ! grcflav: 78, d-bar s-bar
         TERM(7) = XPP(-4)*XPPBAR(-2)  ! grcflav: 79, c-bar u-bar
         TERM(8) = XPP(-3)*XPPBAR(-1)  ! grcflav: 80, s-bar d-bar

         IF (INPFL.GT.4) THEN
            INPA = 16
            TERM(9)  = XPP(5)*XPPBAR(1)   ! grcflav: 81, b     d
            TERM(10) = XPP(5)*XPPBAR(3)   ! grcflav: 82, b     s
            TERM(11) = XPP(1)*XPPBAR(5)   ! grcflav: 83, d     b
            TERM(12) = XPP(3)*XPPBAR(5)   ! grcflav: 84, s     b
            TERM(13) = XPP(-5)*XPPBAR(-1) ! grcflav: 85, b-bar d-bar
            TERM(14) = XPP(-5)*XPPBAR(-3) ! grcflav: 86, b-bar s-bar
            TERM(15) = XPP(-1)*XPPBAR(-5) ! grcflav: 87, d-bar b-bar
            TERM(16) = XPP(-3)*XPPBAR(-5) ! grcflav: 88, s-bar b-bar
         ENDIF

         IF (INPFL.EQ.6) THEN
            INPA = 24
            TERM(17) = XPP(6)*XPPBAR(2)   ! grcflav: 89, t     u
            TERM(18) = XPP(6)*XPPBAR(4)   ! grcflav: 90, t     c
            TERM(19) = XPP(2)*XPPBAR(6)   ! grcflav: 91, u     t
            TERM(20) = XPP(4)*XPPBAR(6)   ! grcflav: 92, c     t
            TERM(21) = XPP(-6)*XPPBAR(-2) ! grcflav: 93, t-bar u-bar
            TERM(22) = XPP(-6)*XPPBAR(-4) ! grcflav: 94, t-bar c-bar
            TERM(23) = XPP(-2)*XPPBAR(-6) ! grcflav: 95, u-bar t-bar
            TERM(24) = XPP(-4)*XPPBAR(-6) ! grcflav: 96, c-bar t-bar
         ENDIF
      ENDIF

#elif PDFLIB == 2
C...External PDF
      IF (IGPP.EQ.0) THEN
         CALL STRUCTM(GPRI(5),GRCFAQ,
     $        UPV1,DNV1,USEA1,DSEA1,STR1,CHM1,BOT1,TOP1,GL1)
         CALL STRUCTM(GPRI(6),GRCFAQ,
     $        UPV2,DNV2,USEA2,DSEA2,STR2,CHM2,BOT2,TOP2,GL2)
         UPV1 = UPV1 + USEA1
         DNV1 = DNV1 + DSEA1
         UPV2 = UPV2 + USEA2
         DNV2 = DNV2 + DSEA2
      ELSE IF (IGPP.EQ.1) THEN
         CALL STRUCTM(GPRI(5),GRCFAQ,
     $        UPV1,DNV1,USEA1,DSEA1,STR1,CHM1,BOT1,TOP1,GL1)
         CALL STRUCTM(GPRI(6),GRCFAQ,
     $        USEA2,DSEA2,UPV2,DNV2,STR2,CHM2,BOT2,TOP2,GL2)
         UPV1 = UPV1 + USEA1
         DNV1 = DNV1 + DSEA1
         USEA2 = UPV2 + USEA2
         DSEA2 = DNV2 + DSEA2
      ENDIF

      IF (GPRI(10).LE.XSS) THEN
         FLAGPA = 1
         RNA = GPRI(10)/XSS
         YACOB = YACOB/XSS

         INPA = 8
         TERM(1)  = UPV1*UPV2    ! grcflav: 61, u     u
         TERM(2)  = DNV1*DNV2    ! grcflav: 62, d     d
         TERM(3)  = CHM1*CHM2    ! grcflav: 63, c     c
         TERM(4)  = STR1*STR2    ! grcflav: 64, s     s
         TERM(5)  = USEA1*USEA2  ! grcflav: 65, u-bar u-bar
         TERM(6)  = DSEA1*DSEA2  ! grcflav: 66, d-bar d-bar
         TERM(7)  = CHM1*CHM2    ! grcflav: 67, c-bar c-bar
         TERM(8)  = STR1*STR2    ! grcflav: 68, s-bar s-bar

         IF (INPFL.GT.4) THEN
            INPA = 10
            TERM(9) = BOT1*BOT2  ! grcflav: 69, b     b
            TERM(10) = BOT1*BOT2 ! grcflav: 70, b-bar b-bar
         ENDIF

         IF (INPFL.EQ.6) THEN
            INPA = 12
            TERM(11) = TOP1*TOP2 ! grcflav: 71, t     t
            TERM(12) = TOP1*TOP2 ! grcflav: 72, t-bar t-bar
         ENDIF
      ELSE
         FLAGPA = 2
         RNA = (GPRI(10) - XSS)/(1.0D0 - XSS)
         YACOB = YACOB/(1.0D0 - XSS)

         INPA = 8
         TERM(1) = UPV1*CHM2    ! grcflav: 73, u     c
         TERM(2) = DNV1*STR2    ! grcflav: 74, d     s
         TERM(3) = CHM1*UPV2    ! grcflav: 75, c     u
         TERM(4) = STR1*DNV2    ! grcflav: 76, s     d
         TERM(5) = USEA1*CHM2   ! grcflav: 77, u-bar c-bar
         TERM(6) = DSEA1*STR2   ! grcflav: 78, d-bar s-bar
         TERM(7) = CHM1*USEA2   ! grcflav: 79, c-bar u-bar
         TERM(8) = STR1*DSEA2   ! grcflav: 80, s-bar d-bar

         IF (INPFL.GT.4) THEN
            INPA = 16
            TERM(9)  = BOT1*DNV2  ! grcflav: 81, b     d
            TERM(10) = BOT1*STR2  ! grcflav: 82, b     s
            TERM(11) = DNV1*BOT2  ! grcflav: 83, d     b
            TERM(12) = STR1*BOT2  ! grcflav: 84, s     b
            TERM(13) = BOT1*DSEA2 ! grcflav: 85, b-bar d-bar
            TERM(14) = BOT1*STR2  ! grcflav: 86, b-bar s-bar
            TERM(15) = DSEA1*BOT2 ! grcflav: 87, d-bar b-bar
            TERM(16) = STR1*BOT2  ! grcflav: 88, s-bar b-bar
         ENDIF

         IF (INPFL.EQ.6) THEN
            INPA = 24
            TERM(17) = TOP1*UPV2  ! grcflav: 89, t     u
            TERM(18) = TOP1*CHM2  ! grcflav: 90, t     c
            TERM(19) = UPV1*TOP2  ! grcflav: 91, u     t
            TERM(20) = CHM1*TOP2  ! grcflav: 92, c     t
            TERM(21) = TOP1*USEA2 ! grcflav: 93, t-bar u-bar
            TERM(22) = TOP1*CHM2  ! grcflav: 94, t-bar c-bar
            TERM(23) = USEA1*TOP2 ! grcflav: 95, u-bar t-bar
            TERM(24) = CHM1*TOP2  ! grcflav: 96, c-bar t-bar
         ENDIF
      ENDIF

#elif PDFLIB == 3
C...LHA PDF
      CALL evolvePDF(GPRI(5),GRCFAQ,FLHA1)
      CALL evolvePDF(GPRI(6),GRCFAQ,FLHA2)

      IF (GPRI(10).LE.XSS) THEN
         FLAGPA = 1
         RNA = GPRI(10)/XSS
         YACOB = YACOB/XSS

C...Loop over quark flavours (up to b).
         INPA = 8
         TERM(1) = FLHA1(2)*FLHA2(2*(1-2*IGPP))   ! grcflav: 61, u    u
         TERM(2) = FLHA1(1)*FLHA2(1*(1-2*IGPP))   ! grcflav: 62, d    d
         TERM(3) = FLHA1(4)*FLHA2(4*(1-2*IGPP))   ! grcflav: 63, c    c
         TERM(4) = FLHA1(3)*FLHA2(3*(1-2*IGPP))   ! grcflav: 64, s    s
         TERM(5) = FLHA1(-2)*FLHA2(-2*(1-2*IGPP)) ! grcflav: 65, ubar ubar
         TERM(6) = FLHA1(-1)*FLHA2(-1*(1-2*IGPP)) ! grcflav: 66, dbar dbar
         TERM(7) = FLHA1(-4)*FLHA2(-4*(1-2*IGPP)) ! grcflav: 67, cbar cbar
         TERM(8) = FLHA1(-3)*FLHA2(-3*(1-2*IGPP)) ! grcflav: 68, sbar sbar

         IF (INPFL.GT.4) THEN
            INPA = 10
            TERM(9)  = FLHA1(5)*FLHA2(5*(1-2*IGPP))   ! grcflav: 69, b    b
            TERM(10) = FLHA1(-5)*FLHA2(-5*(1-2*IGPP)) ! grcflav: 70, bbar bbar
         ENDIF

         IF (INPFL.EQ.6) THEN
            INPA = 12
            TERM(11) = FLHA1(6)*FLHA2(6*(1-2*IGPP))   ! grcflav: 71, t    t
            TERM(12) = FLHA1(-6)*FLHA2(-6*(1-2*IGPP)) ! grcflav: 72, tbar tbar
         ENDIF
      ELSE
         FLAGPA = 2
         RNA = (GPRI(10) - XSS)/(1.0D0 - XSS)
         YACOB = YACOB/(1.0D0 - XSS)

C...Loop over quark flavours (up to b).
         INPA = 8
         TERM(1) = FLHA1(2)*FLHA2(4*(1-2*IGPP))    ! grcflav: 73, u    c
         TERM(2) = FLHA1(1)*FLHA2(3*(1-2*IGPP))    ! grcflav: 74, d    s
         TERM(3) = FLHA1(4)*FLHA2(2*(1-2*IGPP))    ! grcflav: 75, c    u
         TERM(4) = FLHA1(3)*FLHA2(1*(1-2*IGPP))    ! grcflav: 76, s    d
         TERM(5) = FLHA1(-2)*FLHA2(-4*(1-2*IGPP))  ! grcflav: 77, ubar cbar
         TERM(6) = FLHA1(-1)*FLHA2(-3*(1-2*IGPP))  ! grcflav: 78, dbar sbar
         TERM(7) = FLHA1(-4)*FLHA2(-2*(1-2*IGPP))  ! grcflav: 79, cbar ubar
         TERM(8) = FLHA1(-3)*FLHA2(-1*(1-2*IGPP))  ! grcflav: 80, sbar dbar

         IF (INPFL.GT.4) THEN
            INPA = 16
            TERM(9)  = FLHA1(5)*FLHA2(1*(1-2*IGPP))   ! grcflav: 81, b    d
            TERM(10) = FLHA1(5)*FLHA2(3*(1-2*IGPP))   ! grcflav: 82, b    s
            TERM(11) = FLHA1(1)*FLHA2(5*(1-2*IGPP))   ! grcflav: 83, d    b
            TERM(12) = FLHA1(3)*FLHA2(5*(1-2*IGPP))   ! grcflav: 84, s    b
            TERM(13) = FLHA1(-5)*FLHA2(-1*(1-2*IGPP)) ! grcflav: 85, bbar dbar
            TERM(14) = FLHA1(-5)*FLHA2(-3*(1-2*IGPP)) ! grcflav: 86, bbar sbar
            TERM(15) = FLHA1(-1)*FLHA2(-5*(1-2*IGPP)) ! grcflav: 87, dbar bbar
            TERM(16) = FLHA1(-3)*FLHA2(-5*(1-2*IGPP)) ! grcflav: 88, sbar bbar
         ENDIF

         IF (INPFL.EQ.6) THEN
            INPA = 24
            TERM(17) = FLHA1(6)*FLHA2(2*(1-2*IGPP))   ! grcflav: 89, t    u
            TERM(18) = FLHA1(6)*FLHA2(4*(1-2*IGPP))   ! grcflav: 90, t    c
            TERM(19) = FLHA1(2)*FLHA2(6*(1-2*IGPP))   ! grcflav: 91, u    t
            TERM(20) = FLHA1(4)*FLHA2(6*(1-2*IGPP))   ! grcflav: 92, c    t
            TERM(21) = FLHA1(-6)*FLHA2(-2*(1-2*IGPP)) ! grcflav: 93, tbar ubar
            TERM(22) = FLHA1(-6)*FLHA2(-4*(1-2*IGPP)) ! grcflav: 94, tbar cbar
            TERM(23) = FLHA1(-2)*FLHA2(-6*(1-2*IGPP)) ! grcflav: 95, ubar tbar
            TERM(24) = FLHA1(-4)*FLHA2(-6*(1-2*IGPP)) ! grcflav: 96, cbar tbar
         ENDIF
      ENDIF
#endif
 
C      IF (INPA.EQ.10.AND.TERM(9).EQ.0.0D0) THEN
C         WRITE(*,*) 'Error: PDF, wrong num. of flavor(5).'
C         STOP
C      ELSE IF (INPA.EQ.12.AND.TERM(11).EQ.0.0D0) THEN
C         WRITE(*,*) 'Error: PDF, wrong num. of flavor(6).'
C         STOP
C      ELSE IF (INPA.EQ.16.AND.TERM(9).EQ.0.0D0) THEN
C         WRITE(*,*) 'Error: PDF, wrong num. of flavor(5).'
C         STOP
C      ELSE IF (INPA.EQ.24.AND.TERM(17).EQ.0.0D0) THEN
C         WRITE(*,*) 'Error: PDF, wrong num. of flavor(6).'
C         STOP
C      ENDIF
 
      SUM = 0.0D0
      DO I = 1,INPA
         SUM = SUM + TERM(I)
      ENDDO

C...Reject very small/large x
      IF (SUM.EQ.0.0D0) THEN
C         write(*,*) 'reject',sum,GPRI(5),GPRI(6),GRCFAQ
         GO TO 999
      ENDIF
 
C...Select one of the possible terms at random.
      RANSUM = RNA*SUM
      ISEL = 0
      DO WHILE (RANSUM.GT.0.0D0)
         ISEL = ISEL + 1
         RANSUM = RANSUM - TERM(ISEL)
      ENDDO

      IF (ISEL.EQ.0) THEN
         WRITE(*,*) 'GRC::Error Mismatch intial flavor, ISEL=0 .'
         GO TO 999
      ENDIF

      IF (FLAGPA.EQ.1) THEN
         GRCFLAV = ISEL + 60
      ELSE IF (FLAGPA.EQ.2) THEN
         GRCFLAV = ISEL + 72
      ELSE
         WRITE(*,*) 'GRC::Error: Strange random num. for PDF.',RNA
         GO TO 999
      ENDIF
      YACOB = SUM*YACOB

C...Set flavor
      CALL GRCFFIL
 
      RETURN
 999  CONTINUE
      IREJ = 2
      YACOB = 0.0D0
      RETURN
      END

C-----------------------------------------
C Initial up(down)-quark down(up)-quark'.
C-----------------------------------------
      SUBROUTINE GRCPRTNQQS
      IMPLICIT REAL*8(A-H,O-Z)
 
#include "define.h"
 
      INCLUDE 'grchad.inc'
 
#if PSMODEL > 0
C...Parameters.
      COMMON/PYPARS/MSTP(200),PARP(200),MSTI(200),PARI(200)
#endif
 
C...Proton Variable
      INTEGER I
      REAL*8 TERM(36)

C...PYTHIA PDF
      REAL*8 XPP(-25:25),XPPBAR(-25:25)
C...LHA PDF
      REAL*8 FLHA1(-6:6),FLHA2(-6:6)
 
C...Grace Variable
      INTEGER ISEL,INPA
      REAL*8 SUM,RANSUM,RNA

      IF (GPRI(5).GT.1.OR.GPRI(6).GT.1) THEN
         IREJ = 1
         GO TO 999
      ENDIF

#if PSMODEL > 0
      IF (INPFL.NE.MSTP(58)) THEN
         WRITE(*,*) 'Mismacth Nr. of flavor in PDF: INPFL/MSTP(58): ',
     $        INPFL,'/',MSTP(58)
         WRITE(*,*) 'processed as INPFL:',INPFL
         MSTP(58) = INPFL
      ENDIF
#endif

#if PDFLIB == 1
C...Internal PDF from PYTHIA
C...External PDF
      IF (IGPP.EQ.0) THEN
         CALL PYPDFU( 2212,GPRI(5),GRCFAQ**2,XPP)
         CALL PYPDFU( 2212,GPRI(6),GRCFAQ**2,XPPBAR)
      ELSE IF (IGPP.EQ.1) THEN
         CALL PYPDFU( 2212,GPRI(5),GRCFAQ**2,XPP)
         CALL PYPDFU(-2212,GPRI(6),GRCFAQ**2,XPPBAR)
      ENDIF
         
C...Loop over quark flavours (up to b).
      INPA = 16
      TERM(1)  = XPP(2)*XPPBAR(1)   ! grcflav:  97, u     d
      TERM(2)  = XPP(2)*XPPBAR(3)   ! grcflav:  98, u     s
      TERM(3)  = XPP(4)*XPPBAR(1)   ! grcflav:  99, c     d
      TERM(4)  = XPP(4)*XPPBAR(3)   ! grcflav: 100, c     s
      TERM(5)  = XPP(1)*XPPBAR(2)   ! grcflav: 101, d     u
      TERM(6)  = XPP(1)*XPPBAR(4)   ! grcflav: 102, d     c
      TERM(7)  = XPP(3)*XPPBAR(2)   ! grcflav: 103, s     u
      TERM(8)  = XPP(3)*XPPBAR(4)   ! grcflav: 104, s     c
      TERM(9)  = XPP(-2)*XPPBAR(-1) ! grcflav: 105, u-bar d-bar
      TERM(10) = XPP(-2)*XPPBAR(-3) ! grcflav: 106, u-bar s-bar
      TERM(11) = XPP(-4)*XPPBAR(-1) ! grcflav: 107, c-bar d-bar
      TERM(12) = XPP(-4)*XPPBAR(-3) ! grcflav: 108, c-bar s-bar
      TERM(13) = XPP(-1)*XPPBAR(-2) ! grcflav: 109, d-bar u-bar
      TERM(14) = XPP(-1)*XPPBAR(-4) ! grcflav: 110, d-bar c-bar
      TERM(15) = XPP(-3)*XPPBAR(-2) ! grcflav: 111, s-bar u-bar
      TERM(16) = XPP(-3)*XPPBAR(-4) ! grcflav: 112, s-bar c-bar

      IF (INPFL.GT.4) THEN
         INPA = 24
         TERM(17) = XPP(2)*XPPBAR(5)   ! grcflav: 113, u     b
         TERM(18) = XPP(4)*XPPBAR(5)   ! grcflav: 114, c     b
         TERM(19) = XPP(5)*XPPBAR(2)   ! grcflav: 115, b     u
         TERM(20) = XPP(5)*XPPBAR(4)   ! grcflav: 116, b     c
         TERM(21) = XPP(-2)*XPPBAR(-5) ! grcflav: 117, u-bar b-bar
         TERM(22) = XPP(-4)*XPPBAR(-5) ! grcflav: 118, c-bar b-bar
         TERM(23) = XPP(-5)*XPPBAR(-2) ! grcflav: 119, b-bar u-bar
         TERM(24) = XPP(-5)*XPPBAR(-4) ! grcflav: 120, b-bar c-bar
      ENDIF

      IF (INPFL.EQ.6) THEN
         INPA = 36
         TERM(25) = XPP(6)*XPPBAR(1)   ! grcflav: 121, t     d
         TERM(26) = XPP(6)*XPPBAR(3)   ! grcflav: 122, t     s
         TERM(27) = XPP(6)*XPPBAR(5)   ! grcflav: 123, t     b
         TERM(28) = XPP(1)*XPPBAR(6)   ! grcflav: 124, d     t
         TERM(29) = XPP(3)*XPPBAR(6)   ! grcflav: 125, s     t
         TERM(30) = XPP(5)*XPPBAR(6)   ! grcflav: 126, b     t
         TERM(31) = XPP(-6)*XPPBAR(-1) ! grcflav: 127, t-bar d-bar
         TERM(32) = XPP(-6)*XPPBAR(-3) ! grcflav: 128, t-bar s-bar
         TERM(33) = XPP(-6)*XPPBAR(-5) ! grcflav: 129, t-bar b-bar
         TERM(34) = XPP(-1)*XPPBAR(-6) ! grcflav: 130, d-bar t-bar
         TERM(35) = XPP(-3)*XPPBAR(-6) ! grcflav: 131, s-bar t-bar
         TERM(36) = XPP(-5)*XPPBAR(-6) ! grcflav: 132, b-bar t-bar
      ENDIF

#elif PDFLIB == 2
C...External PDF
      IF (IGPP.EQ.0) THEN
         CALL STRUCTM(GPRI(5),GRCFAQ,
     $        UPV1,DNV1,USEA1,DSEA1,STR1,CHM1,BOT1,TOP1,GL1)
         CALL STRUCTM(GPRI(6),GRCFAQ,
     $        UPV2,DNV2,USEA2,DSEA2,STR2,CHM2,BOT2,TOP2,GL2)
         UPV1 = UPV1 + USEA1
         DNV1 = DNV1 + DSEA1
         UPV2 = UPV2 + USEA2
         DNV2 = DNV2 + DSEA2
      ELSE IF (IGPP.EQ.1) THEN
         CALL STRUCTM(GPRI(5),GRCFAQ,
     $        UPV1,DNV1,USEA1,DSEA1,STR1,CHM1,BOT1,TOP1,GL1)
         CALL STRUCTM(GPRI(6),GRCFAQ,
     $        USEA2,DSEA2,UPV2,DNV2,STR2,CHM2,BOT2,TOP2,GL2)
         UPV1 = UPV1 + USEA1
         DNV1 = DNV1 + DSEA1
         USEA2 = UPV2 + USEA2
         DSEA2 = DNV2 + DSEA2
      ENDIF

      INPA = 16
      TERM(1)  = UPV1*DNV2      ! grcflav:  97, u     d
      TERM(2)  = UPV1*STR2      ! grcflav:  98, u     s
      TERM(3)  = CHM1*DNV2      ! grcflav:  99, c     d
      TERM(4)  = CHM1*STR2      ! grcflav: 100, c     s
      TERM(5)  = DNV1*UPV2      ! grcflav: 101, d     u
      TERM(6)  = DNV1*CHM2      ! grcflav: 102, d     c
      TERM(7)  = STR1*UPV2      ! grcflav: 103, s     u
      TERM(8)  = STR1*CHM2      ! grcflav: 104, s     c
      TERM(9)  = USEA1*DSEA2    ! grcflav: 105, u-bar d-bar
      TERM(10) = USEA1*STR2     ! grcflav: 106, u-bar s-bar
      TERM(11) = CHM1*DSEA2     ! grcflav: 107, c-bar d-bar
      TERM(12) = CHM1*STR2      ! grcflav: 108, c-bar s-bar
      TERM(13) = DSEA1*USEA2    ! grcflav: 109, d-bar u-bar
      TERM(14) = DSEA1*CHM2     ! grcflav: 110, d-bar c-bar
      TERM(15) = STR1*USEA2     ! grcflav: 111, s-bar u-bar
      TERM(16) = STR1*CHM2      ! grcflav: 112, s-bar c-bar


      IF (INPFL.GT.4) THEN
         INPA = 24
         TERM(17) = UPV1*BOT2   ! grcflav: 113, u     b
         TERM(18) = CHM1*BOT2   ! grcflav: 114, c     b
         TERM(19) = BOT1*UPV2   ! grcflav: 115, b     u
         TERM(20) = BOT1*CHM2   ! grcflav: 116, b     c
         TERM(21) = USEA1*BOT2  ! grcflav: 117, u-bar b-bar
         TERM(22) = CHM1*BOT2   ! grcflav: 118, c-bar b-bar
         TERM(23) = BOT1*USEA2  ! grcflav: 119, b-bar u-bar
         TERM(24) = BOT1*CHM2   ! grcflav: 120, b-bar c-bar
      ENDIF

      IF (INPFL.EQ.6) THEN
         INPA = 36
         TERM(25) = TOP1*DNV2   ! grcflav: 121, t     d
         TERM(26) = TOP1*STR2   ! grcflav: 122, t     s
         TERM(27) = TOP1*BOT2   ! grcflav: 123, t     b
         TERM(28) = DNV1*TOP2   ! grcflav: 124, d     t
         TERM(29) = STR1*TOP2   ! grcflav: 125, s     t
         TERM(30) = BOT1*TOP2   ! grcflav: 126, b     t
         TERM(31) = TOP1*DSEA2  ! grcflav: 127, t-bar d-bar
         TERM(32) = TOP1*STR2   ! grcflav: 128, t-bar s-bar
         TERM(33) = TOP1*BOT2   ! grcflav: 129, t-bar b-bar
         TERM(34) = DSEA1*TOP2  ! grcflav: 130, d-bar t-bar
         TERM(35) = STR1*TOP2   ! grcflav: 131, s-bar t-bar
         TERM(36) = BOT1*TOP2   ! grcflav: 132, b-bar t-bar
      ENDIF

#elif PDFLIB == 3
C...LHA PDF
      CALL evolvePDF(GPRI(5),GRCFAQ,FLHA1)
      CALL evolvePDF(GPRI(6),GRCFAQ,FLHA2)

C...Loop over quark flavours (up to b).
      INPA = 16
      TERM(1)  = FLHA1(2)*FLHA2(1*(1-2*IGPP))   ! grcflav:  97, u     d
      TERM(2)  = FLHA1(2)*FLHA2(3*(1-2*IGPP))   ! grcflav:  98, u     s
      TERM(3)  = FLHA1(4)*FLHA2(1*(1-2*IGPP))   ! grcflav:  99, c     d
      TERM(4)  = FLHA1(4)*FLHA2(3*(1-2*IGPP))   ! grcflav: 100, c     s
      TERM(5)  = FLHA1(1)*FLHA2(2*(1-2*IGPP))   ! grcflav: 101, d     u
      TERM(6)  = FLHA1(1)*FLHA2(4*(1-2*IGPP))   ! grcflav: 102, d     c
      TERM(7)  = FLHA1(3)*FLHA2(2*(1-2*IGPP))   ! grcflav: 103, s     u
      TERM(8)  = FLHA1(3)*FLHA2(4*(1-2*IGPP))   ! grcflav: 104, s     c
      TERM(9)  = FLHA1(-2)*FLHA2(-1*(1-2*IGPP)) ! grcflav: 105, u-bar d-bar
      TERM(10) = FLHA1(-2)*FLHA2(-3*(1-2*IGPP)) ! grcflav: 106, u-bar s-bar
      TERM(11) = FLHA1(-4)*FLHA2(-1*(1-2*IGPP)) ! grcflav: 107, c-bar d-bar
      TERM(12) = FLHA1(-4)*FLHA2(-3*(1-2*IGPP)) ! grcflav: 108, c-bar s-bar
      TERM(13) = FLHA1(-1)*FLHA2(-2*(1-2*IGPP)) ! grcflav: 109, d-bar u-bar
      TERM(14) = FLHA1(-1)*FLHA2(-4*(1-2*IGPP)) ! grcflav: 110, d-bar c-bar
      TERM(15) = FLHA1(-3)*FLHA2(-2*(1-2*IGPP)) ! grcflav: 111, s-bar u-bar
      TERM(16) = FLHA1(-3)*FLHA2(-4*(1-2*IGPP)) ! grcflav: 112, s-bar c-bar

      IF (INPFL.GT.4) THEN
         INPA = 24
         TERM(17) = FLHA1(2)*FLHA2(5*(1-2*IGPP))   ! grcflav: 113, u     b
         TERM(18) = FLHA1(4)*FLHA2(5*(1-2*IGPP))   ! grcflav: 114, c     b
         TERM(19) = FLHA1(5)*FLHA2(2*(1-2*IGPP))   ! grcflav: 115, b     u
         TERM(20) = FLHA1(5)*FLHA2(4*(1-2*IGPP))   ! grcflav: 116, b     c
         TERM(21) = FLHA1(-2)*FLHA2(-5*(1-2*IGPP)) ! grcflav: 117, u-bar b-bar
         TERM(22) = FLHA1(-4)*FLHA2(-5*(1-2*IGPP)) ! grcflav: 118, c-bar b-bar
         TERM(23) = FLHA1(-5)*FLHA2(-2*(1-2*IGPP)) ! grcflav: 119, b-bar u-bar
         TERM(24) = FLHA1(-5)*FLHA2(-4*(1-2*IGPP)) ! grcflav: 120, b-bar c-bar
      ENDIF

      IF (INPFL.EQ.6) THEN
         INPA = 36
         TERM(25) = FLHA1(6)*FLHA2(1*(1-2*IGPP))   ! grcflav: 121, t     d
         TERM(26) = FLHA1(6)*FLHA2(3*(1-2*IGPP))   ! grcflav: 122, t     s
         TERM(27) = FLHA1(6)*FLHA2(5*(1-2*IGPP))   ! grcflav: 123, t     b
         TERM(28) = FLHA1(1)*FLHA2(6*(1-2*IGPP))   ! grcflav: 124, d     t
         TERM(29) = FLHA1(3)*FLHA2(6*(1-2*IGPP))   ! grcflav: 125, s     t
         TERM(30) = FLHA1(5)*FLHA2(6*(1-2*IGPP))   ! grcflav: 126, b     t
         TERM(31) = FLHA1(-6)*FLHA2(-1*(1-2*IGPP)) ! grcflav: 127, t-bar d-bar
         TERM(32) = FLHA1(-6)*FLHA2(-3*(1-2*IGPP)) ! grcflav: 128, t-bar s-bar
         TERM(33) = FLHA1(-6)*FLHA2(-5*(1-2*IGPP)) ! grcflav: 129, t-bar b-bar
         TERM(34) = FLHA1(-1)*FLHA2(-6*(1-2*IGPP)) ! grcflav: 130, d-bar t-bar
         TERM(35) = FLHA1(-3)*FLHA2(-6*(1-2*IGPP)) ! grcflav: 131, s-bar t-bar
         TERM(36) = FLHA1(-5)*FLHA2(-6*(1-2*IGPP)) ! grcflav: 132, b-bar t-bar
      ENDIF

#endif
 
C      IF (INPA.EQ.24.AND.TERM(17).EQ.0.0D0) THEN
C         WRITE(*,*) 'Error: PDF, wrong num. of flavor(5).'
C         STOP
C      ELSE IF (INPA.EQ.36.AND.TERM(25).EQ.0.0D0) THEN
C         WRITE(*,*) 'Error: PDF, wrong num. of flavor(6).'
C         STOP
C      ENDIF
 
      SUM = 0.0D0
      DO I = 1,INPA
         SUM = SUM + TERM(I)
      ENDDO

C...Reject very small/large x
      IF (SUM.EQ.0.0D0) THEN
C         write(*,*) 'reject',sum,GPRI(5),GPRI(6),GRCFAQ
         GO TO 999
      ENDIF
 
C...Select one of the possible terms at random.
      RNA = GPRI(10)
      RANSUM = RNA*SUM
      ISEL = 0
      DO WHILE (RANSUM.GT.0.0D0)
         ISEL = ISEL + 1
         RANSUM = RANSUM - TERM(ISEL)
      ENDDO

      IF (ISEL.EQ.0) THEN
         WRITE(*,*) 'GRC::Error Mismatch intial flavor, ISEL=0 .'
         GO TO 999
      ENDIF

      GRCFLAV = ISEL + 96
      YACOB = SUM*YACOB

C...Set flavor
      CALL GRCFFIL
 
      RETURN
 999  CONTINUE
      IREJ = 2
      YACOB = 0.0D0
      RETURN
      END

C--------------------------------------
C Set flavors.
C Set kinematics of initial particles.
C--------------------------------------
      SUBROUTINE GRCFFIL
      IMPLICIT REAL*8(A-H,O-Z)
 
      INCLUDE 'inclk.h'
      INCLUDE 'grchad.inc'

      INTEGER I
      REAL*8 E1,E2,TMPCKMR(30)
      INTEGER IDCYST,IJBR,IJNF,IDJBR(30),IDJBR2(30),IDJBR3(30)
      REAL*8 SUMJBR,XS1,XS2,XSS,RNA

      INTEGER IKFGP(157),IKFGA(157),IGPCOMB(157)
      INTEGER IGENE(6),IPARA(6)

C...large matrix for flavor combination.
      DATA IKFGP/21, 2, 1, 4, 3,-2,-1,-4,-3, 5,-5, 6,-6, 2, 4,-1,
     $           -3,-2,-4, 1, 3, 2, 4,-3,-1,-2,-4, 3, 1, 2, 4,-5,
     $           -5,-2,-4, 5, 5, 2, 1, 4, 3,-2,-1,-4,-3,21,21,21,
     $           21,21,21,21,21, 5,-5,21,21, 6,-6,21,21, 2, 1, 4,
     $            3,-2,-1,-4,-3, 5,-5, 6,-6, 2, 1, 4, 3,-2,-1,-4,
     $           -3, 5, 5, 1, 3,-5,-5,-1,-3, 6, 6, 2, 4,-6,-6,-2,
     $           -4, 2, 2, 4, 4, 1, 1, 3, 3,-2,-2,-4,-4,-1,-1,-3,
     $           -3, 2, 4, 5, 5,-2,-4,-5,-5, 6, 6, 6, 1, 3, 5,-6,
     $           -6,-6,-1,-3,-5, 2, 1, 4, 3,-2,-1,-4,-3, 1, 3, 5,
     $            5,-5,-5,-1,-3, 2, 4, 6, 6,-2,-4,-6,-6/
      DATA IKFGA/21,-2,-1,-4,-3, 2, 1, 4, 3,-5, 5,-6, 6,-1,-3, 2, 
     $            4, 1, 3,-2,-4,-3,-1, 2, 4, 3, 1,-2,-4,-5,-5, 2, 
     $            4, 5, 5,-2,-4,21,21,21,21,21,21,21,21, 2, 1, 4, 
     $            3,-2,-1,-4,-3,21,21, 5,-5,21,21, 6,-6, 2, 1, 4,
     $            3,-2,-1,-4,-3, 5,-5, 6,-6, 4, 3, 2, 1,-4,-3,-2,
     $           -1, 1, 3, 5, 5,-1,-3,-5,-5, 2, 4, 6, 6,-2,-4,-6,
     $           -6, 1, 3, 1, 3, 2, 4, 2, 4,-1,-3,-1,-3,-2,-4,-2,
     $           -4, 5, 5, 2, 4,-5,-5,-2,-4, 1, 3, 5, 6, 6, 6,-1,
     $           -3,-5,-6,-6,-6,-4,-3,-2,-1, 4, 3, 2, 1,-5,-5,-1,
     $           -3, 1, 3, 5, 5,-6,-6,-2,-4, 6, 6, 2, 4/

C...Symmetric conversion array
      DATA IGPCOMB/0,0,0,0,0,1,1,1,1,0,1,0,1,0,0,1,1,0,0,1,1,0,0,1,1,
     $             0,0,1,1,0,0,1,1,0,0,1,1,0,0,0,0,0,0,0,0,1,1,1,1,1,
     $             1,1,1,0,0,1,1,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
     $             0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
     $             0,1,1,1,1,0,0,0,0,1,1,1,1,0,0,1,1,0,0,1,1,0,0,0,1,
     $             1,1,0,0,0,1,1,1,0,0,0,0,1,1,1,1,0,0,0,0,1,1,1,1,0,
     $             0,0,0,1,1,1,1/

C...Diagonal partner.
      DATA IPARA/4,3,2,1,6,5/

C...quark generation.
      DATA IGENE/1,1,2,2,3,3/

C...Masses, Charge*3 and KFcode of initial state particles
      KFCODE(1) = IKFGP(GRCFLAV+1)
      KFCODE(2) = IKFGA(GRCFLAV+1)

      DO I = 1,2
         IF (KFCODE(I).EQ.21) THEN
            KCHARG(I) = 0
         ELSE IF (KFCODE(I).GT.0
     $           .AND.MOD(ABS(KFCODE(I)),2).EQ.0) THEN
            KCHARG(I) = 2
         ELSE IF (KFCODE(I).GT.0
     $           .AND.MOD(ABS(KFCODE(I)),2).EQ.1) THEN
            KCHARG(I) = -1
         ELSE IF (KFCODE(I).LT.0
     $           .AND.MOD(ABS(KFCODE(I)),2).EQ.0) THEN
            KCHARG(I) = -2
         ELSE IF (KFCODE(I).LT.0
     $           .AND.MOD(ABS(KFCODE(I)),2).EQ.1) THEN
            KCHARG(I) = 1
         ELSE
            WRITE(*,*) 'GRC::Error: Strange flavor in KFCODE(',I,').'
         ENDIF
      ENDDO
 
      IF (GRCFLAV.EQ.0) THEN
         AMASS1(1) = AMG
         AMASS1(2) = AMG
      ELSE IF (GRCFLAV.LT.13) THEN
         AMASS1(1) = GWDCYMS(ABS(KFCODE(1)))
         AMASS1(2) = GWDCYMS(ABS(KFCODE(2)))
         AMUQ(1) = GWDCYMS(ABS(KFCODE(1)))
      ELSE IF (GRCFLAV.LT.37) THEN
         IF (MOD(ABS(KFCODE(1)),2).EQ.0) THEN
            YACOB = YACOB*
     $           GRCCKM(IGENE(ABS(KFCODE(1))),IGENE(ABS(KFCODE(2))))
            AMUQ(1) = AMUQ(IGENE(ABS(KFCODE(1))))
            AMDQ(1) = AMDQ(IGENE(ABS(KFCODE(2))))
         ELSE
            YACOB = YACOB*
     $           GRCCKM(IGENE(ABS(KFCODE(2))),IGENE(ABS(KFCODE(1))))
            AMUQ(1) = AMUQ(IGENE(ABS(KFCODE(2))))
            AMDQ(1) = AMDQ(IGENE(ABS(KFCODE(1))))
         ENDIF
         AMASS1(1) = GWDCYMS(ABS(KFCODE(1)))
         AMASS1(2) = GWDCYMS(ABS(KFCODE(2)))
      ELSE IF (GRCFLAV.LT.61) THEN
         IF (KFCODE(1).EQ.21) THEN
            AMASS1(1) = AMG
            AMASS1(2) = GWDCYMS(ABS(KFCODE(2)))
            AMUQ(1) = AMASS1(2)
         ELSE
            AMASS1(1) = GWDCYMS(ABS(KFCODE(1)))
            AMASS1(2) = AMG
            AMUQ(1) = AMASS1(1)
         ENDIF
         AMDQ(1) = GWDCYMS(ABS(KFCODE(5)))
      ELSE IF (GRCFLAV.LT.157) THEN
         AMASS1(1) = GWDCYMS(ABS(KFCODE(1)))
         AMASS1(2) = GWDCYMS(ABS(KFCODE(2)))
      ENDIF

      AMASS2(1) = AMASS1(1)**2
      AMASS2(2) = AMASS1(2)**2
 
C...Set Initial Energy and Momentum
      E1 = (GPRI(3)+AMASS2(1)-AMASS2(2))/2.0D0/GPRI(4)
      E2 = (GPRI(3)+AMASS2(2)-AMASS2(1))/2.0D0/GPRI(4)

      IF (E1.LT.AMASS1(1).OR.E2.LT.AMASS1(2)) THEN
         IREJ = 1
         GO TO 999
      ENDIF
      IF (0.5D0*GPRI(5)*GRCECM.LT.AMASS1(1)) THEN
         IREJ = 3
         GO TO 999
      ENDIF
      IF (0.5D0*GPRI(6)*GRCECM.LT.AMASS1(2)) THEN
         IREJ = 3
         GO TO 999
      ENDIF

      IF (GRCFLAV.LT.133) THEN
         IF (IGPCOMB(GRCFLAV+1).EQ.1) THEN
            TMPMAS = AMASS1(1)
            AMASS1(1) = AMASS1(2)
            AMASS1(2) = TMPMAS
            AMASS2(1) = AMASS1(1)**2
            AMASS2(2) = AMASS1(2)**2
         ENDIF
      ENDIF

C...Decide jet flavor
      IF (IGSUB.EQ.422.OR.IGSUB.EQ.424.OR.IGSUB.EQ.431.OR.IGSUB.EQ.439
     $     .OR.IGSUB.EQ.464.OR.IGSUB.EQ.465) THEN

         IF (IGSUB.EQ.422.OR.IGSUB.EQ.464) THEN
            RNA = XRUN(10)
         ELSE IF (IGSUB.EQ.424.OR.IGSUB.EQ.465) THEN
            RNA = XRUN(13)
         ELSE IF (IGSUB.EQ.431) THEN
            RNA = XRUN(16)
         ELSE IF (IGSUB.EQ.439) THEN
            RNA = XRUN(19)
         ENDIF

         CALL GRCFJQG(RNA)
         AMDQ(1) = AMASS1(5)

      ELSE IF (IGSUB.EQ.425.OR.IGSUB.EQ.433.OR.IGSUB.EQ.467) THEN

C...Initialize
         IJBR = 0
         SUMJBR = 0.0
         DO I = 1,5
            IDJBR(I) = 0
            IDJBR2(I) = 0
            TMPCKMR(I) = 0.0D0
         ENDDO

C...Set ratio for jet flavor configuration.
C...Without CKM
         IF (GRCCKM(1,1).EQ.1.0D0.AND.GRCCKM(1,2).EQ.0.0D0) THEN
            XS1 = 1.0D0
            XS2 = 0.0D0
C...With CKM
         ELSE
            XS1 = 10.0D0
            XS2 = 1.0D0
         ENDIF
         XSS = XS1/(XS1 + XS2)

         IF (IGSUB.EQ.425.OR.IGSUB.EQ.467) THEN
            RNA = XRUN(13)
         ELSE IF (IGSUB.EQ.433) THEN
            RNA = XRUN(16)
         ENDIF

         IF (RNA.LE.XSS) THEN
C...Same jet flavor (no CKM)
            RNA = RNA/XSS
            YACOB = YACOB/XSS

            DO I = 1,5
               IF (IGJFLV(I).EQ.1) THEN
                  IJBR = IJBR + 1
                  IDJBR(IJBR) = I
                  TMPCKMR(IJBR) = 1.0D0
                  SUMJBR = TMPCKMR(IJBR) + SUMJBR
               ENDIF
            ENDDO
            IF (IJBR.EQ.0) THEN
               IREJ = 3
               GO TO 999
            ENDIF

C...Pick up final state flavor.
            RANSUM = RNA*SUMJBR
            YACOB = SUMJBR*YACOB
            IDCYST = 0
            DO WHILE (RANSUM.GT.0.0D0)
               IDCYST = IDCYST + 1
               RANSUM = RANSUM - TMPCKMR(IDCYST)
            ENDDO

            IF (KFCODE(1).GT.0) THEN
               KFCODE(5) = IDJBR(IDCYST)
               KFCODE(6) = -IDJBR(IDCYST)
            ELSE
               KFCODE(5) = -IDJBR(IDCYST)
               KFCODE(6) = IDJBR(IDCYST)
            ENDIF

         ELSE
C...CKM jet
            RNA = (RNA - XSS)/(1.0D0 - XSS)
            YACOB = YACOB/(1.0D0 - XSS)
            IF (MOD(ABS(KFCODE(1)),2).EQ.0) THEN
               YACOB = YACOB/
     $              GRCCKM(IGENE(ABS(KFCODE(1))),IGENE(ABS(KFCODE(2))))
            ELSE
               YACOB = YACOB/
     $              GRCCKM(IGENE(ABS(KFCODE(2))),IGENE(ABS(KFCODE(1))))
            ENDIF

            IF (MOD(ABS(KFCODE(1)),2).EQ.0) THEN

C...up-type jet
               IF (IGJFLV(ABS(KFCODE(1))).EQ.1
     $              .AND.IGJFLV(IPARA(ABS(KFCODE(2)))).EQ.1) THEN
                  IJBR = IJBR + 1
                  IDJBR(IJBR) = KFCODE(1)
                  IF (KFCODE(1).GT.0) THEN
                     IDJBR2(IJBR) = -IPARA(ABS(KFCODE(2)))
                  ELSE
                     IDJBR2(IJBR) = IPARA(ABS(KFCODE(2)))
                  ENDIF
                  TMPCKMR(IJBR) = GRCCKM(IGENE(ABS(IDJBR2(IJBR))),
     $                 IGENE(ABS(KFCODE(2))))
                  SUMJBR = TMPCKMR(IJBR) + SUMJBR
               ENDIF

C...down-type jet
               IF (IGJFLV(ABS(KFCODE(2))).EQ.1
     $              .AND.IGJFLV(IPARA(ABS(KFCODE(1)))).EQ.1) THEN
                  IJBR = IJBR + 1
                  IDJBR2(IJBR) = KFCODE(2)
                  IF (KFCODE(2).GT.0) THEN
                     IDJBR(IJBR) = -IPARA(ABS(KFCODE(1)))
                  ELSE
                     IDJBR(IJBR) = IPARA(ABS(KFCODE(1)))
                  ENDIF
                  TMPCKMR(IJBR) = GRCCKM(IGENE(ABS(KFCODE(1))),
     $                 IGENE(ABS(IDJBR(IJBR))))
                  SUMJBR = TMPCKMR(IJBR) + SUMJBR
               ENDIF

C...b-quark jet
               IF (IGJFLV(ABS(KFCODE(2))).EQ.1
     $              .AND.IGJFLV(5).EQ.1) THEN
                  IJBR = IJBR + 1
                  IDJBR2(IJBR) = KFCODE(2)
                  IF (KFCODE(2).GT.0) THEN
                     IDJBR(IJBR) = -5
                  ELSE
                     IDJBR(IJBR) = 5
                  ENDIF
                  TMPCKMR(IJBR) = GRCCKM(IGENE(ABS(KFCODE(1))),3)
                  SUMJBR = TMPCKMR(IJBR) + SUMJBR
               ENDIF

            ELSE

C...down-type jet
               IF (IGJFLV(ABS(KFCODE(1))).EQ.1
     $              .AND.IGJFLV(IPARA(ABS(KFCODE(2)))).EQ.1) THEN
                  IJBR = IJBR + 1
                  IDJBR(IJBR) = KFCODE(1)
                  IF (KFCODE(1).GT.0) THEN
                     IDJBR2(IJBR) = -IPARA(ABS(KFCODE(2)))
                  ELSE
                     IDJBR2(IJBR) = IPARA(ABS(KFCODE(2)))
                  ENDIF
                  TMPCKMR(IJBR) = GRCCKM(IGENE(ABS(KFCODE(2))),
     $                 IGENE(ABS(IDJBR2(IJBR))))
                  SUMJBR = TMPCKMR(IJBR) + SUMJBR
               ENDIF

C...b-quark jet
               IF (IGJFLV(ABS(KFCODE(1))).EQ.1
     $              .AND.IGJFLV(5).EQ.1) THEN
                  IJBR = IJBR + 1
                  IDJBR(IJBR) = KFCODE(1)
                  IF (KFCODE(1).GT.0) THEN
                     IDJBR2(IJBR) = -5
                  ELSE
                     IDJBR2(IJBR) = 5
                  ENDIF
                  TMPCKMR(IJBR) = GRCCKM(IGENE(ABS(KFCODE(2))),3)
                  SUMJBR = TMPCKMR(IJBR) + SUMJBR
               ENDIF

C...up-type jet
               IF (IGJFLV(ABS(KFCODE(2))).EQ.1
     $              .AND.IGJFLV(IPARA(ABS(KFCODE(1)))).EQ.1) THEN
                  IJBR = IJBR + 1
                  IDJBR2(IJBR) = KFCODE(2)
                  IF (KFCODE(2).GT.0) THEN
                     IDJBR(IJBR) = -IPARA(ABS(KFCODE(1)))
                  ELSE
                     IDJBR(IJBR) = IPARA(ABS(KFCODE(1)))
                  ENDIF
                  TMPCKMR(IJBR) = GRCCKM(IGENE(ABS(IDJBR(IJBR))),
     $                 IGENE(ABS(KFCODE(1))))
                  SUMJBR = TMPCKMR(IJBR) + SUMJBR
               ENDIF
            ENDIF

            IF (IJBR.EQ.0) THEN
               IREJ = 3
               GO TO 999
            ENDIF

C...Pick up final state flavor.
            RANSUM = RNA*SUMJBR
            YACOB = SUMJBR*YACOB
            IDCYST = 0
            DO WHILE (RANSUM.GT.0.0D0)
               IDCYST = IDCYST + 1
               RANSUM = RANSUM - TMPCKMR(IDCYST)
            ENDDO

            KFCODE(5) = IDJBR(IDCYST)
            KFCODE(6) = IDJBR2(IDCYST)

         ENDIF

         AMASS1(5) = GWDCYMS(ABS(KFCODE(5)))
         AMASS1(6) = GWDCYMS(ABS(KFCODE(6)))
         AMASS2(5) = AMASS1(5)**2
         AMASS2(6) = AMASS1(6)**2

      ELSE IF (IGSUB.EQ.427.OR.IGSUB.EQ.435) THEN

C...Set ratio for jet flavor configuration.
C...Without CKM
         IF (GRCCKM(1,1).EQ.1.0D0.AND.GRCCKM(1,2).EQ.0.0D0) THEN
            XS1 = 1.0D0
            XS2 = 0.0D0
C...With CKM
         ELSE
            XS1 = 10.0D0
            XS2 = 1.0D0
         ENDIF
         XSS = XS1/(XS1 + XS2)

         IF (IGSUB.EQ.427) THEN
            RNA = XRUN(13)
         ELSE IF (IGSUB.EQ.435) THEN
            RNA = XRUN(16)
         ENDIF

         IF (RNA.LE.XSS) THEN
C...Diagonal component of CKM
            RNA = RNA/XSS
            YACOB = YACOB/XSS
            CALL GRCFJQQ(2,RNA)
         ELSE
C...Off-Diagonal component of CKM
            RNA = (RNA - XSS)/(1.0D0 - XSS)
            YACOB = YACOB/(1.0D0 - XSS)
            CALL GRCFJQQ(3,RNA)
         ENDIF
         AMUQ(1)   = AMASS1(5)
         AMDQ(1)   = AMASS1(6)

      ELSE IF (IGSUB.EQ.428.OR.IGSUB.EQ.436) THEN

C...Set ratio for jet flavor configuration.
C...Without CKM
         IF (GRCCKM(1,1).EQ.1.0D0.AND.GRCCKM(1,2).EQ.0.0D0) THEN
            XS1 = 1.0D0
            XS2 = 0.0D0
C...With CKM
         ELSE
            XS1 = 10.0D0
            XS2 = 1.0D0
         ENDIF
         XSS = XS1/(XS1 + XS2)

         IF (IGSUB.EQ.428) THEN
            RNA = XRUN(13)
         ELSE IF (IGSUB.EQ.436) THEN
            RNA = XRUN(16)
         ENDIF

         IF (ABS(KFCODE(1)).EQ.ABS(KFCODE(2))) THEN
            IF (RNA.LE.XSS) THEN
C...Diagonal component of CKM
               RNA = RNA/XSS
               YACOB = YACOB/XSS
               CALL GRCFJQQ(2,RNA)
            ELSE
C...Off-Diagonal component of CKM
               RNA = (RNA - XSS)/(1.0D0 - XSS)
               YACOB = YACOB/(1.0D0 - XSS)
               CALL GRCFJQQ(3,RNA)
            ENDIF
         ELSE
            IF (RNA.LE.XSS) THEN
C...Diagonal component of CKM
               RNA = RNA/XSS
               YACOB = YACOB/XSS
               CALL GRCFJQQ(4,RNA)
            ELSE
C...Off-Diagonal component of CKM
               RNA = (RNA - XSS)/(1.0D0 - XSS)
               YACOB = YACOB/(1.0D0 - XSS)
               CALL GRCFJQQ(5,RNA)
            ENDIF
         ENDIF

      ELSE IF (IGSUB.EQ.429.OR.IGSUB.EQ.437) THEN
C...Initialize
         IJBR = 0
         IJNF = 0
         SUMJBR = 0.0
         DO I = 1,6
            IDJBR(I) = 0
            TMPCKMR(I) = 0.0D0
         ENDDO

         IF (IGSUB.EQ.429) THEN
            RNA = XRUN(13)
         ELSE IF (IGSUB.EQ.437) THEN
            RNA = XRUN(16)
         ENDIF

C...Down type jet flavor
         IF(KFCODE(1).EQ.KFCODE(2)
     $        .AND.MOD(ABS(KFCODE(1)),2).EQ.0) THEN
            DO I = 1,3
               IF (IGJFLV(2*I-1).EQ.1) THEN
                  IJBR = IJBR + 1
                  IDJBR(IJBR) = 2*I-1
                  TMPCKMR(IJBR) = GRCCKM(IGENE(ABS(KFCODE(1))),I)
                  SUMJBR = TMPCKMR(IJBR) + SUMJBR
               ENDIF
            ENDDO
            IJNF = 1
C...Up type jet flavor
         ELSE IF(KFCODE(1).EQ.KFCODE(2)
     $           .AND.MOD(ABS(KFCODE(1)),2).EQ.1) THEN
            DO I = 1,2
               IF (IGJFLV(2*I).EQ.1) THEN
                  IJBR = IJBR + 1
                  IDJBR(IJBR) = 2*I
                  TMPCKMR(IJBR) = GRCCKM(I,IGENE(ABS(KFCODE(1))))
                  SUMJBR = TMPCKMR(IJBR) + SUMJBR
               ENDIF
            ENDDO
            IJNF = 1
C...Down type jet flavor
         ELSE IF(MOD(ABS(KFCODE(1)),2).EQ.0) THEN
            DO J = 1,2
               DO I = 1,3
                  IF (IGJFLV(2*I-1).EQ.1) THEN
                     IJBR = IJBR + 1
                     IDJBR(IJBR) = 2*I-1
                     TMPCKMR(IJBR) = GRCCKM(IGENE(ABS(KFCODE(J))),I)
                     SUMJBR = TMPCKMR(IJBR) + SUMJBR
                  ENDIF
               ENDDO
            ENDDO
            IJNF = 2
C...Up type jet flavor
         ELSE IF(MOD(ABS(KFCODE(1)),2).EQ.1) THEN
            DO J = 1,2
               DO I = 1,2
                  IF (IGJFLV(2*I).EQ.1) THEN
                     IJBR = IJBR + 1
                     IDJBR(IJBR) = 2*I
                     TMPCKMR(IJBR) = GRCCKM(I,IGENE(ABS(KFCODE(J))))
                     SUMJBR = TMPCKMR(IJBR) + SUMJBR
                  ENDIF
               ENDDO
            ENDDO
            IJNF = 2
         ELSE
            WRITE(*,*) 'GRC::Error: Not supported top content in PDF.'
            IREJ = 3
            STOP
         ENDIF

         IF (IJBR.EQ.0) THEN
            IREJ = 3
            GO TO 999
         ENDIF

C...Pick up final state flavor.
         RANSUM = RNA*SUMJBR
         YACOB = SUMJBR*YACOB
         IDCYST = 0
         DO WHILE (RANSUM.GT.0.0D0)
            IDCYST = IDCYST + 1
            RANSUM = RANSUM - TMPCKMR(IDCYST)
         ENDDO

         IF (IDCYST.LE.IJBR/IJNF) THEN
            IF(MOD(ABS(KFCODE(1)),2).EQ.0) THEN
               KFCODE(5) = ABS(KFCODE(1))
               KFCODE(6) = IDJBR(IDCYST)
               AMDQ(1)   = AMASS1(6)
            ELSE IF(MOD(ABS(KFCODE(1)),2).EQ.1) THEN
               KFCODE(5) = IDJBR(IDCYST)
               KFCODE(6) = ABS(KFCODE(1))
               AMUQ(1)   = AMASS1(5)
            ENDIF
         ELSE IF (IDCYST.GT.IJBR/IJNF) THEN
            IF(MOD(ABS(KFCODE(1)),2).EQ.0) THEN
               KFCODE(5) = ABS(KFCODE(2))
               KFCODE(6) = IDJBR(IDCYST)
               AMDQ(1)   = AMASS1(6)
            ELSE IF(MOD(ABS(KFCODE(1)),2).EQ.1) THEN
               KFCODE(5) = IDJBR(IDCYST)
               KFCODE(6) = ABS(KFCODE(2))
               AMUQ(1)   = AMASS1(5)
            ENDIF
         ENDIF

         AMASS1(5) = GWDCYMS(ABS(KFCODE(5)))
         AMASS1(6) = GWDCYMS(ABS(KFCODE(6)))
         AMASS2(5) = AMASS1(5)**2
         AMASS2(6) = AMASS1(6)**2

      ELSE IF (IGSUB.EQ.430.OR.IGSUB.EQ.438) THEN
C...Initialize
         IJBR = 0
         SUMJBR = 0.0
         DO I = 1,5
            IDJBR(I) = 0
            IDJBR2(I) = 0
            TMPCKMR(I) = 0.0D0
         ENDDO

         IF (IGSUB.EQ.430) THEN
            RNA = XRUN(13)
         ELSE IF (IGSUB.EQ.438) THEN
            RNA = XRUN(16)
         ENDIF

C...Scan d/u/s/c/b
C...Decide Down-type jet.
         DO I = 1,3
            IF (IGJFLV(2*I-1).EQ.1) THEN
               IJBR = IJBR + 1
               IDJBR(IJBR) = 2*I-1
               IF (MOD(ABS(KFCODE(1)),2).EQ.0) THEN
                  TMPCKMR(IJBR) = 
     $                 GRCCKM(IGENE(ABS(KFCODE(1))),IGENE(2*I-1))
                  IDJBR2(IJBR) = KFCODE(2)
               ELSE IF (MOD(ABS(KFCODE(2)),2).EQ.0) THEN
                  TMPCKMR(IJBR) = 
     $                 GRCCKM(IGENE(ABS(KFCODE(2))),IGENE(2*I-1))
                  IDJBR2(IJBR) = KFCODE(1)
               ENDIF
               SUMJBR = TMPCKMR(IJBR) + SUMJBR
            ENDIF
         ENDDO
C...Decide Up-type jet.
         DO I = 1,2
            IF (IGJFLV(2*I).EQ.1) THEN
               IJBR = IJBR + 1
               IDJBR(IJBR) = 2*I
               IF (MOD(ABS(KFCODE(1)),2).EQ.1) THEN
                  TMPCKMR(IJBR) = 
     $                 GRCCKM(IGENE(2*I),IGENE(ABS(KFCODE(1))))
                  IDJBR2(IJBR) = KFCODE(2)
               ELSE IF (MOD(ABS(KFCODE(2)),2).EQ.1) THEN
                  TMPCKMR(IJBR) = 
     $                 GRCCKM(IGENE(2*I),IGENE(ABS(KFCODE(2))))
                  IDJBR2(IJBR) = KFCODE(1)
               ENDIF
               SUMJBR = TMPCKMR(IJBR) + SUMJBR
            ENDIF
         ENDDO

CCCC...If pick up the same flavor jet.
CCC         IJBR = 2
CCC         IDJBR2(1) = KFCODE(2)
CCC         IDJBR(1)  = KFCODE(2)
CCC         TMPCKMR(1) = 
CCC     $        GRCCKM(IGENE(ABS(KFCODE(1))),IGENE(ABS(KFCODE(2))))
CCC         IDJBR2(2) = KFCODE(1)
CCC         IDJBR(2)  = KFCODE(1)
CCC         TMPCKMR(2) = 
CCC     $        GRCCKM(IGENE(ABS(KFCODE(1))),IGENE(ABS(KFCODE(2))))
CCC         SUMJBR = TMPCKMR(1) + TMPCKMR(2)

         IF (IJBR.EQ.0) THEN
            WRITE(*,*) 'GRC::Error: No allowed jet flavor.'
            IREJ = 3
            GO TO 999
         ENDIF

C...Pick up final state flavor.
         RANSUM = RNA*SUMJBR
         YACOB = SUMJBR*YACOB
         IDCYST = 0
         DO WHILE (RANSUM.GT.0.0D0)
            IDCYST = IDCYST + 1
            RANSUM = RANSUM - TMPCKMR(IDCYST)
         ENDDO

         IF (IDCYST.EQ.0) THEN
            WRITE(*,*) 'GRC::Error strange jet combination.:',IDCYST
            GO TO 999
         ENDIF

         KFCODE(5) = IDJBR2(IDCYST)
         KFCODE(6) = IDJBR(IDCYST)
         IF (KFCODE(5).LT.0) THEN
            KFCODE(6) = -IDJBR(IDCYST)
         ENDIF

CCC         YACOB = GRCCKM(1,2)*YACOB
CCC         KFCODE(5) = -4
CCC         KFCODE(6) = -2

         AMASS1(5) = GWDCYMS(ABS(KFCODE(5)))
         AMASS1(6) = GWDCYMS(ABS(KFCODE(6)))
         AMASS2(5) = AMASS1(5)**2
         AMASS2(6) = AMASS1(6)**2

      ELSE IF (IGSUB.EQ.432) THEN

C...Initialize
         IJBR = 0
         SUMJBR = 0.0
         DO I = 1,30
            IDJBR(I) = 0
            IDJBR2(I) = 0
            IDJBR3(I) = 0
            TMPCKMR(I) = 0.0D0
         ENDDO

C...Initial up-type quark.
         IF (MOD(ABS(MIN(KFCODE(1),KFCODE(2))),2).EQ.0) THEN
C...Scan d/u/s/c/b
C...Decide Down-type jet.
            DO I = 1,3
               IF (IGJFLV(2*I-1).EQ.1) THEN
                  DO J = 1,5
                     IF (IGJFLV(J).EQ.1) THEN
                        IJBR = IJBR + 1
                        IDJBR(IJBR)  = 2*I-1
                        IDJBR2(IJBR) = J
                        IDJBR3(IJBR) = J
                        TMPCKMR(IJBR) = GRCCKM(
     $                       IGENE(ABS(MIN(KFCODE(1),KFCODE(2)))),
     $                       IGENE(2*I-1))
                        SUMJBR = TMPCKMR(IJBR) + SUMJBR
                     ENDIF
                  ENDDO

                  IF (ABS(MIN(KFCODE(1),KFCODE(2))).EQ.2
     $                 .AND.IGJFLV(4).EQ.1) THEN
                     IJBR = IJBR + 1
                     IDJBR(IJBR)  = 2*I-1
                     IDJBR2(IJBR) = 2
                     IDJBR3(IJBR) = 4
                     TMPCKMR(IJBR) = GRCCKM(2,IGENE(2*I-1))
                     SUMJBR = TMPCKMR(IJBR) + SUMJBR
                  ELSE IF (ABS(MIN(KFCODE(1),KFCODE(2))).EQ.4
     $                    .AND.IGJFLV(2).EQ.1) THEN
                     IJBR = IJBR + 1
                     IDJBR(IJBR) = 2*I-1
                     IDJBR2(IJBR) = 4
                     IDJBR3(IJBR) = 2
                     TMPCKMR(IJBR) = GRCCKM(1,IGENE(2*I-1))
                     SUMJBR = TMPCKMR(IJBR) + SUMJBR
                  ELSE
                     WRITE(*,*) 'GRC:Error: Strange flavor in 432.'
                  ENDIF
               ENDIF
            ENDDO
C...Decide Up-type jet.
            IF (IGJFLV(ABS(MIN(KFCODE(1),KFCODE(2)))).EQ.1) THEN
               DO J = 1,2
                  IF (IGJFLV(2*J).EQ.1) THEN
                     DO I = 1,3
                        IF (IGJFLV(2*I-1).EQ.1) THEN
                           IJBR = IJBR + 1
                           IDJBR(IJBR)  = ABS(MIN(KFCODE(1),KFCODE(2)))
                           IDJBR2(IJBR) = 2*J
                           IDJBR3(IJBR) = 2*I-1
                           TMPCKMR(IJBR) = 
     $                          GRCCKM(IGENE(2*J),IGENE(2*I-1))
                           SUMJBR = TMPCKMR(IJBR) + SUMJBR
                        ENDIF
                     ENDDO
                  ENDIF
               ENDDO
            ENDIF
C...Initial down-type quark.
         ELSE IF (MOD(ABS(MIN(KFCODE(1),KFCODE(2))),2).EQ.1) THEN
C...Decide Up-type jet.
            DO I = 1,2
               IF (IGJFLV(2*I).EQ.1) THEN
                  DO J = 1,5
                     IF (IGJFLV(J).EQ.1) THEN
                        IJBR = IJBR + 1
                        IDJBR(IJBR)  = 2*I
                        IDJBR2(IJBR) = J
                        IDJBR3(IJBR) = J
                        TMPCKMR(IJBR) = GRCCKM(
     $                       IGENE(2*I),
     $                       IGENE(ABS(MIN(KFCODE(1),KFCODE(2)))))
                        SUMJBR = TMPCKMR(IJBR) + SUMJBR
                     ENDIF
                  ENDDO

                  IF (ABS(MIN(KFCODE(1),KFCODE(2))).EQ.1) THEN
                     DO J = 1,2
                        IF (IGJFLV(2*J+1).EQ.1) THEN
                           IJBR = IJBR + 1
                           IDJBR(IJBR)  = 2*I
                           IDJBR2(IJBR) = 1
                           IDJBR3(IJBR) = 2*J+1
                           TMPCKMR(IJBR) = 
     $                          GRCCKM(IGENE(2*I),IGENE(2*J+1))
                           SUMJBR = TMPCKMR(IJBR) + SUMJBR
                        ENDIF
                     ENDDO
                  ELSE IF (ABS(MIN(KFCODE(1),KFCODE(2))).EQ.3) THEN
                     DO J = 1,2
                        IF (IGJFLV(4*J-3).EQ.1) THEN
                           IJBR = IJBR + 1
                           IDJBR(IJBR)  = 2*I
                           IDJBR2(IJBR) = 3
                           IDJBR3(IJBR) = 4*J-3
                           TMPCKMR(IJBR) = 
     $                          GRCCKM(IGENE(2*I),IGENE(4*J-3))
                           SUMJBR = TMPCKMR(IJBR) + SUMJBR
                        ENDIF
                     ENDDO
                  ELSE IF (ABS(MIN(KFCODE(1),KFCODE(2))).EQ.5) THEN
                     DO J = 1,2
                        IF (IGJFLV(2*J-1).EQ.1) THEN
                           IJBR = IJBR + 1
                           IDJBR(IJBR)  = 2*I
                           IDJBR2(IJBR) = 5
                           IDJBR3(IJBR) = 2*J-1
                           TMPCKMR(IJBR) = 
     $                          GRCCKM(IGENE(2*I),IGENE(2*J-1))
                           SUMJBR = TMPCKMR(IJBR) + SUMJBR
                        ENDIF
                     ENDDO
                  ENDIF
               ENDIF
            ENDDO
C...Decide Down-type jet.
            IF (IGJFLV(ABS(MIN(KFCODE(1),KFCODE(2)))).EQ.1) THEN
               DO J = 1,3
                  IF (IGJFLV(2*J-1).EQ.1) THEN
                     DO I = 1,2
                        IF (IGJFLV(2*I).EQ.1) THEN
                           IJBR = IJBR + 1
                           IDJBR(IJBR) = ABS(MIN(KFCODE(1),KFCODE(2)))
                           IDJBR2(IJBR) = 2*J-1
                           IDJBR3(IJBR) = 2*I
                           TMPCKMR(IJBR) = 
     $                          GRCCKM(IGENE(2*I),IGENE(2*J-1))
                           SUMJBR = TMPCKMR(IJBR) + SUMJBR
                        ENDIF
                     ENDDO
                  ENDIF
               ENDDO
            ENDIF
         ENDIF

         IF (IJBR.EQ.0) THEN
            WRITE(*,*) 'GRC::Error: No allowed jet flavor.'
            IREJ = 3
            GO TO 999
         ENDIF

C...Pick up final state flavor.
         RANSUM = XRUN(16)*SUMJBR
         YACOB = SUMJBR*YACOB
         IDCYST = 0
         DO WHILE (RANSUM.GT.0.0D0)
            IDCYST = IDCYST + 1
            RANSUM = RANSUM - TMPCKMR(IDCYST)
         ENDDO

         IF (IDCYST.EQ.0) THEN
            WRITE(*,*) 'GRC::Error strange jet combination.:',IDCYST
            GO TO 999
         ENDIF

         KFCODE(5) = IDJBR(IDCYST)
         KFCODE(6) = IDJBR2(IDCYST)
         KFCODE(7) = IDJBR3(IDCYST)

         AMASS1(5) = GWDCYMS(ABS(KFCODE(5)))
         AMASS1(6) = GWDCYMS(ABS(KFCODE(6)))
         AMASS1(7) = GWDCYMS(ABS(KFCODE(7)))
         AMASS2(5) = AMASS1(5)**2
         AMASS2(6) = AMASS1(6)**2
         AMASS2(7) = AMASS1(7)**2

      ELSE IF (IGSUB.EQ.501.OR.IGSUB.EQ.503) THEN
         KFCODE(5) = MIN(KFCODE(1),KFCODE(2))
         AMASS1(5) = GWDCYMS(ABS(KFCODE(5)))
         AMASS2(5) = AMASS1(5)**2

      ELSE IF (IGSUB.EQ.504.OR.IGSUB.EQ.506) THEN

C...Initialize
         IJBR = 0
         SUMJBR = 0.0
         DO I = 1,5
            IDJBR(I) = 0
            TMPCKMR(I) = 0.0D0
         ENDDO

         IF (IGSUB.EQ.504.OR.IGSUB.EQ.506) THEN
            RNA = XRUN(13)
         ENDIF

         IF (KFCODE(1)+KFCODE(2).EQ.0.OR.IGSUB.EQ.506) THEN
C...Same jet flavor (no CKM)
            DO I = 1,5
               IF (IGJFLV(I).EQ.1) THEN
                  IJBR = IJBR + 1
                  IDJBR(IJBR) = I
                  TMPCKMR(IJBR) = 1.0D0
                  SUMJBR = TMPCKMR(IJBR) + SUMJBR
               ENDIF
            ENDDO
            IF (IJBR.EQ.0) THEN
               IREJ = 3
               GO TO 999
            ENDIF

C...Pick up final state flavor.
            RANSUM = RNA*SUMJBR
            YACOB = SUMJBR*YACOB
            IDCYST = 0
            DO WHILE (RANSUM.GT.0.0D0)
               IDCYST = IDCYST + 1
               RANSUM = RANSUM - TMPCKMR(IDCYST)
            ENDDO

            IF (KFCODE(1).GT.0) THEN
               KFCODE(5) = IDJBR(IDCYST)
               KFCODE(6) = -IDJBR(IDCYST)
            ELSE
               KFCODE(5) = -IDJBR(IDCYST)
               KFCODE(6) = IDJBR(IDCYST)
            ENDIF
         ELSE
            KFCODE(5) = KFCODE(1)
            KFCODE(6) = KFCODE(2)
         ENDIF
         AMASS1(5) = GWDCYMS(ABS(KFCODE(5)))
         AMASS1(6) = GWDCYMS(ABS(KFCODE(6)))
         AMASS2(5) = AMASS1(5)**2
         AMASS2(6) = AMASS1(6)**2

      ELSE IF (IGSUB.EQ.507.OR.IGSUB.EQ.508.OR.IGSUB.EQ.509) THEN
C...With CKM
         IF (IGSUB.EQ.507.AND.GRCCKM(1,2).GT.0.0D0) THEN
            IF (MOD(ABS(KFCODE(1)),2).EQ.0) THEN
               YACOB = YACOB/
     $              GRCCKM(IGENE(ABS(KFCODE(1))),IGENE(ABS(KFCODE(2))))
            ELSE
               YACOB = YACOB/
     $              GRCCKM(IGENE(ABS(KFCODE(2))),IGENE(ABS(KFCODE(1))))
            ENDIF
         ENDIF
         KFCODE(5) = KFCODE(1)
         KFCODE(6) = KFCODE(2)
         AMASS1(5) = GWDCYMS(ABS(KFCODE(5)))
         AMASS1(6) = GWDCYMS(ABS(KFCODE(6)))
         AMASS2(5) = AMASS1(5)**2
         AMASS2(6) = AMASS1(6)**2

      ELSE IF (IGSUB.EQ.551) THEN
C...Decide internal propagator.
C...Initialize
         IJBR = 0
         SUMJBR = 0.0
         DO I = 1,5
            IDJBR(I) = 0
            TMPCKMR(I) = 0.0D0
         ENDDO

         RNA = XRUN(12)

         IF (MOD(ABS(KFCODE(1)),2).EQ.0) THEN
C...Decide down-type flavor.
            DO I = 1,3
               IJBR = IJBR + 1
               IDJBR(IJBR) = 2*I-1
               TMPCKMR(IJBR) = GRCCKM(IGENE(ABS(KFCODE(1))),I)
     $              *GRCCKM(IGENE(ABS(KFCODE(2))),I)
               SUMJBR = TMPCKMR(IJBR) + SUMJBR
            ENDDO
         ELSE
C...Decide up-type flavor.
            DO I = 1,2
               IJBR = IJBR + 1
               IDJBR(IJBR) = 2*I
               TMPCKMR(IJBR) = GRCCKM(I,IGENE(ABS(KFCODE(1))))
     $              *GRCCKM(I,IGENE(ABS(KFCODE(2))))
               SUMJBR = TMPCKMR(IJBR) + SUMJBR
            ENDDO
         ENDIF
         IF (IJBR.EQ.0) THEN
            IREJ = 3
            GO TO 999
         ENDIF

C...Pick up internal propagator.
         RANSUM = RNA*SUMJBR
         YACOB = SUMJBR*YACOB
         IDCYST = 0
         DO WHILE (RANSUM.GT.0.0D0)
            IDCYST = IDCYST + 1
            RANSUM = RANSUM - TMPCKMR(IDCYST)
         ENDDO
         AMDQ(1) = GWDCYMS(IDJBR(IDCYST))

      ENDIF

      RETURN
 999  CONTINUE
      IREJ = 3
      YACOB = 0.0D0
      RETURN
      END

C---------------------------------------------------------
C Set Jet flavor for 2 parton jet system.
C Flavors are arbitrary decided.
C     IMODE = 1 : q q-bar
C           = 2 : q q'-bar  Diagonal component of CKM
C           = 3 : q q'-bar  Off-Diagonal component of CKM
C           = 4 : q q'-bar  Forced by initial parton flavor 
C                           with diagonal component of CKM
C           = 5 : q q'-bar  Forced by initial parton flavor 
C                           with off-diagonal component of CKM
C---------------------------------------------------------
      SUBROUTINE GRCFJQQ(IMODE,RNA)
      IMPLICIT REAL*8(A-H,O-Z)
 
      INCLUDE 'inclk.h'
      INCLUDE 'grchad.inc'

      INTEGER IMODE,I,IJBR,IDCYST,IDJBR(12),IDJBR2(12)
      REAL*8 SUMJBR,RNA
      REAL*8 TMPCKMR(12)

      INTEGER ICHGP(12),ICHGA(12)
      INTEGER IKFGP(12),IKFGA(12)
      INTEGER IYBCKM1(12),IYBCKM2(12)
      INTEGER IPART(6),IPARA(6),IGENE(6)

C...Large matrix.
      DATA ICHGP/ 2, 2,-1,-1, 2, 2,-1,-1, 2, 2,-1,-1/
      DATA ICHGA/ 1, 1,-2,-2, 1, 1,-2,-2, 1, 1,-2,-2/
      DATA IKFGP/ 2, 4, 1, 3, 2, 4, 3, 1, 2, 4, 5, 5/
      DATA IKFGA/-1,-3,-2,-4,-3,-1,-2,-4,-5,-5,-2,-4/

      DATA IYBCKM1/1,2,1,2,1,2,1,2,1,2,1,2/
      DATA IYBCKM2/1,2,1,2,2,1,2,1,3,3,3,3/

C...Diagonal partner.
      DATA IPART/2,1,4,3,6,5/
      DATA IPARA/4,3,2,1,6,5/
C...quark generation.
      DATA IGENE/1,1,2,2,3,3/

C...Initialize
      IJBR = 0
      SUMJBR = 0.0
      DO I = 1,12
         IDJBR(I) = 0
         IDJBR2(I) = 0
         TMPCKMR(I) = 0.0D0
      ENDDO

      IF (IGJFLV(6).EQ.1) THEN
         WRITE(*,*) 'GRC::Warning: Not supported top-quark jet in',
     $        IGSUB,'process.'
         WRITE(*,*) 'processed as IGJFLV(6)=0.'
         IGJFLV(6) = 0
      ENDIF

      IF (IMODE.EQ.1) THEN
      ELSE IF (IMODE.EQ.2) THEN
C...Loop over quark flavours (up to b).
C...Diagonal component of CKM
         DO I = 1,4
            IF (IGJFLV(ABS(IKFGP(I))).EQ.1
     $           .AND.IGJFLV(ABS(IKFGA(I))).EQ.1) THEN
               IJBR = IJBR + 1
               IDJBR(IJBR) = I
               TMPCKMR(IJBR) = GRCCKM(IYBCKM1(I),IYBCKM2(I))
               SUMJBR = GRCCKM(IYBCKM1(I),IYBCKM2(I)) + SUMJBR
            ENDIF
         ENDDO
      ELSE IF (IMODE.EQ.3) THEN
C...Off-Diagonal component of CKM
         DO I = 1,8
            IF (IGJFLV(ABS(IKFGP(I+4))).EQ.1
     $           .AND.IGJFLV(ABS(IKFGA(I+4))).EQ.1) THEN
               IJBR = IJBR + 1
               IDJBR(IJBR) = I + 4
               TMPCKMR(IJBR) = GRCCKM(IYBCKM1(I+4),IYBCKM2(I+4))
               SUMJBR = GRCCKM(IYBCKM1(I+4),IYBCKM2(I+4)) + SUMJBR
            ENDIF
         ENDDO
      ELSE IF (IMODE.EQ.4) THEN
C...In the case of Initial different up(down)-type quark combination.
C...Diagonal component of CKM
         DO I = 1,2
            IF (IGJFLV(ABS(KFCODE(I))).EQ.1
     $           .AND.IGJFLV(IPART(ABS(KFCODE(3-I)))).EQ.1) THEN
               IJBR = IJBR + 1
               IDJBR(IJBR) = KFCODE(I)
               IF (KFCODE(I).GT.0) THEN
                  IDJBR2(IJBR) = -IPART(ABS(KFCODE(3-I)))
               ELSE
                  IDJBR2(IJBR) = IPART(ABS(KFCODE(3-I)))
               ENDIF
               TMPCKMR(IJBR) = GRCCKM(IGENE(ABS(KFCODE(3-I))),
     $              IGENE(ABS(KFCODE(3-I))))
               SUMJBR = TMPCKMR(IJBR) + SUMJBR
            ENDIF
         ENDDO
      ELSE IF (IMODE.EQ.5) THEN
C...Off-Diagonal component of CKM
         DO I = 1,2
            IF (IGJFLV(ABS(KFCODE(I))).EQ.1
     $           .AND.IGJFLV(IPARA(ABS(KFCODE(3-I)))).EQ.1) THEN
               IJBR = IJBR + 1
               IDJBR(IJBR) = KFCODE(I)
               IF (KFCODE(I).GT.0) THEN
                  IDJBR2(IJBR) = -IPARA(ABS(KFCODE(3-I)))
               ELSE
                  IDJBR2(IJBR) = IPARA(ABS(KFCODE(3-I)))
               ENDIF
               IF (MOD(ABS(KFCODE(3-I)),2).EQ.0) THEN
                  TMPCKMR(IJBR)=GRCCKM(IGENE(ABS(KFCODE(3-I))),
     $                 IGENE(IPARA(ABS(KFCODE(3-I)))))
               ELSE
                  TMPCKMR(IJBR)=GRCCKM(IGENE(IPARA(ABS(KFCODE(3-I)))),
     $                 IGENE(ABS(KFCODE(3-I))))
               ENDIF
               SUMJBR = TMPCKMR(IJBR) + SUMJBR
            ENDIF
         ENDDO

C...For b-quark jet.
         IF (MOD(ABS(KFCODE(1)),2).EQ.0) THEN
            DO I = 1,2
               IF (IGJFLV(ABS(KFCODE(I))).EQ.1
     $              .AND.IGJFLV(5).EQ.1) THEN
                  IJBR = IJBR + 1
                  IDJBR(IJBR) = KFCODE(I)
                  IF (KFCODE(I).GT.0) THEN
                     IDJBR2(IJBR) = -5
                  ELSE
                     IDJBR2(IJBR) = 5
                  ENDIF
                  TMPCKMR(IJBR) = GRCCKM(IGENE(ABS(KFCODE(3-I))),3)
                  SUMJBR = TMPCKMR(IJBR) + SUMJBR
               ENDIF
            ENDDO
         ENDIF

         IF (MAX(ABS(KFCODE(1)),ABS(KFCODE(2))).EQ.5) THEN
            DO I= 1,2
               IF (IGJFLV(MIN(ABS(KFCODE(1)),ABS(KFCODE(2)))).EQ.1
     $              .AND.IGJFLV(2*I).EQ.1) THEN
                  IJBR = IJBR + 1
                  IF (MAX(KFCODE(1),KFCODE(2)).EQ.5) THEN
                     IDJBR(IJBR) = -MIN(ABS(KFCODE(1)),ABS(KFCODE(2)))
                     IDJBR2(IJBR) = 2*I
                  ELSE
                     IDJBR(IJBR) = MIN(ABS(KFCODE(1)),ABS(KFCODE(2)))
                     IDJBR2(IJBR) = -2*I
                  ENDIF
                  TMPCKMR(IJBR) = GRCCKM(I,3)
                  SUMJBR = TMPCKMR(IJBR) + SUMJBR
               ENDIF
            ENDDO
         ENDIF

      ELSE
         WRITE(*,*) 'GRC::Error: Wrong argument in GRCFJQQ.',IMODE
      ENDIF

      IF (IJBR.EQ.0) THEN
         IREJ = 3
         GO TO 999
      ENDIF

C...Pick up final state flavor.
      RANSUM = RNA*SUMJBR
      YACOB = SUMJBR*YACOB
      IDCYST = 0
      DO WHILE (RANSUM.GT.0.0D0)
         IDCYST = IDCYST + 1
         RANSUM = RANSUM - TMPCKMR(IDCYST)
      ENDDO

      IF (IDCYST.EQ.0) THEN
         WRITE(*,*) 'GRC:strange jet combination.:',IDCYST
         GO TO 999
      ENDIF

      IF (IMODE.LT.4) THEN
         KFCODE(5) = IKFGP(IDJBR(IDCYST))
         KFCODE(6) = IKFGA(IDJBR(IDCYST))
         IF (KFCODE(1).LT.0) THEN
            KFCODE(5) = -IKFGP(IDJBR(IDCYST))
            KFCODE(6) = -IKFGA(IDJBR(IDCYST))
         ENDIF
      ELSE IF (IMODE.LT.6) THEN
         IF (IDJBR(IDCYST).EQ.KFCODE(1)) THEN
            KFCODE(5) = IDJBR(IDCYST)
            KFCODE(6) = IDJBR2(IDCYST)
         ELSE IF (IDJBR(IDCYST).EQ.KFCODE(2)) THEN
            KFCODE(6) = IDJBR(IDCYST)
            KFCODE(5) = IDJBR2(IDCYST)
         ELSE
            WRITE(*,*) 'GRC::Error: Strange jet flavor in GRCFJQQ',
     $           kfcode(1),kfcode(2),kfcode(5),kfcode(6)
         ENDIF
      ENDIF

C      if (mod(abs(kfcode(5)),2).eq.1) then
C         write(*,*) 'w+A',kfcode(1),kfcode(2),kfcode(5),kfcode(6)
C      endif

      AMASS1(5) = GWDCYMS(ABS(KFCODE(5)))
      AMASS2(5) = AMASS1(5)**2
      AMASS1(6) = GWDCYMS(ABS(KFCODE(6)))
      AMASS2(6) = AMASS1(6)**2

      RETURN
 999  CONTINUE
      YACOB = 0.0D0
      RETURN
      END

C-----------------------------------------
C Set Jet flavor for 1 parton jet system.
C One jet should be gluon.
C The other flavor is arbitrary decided.
C-----------------------------------------
      SUBROUTINE GRCFJQG(RNA)
      IMPLICIT REAL*8(A-H,O-Z)
 
      INCLUDE 'inclk.h'
      INCLUDE 'grchad.inc'

      INTEGER I,IJBR,IDCYST,IJNF,IDJBR(3)
      REAL*8 SUMJBR,XS1,XS2,XSS,RNA
      REAL*8 TMPCKMR(3)

C...Initialize
      IJBR = 0
      IJNF = 0
      SUMJBR = 0.0
      DO I = 1,3
         IDJBR(I) = 0
         TMPCKMR(I) = 0.0D0
      ENDDO

C...Check intial parton flavor combination.
      IF (GRCFLAV.GT.36.AND.GRCFLAV.LT.61) THEN
      ELSE
         WRITE(*,*) 'GRC::Error: Wrong intitial flavor in GRCFJQG.'
         GO TO 999
      ENDIF

      IF (GRCFLAV.LT.53) THEN
         IF(MOD(GRCFLAV,4).EQ.1) THEN
C...Down type jet flavor
            DO I = 1,3
               IF (IGJFLV(2*I-1).EQ.1) THEN
                  IJBR = IJBR + 1
                  IDJBR(IJBR) = I
                  SUMJBR = GRCCKM(I,1) + SUMJBR
               ENDIF
            ENDDO
            IJNF = 1
         ELSE IF(MOD(GRCFLAV,4).EQ.2) THEN
C...Up type jet flavor
            DO I = 1,2
               IF (IGJFLV(2*I).EQ.1) THEN
                  IJBR = IJBR + 1
                  IDJBR(IJBR) = I
                  SUMJBR = GRCCKM(I,1) + SUMJBR
               ENDIF
            ENDDO
            IJNF = 1
         ELSE IF(MOD(GRCFLAV,4).EQ.3) THEN
C...Down type jet flavor
            DO I = 1,3
               IF (IGJFLV(2*I-1).EQ.1) THEN
                  IJBR = IJBR + 1
                  IDJBR(IJBR) = I
                  SUMJBR = GRCCKM(I,2) + SUMJBR
               ENDIF
            ENDDO
            IJNF = 2
         ELSE IF(MOD(GRCFLAV,4).EQ.0) THEN
C...Up type jet flavor
            DO I = 1,2
               IF (IGJFLV(2*I).EQ.1) THEN
                  IJBR = IJBR + 1
                  IDJBR(IJBR) = I
                  SUMJBR = GRCCKM(I,2) + SUMJBR
               ENDIF
            ENDDO
            IJNF = 2
         ENDIF
      ELSE IF (GRCFLAV.LT.57) THEN
C...Up type jet flavor
         DO I = 1,2
            IF (IGJFLV(2*I).EQ.1) THEN
               IJBR = IJBR + 1
               IDJBR(IJBR) = I
               SUMJBR = GRCCKM(I,3) + SUMJBR
            ENDIF
         ENDDO
         IJNF = 3
      ELSE
         WRITE(*,*) 'GR@PPA: Not supported top content in PDF.'
         IREJ = 3
         STOP
      ENDIF

      IF (IJBR.GT.0) THEN
         DO I = 1,IJBR
            TMPCKMR(I) = GRCCKM(IDJBR(I),IJNF)
         ENDDO
      ELSE
         IREJ = 3
         GO TO 999
      ENDIF

C...Pick up final state flavor.
      RANSUM = RNA*SUMJBR
      YACOB = SUMJBR*YACOB
      IDCYST = 0
      DO WHILE (RANSUM.GT.0.0D0)
         IDCYST = IDCYST + 1
         RANSUM = RANSUM - TMPCKMR(IDCYST)
      ENDDO

      IF (MOD(GRCFLAV,2).EQ.1) THEN
         KFCODE(5) = 2*IDJBR(IDCYST) - 1
      ELSE
         KFCODE(5) = 2*IDJBR(IDCYST)
      ENDIF

      AMASS1(5) = GWDCYMS(ABS(KFCODE(5)))
      AMASS2(5) = AMASS1(5)**2

      RETURN
 999  CONTINUE
      YACOB = 0.0D0
      RETURN
      END

C-----------------------------------
C Set integration/event generation.
C-----------------------------------
      SUBROUTINE GRCPYGEN(PRT,ISUB,INIT,SIGEV)
      IMPLICIT REAL*8(A-H,O-Z)
 
#include "define.h"

#if PSMODEL == 61
      COMMON/PYDAT3/MDCY(500,3),MDME(4000,2),BRAT(4000),KFDP(4000,5)
#elif PSMODEL == 62
      COMMON/PYDAT3/MDCY(500,3),MDME(8000,2),BRAT(8000),KFDP(8000,5)
#endif

C...Include HEP common parameter.
      INTEGER MAXPUP
      PARAMETER (MAXPUP=100)
      INTEGER IDBMUP,PDFGUP,PDFSUP,IDWTUP,NPRUP,LPRUP
      DOUBLE PRECISION EBMUP,XSECUP,XERRUP,XMAXUP
      COMMON/HEPRUP/IDBMUP(2),EBMUP(2),PDFGUP(2),PDFSUP(2),
     &IDWTUP,NPRUP,XSECUP(MAXPUP),XERRUP(MAXPUP),XMAXUP(MAXPUP),
     &LPRUP(MAXPUP)
      SAVE /HEPRUP/

      INTEGER MAXNUP
      PARAMETER (MAXNUP=500) 
      INTEGER NUP,IDPRUP,IDUP,ISTUP,MOTHUP,ICOLUP
      DOUBLE PRECISION XWGTUP,SCALUP,AQEDUP,AQCDUP,PUP,VTIMUP,SPINUP  
      COMMON/HEPEUP/NUP,IDPRUP,XWGTUP,SCALUP,AQEDUP,AQCDUP,IDUP(MAXNUP),
     &ISTUP(MAXNUP),MOTHUP(2,MAXNUP),ICOLUP(2,MAXNUP),PUP(5,MAXNUP),
     &VTIMUP(MAXNUP),SPINUP(MAXNUP)  
      SAVE /HEPEUP/ 


      PARAMETER (MXDIM = 50, NDMX = 50 )
      COMMON /BPARM1/ XL(MXDIM),XU(MXDIM),NDIM,NWILD,
     $     IG(MXDIM),NCALL
      COMMON /BPARM2/ ACC1,ACC2,ITMX1,ITMX2
 
      INCLUDE 'grchad.inc'
      COMMON /AMJPRC/JPROC
      EXTERNAL FUNC
 
C      INCLUDE 'inclk.h'

      CHARACTER XSTR*14
 
      INTEGER ISUB,INIT
      INTEGER I,J,NI,NJ,MXTRY
      REAL*8 SIGEV,ESTIM,SIGMA,CTIME
      CHARACTER GRCFILE*60,GRCPROC*40,PRT*3
      CHARACTER GRESO*60,GDATA*60,RESO*7,BDAT*5,BLANK*1
 
      CHARACTER*60 GRCBSFILE(64)
      SAVE GRCBSFILE
 
      LOGICAL FOREVER
      PARAMETER (FOREVER=.TRUE.)
      DATA BLANK/' '/, RESO/'.result'/, BDAT/'.data'/
 
      INTEGER ICHCOUP,ICHFAQ,ICHSUB
      INTEGER IDCY,ICHIDCY,ICHNDCY(40),ICHMDCY(40),IDCYTMP
      CHARACTER TMPA*40,TMPB*6,TMPC*2,TMPD*40,OPTF*60,COLP*30
      REAL*8 CHECM,CHPTCUT(6),CHETACUT(6),CHRCONCUT(6),CHMETCUT
      REAL*8 CHBDCY(40)
      REAL*8 CRGET,CHQ2,CHFAQ2,DUMMY
      REAL*8 XDUMMY
 
C...Set Subprocess ID
      IGSUB = ISUB
 
C...Initialization of BASES.
      IBSINT = INIT

#ifndef GRCDEBUG
      CALL BSINIT
      CALL GRCPYPRO(ISUB,GRCFILE,GRCPROC)
      CALL GRCBSIT
#endif

C...Integration (ibsint=1)
      IF (IBSINT.EQ.1) THEN
 
C...For Debug
C+-------------------------------------------------+
#ifdef GRCDEBUG
         CALL BSINIT
         CALL GRCPYPRO(ISUB,GRCFILE,GRCPROC)
         CALL GRCBSIT
#endif
C+-------------------------------------------------+

C...Making Title.
         CALL GRCLOGO

C...Make Decay Table.
         CALL SETMAS
         CALL GRCSDCY

C...Init ICHMDCY(I)
         IF (IGSUB.EQ.408.OR.IGSUB.EQ.410.OR.IGSUB.EQ.411
     $        .OR.IGSUB.EQ.412.OR.IGSUB.EQ.414.OR.IGSUB.EQ.415
     $        .OR.IGSUB.EQ.419.OR.IGSUB.EQ.420
     $        .OR.IGSUB.EQ.421.OR.IGSUB.EQ.422.OR.IGSUB.EQ.423
     $        .OR.IGSUB.EQ.424.OR.IGSUB.EQ.425.OR.IGSUB.EQ.426
     $        .OR.IGSUB.EQ.427.OR.IGSUB.EQ.428.OR.IGSUB.EQ.429
     $        .OR.IGSUB.EQ.430.OR.IGSUB.EQ.431.OR.IGSUB.EQ.432
     $        .OR.IGSUB.EQ.433.OR.IGSUB.EQ.434.OR.IGSUB.EQ.435
     $        .OR.IGSUB.EQ.436.OR.IGSUB.EQ.437.OR.IGSUB.EQ.438
     $        .OR.IGSUB.EQ.439
     $        .OR.IGSUB.EQ.500.OR.IGSUB.EQ.501.OR.IGSUB.EQ.502
     $        .OR.IGSUB.EQ.503.OR.IGSUB.EQ.504.OR.IGSUB.EQ.505
     $        .OR.IGSUB.EQ.506.OR.IGSUB.EQ.507.OR.IGSUB.EQ.508
     $        .OR.IGSUB.EQ.509
     $        .OR.IGSUB.EQ.550.OR.IGSUB.EQ.551.OR.IGSUB.EQ.552
     $        .OR.IGSUB.EQ.560.OR.IGSUB.EQ.561
     $        .OR.IGSUB.EQ.101.OR.IGSUB.EQ.102.OR.IGSUB.EQ.103
     $        .OR.IGSUB.EQ.111.OR.IGSUB.EQ.112) THEN
            ICHIDCY = 0
            DO I = 1,40
               ICHNDCY(I) = 0
               ICHMDCY(I) = 10
               CHBDCY(I) = 0
            ENDDO
         ENDIF
 
C...Set Collision Particles
         WRITE(*,*) 'Accepted CM Energy :', GRCECM, 'GeV'
         IF (PRT(1:2).EQ.'PP') THEN
            IGPP = 0
            WRITE(*,*) '         Beam type : ',
     $           'Proton-Proton Collision\n'
         ELSE IF (PRT(1:2).EQ.'PA') THEN
            IGPP = 1
            WRITE(*,*) '         Beam type : ',
     $           'Proton-Anti-Proton Collision\n'
         ELSE
            WRITE(*,*) 'GR@PPA: Wrong parameter. Put PP/PAP.'
         ENDIF
 
C...Set BASES Output File Name
         J = 0
 100     J = J + 1
         IF (GRCFILE(J+1:J+1).NE.BLANK) GO TO 100
         GRESO = GRCFILE(1:J)//RESO
         GDATA = GRCFILE(1:J)//BDAT
 
         GRCBSFILE(JPROC) = GDATA
 
C...Make BASES File (ibswrt=0)
         IF (IBSWRT.EQ.0) THEN
 
            CALL GRCIKIN
 
            WRITE(*,*) 'Process : [',IGSUB,'] ',GRCPROC,'\n'
            WRITE(*,*) 'Set BASES file'
            WRITE(*,*) '   Filename : ',GRESO
            WRITE(*,*) '              ',GDATA,'\n'
 
            IF (KTYPE.LT.2) THEN
               NI = 2
            ELSE IF (KTYPE.LT.5) THEN
               NI = 3
            ELSE IF (KTYPE.LT.10) THEN
               NI = 4
            ELSE IF (KTYPE.LT.13) THEN
               NI = 5
            ELSE IF (KTYPE.EQ.13) THEN
               NI = 6
            ENDIF
 
            WRITE(*,*) NI,' body final state'
            DO I=1,NI
               IF (GRCPTCUT(I).GT.0.0) THEN
                  WRITE(*,*) '\tPt Cut of Particle',I,
     $                 ' : ',GRCPTCUT(I),' GeV'
               ELSE
                  WRITE(*,*) '\tPt Cut of Particle',I,
     $                 ' : none'
               ENDIF
            ENDDO
            DO I=1,NI
               IF (GRCETACUT(I).LT.100.0) THEN
                  WRITE(*,*) '\tEta Cut of Particle',I,
     $                 ' : ',GRCETACUT(I)
               ELSE
                  WRITE(*,*) '\tEta Cut of Particle',I,
     $                 ' : none'
               ENDIF
            ENDDO
            DO I=1,NI
               IF (GRCRCONCUT(I).GT.0.0) THEN
                  WRITE(*,*) '\tRcone Cut of Particle',I,
     $                 ' : ',GRCRCONCUT(I)
               ELSE
                  WRITE(*,*) '\tRcone Cut of Particle',I,
     $                 ' : none'
               ENDIF
            ENDDO
            IF (GRCMETCUT.GT.0.0) THEN
               WRITE(*,*) '\tMissing Pt(Et) Cut : ',GRCMETCUT,' GeV\n'
            ELSE
               WRITE(*,*) '\tMissing Pt(Et) Cut : none\n'
            ENDIF
            IF (ICOUP.EQ.5) THEN
               WRITE(*,*) '\tOption of Renormalization Scale :',
     $              ICOUP,GRCQ,'GeV'
            ELSE
               WRITE(*,*) '\tOption of Renormalization Scale :',ICOUP
            ENDIF
 
            IF (IFACT.EQ.0) THEN
               WRITE(*,*) '\tOption of   Factorization Scale :',IFACT
            ELSE IF (IFACT.EQ.5) THEN
               WRITE(*,*) '\tOption of   Factorization Scale :',
     $              IFACT,GRCFAQ,'GeV'
            ELSE
               WRITE(*,*) '\tOption of   Factorization Scale :',IFACT
            ENDIF

#if PSMODEL > 0
            IF (IGSUB.EQ.408.OR.IGSUB.EQ.412.OR.IGSUB.EQ.419
     $           .OR.IGSUB.EQ.420.OR.IGSUB.EQ.421.OR.IGSUB.EQ.422
     $           .OR.IGSUB.EQ.423.OR.IGSUB.EQ.424.OR.IGSUB.EQ.425
     $           .OR.IGSUB.EQ.426.OR.IGSUB.EQ.427.OR.IGSUB.EQ.428
     $           .OR.IGSUB.EQ.429.OR.IGSUB.EQ.430.OR.IGSUB.EQ.431
     $           .OR.IGSUB.EQ.432.OR.IGSUB.EQ.433.OR.IGSUB.EQ.434
     $           .OR.IGSUB.EQ.435.OR.IGSUB.EQ.436.OR.IGSUB.EQ.437
     $           .OR.IGSUB.EQ.438.OR.IGSUB.EQ.439
     $           .OR.IGSUB.EQ.551.OR.IGSUB.EQ.552
     $           .OR.IGSUB.EQ.560.OR.IGSUB.EQ.561
     $           .OR.IGSUB.EQ.101.OR.IGSUB.EQ.102
     $           .OR.IGSUB.EQ.103) THEN
               WRITE(*,*)
               DO I = 1,MDCY(24,3)
                  IF (MDME(189+I,1).EQ.1
     $                 .AND.BRAT(189+I).GT.0.000) THEN
                     WRITE(*,*) 'W Forced Decay :',
     $                    189+I,MDME(189+I,1),BRAT(189+I)
                  ENDIF
               ENDDO
            ENDIF

            IF (IGSUB.EQ.410.OR.IGSUB.EQ.411.OR.IGSUB.EQ.414
     $           .OR.IGSUB.EQ.415.OR.IGSUB.EQ.500.OR.IGSUB.EQ.501
     $           .OR.IGSUB.EQ.502.OR.IGSUB.EQ.503.OR.IGSUB.EQ.504
     $           .OR.IGSUB.EQ.505.OR.IGSUB.EQ.506.OR.IGSUB.EQ.507
     $           .OR.IGSUB.EQ.508.OR.IGSUB.EQ.509
     $           .OR.IGSUB.EQ.550.OR.IGSUB.EQ.552
     $           .OR.IGSUB.EQ.111.OR.IGSUB.EQ.112) THEN
               WRITE(*,*)
               DO I = 1,MDCY(23,3)
                  IF (MDME(173+I,1).EQ.1
     $                 .AND.BRAT(173+I).GT.0.000) THEN
                     WRITE(*,*) 'Z Forced Decay :',
     $                    173+I,MDME(173+I,1),BRAT(173+I)
                  ENDIF
               ENDDO
            ENDIF
#endif
 
            OPEN(11,FILE = GRESO,
     $           STATUS='unknown',FORM='formatted')
            DO I = 1, NDIM
               WRITE(XSTR, 120) I
 120           FORMAT('x(',I2,') spectrum')
               CALL XHINIT(I, XL(I), XU(I), 50, XSTR)
            ENDDO
 
C...BASES Integration
            WRITE(*,*)
            WRITE(*,*) 'Start BASES Integration!!'
            CALL BASES( FUNC, ESTIM, SIGMA, CTIME, IT1, IT2 )
            GRCSIG(JPROC) = ESTIM
 
            CALL BSINFO(11)
            CALL BHPLOT(11)
 
            WRITE(11,*) 'Summary Table'
            WRITE(11,*) '============='
 
            IF (IGPP.EQ.0) THEN
               WRITE(11,30) 'CM Energy :', GRCECM, 'GeV'
               WRITE(11,32) 'Beam type :','Proton-Proton Collision'
            ELSE IF (IGPP.EQ.1) THEN
               WRITE(11,30) 'CM Energy :', GRCECM, 'GeV'
               WRITE(11,32) 'Beam type :','Proton-Anti-Proton Collision'
            ENDIF
 30         FORMAT(A14,F10.3,A4)
 32         FORMAT(A14,A30)
 
            WRITE(11,*)
            WRITE(11,35) 'Process : [',IGSUB,'] ',GRCPROC
 35         FORMAT(A14,I3,A2,A40)
 
            WRITE(11,*)
            WRITE(11,40) NI,'body final state'
 40         FORMAT(I4,A17)
            DO I=1,NI
               WRITE(11,50) 'Pt Cut of Particle',I,
     $              ':',GRCPTCUT(I),'GeV'
            ENDDO
 50         FORMAT(A25,I2,A2,F7.3,A4)
            DO I=1,NI
               WRITE(11,60) 'Eta Cut of Particle',I,
     $              ':',GRCETACUT(I)
            ENDDO
            DO I=1,NI
               WRITE(11,60) 'Rcone Cut of Particle',I,
     $              ':',GRCRCONCUT(I)
            ENDDO
 60         FORMAT(A25,I2,A2,F7.3)
            WRITE(11,70) 'Missing Pt(Et) Cut :',GRCMETCUT,'GeV'
 70         FORMAT(A29,F7.3,A4)
            WRITE(11,*)
            IF (ICOUP.EQ.5) THEN
               WRITE(11,75) 'Option of Renormalization Scale :',
     $              ICOUP,GRCQ,'GeV'
 75            FORMAT(A35,I3,F8.3,A6)
            ELSE
               WRITE(11,80) 'Option of Renormalization Scale :',ICOUP
 80            FORMAT(A35,I3)
            ENDIF
 
            IF (IFACT.EQ.0) THEN
               WRITE(11,80) 'Option of   Factorization Scale :',IFACT
 83            FORMAT(A35,I3)
            ELSE IF (IFACT.EQ.5) THEN
               WRITE(11,75) 'Option of   Factorization Scale :',
     $              IFACT,GRCFAQ,'GeV'
 85            FORMAT(A35,I3,F8.3,A6)
            ELSE
               WRITE(11,80) 'Option of   Factorization Scale :',IFACT
 87            FORMAT(A35,I3)
            ENDIF

#if PSMODEL > 0
            IF (IGSUB.EQ.408.OR.IGSUB.EQ.412.OR.IGSUB.EQ.419
     $           .OR.IGSUB.EQ.420.OR.IGSUB.EQ.421.OR.IGSUB.EQ.422
     $           .OR.IGSUB.EQ.423.OR.IGSUB.EQ.424.OR.IGSUB.EQ.425
     $           .OR.IGSUB.EQ.426.OR.IGSUB.EQ.427.OR.IGSUB.EQ.428
     $           .OR.IGSUB.EQ.429.OR.IGSUB.EQ.430.OR.IGSUB.EQ.431
     $           .OR.IGSUB.EQ.432.OR.IGSUB.EQ.433.OR.IGSUB.EQ.434
     $           .OR.IGSUB.EQ.435.OR.IGSUB.EQ.436.OR.IGSUB.EQ.437
     $           .OR.IGSUB.EQ.438.OR.IGSUB.EQ.439
     $           .OR.IGSUB.EQ.551
     $           .OR.IGSUB.EQ.560.OR.IGSUB.EQ.561
     $           .OR.IGSUB.EQ.101.OR.IGSUB.EQ.102
     $           .OR.IGSUB.EQ.103) THEN
               IDCY = 0
               DO I = 1,MDCY(24,3)
                  IF (MDME(189+I,1).EQ.1
     $                 .AND.BRAT(189+I).GT.0.000) THEN
                     IDCY = IDCY + 1
                  ENDIF
               ENDDO
               WRITE(11,*)
               WRITE(11,88) 'W Forced decay :',IDCY

               DO I = 1,MDCY(24,3)
                  IF (MDME(189+I,1).EQ.1
     $                 .AND.BRAT(189+I).GT.0.000) THEN
                     WRITE(11,89) ' :',
     $                    189+I,MDME(189+I,1),BRAT(189+I)
                  ENDIF
               ENDDO
            ENDIF

            IF (IGSUB.EQ.410.OR.IGSUB.EQ.411.OR.IGSUB.EQ.414
     $           .OR.IGSUB.EQ.415.OR.IGSUB.EQ.500.OR.IGSUB.EQ.501
     $           .OR.IGSUB.EQ.502.OR.IGSUB.EQ.503.OR.IGSUB.EQ.504
     $           .OR.IGSUB.EQ.505.OR.IGSUB.EQ.506.OR.IGSUB.EQ.507
     $           .OR.IGSUB.EQ.508.OR.IGSUB.EQ.509
     $           .OR.IGSUB.EQ.550
     $           .OR.IGSUB.EQ.111.OR.IGSUB.EQ.112) THEN
               IDCY = 0
               DO I = 1,MDCY(23,3)
                  IF (MDME(173+I,1).EQ.1
     $                 .AND.BRAT(173+I).GT.0.000) THEN
                     IDCY = IDCY + 1
                  ENDIF
               ENDDO
               WRITE(11,*)
               WRITE(11,88) 'Z Forced decay :',IDCY

               DO I = 1,MDCY(23,3)
                  IF (MDME(173+I,1).EQ.1
     $                 .AND.BRAT(173+I).GT.0.000) THEN
                     WRITE(11,89) ' :',
     $                    173+I,MDME(173+I,1),BRAT(173+I)
                  ENDIF
               ENDDO
            ENDIF

            IF (IGSUB.EQ.552) THEN
               IDCY = 0
               DO I = 1,MDCY(23,3)
                  IF (MDME(173+I,1).EQ.1
     $                 .AND.BRAT(173+I).GT.0.000) THEN
                     IDCY = IDCY + 1
                  ENDIF
               ENDDO
               DO I = 1,MDCY(24,3)
                  IF (MDME(189+I,1).EQ.1
     $                 .AND.BRAT(189+I).GT.0.000) THEN
                     IDCY = IDCY + 1
                  ENDIF
               ENDDO

               WRITE(11,*)
               WRITE(11,88) 'Z and W Forced decay :',IDCY

               DO I = 1,MDCY(23,3)
                  IF (MDME(173+I,1).EQ.1
     $                 .AND.BRAT(173+I).GT.0.000) THEN
                     WRITE(11,89) ' :',
     $                    173+I,MDME(173+I,1),BRAT(173+I)
                  ENDIF
               ENDDO
               DO I = 1,MDCY(24,3)
                  IF (MDME(189+I,1).EQ.1
     $                 .AND.BRAT(189+I).GT.0.000) THEN
                     WRITE(11,89) ' :',
     $                    189+I,MDME(189+I,1),BRAT(189+I)
                  ENDIF
               ENDDO
            ENDIF

 88         FORMAT(A20,I5)
 89         FORMAT(A20,2I5,F8.3)
#endif

            WRITE(11,*)
            WRITE(11,*) ' Num. of Mapping for Integration : ', INBS
            WRITE(11,*) ' Num. of Variable Rejection      : ', INKV
            WRITE(11,*) ' Num. of Cut Rejection           : ', INKC
            WRITE(11,*) ' Num. of Kinematical Rejection   : ', IN4M
            WRITE(11,*) ' Num. of Additional Rejection    : ', INAC
            WRITE(11,*) '==========================================='
            WRITE(11,90) ' Total Cross Section : ', GRCSIG(JPROC), ' pb'
 90         FORMAT(A24,D15.6,A5)
            CLOSE(11)
 
            OPEN(23,FILE = GDATA,
     $           STATUS='unknown',FORM='unformatted')
            CALL BSWRIT(23)
            CLOSE (23)
 
C...Read BASES File (ibswrt=1)
         ELSE IF (IBSWRT.EQ.1) THEN
C...Initialize of Kinematics
            CALL GRCIKIN

#if PSMODEL > 0
C...Check forced decay
            IF (IGSUB.EQ.408.OR.IGSUB.EQ.412.OR.IGSUB.EQ.419
     $           .OR.IGSUB.EQ.420.OR.IGSUB.EQ.421.OR.IGSUB.EQ.422
     $           .OR.IGSUB.EQ.423.OR.IGSUB.EQ.424.OR.IGSUB.EQ.425
     $           .OR.IGSUB.EQ.426.OR.IGSUB.EQ.427.OR.IGSUB.EQ.428
     $           .OR.IGSUB.EQ.429.OR.IGSUB.EQ.430.OR.IGSUB.EQ.431
     $           .OR.IGSUB.EQ.432.OR.IGSUB.EQ.433.OR.IGSUB.EQ.434
     $           .OR.IGSUB.EQ.435.OR.IGSUB.EQ.436.OR.IGSUB.EQ.437
     $           .OR.IGSUB.EQ.438.OR.IGSUB.EQ.439
     $           .OR.IGSUB.EQ.551.OR.IGSUB.EQ.552
     $           .OR.IGSUB.EQ.560.OR.IGSUB.EQ.561
     $           .OR.IGSUB.EQ.101.OR.IGSUB.EQ.102
     $           .OR.IGSUB.EQ.103) THEN
               IDCYTMP = 0
               DO I = 1,MDCY(24,3)
                  IF (MDME(189+I,1).EQ.1
     $                 .AND.BRAT(189+I).GT.0.000) THEN
                     IDCYTMP = IDCYTMP + 1
                  ENDIF
               ENDDO
            ENDIF
 
            IF (IGSUB.EQ.410.OR.IGSUB.EQ.411.OR.IGSUB.EQ.414
     $           .OR.IGSUB.EQ.415.OR.IGSUB.EQ.500.OR.IGSUB.EQ.501
     $           .OR.IGSUB.EQ.502.OR.IGSUB.EQ.503.OR.IGSUB.EQ.504
     $           .OR.IGSUB.EQ.505.OR.IGSUB.EQ.506.OR.IGSUB.EQ.507
     $           .OR.IGSUB.EQ.508.OR.IGSUB.EQ.509
     $           .OR.IGSUB.EQ.550.OR.IGSUB.EQ.552
     $           .OR.IGSUB.EQ.111.OR.IGSUB.EQ.112) THEN
               IDCYTMP = 0
               DO I = 1,MDCY(23,3)
                  IF (MDME(173+I,1).EQ.1
     $                 .AND.BRAT(173+I).GT.0.000) THEN
                     IDCYTMP = IDCYTMP + 1
                  ENDIF
               ENDDO
            ENDIF
#endif

            WRITE(*,*) 'Read from BASES file : ',GRESO,'\n'
            OPEN(11,FILE = GRESO,
     $           STATUS='old',FORM='formatted')

            DO WHILE (FOREVER)
               READ(11,'(a)',END=1000) OPTF
 
               IF (OPTF(2:8).EQ.'Summary') THEN
                  READ(11,'(a)',END=1000)
                  READ(11,'(a14,f10.3,a4)',END=1000)
     $                 TMPA,CHECM,TMPB
                  READ(11,'(a14,a30)',END=1000)
     $                 TMPA,COLP
 
                  READ(11,'(a)',END=1000)
                  READ(11,'(a14,i3,a2,a40)',END=1000)
     $                 TMPA,ICHSUB,TMPC,TMPD
 
                  READ(11,'(a)',END=1000)
                  READ(11,'(i4,a17)',END=1000) NI,TMPA
                  DO I=1,NI
                     READ(11,'(a25,i2,a2,f7.3,a4)',END=1000)
     $                    TMPA,NJ,TMPC,CHPTCUT(I),TMPB
                  ENDDO
                  DO I=1,NI
                     READ(11,'(a25,i2,a2,f7.3)',END=1000)
     $                    TMPA,NJ,TMPC,CHETACUT(I)
                  ENDDO
                  DO I=1,NI
                     READ(11,'(a25,i2,a2,f7.3)',END=1000)
     $                    TMPA,NJ,TMPC,CHRCONCUT(I)
                  ENDDO
                  READ(11,'(a29,f7.3,a4)',END=1000)
     $                 TMPA,CHMETCUT,TMPB
 
                  READ(11,'(f3.1)',END=1000) DUMMY
 
                  IF (ICOUP.EQ.5) THEN
                     READ(11,'(a35,i3,f8.3,a6)',END=1000)
     $                    TMPA,ICHCOUP,CHQ2,TMPB
                  ELSE
                     READ(11,'(a35,i3)',END=1000)
     $                    TMPA,ICHCOUP
                  ENDIF
 
                  IF (IFACT.EQ.0) THEN
                     READ(11,'(a35,i3)',END=1000)
     $                    TMPA,ICHFAQ
                  ELSE IF (IFACT.EQ.5) THEN
                     READ(11,'(a35,i3,f8.3,a6)',END=1000)
     $                    TMPA,ICHFAQ,CHFAQ2,TMPB
                  ELSE
                     READ(11,'(a35,i3)',END=1000)
     $                    TMPA,ICHFAQ
                  ENDIF

#if PSMODEL > 0
                  IF (IGSUB.EQ.408.OR.IGSUB.EQ.410.OR.IGSUB.EQ.411
     $                 .OR.IGSUB.EQ.412.OR.IGSUB.EQ.414.OR.IGSUB.EQ.415
     $                 .OR.IGSUB.EQ.419.OR.IGSUB.EQ.420.OR.IGSUB.EQ.421
     $                 .OR.IGSUB.EQ.422.OR.IGSUB.EQ.423.OR.IGSUB.EQ.424
     $                 .OR.IGSUB.EQ.425.OR.IGSUB.EQ.426.OR.IGSUB.EQ.428
     $                 .OR.IGSUB.EQ.429.OR.IGSUB.EQ.430.OR.IGSUB.EQ.427
     $                 .OR.IGSUB.EQ.431.OR.IGSUB.EQ.432.OR.IGSUB.EQ.433
     $                 .OR.IGSUB.EQ.434.OR.IGSUB.EQ.435.OR.IGSUB.EQ.436
     $                 .OR.IGSUB.EQ.437.OR.IGSUB.EQ.438.OR.IGSUB.EQ.439
     $                 .OR.IGSUB.EQ.500.OR.IGSUB.EQ.501.OR.IGSUB.EQ.502
     $                 .OR.IGSUB.EQ.503.OR.IGSUB.EQ.504.OR.IGSUB.EQ.505
     $                 .OR.IGSUB.EQ.506.OR.IGSUB.EQ.507.OR.IGSUB.EQ.508
     $                 .OR.IGSUB.EQ.509
     $                 .OR.IGSUB.EQ.550.OR.IGSUB.EQ.551.OR.IGSUB.EQ.552
     $                 .OR.IGSUB.EQ.560.OR.IGSUB.EQ.561
     $                 .OR.IGSUB.EQ.101.OR.IGSUB.EQ.102.OR.IGSUB.EQ.103
     $                 .OR.IGSUB.EQ.111.OR.IGSUB.EQ.112) THEN
                     IF (IDCYTMP.GT.0) THEN
                        READ(11,'(f3.1)',END=1000) DUMMY

                        READ(11,'(a20,i5)',END=1000) TMPA,ICHIDCY

                        DO I = 1,ICHIDCY
                           READ(11,'(a20,2i5,f8.3)',END=1000) 
     $                          TMPA,ICHNDCY(I),ICHMDCY(I),CHBDCY(I)
                        ENDDO
                     ENDIF
                  ENDIF
#endif

C...Check Parameter for Event Generation
C...Check Ecm
                  IF (ABS(CHECM-GRCECM).GT.0.0001) THEN
                     WRITE(*,*) 'Different CM Energy :',
     $                    '(BASES file):',CHECM,
     $                    'GeV and (PROGRAM):',GRCECM,'GeV'
                     GO TO 999
                  ENDIF
C...Check Collision Particles
                  IF (IGPP.EQ.0
     $                 .AND.COLP(8:20).EQ.'Proton-Proton') THEN
                  ELSE IF (IGPP.EQ.1
     $                    .AND.COLP(3:15).EQ.'Proton-Anti-P') THEN
                  ELSE
                     WRITE(*,*) COLP
                     WRITE(*,*) 'Mismatch in the Collision Particles.'
                     GO TO 999
                  ENDIF
C...Check Process
                  IF (ICHSUB.EQ.IGSUB) THEN
                     WRITE(*,*) '  Process : [',IGSUB,'] ',GRCPROC
                     WRITE(*,*)
                  ELSE
                     WRITE(*,*) 'GRC::Error: Different subprocess',
     $                    ' (BASES file):',ICHSUB,' and (PROGRAM):',
     $                    IGSUB
                  ENDIF
C...Check Pt Cut
                  DO I=1,NI
                     IF (ABS(CHPTCUT(I)-GRCPTCUT(I)).LT.0.001) THEN
                        IF (GRCPTCUT(I).GT.0.0) THEN
                           WRITE(*,*) '\tPt Cut of Particle',I,
     $                          ' : ',GRCPTCUT(I),' GeV'
                        ELSE
                           WRITE(*,*) '\tPt Cut of Particle',I,
     $                          ' : none'
                        ENDIF
                     ELSE
                        WRITE(*,*) 'Different Pt Cut for Particle',
     $                       I,' : (BASES file):',CHPTCUT(I),
     $                       'GeV and (PROGRAM):',GRCPTCUT(I),
     $                       'GeV'
                        GO TO 999
                     ENDIF
                  ENDDO
C...Check Eta Cut
                  DO I=1,NI
                     IF (ABS(CHETACUT(I)-GRCETACUT(I)).LT.0.001) THEN
                        IF (GRCETACUT(I).LT.100.0) THEN
                           WRITE(*,*) '\tEta Cut of Particle',I,
     $                          ' : ',GRCETACUT(I)
                        ELSE
                           WRITE(*,*) '\tEta Cut of Particle',I,
     $                          ' : none'
                        ENDIF
                     ELSE
                        WRITE(*,*) 'Different Eta Cut for Particle',
     $                       I,' : (BASES file):',CHETACUT(I),
     $                       ' and (PROGRAM):',GRCETACUT(I)
                        GO TO 999
                     ENDIF
                  ENDDO
C...Check Rcon Cut
                  DO I=1,NI
                     IF (ABS(CHRCONCUT(I)-GRCRCONCUT(I)).LT.0.001) THEN
                        IF (GRCRCONCUT(I).GT.0.0) THEN
                           WRITE(*,*) '\tRcone Cut of Particle',I,
     $                          ' : ',GRCRCONCUT(I)
                        ELSE
                           WRITE(*,*) '\tRcone Cut of Particle',I,
     $                          ' : none'
                        ENDIF
                     ELSE
                        WRITE(*,*) 'Different Rcone Cut for Particle',
     $                       I,' : (BASES file):',CHRCONCUT(I),
     $                       ' and (PROGRAM):',GRCRCONCUT(I)
                        GO TO 999
                     ENDIF
                  ENDDO
C...Check Missing Et(Pt) Cut
                  IF (ABS(CHMETCUT-GRCMETCUT).LT.0.001) THEN
                     IF (GRCMETCUT.GT.0.0) THEN
                        WRITE(*,*) '\tMissing Pt(Et) Cut : ',
     $                       GRCMETCUT,' GeV\n'
                     ELSE
                        WRITE(*,*) '\tMissing Pt(Et) Cut : none\n'
                     ENDIF
                  ELSE
                     WRITE(*,*) 'Different Missing Et(Pt) Cut :',
     $                    '(BASES file):',CHMETCUT,
     $                    'GeV and (PROGRAM):',GRCMETCUT,'GeV'
                     GO TO 999
                  ENDIF
 
                  WRITE(*,*)
C...Check Option of Renormalization Scale (GeV)
                  IF (ICHCOUP.EQ.ICOUP) THEN
                     WRITE(*,*) '\tOption of Renormalization Scale : ',
     $                    ICOUP
                  ELSE
                     WRITE(*,*)
     $                    'Different Option of Renormalization Scale :',
     $                    '(BASES file):',ICHCOUP,
     $                    ' and (PROGRAM):',ICOUP
                     GO TO 999
                  ENDIF
                  IF (ICHCOUP.EQ.5) THEN
                     IF (ABS(CHQ2-GRCQ).LT.0.001) THEN
                        WRITE(*,*) '\tFixed Renormalization Scale : ',
     $                       GRCQ,' GeV'
                     ELSE
                        WRITE(*,*)
     $                       'Different Renormalization Scale (GeV)',
     $                       ' or Option :','(BASES file):',CHQ2,
     $                       'GeV and (PROGRAM):',GRCQ,'GeV'
                        GO TO 999
                     ENDIF
                  ENDIF
 
C...Check Option of Factorization Scale (GeV)
                  IF (ICHFAQ.EQ.IFACT) THEN
                     WRITE(*,*) '\tOption of Factorization Scale : ',
     $                    IFACT
                  ELSE
                     WRITE(*,*)
     $                    'Different Option of Factorization Scale :',
     $                    '(BASES file):',ICHFAQ,
     $                    ' and (PROGRAM):',IFACT
                     GO TO 999
                  ENDIF
                  IF (ICHFAQ.EQ.0) THEN
                     WRITE(*,*) '\t    (taken as same as ',
     $                    'the renormalization scale.)'
                  ELSE IF (ICHFAQ.EQ.5) THEN
                     IF (ABS(CHFAQ2-GRCFAQ).LT.0.001) THEN
                        WRITE(*,*) '\tFixed Factorization Scale : ',
     $                       GRCFAQ,' GeV'
                     ELSE
                        WRITE(*,*)
     $                       'Different Factorization Scale (GeV)',
     $                       ' or Option :','(BASES file):',CHFAQ2,
     $                       'GeV and (PROGRAM):',GRCFAQ,'GeV'
                        GO TO 999
                     ENDIF
                  ENDIF

#if PSMODEL > 0
C...Check Forced Decay Option
                  IF (IGSUB.EQ.408.OR.IGSUB.EQ.410.OR.IGSUB.EQ.411
     $                 .OR.IGSUB.EQ.412.OR.IGSUB.EQ.414.OR.IGSUB.EQ.415
     $                 .OR.IGSUB.EQ.419.OR.IGSUB.EQ.420.OR.IGSUB.EQ.421
     $                 .OR.IGSUB.EQ.422.OR.IGSUB.EQ.423.OR.IGSUB.EQ.424
     $                 .OR.IGSUB.EQ.425.OR.IGSUB.EQ.426.OR.IGSUB.EQ.428
     $                 .OR.IGSUB.EQ.429.OR.IGSUB.EQ.430.OR.IGSUB.EQ.427
     $                 .OR.IGSUB.EQ.431.OR.IGSUB.EQ.432.OR.IGSUB.EQ.433
     $                 .OR.IGSUB.EQ.434.OR.IGSUB.EQ.435.OR.IGSUB.EQ.436
     $                 .OR.IGSUB.EQ.437.OR.IGSUB.EQ.438.OR.IGSUB.EQ.439
     $                 .OR.IGSUB.EQ.500.OR.IGSUB.EQ.501.OR.IGSUB.EQ.502
     $                 .OR.IGSUB.EQ.503.OR.IGSUB.EQ.504.OR.IGSUB.EQ.505
     $                 .OR.IGSUB.EQ.506.OR.IGSUB.EQ.507.OR.IGSUB.EQ.508
     $                 .OR.IGSUB.EQ.509
     $                 .OR.IGSUB.EQ.550.OR.IGSUB.EQ.551.OR.IGSUB.EQ.552
     $                 .OR.IGSUB.EQ.560.OR.IGSUB.EQ.561
     $                 .OR.IGSUB.EQ.101.OR.IGSUB.EQ.102.OR.IGSUB.EQ.103
     $                 .OR.IGSUB.EQ.111.OR.IGSUB.EQ.112) THEN
                     IF (IDCYTMP.GT.0) THEN
                        WRITE(*,*)
                        DO I = 1,IDCYTMP
                           IF (MDME(ICHNDCY(I),1).EQ.1
     $                          .AND.MDME(ICHNDCY(I),1).EQ.ICHMDCY(I)
     $                          .AND.ABS(BRAT(ICHNDCY(I))-CHBDCY(I))
     $                          .LT.0.001) THEN
                              WRITE(*,*) '\tW or Z decay :',ICHNDCY(I),
     $                             ' Br. ',BRAT(ICHNDCY(I)),' ON'
                           ELSE
                              WRITE(*,*) 'GRC::Error:',
     $                             ' different decay combination',
     $                             ' or different Br..'
                              GO TO 999
                           ENDIF
                        ENDDO
                     ENDIF
                  ENDIF
#endif
               ENDIF 
               IF (OPTF(15:25).EQ.'===========') THEN
                  READ(11,'(a24,d15.6,a5)',END=1000) TMPA,CRGET,TMPB
                  WRITE(*,*)
                  WRITE(*,*) TMPA,CRGET,TMPB
                  GO TO 1000
               ENDIF
            ENDDO
 1000       CONTINUE
            CLOSE(11)
 
            GRCSIG(JPROC) = CRGET

         ELSE IF (IBSWRT.EQ.2) THEN

C...Check process.
            IF (NPRUP.GT.1) THEN
               WRITE(*,*) 'GRC::Error: Option IBSWRT=2',
     $              ' is only available for single process.'
               GO TO 999
            ENDIF

            WRITE(*,*) 'Read from ASCII file : ',GDATA,'\n'
            OPEN(14,FILE = GDATA,STATUS='old',FORM='formatted')

            DO WHILE (FOREVER)
               READ(14,'(a)',END=222) OPTF
               WRITE(*,*) OPTF
 
               IF (OPTF(2:8).EQ.'Process') THEN
                  READ(14,'(a4,a6,f10.3)',END=222) TMPA,TMPC,CHECM
                  WRITE(*,*) ' ECM     :',CHECM
               ELSE IF (OPTF(2:8).EQ.'StrFun1') THEN
                  IF (OPTF(16:18).EQ.'CTE'
     $                 .AND.OPTF(20:21).EQ.'5L') THEN
                     PDFGUP(1) = 4
                     PDFSUP(1) = 46
                  ELSE IF (OPTF(16:18).EQ.'CTE'
     $                 .AND.OPTF(20:21).EQ.'5D') THEN
                     PDFGUP(1) = 4
                     PDFSUP(1) = 47
                  ELSE IF (OPTF(16:18).EQ.'CTE'
     $                 .AND.OPTF(20:21).EQ.'5M') THEN
                     PDFGUP(1) = 4
                     PDFSUP(1) = 48
                  ENDIF

                  IF (OPTF(24:27).EQ.'prot') THEN
                     IDBMUP(1) = 2212
                  ELSE IF (OPTF(24:27).EQ.'anti') THEN
                     IDBMUP(1) = -2212
                  ELSE
                     WRITE(*,*) 'GRC::Error: Strange Col. particle 1.'
                     GO TO 999
                  ENDIF
               ELSE IF (OPTF(2:8).EQ.'StrFun2') THEN
                  IF (OPTF(16:18).EQ.'CTE'
     $                 .AND.OPTF(20:21).EQ.'5L') THEN
                     PDFGUP(2) = 4
                     PDFSUP(2) = 46
                  ELSE IF (OPTF(16:18).EQ.'CTE'
     $                 .AND.OPTF(20:21).EQ.'5D') THEN
                     PDFGUP(2) = 4
                     PDFSUP(2) = 47
                  ELSE IF (OPTF(16:18).EQ.'CTE'
     $                 .AND.OPTF(20:21).EQ.'5M') THEN
                     PDFGUP(2) = 4
                     PDFSUP(2) = 48
                  ENDIF

                  IF (OPTF(24:27).EQ.'prot') THEN
                     IDBMUP(2) = 2212
                  ELSE IF (OPTF(24:27).EQ.'anti') THEN
                     IDBMUP(2) = -2212
                  ELSE
                     WRITE(*,*) 'GRC::Error: Strange Col. particle 2.'
                     GO TO 999
                  ENDIF

                  READ(14,'(a16,f12.5,a6)',END=222) TMPA,SIGEV,TMPB
                  WRITE(*,*) ' Cross Section :',SIGEV,' (pb)'

                  READ(14,'(a10,i3)',END=222) TMPA,NUP
                  WRITE(*,*) ' NUP     :',NUP

                  WRITE(*,*)
                  WRITE(*,FMT='(a11,i6,a15,i6)') 
     $                 'IDBMUP(1)=',IDBMUP(1),'IDBMUP(2)=',IDBMUP(2)
                  WRITE(*,FMT='(a11,i6,a15,i6)') 
     $                 'PDFGUP(1)=',PDFGUP(1),'PDFGUP(2)=',PDFGUP(2)
                  WRITE(*,FMT='(a11,i6,a15,i6)') 
     $                 'PDFSUP(1)=',PDFSUP(1),'PDFSUP(2)=',PDFSUP(2)

C...Check Parameter for Event Generation
C...Check Ecm
                  IF (ABS(CHECM-GRCECM).GT.0.0001) THEN
                     WRITE(*,*) 'Different CM Energy :',
     $                    '(Ascii file):',CHECM,
     $                    'GeV and (PROGRAM):',GRCECM,'GeV'
                     GO TO 999
                  ENDIF
               ENDIF

               IF (OPTF(2:3).EQ.'#') GO TO 222
            ENDDO

 222        CONTINUE

            WRITE(*,*) 'Read Ascii file!!'

            GRCSIG(JPROC) = SIGEV
         ELSE
            WRITE(*,*) 'GRC::Error: Wrong IBSWRT=',IBSWRT,'.'
            WRITE(*,*) '            Set IBSWRT <= 2 .'
            GO TO 999
         ENDIF
 
#if PSMODEL == 61
         CALL PYUPIN(ISUB,GRCPROC,GRCSIG(JPROC)*1E-9)
#endif
 
C...For Debug
C+-------------------------------------------------+
#ifdef GRCDEBUG
         OPEN(23,FILE = GRCBSFILE(JPROC),
     $        STATUS='old',FORM='unformatted')
         CALL BSREAD(23)
         CLOSE(23)
#endif
C+-------------------------------------------------+
 
C...Event Generation (ibsint=0)
      ELSE IF (IBSINT.EQ.0) THEN
 
         IF (IBSWRT.LT.2) THEN

#ifndef GRCDEBUG
            OPEN(23,FILE = GRCBSFILE(JPROC),
     $           STATUS='old',FORM='unformatted')
            CALL BSREAD(23)
            CLOSE(23)
#endif

#if PSMODEL == 0
C...Set Seed for random number
            IRSEED=48828125*IRSEED
            IRSEED=IAND(IRSEED,NZZ)
            CALL DRNSET( IRSEED )
#endif

C...SPRING Event Generation
            MXTRY = 100000
            CALL SPRING( FUNC, MXTRY )

C...OK, accept
            IBSINT = 2
            YACOB = 1.0D0
            XDUMMY = FUNC(X)

C...For Debug
C+-------------------------------------------------+
#ifdef GRCDEBUG
            DO I = 1, NDIM
               CALL XHFILL(I,XRUN(I),XDUMMY)
            ENDDO
#endif
C+-------------------------------------------------+

C...Smear to PYTHIA
            CALL GRCSSMR

         ELSE IF (IBSWRT.EQ.2) THEN

C...Smear to PYTHIA
            CALL GRCSSMR
 
         ELSE
            WRITE(*,*) 'GRC::Error: Wrong ibswrt'
            GO TO 999
         ENDIF
 
      ENDIF
 
C...Set Cross Section
      IF (IBSINT.EQ.1.OR.IREJ.LT.1) THEN
#if PSMODEL == 61
         SIGEV = GRCSIG(JPROC)*1E-9
#else
         SIGEV = GRCSIG(JPROC)
#endif
      ELSE
         SIGEV = 0.0D0
      ENDIF
 
      RETURN
 999  CONTINUE
      STOP
      END
 
C-------------
C Write logo.
C-------------
      SUBROUTINE GRCLOGO
      IMPLICIT REAL*8(A-H,O-Z)
 
      INTEGER I,J
      CHARACTER LOGO(13)*48, LINE*79
 
      DATA (LOGO(J),J=1,13)/
     $     '      GGG  RRRR          PPPP  PPPP   AAA       ',
     $     '     G   G R   R   @@@@  P   P P   P A   A      ',
     $     '     G     R   R  @ @@ @ P   P P   P A   A      ',
     $     '     G GGG RRRR  @ @ @ @ PPPP  PPPP  AAAAA      ',
     $     '     G   G R  R  @ @@ @  P     P     A   A      ',
     $     '      GGG  R   R  @@@@@@ P     P     A   A      ',
     $     '    =======================================     ',
     $     '      GRace At Proton-Proton/Anti-proton        ',
     $     '        This is GR@PPA_4b version 2.01,         ',
     $     '       coded by S.Tsuno (tsuno@fnal.gov)        ',
     $     '      with Minami Tateya Collab. and ATLAS-J.   ',
     $     '  http://atlas.kek.jp/physics/nlo-wg/grappa.html',
     $     '             Ref: hep-ph/0204222                '/
 
      DO I = 1,22
         LINE = ' '
         LINE(2:3) = '**'
         LINE(78:79) = '**'
 
         IF (I.LT.3) THEN
            LINE(4:77) = '**************************************'//
     $           '************************************'
         ELSE IF (I.EQ.3) THEN
            LINE(6:15) = 'Welcome to'
         ELSE IF (I.GT.4.AND.I.LT.13) THEN
            LINE(5:53) = LOGO(I-4)
         ELSE IF (I.GT.13.AND.I.LT.17) THEN
            LINE(5:53) = LOGO(I-5)
         ELSE IF (I.GT.17.AND.I.LT.20) THEN
            LINE(5:53) = LOGO(I-6)
         ELSE IF (I.GT.20) THEN
            LINE(4:77) = '**************************************'//
     $           '************************************'
         ENDIF
         WRITE(*,'(a79)') LINE
      ENDDO
 
      RETURN
      END

C-------------------
C Make Decay Table.
C-------------------
      SUBROUTINE GRCSDCY
      IMPLICIT REAL*8(A-H,O-Z)

#include "define.h"

      INCLUDE 'inclk.h'
      INCLUDE 'grchad.inc'

C...Particle properties + some flavour parameters.
#if PSMODEL == 61
      COMMON/PYDAT2/KCHG(500,4),PMAS(500,4),PARF(2000),VCKM(4,4)
      COMMON/PYDAT3/MDCY(500,3),MDME(4000,2),BRAT(4000),KFDP(4000,5)
#elif PSMODEL == 62
      COMMON/PYDAT2/KCHG(500,4),PMAS(500,4),PARF(2000),VCKM(4,4)
      COMMON/PYDAT3/MDCY(500,3),MDME(8000,2),BRAT(8000),KFDP(8000,5)
#endif

      INTEGER I,J,NUMWDCY,NUMZDCY
      INTEGER IGWDCYTMP(40),IGZDCYTMP(32),IPOSDCY(36),ITMPDCY(36)
      REAL*8 TMPZBR(16),TMPWBR(20),TMPCKM(4,4)

C...W+/Z decay.
      DATA IGWDCYTMP/ -1,  2, -1,  4, -1,  6, -1,  8,
     $                -3,  2, -3,  4, -3,  6, -3,  8,
     $                -5,  2, -5,  4, -5,  6, -5,  8,
     $                -7,  2, -7,  4, -7,  6, -7,  8,
     $               -11, 12,-13, 14,-15, 16,-17, 18/

      DATA IGZDCYTMP/  1, -1,  2, -2,  3, -3,  4, -4,
     $                 5, -5,  6, -6,  7, -7,  8, -8,
     $                11,-11, 12,-12, 13,-13, 14,-14,
     $                15,-15, 16,-16, 17,-17, 18,-18/

      DATA (TMPZBR(I),I=1,16)/0.153995D0, 0.11942D0,  0.153984D0,
     $                        0.119259D0, 0.152272D0, 0.0D0,
     $                        0.0D0,      0.0D0,      0.033576D0,
     $                        0.066806D0, 0.033576D0, 0.066806D0,
     $                        0.0335D0,   0.066806D0, 0.0D0,
     $                        0.0D0/      

      DATA (TMPWBR(I),I=1,20)/0.321369D0, 0.016494D0, 0.0D0,
     $                        0.0D0,      0.016502D0, 0.320615D0,
     $                        0.0D0,      0.0D0,      0.00001D0,
     $                        0.000591D0, 0.0D0,      0.0D0,
     $                        0.0D0,      0.0D0,      0.0D0,
     $                        0.0D0,      0.108166D0, 0.108166D0,
     $                        0.108087D0, 0.0D0/

C...CKM
      DATA ((TMPCKM(I,J),J=1,4),I=1,4)/
     $     0.95113D0,  0.04884D0,  0.00003D0,  0.00000D0,
     $     0.04884D0,  0.94940D0,  0.00176D0,  0.00000D0,
     $     0.00003D0,  0.00176D0,  0.99821D0,  0.00000D0,
     $     0.00000D0,  0.00000D0,  0.00000D0,  1.00000D0/

      DATA ITMPDCY/ 1, 0, 0,-1, 0, 1, 0,-1, 0, 0, 0,-1,-1,-1,-1,-1,
     $              1, 1, 1,-1, 1, 1, 1, 1, 1, 1,-1,-1, 1, 1, 1, 1,
     $              1, 1,-1,-1/

      DO I = 1,40
         IGWDCY(I) = IGWDCYTMP(I)
      ENDDO
      DO I = 1,32
         IGZDCY(I) = IGZDCYTMP(I)
      ENDDO

      GWDCYMS(1)  = AMDQ(1)
      GWDCYMS(2)  = AMUQ(1)
      GWDCYMS(3)  = AMDQ(2)
      GWDCYMS(4)  = AMUQ(2)
      GWDCYMS(5)  = AMDQ(3)
      GWDCYMS(6)  = AMUQ(3)
      GWDCYMS(7)  = AMX
      GWDCYMS(8)  = AMY
      GWDCYMS(9)  = 0.0D0
      GWDCYMS(10) = 0.0D0
      GWDCYMS(11) = AMLP(1)
      GWDCYMS(12) = AMNU(1)
      GWDCYMS(13) = AMLP(2)
      GWDCYMS(14) = AMNU(2)
      GWDCYMS(15) = AMLP(3)
      GWDCYMS(16) = AMNU(3)

#if PSMODEL > 0
C...Set W/Z Br.
      DO I=1,20
         GRCWBR(I) = BRAT(189+I)
      ENDDO
      DO I=1,16
         GRCZBR(I) = BRAT(173+I)
      ENDDO

C...Set CKM
      DO I = 1,4
         DO J = 1,4
            GRCCKM(I,J) = VCKM(I,J)
         ENDDO
      ENDDO

      NUMWDCY = MDCY(24,3)
      NUMZDCY = MDCY(23,3)

      DO I = 1,NUMWDCY
         IPOSDCY(I) = MDME(189+I,1)
      ENDDO
      DO I = 1,NUMZDCY
         IPOSDCY(I+20) = MDME(173+I,1)
      ENDDO

#else
C...Set W/Z Br.
      DO I=1,20
         GRCWBR(I) = TMPWBR(I)
      ENDDO
      DO I=1,16
         GRCZBR(I) = TMPZBR(I)
      ENDDO

C...Default is CKM on.
      DO I=1,4
         DO J=1,4
            GRCCKM(I,J) = TMPCKM(I,J)
         ENDDO
      ENDDO

      NUMWDCY = 20
      NUMZDCY = 16

      DO I=1,36
         IPOSDCY(I) = ITMPDCY(I)
      ENDDO
#endif

      IF (IGSUB.EQ.419.OR.IGSUB.EQ.420.OR.IGSUB.EQ.551) THEN
         IGNDCY = 0
         GDCYSUM = 0.0d0
         DO I = 1,NUMWDCY
            IF (IPOSDCY(I).EQ.1.AND.GRCWBR(I).GT.0.0D0) THEN
               DO J = 1,NUMWDCY
                  IF (IPOSDCY(I).EQ.1.AND.GRCWBR(J).GT.1.0D-4) THEN
                     IGNDCY = IGNDCY + 1
                     GDCYPR1(IGNDCY) = I
                     GDCYPR2(IGNDCY) = J
                     GDCYRAT(IGNDCY) = GRCWBR(I)*GRCWBR(J)
                     GDCYSUM = GDCYSUM + GDCYRAT(IGNDCY)
                  ENDIF
               ENDDO
            ENDIF
         ENDDO
      ELSE IF (IGSUB.EQ.408.OR.IGSUB.EQ.412.OR.IGSUB.EQ.421
     $        .OR.IGSUB.EQ.422.OR.IGSUB.EQ.423.OR.IGSUB.EQ.424
     $        .OR.IGSUB.EQ.425.OR.IGSUB.EQ.426.OR.IGSUB.EQ.427
     $        .OR.IGSUB.EQ.428.OR.IGSUB.EQ.429.OR.IGSUB.EQ.430
     $        .OR.IGSUB.EQ.431.OR.IGSUB.EQ.432.OR.IGSUB.EQ.433
     $        .OR.IGSUB.EQ.434.OR.IGSUB.EQ.435.OR.IGSUB.EQ.436
     $        .OR.IGSUB.EQ.437.OR.IGSUB.EQ.438.OR.IGSUB.EQ.439
     $        .OR.IGSUB.EQ.464.OR.IGSUB.EQ.465.OR.IGSUB.EQ.467
     $        .OR.IGSUB.EQ.560.OR.IGSUB.EQ.561
     $        .OR.IGSUB.EQ.101.OR.IGSUB.EQ.102.OR.IGSUB.EQ.103) THEN
         IGNDCY = 0
         GDCYSUM = 0.0d0
         DO I = 1,NUMWDCY
            IF (IPOSDCY(I).EQ.1.AND.GRCWBR(I).GT.1.0D-4) THEN
               IGNDCY = IGNDCY + 1
               GDCYPR1(IGNDCY) = I
               GDCYRAT(IGNDCY) = GRCWBR(I)
               GDCYSUM = GDCYSUM + GDCYRAT(IGNDCY)
            ENDIF
         ENDDO
      ELSE IF (IGSUB.EQ.410.OR.IGSUB.EQ.411.OR.IGSUB.EQ.414
     $        .OR.IGSUB.EQ.415.OR.IGSUB.EQ.500.OR.IGSUB.EQ.501
     $        .OR.IGSUB.EQ.502.OR.IGSUB.EQ.503.OR.IGSUB.EQ.504
     $        .OR.IGSUB.EQ.505.OR.IGSUB.EQ.506.OR.IGSUB.EQ.507
     $        .OR.IGSUB.EQ.508.OR.IGSUB.EQ.509
     $        .OR.IGSUB.EQ.111.OR.IGSUB.EQ.112) THEN
         IGNDCY = 0
         GDCYSUM = 0.0d0
         DO I = 1,NUMZDCY
            IF (IPOSDCY(I+20).EQ.1.AND.GRCZBR(I).GT.1.0D-4) THEN
               IGNDCY = IGNDCY + 1
               GDCYPR1(IGNDCY) = I
               GDCYRAT(IGNDCY) = GRCZBR(I)
               GDCYSUM = GDCYSUM + GDCYRAT(IGNDCY)
            ENDIF
         ENDDO
      ELSE IF (IGSUB.EQ.550) THEN
         IGNDCY = 0
         GDCYSUM = 0.0d0
         DO I = 1,NUMZDCY
            IF (IPOSDCY(I+20).EQ.1.AND.GRCZBR(I).GT.1.0D-4) THEN
               DO J = 1,NUMZDCY
                  IF (IPOSDCY(I+20).EQ.1.AND.GRCZBR(J).GT.1.0D-4) THEN
                     IGNDCY = IGNDCY + 1
                     GDCYPR1(IGNDCY) = I
                     GDCYPR2(IGNDCY) = J
                     GDCYRAT(IGNDCY) = GRCZBR(I)*GRCZBR(J)
                     GDCYSUM = GDCYSUM + GDCYRAT(IGNDCY)
                  ENDIF
               ENDDO
            ENDIF
         ENDDO
      ELSE IF (IGSUB.EQ.552) THEN
         IGNDCY = 0
         GDCYSUM = 0.0d0
         DO I = 1,NUMZDCY
            IF (IPOSDCY(I+20).EQ.1.AND.GRCZBR(I).GT.1.0D-4) THEN
               DO J = 1,NUMWDCY
                  IF (IPOSDCY(I).EQ.1.AND.GRCWBR(J).GT.1.0D-4) THEN
                     IGNDCY = IGNDCY + 1
                     GDCYPR1(IGNDCY) = I
                     GDCYPR2(IGNDCY) = J
                     GDCYRAT(IGNDCY) = GRCZBR(I)*GRCWBR(J)
                     GDCYSUM = GDCYSUM + GDCYRAT(IGNDCY)
                  ENDIF
               ENDDO
            ENDIF
         ENDDO
      ENDIF

      RETURN
      END
